//###### HEADER ######
//
//		####################################
// 			This is SpAcAn 1.04 (June 3, 2010)
//			Project website: http://www.spacan.net
//			Email: postmaster@spacan.net
//		####################################
//
//### NOTE ###
//
// Compatibility issue with WINE (Wine 1.1.42)
// PopupMenu controls not functional (therefore not used in SpAcAn)
//
//### CHANGELOG OF v. 1.04 ###
//
// 1. A long awaited function is finally implemented: an automatic fit (double exponential) of the decay of all tagged events. As the other fit functions, this new function
// is intended to be used with postsynaptic currents. It is launched using the "Fit all events" button in the "Kin." tab of the window DetectedEvents.
// It is necessary to calculate the average event first, since the function will use the fit parameters of the average as a first guess for individual events.
// 2. New tab "Rasters" in MainWindow. Samples of traces can be displayed easily with this new tab.
// The range to show is defined by the x scaling of the channels in their respective subwindows (inside MainWindow).
// The user defines the number of rows to use to display the traces. The original recording is split into segments of
// equal length to achieve the number of rows requested.
// Channel1 and 2 can be displayed as traces or as ticks marking the time of detected events.
// Tags can be used optionally to only display tagged events as ticks.
// 3. New scale bar management system, available in certain windows (rasters and superimpose tagged events).
// Simply drag out a marquee in the window and select the function "DrawScale". A scale bar will appear with the x and y scaling of the marquee.
// A set of controls allow to 1) choose which axis to use when drawing the scale bar, 2) select existing scale bars and 3) modify their length.
// 4. New tab "TTLs" in MainWindow. This new tab is designed to exploit recordings in which one channel contains stimulation pulses.
// Pulses are treated as events (and must be detected beforehand) and are shown as blue boxes.
// The user can align traces corresponding to successive pulses by entering the range of pulse numbers to use, 
// or can display the recording with the x scaling used in MainWindow.
// Pulses can be used to plot PSTHs of events in the other channel (using a range of pulse numbers).
// Example of application: optogenetic stimulations with one channel containing the light pulses data.
// 5. SpAcAn now keeps a memory of all variables and checkboxes of the MainWindow (it stores them in a multidimensional wave and 
// loads them at the next initialization).
// 6. Change in loading waves from Igor folders: the user can now type in a wave name.
// 7. Fixes a bug that occurred in average calculation when using 2 channels with different sampling intervals.
// 8. Added a button "D&D" (Diff & Detect) to perform the differentiate operation and the detection in one click.
// 9. General improvements of the GUI.
// 10. Code cleanup: major functions rewritten to work with both channels (saves space and will facilitate future modifications).
// 11. Fits more accurate. Possibilty to extrapolate to peak or to fit only from the desired decay percentage.
// 12. Better display of fit parameters also for sigmoid fits.
// 13. Added a statement to restrict use of SpAcAn with compatible versions of igor pro
//
//### FIXME ###
//
// Pb in sig + dbexp fit
//
//### TODO ###
//
// Use WaveList in ScaleBars folder to speed up scale bar management functions
// improve distance between text and scale bar (take a percentage of the graph limits
// Extend scale bar management to TTLs plot
// Check if pulses in pulse range have the same width (tab TTLs)
// Put Sync tags (in pairs tab) into manage tags; use the remaining tab for MAS
// Group variables associated to checkboxes states into a specific folder?
// Group waves related to each channel in distinct folders?
// More info on the notebook that gives the name and location of important waves
// Generalize as many functions as possible.
//
//###### END OF HEADER ######

#pragma rtGlobals=1
#include <WaveSelectorWidget>
#pragma IgorVersion=5.05

MENU 	"SpAcAn"
	"New Analysis", Initialize(0)
	SubMenu "Bring to Front..."
		"Main Window", DoWindow/F MainWindow
		SubMenu "Detected Events"
			"Channel 1", DoWindow/F DetectedEvents1
			"Channel 2", DoWindow/F DetectedEvents2
		End
	End
	"------------------------"
	SubMenu "Help"
		"List of useful waves", ListOfWaves(1)
	End
	"Show procedure", ShowCode(2)
	"About SpAcAn...", About(3)
END

// ========== SHOW CODE ========== 
Function ShowCode(numero)
	Variable numero
	DisplayProcedure "Initialize"
End

// ========== ABOUT SPACAN ==========
FUNCTION About(numero)
	Variable numero
	// Builds panel
	DoWindow/K Window_About; Display/W=(300,100,600,300) as "About SpAcAn..."; DoWindow/C Window_About
	NewPanel/K=1/N=Panel_About/HOST=Window_About/FG=(FL, FT, gRight, FB )/W=(1,1,1,1)
	ModifyPanel cbRGB=(65500,65500,65500)
	// Add Logo on top
	DrawPict/W=Window_About#Panel_About 140, 16, 1, 1, ProcGlobal#SpAcAnLogo
	// Close button
	Button ButtonCloseAbout pos={350,10},	size={40,20}, proc=ButtonProcCloseAbout, 	title="Close"
	// Text
	SetDrawEnv fsize= 14, fstyle=1;DrawText 105,76,"Version 1.04 - June 3, 2010"
	
	SetDrawEnv fsize= 14;DrawText 10,115,"SpAcAn (Spontaneous Activity Analysis) is a collection of"
	SetDrawEnv fsize= 14;DrawText 10,135,"IGOR Pro functions for the detection and analysis of"
	SetDrawEnv fsize= 14;DrawText 10,155,"spontaneous activity in 1 or 2-channel electrophysiological"
	SetDrawEnv fsize= 14;DrawText 10,175,"recordings."
	SetDrawEnv fsize= 14;DrawText 10,215,"By Guillaume P. Dugu & Charly Rousseau."
//	SetDrawEnv fsize= 14;DrawText 10,235,"Project website:"
//	SetDrawEnv fsize= 14, fstyle=4;DrawText 115,235,"http://www.spacan.net/"
	Button ButtonBrowseSpAcAnWebsite, pos={115,235},	fstyle=4, size={160,20}, proc=browseSpAcAnWebsite, 	title="\\K(0,0,65280)http://www.spacan.net/"
END

FUNCTION ButtonProcCloseAbout(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_About
END

Function browseSpAcAnWebsite(ctrlName) : ButtonControl
	string ctrlName
	
	string spacanUrl = "http://www.spacan.net/"
	browseUrl spacanUrl
end
// ========== END OF ABOUT SPACAN ==========

FUNCTION ListOfWaves(numero)
	Variable numero
	DoWindow/K SpAcAnWaves
	
	NewNotebook/F=1/K=1/N=SpAcAnWaves/W=(80,100,800,400) as "SpAcAn waves"
	
	Notebook SpAcAnWaves fsize=14, fstyle=1, text="Category \t \t \t \t Wave name \t \t Description\r"
	Notebook SpAcAnWaves text="\r"
	
	Notebook SpAcAnWaves fsize=12, fstyle=1, text="SpAcAn's memory"
	
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t Params"
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t blabla\r"
	Notebook SpAcAnWaves text="\r"
	
	Notebook SpAcAnWaves fsize=12, fstyle=1, text="Timing of detected events"
	
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t Occurrence#"
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t Half-rise time\r"

	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t \t \t RealPeakPosition#"
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t time of peak\r"

	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t \t \t RT20#"
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t time of 20% rise\r"

	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t \t \t RT80#"
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t time of 80% rise\r"

	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t \t \t RiseTime2080#"
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t blabla\r"

	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t \t \t EventFoorTime#"
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t blabla\r"
	Notebook SpAcAnWaves text="\r"

	Notebook SpAcAnWaves fsize=12, fstyle=1, text="Amplitude of detected events"
	
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t RealAmp#"
	Notebook SpAcAnWaves fsize=12, fstyle=0, text="\t \t \t blabla\r"
	Notebook SpAcAnWaves text="\r"

	Notebook SpAcAnWaves fsize=12, fstyle=1, text="Tags\r"
	
END

// ========== TABS HIDDING IN MAIN WINDOW ==========

// First group of tabs: detection parameters
FUNCTION MainWindowTab1Proc(CtrlName, TabNum) : TabControl
	String CtrlName
	Variable TabNum
	Nvar MainWindowTab1	// Memory of the tab state
	MainWindowTab1 = TabNum
	Variable isTab0= tabNum==0
	Variable isTab1= tabNum==1
	// To add new tabs:
	// Variable isTab2= tabNum==2
	// Variable isTab3= tabNum==3
	// Note: disable=0 means "show", disable=1 means "hide"
	
	// Detection params channel1
	ModifyControl SetVarBSLpnts1		disable= !isTab0	// hide if not Tab 0
	ModifyControl SetVarRTPnts1 		disable= !isTab0
	ModifyControl SetVarPpnts1 		disable= !isTab0
	ModifyControl DisplayBSLms1 		disable= !isTab0
	ModifyControl DisplayRTms1 		disable= !isTab0
	ModifyControl DisplayPms1 		disable= !isTab0
	ModifyControl SetWPD1 			disable= !isTab0
	ModifyControl SetThreshold1 		disable= !isTab0
	ModifyControl DW1 				disable= !isTab0
	ModifyControl NoEch1 			disable= !isTab0
	ModifyControl FreqEch1 			disable= !isTab0
	ModifyControl PosDeflection1		disable= !isTab0
	ModifyControl NegDeflection1		disable= !isTab0
	ModifyControl DisplaySI1			disable= !isTab0
	ModifyControl DiffAndDetect1		disable= !isTab0
	
	If (tabNum==0) // If the selected tab is the first
		NVar Diff1Clicked
		If (Diff1Clicked ==1) // If diff already performed
			ModifyControl Detect1 		disable = 0 	// Usable control
		Else
			ModifyControl Detect1 		disable = 2 	// Grayed control
		EndIf
	Else
		ModifyControl Detect1 			disable= 1	// hidden control
	EndIf
	
	// Part not in use but that may be usefull to call certain functions
	// If (TabNum == 0)
	//	SetActiveSubWindow MainWindow#MainPanel	
	// EndIf
		
	// Detection params channel2
	ModifyControl SetVarBSLpnts2 	disable= !isTab1	// hide if not Tab 1
	ModifyControl SetVarRTPnts2 		disable= !isTab1
	ModifyControl SetVarPpnts2 		disable= !isTab1
	ModifyControl DisplayBSLms2 		disable= !isTab1
	ModifyControl DisplayRTms2 		disable= !isTab1
	ModifyControl DisplayPms2 		disable= !isTab1
	ModifyControl SetWPD2 			disable= !isTab1
	ModifyControl SetThreshold2 		disable= !isTab1
	ModifyControl DW2 				disable= !isTab1
	ModifyControl NoEch2 			disable= !isTab1
	ModifyControl FreqEch2 			disable= !isTab1
	ModifyControl PosDeflection2		disable= !isTab1
	ModifyControl NegDeflection2		disable= !isTab1
	ModifyControl DisplaySI2			disable= !isTab1
	ModifyControl DiffAndDetect2		disable= !isTab1
	
	If (tabNum == 1) // If the selected tab is the second
		NVar Diff2Clicked
		If (Diff2Clicked == 1)// If diff already performed
			ModifyControl Detect2			disable = 0 // Usable control
		Else
			ModifyControl Detect2			disable = 2 // Grayed control
		EndIf
	Else
		ModifyControl Detect2 			disable= 1	// hidden control
	EndIf

	// Part not in use but that may be usefull to call certain functions
	// If (TabNum == 1)	
	// 	SetActiveSubWindow MainWindow#MainPanel
	// EndIf
	
	// ADD YOU TAB HERE

	return 0
END

// Second group of tabs: Plots and Correlogram
FUNCTION MainWindowTab2Proc(CtrlName, TabNum) : TabControl
	String CtrlName
	Variable TabNum
	Nvar MainWindowTab2	// Memory of the tab state
	MainWindowTab2 = TabNum
	Variable isTab0= tabNum==0
	Variable isTab1= tabNum==1
	Variable isTab2= tabNum==2
	Variable isTab3= tabNum==3
	
	// To add new tabs:
	// Variable isTab2= tabNum==2
	// Variable isTab3= tabNum==3
	// Note: disable=0 means "show", disable=1 means "hide"

	// Freq/Amp tab
	ModifyControl CheckBoxUseTags	disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonPlot 			disable= !isTab0
	ModifyControl SetFvsTBin 			disable= !isTab0
	ModifyControl SetAvsTBin 			disable= !isTab0
	ModifyControl SetAmpHistoBin 		disable= !isTab0
	ModifyControl TitleBoxFAvsT1 		disable= !isTab0
	ModifyControl TitleBoxFAvsT2 		disable= !isTab0
	ModifyControl CBAvsTShowAv 		disable= !isTab0
	ModifyControl CBAvsTShowEvents 	disable= !isTab0
	ModifyControl SetISIBin 			disable= !isTab0
	ModifyControl GBFvsT 			disable= !isTab0
	ModifyControl GBAvsT 			disable= !isTab0
	ModifyControl GBAmpHisto 			disable= !isTab0
	ModifyControl GBISIHisto 			disable= !isTab0

	// Part not in use but that may be usefull to call certain functions
	// If (TabNum == 0)	
	//	SetActiveSubWindow MainWindow#MainPanel
	// EndIf
	
	// Correlogram tab
	ModifyControl DoCorrelogram 				disable= !isTab1	// hide if not Tab 1
	ModifyControl SetCorrTimeWindow 		disable= !isTab1
	ModifyControl SetCorrBin 					disable= !isTab1
	ModifyControl CheckBoxUsePeakRef 		disable= !isTab1
	ModifyControl CheckBoxUseHRRef 		disable= !isTab1
	ModifyControl CheckBoxUsePeakTest 		disable= !isTab1
	ModifyControl CheckBoxUseHRTest 		disable= !isTab1
	ModifyControl RefCh1Box 					disable= !isTab1
	ModifyControl RefCh2Box						disable= !isTab1
	ModifyControl TestCh1Box 					disable= !isTab1
	ModifyControl TestCh2Box					disable= !isTab1
	ModifyControl RefTitleBox  					disable= !isTab1
	ModifyControl TestTitleBox  					disable= !isTab1
	ModifyControl CheckBoxCCTypeRaw		disable= !isTab1
	ModifyControl CheckBoxCCTypeShuff		disable= !isTab1
	ModifyControl CheckBoxCCTypeNorm		disable= !isTab1
	ModifyControl TitleCorr		disable= !isTab1
	ModifyControl CBUseTagsCorr		disable= !isTab1
	
	// Tab Rasters
	ModifyControl TitleRasters					disable= !isTab2
	ModifyControl TitleBoxTickCh				disable= !isTab2
	ModifyControl CB_Ticks_CH1				disable= !isTab2
	ModifyControl CB_Ticks_CH2				disable= !isTab2
	ModifyControl SetRowsTicks				disable= !isTab2
	ModifyControl ButtonEventsAsTicks			disable= !isTab2
	ModifyControl SetRowsSpacing				disable= !isTab2
	ModifyControl CBRastersUseTags			disable= !isTab2
	
	// Tab TTLs
	ModifyControl TitleTTLDetection		disable= !isTab3	
//	ModifyControl TitleChooseTTLch		disable= !isTab3
	ModifyControl CB_TTL_CH1		disable= !isTab3
	ModifyControl CB_TTL_CH2		disable= !isTab3
//	ModifyControl TitleBoxTickPosition		disable= !isTab3
	ModifyControl CheckTicksAbove		disable= !isTab3
	ModifyControl CheckTicksBelow		disable= !isTab3
	ModifyControl CheckTicksAcross		disable= !isTab3
	ModifyControl CheckFillTicks		disable= !isTab3
	ModifyControl ButtonPlotLightPulses		disable= !isTab3
	ModifyControl SetTagStartTTLs		disable= !isTab3
	ModifyControl SetTagEndTTLs		disable= !isTab3
	ModifyControl CheckUseRangeTTLs		disable= !isTab3
	ModifyControl SetBeforeTTLs		disable= !isTab3
	ModifyControl SetAfterTTLs			disable= !isTab3
	ModifyControl ButtonMakePSTH		disable= !isTab3
	ModifyControl SetPSTHbin				disable= !isTab3
	ModifyControl CBNormPSTH			disable= !isTab3
	ModifyControl CBPSTHUseTags			disable= !isTab3
	ModifyControl GBTTLsAsBoxes			disable= !isTab3
	ModifyControl GBTTLsRange		disable= !isTab3
	ModifyControl GBTTLsIn			disable= !isTab3
	ModifyControl GBTTLsPSTH		disable= !isTab3
	ModifyControl CBShowPulseNumber	disable= !isTab3
	
	// Part not in use but that may be usefull to call certain functions
	// If (TabNum == 1)	
	//	SetActiveSubWindow MainWindow#MainPanel	
	// EndIf
	
	// ADD YOU TAB HERE

	If (isTab3) 	// If tab #4 selected (TTLs)
		NVar CheckRangeTTLs
		If (CheckRangeTTLs == 0)
			CheckBox CBNormPSTH			disable=2
			CheckBox CBShowPulseNumber 	disable=2
			CheckBox CBPSTHUseTags		disable=2
			SetVariable SetPSTHbin			disable=2
			SetVariable SetTagStartTTLs		disable=2
			SetVariable SetTagEndTTLs		disable=2
			SetVariable SetBeforeTTLs		disable=2
			SetVariable SetAfterTTLs			disable=2
			Button ButtonMakePSTH			disable=2
		endif
		if (CheckRangeTTLs == 1)
			CheckBox CBNormPSTH			disable=0
			CheckBox CBShowPulseNumber 	disable=0
			CheckBox CBPSTHUseTags		disable=0
			SetVariable SetPSTHbin			disable=0
			SetVariable SetTagStartTTLs		disable=0
			SetVariable SetTagEndTTLs		disable=0
			SetVariable SetBeforeTTLs		disable=0
			SetVariable SetAfterTTLs			disable=0
			Button ButtonMakePSTH			disable=0
		EndIf
	EndIf

	return 0
END
// ========== END OF TABS HIDDING IN THE MAIN WINDOW ==========

// INITIALIZE: THE FUNCTION THAT INITIALIZES EVERYTHING AND DRAWS THE MAIN WINDOW
FUNCTION Initialize(numero)
	Variable numero

	SetDataFolder root:

	// Get the list of windows including: graphs, tables, notebooks and panels		
	String WinListOutput = WinList("*",";","WIN:83")
	Make/N=0/o/T W_WinList
	Variable WaveCounter = 0
	Variable CharCounter = 0
	String TempString = ""
	Variable i
	For (i = 0; i < strlen(WinListOutput); i += 1)
		if (stringmatch(WinListOutput[i],";") == 0)		// If character is not ";"
			TempString[CharCounter] = WinListOutput[i]
			CharCounter += 1
		else		
			InsertPoints WaveCounter, 1, W_WinList
			W_WinList[WaveCounter] = TempString
			WaveCounter += 1
			CharCounter = 0
			TempString = ""
		endif
	Endfor
	
	For (i = 0; i < numpnts(W_WinList); i += 1)
		TempString = 	W_WinList[i]
		KillWindow $TempString
	Endfor

	KillWaves W_WinList
	
	Newdatafolder/O root:spacan
	Killdatafolder root:spacan
	Newdatafolder/O/S root:spacan
	
	Variable/G NumChannels = 2
	Variable/G FDC1 = 0				// FDC1(2) = First Display Channel 1(2). Tells the functions ButtonLoadWave1 and ButtonLoadWave2 if this is the first time the waves are displayed (FDC1(2)=0).
	Variable/G FDC2 = 0
	Variable/G Ch1Loaded = 0			// Keeping track of loaded channels
	Variable/G Ch2Loaded = 0			// Keeping track of loaded channels
	Variable/G Length = NaN			// The length of the recording
	Variable/G RefNum
	Variable/G BSLpnts1, RTpnts1, Ppnts1, BSLpnts2, RTpnts2, Ppnts2
	Variable/G BSLms1, RTms1, Pms1, BSLms2, RTms2, Pms2
	Variable/G Threshold1, Threshold2
	Variable/G SI1, SI2; SI1 = NaN; SI2 = NaN	// Sample Intervals of waves 1 and 2 are NaN; will be defined later, when loading the traces
	Variable/G C1E = 0; // C1E = 1 if channel 1 exists
	Variable/G C2E = 0; // C2E = 1 if channel 2 exists
	Variable/G CorrTimeWindow, CorrBin
	Variable/G FvsTBin, AvsTBin, AmpHistoBin, UseTags
	Variable/G ISIHistoBin
	Variable/G TWPD1, TWPD2
	Variable/G CheckLoadLocal, CheckLoadDisk
	Variable/G CheckPosDeflection1, CheckNegDeflection1
	Variable/G CheckPosDeflection2, CheckNegDeflection2
	Variable/G CheckAvsTShowAv, CheckAvsTShowEvents
	Variable/G RefCh1BoxVar, TestCh1BoxVar, RefCh2BoxVar, TestCh2BoxVar
	Variable/G RefUsePeak, RefUseHR, TestUsePeak, TestUseHR
	Variable/G CCTypeRaw, CCTypeShuff, CCTypeNorm
	Variable/G CheckUseTagsCorr
	Variable/G CheckTicksCh1, CheckTicksCh2
	Variable/G TicksNumberRows
	Variable/G RowsSpacing
	Variable/G TickAbove, TickBelow, TickAcross
	Variable/G CheckTTLch1, CheckTTLch2
	Variable/G FillTicks_WindowTTLs
	Variable/G CheckRangeTTLs
	Variable/G CheckPSTHUseTags
	Variable/G RangeStartTTLs, RangeEndTTLs
	Variable/G BeforeTTLs, AfterTTLs
	Variable/G BinSizePSTH
	Variable/G CheckNormPSTH
	Variable/G CheckRastersUseTags
	Variable/G CheckShowPulseNumber

	// Loading the detection parameters from waves stored on the disc.
	NewPath/o PathToParams "C:Program Files:WaveMetrics:Igor Pro Folder:Igor Procedures:"
	Open/P=PathToParams/Z=2/R RefNum as "SpAcAnParams.ibw"
	if (V_Flag == 0)
		LoadWave/o/P=PathToParams "SpAcAnParams.ibw" ; Wave/T Params
		BSLpnts1 = str2num(Params[1][1]); BSLms1 = (BSLpnts1-1)*SI1*1e-3
		RTpnts1 = str2num(Params[2][1]); RTms1 = (RTpnts1+1)*SI1*1e-3
		Ppnts1 = str2num(Params[3][1]) ; Pms1 = (Ppnts1-1)*SI1*1e-3
		Threshold1 = str2num(Params[5][1])
		BSLpnts2 = str2num(Params[7][1]) ; BSLms2 = (BSLpnts2-1)*SI2*1e-3
		RTpnts2 = str2num(Params[8][1]) ; RTms2 = (RTpnts2+1)*SI2*1e-3
		Ppnts2 = str2num(Params[9][1]) ; Pms2 = (Ppnts2-1)*SI2*1e-3
		Threshold2 = str2num(Params[11][1])
		CorrTimeWindow = str2num(Params[6][3])
		CorrBin = str2num(Params[7][3])
		FvsTBin = str2num(Params[0][3])
		AvsTBin = str2num(Params[1][3])
		AmpHistoBin = str2num(Params[4][3])
		UseTags = str2num(Params[20][3])
		CheckAvsTShowAv = str2num(Params[3][3])
		CheckAvsTShowEvents = str2num(Params[2][3])
		ISIHistoBin = str2num(Params[5][3])
		TWPD1 = str2num(Params[4][1])
		TWPD2 = str2num(Params[10][1])
		CheckLoadLocal = str2num(Params[0][1])
		CheckLoadDisk = 1 - str2num(Params[0][1])
		if (str2num(Params[6][1]) < 0)
			CheckPosDeflection1 = 0
			CheckNegDeflection1 = 1
		endif
		if (str2num(Params[6][1]) > 0)
			CheckPosDeflection1 = 1
			CheckNegDeflection1 = 0
		endif	
		if (str2num(Params[12][1]) < 0)
			CheckPosDeflection2 = 0
			CheckNegDeflection2 = 1
		endif
		if (str2num(Params[12][1]) > 0)
			CheckPosDeflection2 = 1
			CheckNegDeflection2 = 0
		endif
		RefCh1BoxVar = str2num(Params[8][3])
		TestCh1BoxVar = str2num(Params[9][3])
		RefCh2BoxVar = str2num(Params[10][3])
		TestCh2BoxVar = str2num(Params[11][3])
		RefUsePeak = str2num(Params[12][3])
		RefUseHR = str2num(Params[13][3])
		TestUsePeak = str2num(Params[14][3])
		TestUseHR = str2num(Params[15][3])
		CCTypeRaw = str2num(Params[16][3])
		CCTypeShuff = str2num(Params[17][3])
		CCTypeNorm = str2num(Params[18][3])
		CheckUseTagsCorr = str2num(Params[19][3])
		CheckTicksCh1 = str2num(Params[0][5])
		CheckTicksCh2 = str2num(Params[1][5])
		TicksNumberRows = str2num(Params[2][5])
		RowsSpacing = str2num(Params[3][5])
		TickAbove = str2num(Params[6][5])
		TickBelow = str2num(Params[7][5])
		TickAcross = str2num(Params[8][5])
		CheckTTLch1 = str2num(Params[4][5])
		CheckTTLch2 = str2num(Params[5][5])
		FillTicks_WindowTTLs = str2num(Params[9][5])
		CheckRangeTTLs = str2num(Params[10][5])
		CheckPSTHUseTags = str2num(Params[11][5])
		RangeStartTTLs = str2num(Params[12][5])
		RangeEndTTLs = str2num(Params[13][5])
		BeforeTTLs = str2num(Params[14][5])
		AfterTTLs = str2num(Params[15][5])
		BinSizePSTH = str2num(Params[16][5])
		CheckNormPSTH = str2num(Params[17][5])
		CheckRastersUseTags = str2num(Params[18][5])
		CheckShowPulseNumber = str2num(Params[19][5])
	else
		// Default params if no params if no SpAcAnParams.ibw file
		Make/N=(21,6)/o/T Params; Params=""
		BSLpnts1 = NaN
		RTpnts1 = NaN
		Ppnts1 =NaN
		Threshold1 = NaN
		BSLpnts2 = NaN
		RTpnts2 = NaN
		Ppnts2 = NaN
		Threshold2 = NaN
		CorrTimeWindow = 2
		CorrBin = 0.005
		FvsTBin = 0.5
		AvsTBin = 0.5
		AmpHistoBin = 5e-12
		UseTags = 0
		CheckAvsTShowAv = 1
		CheckAvsTShowEvents = 1
		ISIHistoBin = 0.01
		TWPD1 = 3
		TWPD2 = 3
		CheckLoadLocal = 0
		CheckLoadDisk = 1
		CheckPosDeflection1 = 0
		CheckNegDeflection1 = 1
		CheckPosDeflection2 = 0
		CheckNegDeflection2 = 1
		RefCh1BoxVar = 1
		TestCh1BoxVar = 0
		RefCh2BoxVar = 0
		TestCh2BoxVar = 1
		RefUsePeak = 1
		RefUseHR = 0
		TestUsePeak = 0
		TestUseHR = 1
		CCTypeRaw = 1
		CCTypeShuff = 0
		CCTypeNorm = 0
		CheckUseTagsCorr = 0
		CheckTicksCh1 = 0
		CheckTicksCh2 = 0
		TicksNumberRows = 3
		RowsSpacing = 5
		TickAbove = 0
		TickBelow = 1
		TickAcross = 0
		CheckTTLch1 = 1
		CheckTTLch2 = 0
		FillTicks_WindowTTLs = 1
		CheckRangeTTLs = 0
		CheckPSTHUseTags = 0
		RangeStartTTLs = 0
		RangeEndTTLs = 0
		BeforeTTLs = 2
		AfterTTLs = 4
		BinSizePSTH = 0.1
		CheckNormPSTH = 0
		CheckRastersUseTags = 0
		CheckShowPulseNumber = 0
	endif
	Close RefNum

	// Setting up the main window
	DoWindow/K MainWindow ; Display/W=(0,0,800,476) as "Main window"; DoWindow/C MainWindow
	NewPanel/K=1/N=MainPanel/HOST=MainWindow/FG=(FL, FT, gRight, FB )/W=(0,0,207,470) ; ModifyPanel cbRGB=(65500,65500,65500)
	
	Variable/G Ch1PosOffset = 208 // For programming purpose; easier to move the whole bloc (detection parameters) by changing this variable.
	// The following line creates the tab menu
	Variable/G MainWindowTab1
	TabControl MainWindowTab,pos={7,163},size={257,193}, fstyle=1,proc=MainWindowTab1Proc, win=MainWindow#MainPanel, value= 0
	// The following two lines create the different tabs
	TabControl MainWindowTab,tabLabel(0)="Channel1",value= 0, labelBack=(60000,60000,60000)
	TabControl MainWindowTab,tabLabel(1)="Channel2",value= 0, labelBack=(60000,60000,60000)
	
	// The following line creates the tab menu
	Variable/G Tab2PosOffset = 370
	TabControl MainWindowTab2,pos={7,Tab2PosOffset},size={257,260}, fstyle=1,proc=MainWindowTab2Proc, win=MainWindow#MainPanel, value= 0
	// The following two lines create the different tabs
	TabControl MainWindowTab2,tabLabel(0)="F/A",value= 0, labelBack=(60000,60000,60000)
	TabControl MainWindowTab2,tabLabel(1)="Corr.",value= 0, labelBack=(60000,60000,60000)
	TabControl MainWindowTab2,tabLabel(2)="Rasters",value= 0, labelBack=(60000,60000,60000)
	TabControl MainWindowTab2,tabLabel(3)="TTLs",value= 0, labelBack=(60000,60000,60000)
	
	// Add Logo
	DrawPict/W=MainWindow#MainPanel 77, 8, 1, 1, ProcGlobal#SpAcAnLogo
	
	TitleBox	SpAcAnTitle				pos={53,50},size={60,20},title="Spontaneous Activity Analysis",frame=0,fsize=12
	TitleBox	TodaysDate				pos={82,65},size={60,20},title="\{date()}",frame=0,fsize=12
//	TitleBox	spacanversion			pos={64,60},size={60,20},title="v. 1.04 - May 1, 2010",frame=0
	
	// FIRST GROUP: loading and displaying waves.
	SetDrawLayer UserBack ; SetDrawEnv fstyle= 1, fsize=14 ; DrawText 10,105,"Controls"
//	GroupBox/Z GBTest win=MainWindow#MainPanel, pos={5,50},size={262,125},title="Controls",fstyle=1,fsize=14
	
	Variable/G ControlsPosOffset = 90
	Variable/G GridState1, GridState2; GridState1 = 0; GridState2 = 0
	Button		ButtonSync			pos={191,ControlsPosOffset},		size={36,40},		proc=ButtonSyncProc, 	title="Sync",											Disable=2
	Variable/G LoadingChannel1 = 0; Variable/G LoadingChannel2 = 0
	Button 		LoadWave1 			pos={10,ControlsPosOffset},		size={45,20},		proc=ButtonLoadWave1,	title="Load 1"
	Button 		LoadWave2 			pos={10,ControlsPosOffset+20},	size={45,20},		proc=ButtonLoadWave2,	title="Load 2"		
	Button 		FileStart 			pos={55,ControlsPosOffset},		size={34,20},		proc=ButtonStart,			title="|<----",											Disable=2
	Button 		Leftward 			pos={89,ControlsPosOffset},		size={34,20},		proc=ButtonLeft,			title="<----",											Disable=2
	Button 		Rightward 			pos={123,ControlsPosOffset},		size={34,20},		proc=ButtonRight,		title="---->",											Disable=2
	Button 		FileEnd 				pos={157,ControlsPosOffset},		size={34,20},		proc=ButtonEnd,	 		title="---->|",											Disable=2
	Button 		FullScale 			pos={55,ControlsPosOffset+20},	size={68,20},		proc=ButtonFullScale,		title="Full Scale",										Disable=2
	Button 		Hexpand 			pos={123,ControlsPosOffset+20},	size={34,20},		proc=ButtonHexpand,		title="<---->",											Disable=2
	Button 		Hshrink 				pos={157,ControlsPosOffset+20},	size={34,20},		proc=ButtonHshrink, 		title=">----<",											Disable=2
//	ValDisplay 	DisplayFL 			pos={120,ControlsPosOffset-21}, 	size={140,20}, 							title="Rec. length (s) ="
	Button		GridsMW			pos={227,ControlsPosOffset},		size={34,20},		proc=ButtonGridsMW,	title="Grid",											Disable=2
	Button		FlipWave			pos={227,ControlsPosOffset+20},	size={34,20},		proc=ButtonFlip,			title="Flip",											Disable=2
	String/G WaveNameToUse; WaveNameToUse = ""
	TitleBox		TitleBoxLoad			pos={10,ControlsPosOffset+44},	size={40,20},		frame=0,			title="Load waves from:"
	CheckBox	CheckBoxLoadDisk	pos={110,ControlsPosOffset+44},	size={40,20},		proc=ProcCBLoad,	title="disk", value = CheckLoadDisk
	CheckBox	CheckBoxLoadLocal	pos={155,ControlsPosOffset+44},	size={40,20},		proc=ProcCBLoad,	title="Igor data folder", value = CheckLoadLocal
	
	// SECOND GROUP, FIRST TAB: detection parameters for channel 1.
	Variable/G NumberofEventsch1 = NaN; Variable/G FreqEvents1 = NaN
	//SetDrawLayer UserBack ; SetDrawEnv fstyle= 1, fsize=14 ; DrawText 10,235,"Detection parameters"
	ValDisplay 	DisplaySI1	 		labelBack=(60000,60000,60000), pos={15,Ch1PosOffset-20}, 	size={200,20}, 	title="Sample Interval (s/pt) ="
	Execute "Valdisplay DisplaySI1 value=root:spacan:SI1"
	SetVariable 	SetVarBSLpnts1		labelBack=(60000,60000,60000), pos={15,Ch1PosOffset},			size={147,20}, 		proc=SetBSLms,				title="Baseline (pnts) =",	 						limits={1,inf,2}, 	value = BSLpnts1
	SetVariable 	SetVarRTpnts1 		labelBack=(60000,60000,60000), pos={15,Ch1PosOffset+20}, 		size={147,20},		proc=SetRTms,				title="Rise Time (pnts) =", 	 						limits={1,inf,1}, 	value = RTpnts1
	SetVariable 	SetVarPpnts1		labelBack=(60000,60000,60000), pos={15,Ch1PosOffset+40}, 		size={147,20}, 		proc=SetPms,				title="Peak (pnts) =", 								limits={1,inf,2}, 	value = Ppnts1
	Valdisplay 	DisplayBSLms1 		labelBack=(60000,60000,60000), pos={163,Ch1PosOffset}, 			size={91,20},										title=" = (ms)" 
	Valdisplay 	DisplayRTms1 		labelBack=(60000,60000,60000), pos={163,Ch1PosOffset+20}, 		size={91,20},										title=" = (ms)" 
	Valdisplay 	DisplayPms1 		labelBack=(60000,60000,60000), pos={163,Ch1PosOffset+40}, 		size={91,20},										title=" = (ms)" 
	SetVariable 	SetWPD1			labelBack=(60000,60000,60000), pos={15,Ch1PosOffset+60},		size={241,10},									title="Peak Detection Window (ms) =",		limits={0,inf,1},	value = TWPD1
	SetVariable 	SetThreshold1		labelBack=(60000,60000,60000), pos={15,Ch1PosOffset+80},		size={147,10},									title="Threshold =",							limits={0,inf,0.1},value = Threshold1
	CheckBox	PosDeflection1		labelBack=(60000,60000,60000), pos={165,Ch1PosOffset+80},		size={120,20},		proc=ProcCBDeflection1,		title="Pos."
	CheckBox	NegDeflection1		labelBack=(60000,60000,60000), pos={210,Ch1PosOffset+80},		size={120,20},		proc=ProcCBDeflection1,		title="Neg."
	Execute "CheckBox PosDeflection1 value = CheckPosDeflection1"
	Execute "CheckBox NegDeflection1 value = CheckNegDeflection1"
	Button		DW1				labelBack=(60000,60000,60000), pos={15,Ch1PosOffset+100},		size={52,20},			proc=ButtonDW,			title="Diff",											Disable=2
	Button		Detect1				labelBack=(60000,60000,60000), pos={15,Ch1PosOffset+120},		size={52,20},			proc=ButtonDetect,				title="Detect",										Disable=2
	Button		DiffAndDetect1		labelBack=(60000,60000,60000), pos={69,Ch1PosOffset+100},		size={40,40},			proc=ButtonDandD,				title="D&D"
	ValDisplay	NoEch1				labelBack=(60000,60000,60000), pos={118,Ch1PosOffset+100},		size={136,20},									title="# detected: "
	ValDisplay	FreqEch1			labelBack=(60000,60000,60000), pos={120,Ch1PosOffset+120},		size={134,20},									title="Av Freq (Hz): "
	Execute "ValDisplay DisplayBSLms1 value = root:spacan:BSLms1"
	Execute "ValDisplay DisplayRTms1 value = root:spacan:RTms1"
	Execute "ValDisplay DisplayPms1 value = root:spacan:Pms1"
	Execute "ValDisplay NoEch1 value=root:spacan:NumberofEventsch1"
	Execute "ValDisplay FreqEch1 value=root:spacan:FreqEvents1"
	Variable/G Ch1Detected = 0	// This variable will tell us if the function ButtonD1 has been run.

	// SECOND GROUP, SECOND TAB: detection parameters for channel 2.
	Variable/G NumberofEventsch2 = NaN; Variable/G FreqEvents2 = NaN
	Variable/G Ch2PosOffset = 208 // For programming purpose; easier to move the whole bloc by changing this variable.
	ValDisplay 	DisplaySI2	 		labelBack=(60000,60000,60000), pos={15,Ch2PosOffset-20}, 	size={200,20}, 	title="Sample Interval (s/pt) ="
	Execute "Valdisplay DisplaySI2 value=root:spacan:SI2"
	SetVariable 	SetVarBSLpnts2		labelBack=(60000,60000,60000), pos={15,Ch2PosOffset},			size={147,20}, 		proc=SetBSLms,				title="Baseline (pnts) =",	 						limits={1,inf,2}, 	value = BSLpnts2
	SetVariable 	SetVarRTpnts2 		labelBack=(60000,60000,60000), pos={15,Ch2PosOffset+20}, 		size={147,20},		proc=SetRTms,				title="Rise Time (pnts) =", 	 						limits={1,inf,1}, 	value = RTpnts2
	SetVariable 	SetVarPpnts2		labelBack=(60000,60000,60000), pos={15,Ch2PosOffset+40}, 		size={147,20}, 		proc=SetPms,				title="Peak (pnts) =", 								limits={1,inf,2}, 	value = Ppnts2
	Valdisplay 	DisplayBSLms2 		labelBack=(60000,60000,60000), pos={163,Ch2PosOffset}, 			size={91,20},										title=" = (ms)" 
	Valdisplay 	DisplayRTms2		labelBack=(60000,60000,60000), pos={163,Ch2PosOffset+20}, 		size={91,20},										title=" = (ms)" 
	Valdisplay 	DisplayPms2 		labelBack=(60000,60000,60000), pos={163,Ch2PosOffset+40}, 		size={91,20},										title=" = (ms)" 
	SetVariable 	SetWPD2			labelBack=(60000,60000,60000), pos={15,Ch2PosOffset+60},		size={241,10},									title="Peak Detection Window (ms) =",		limits={0,inf,1},	value = TWPD2
	SetVariable 	SetThreshold2		labelBack=(60000,60000,60000), pos={15,Ch2PosOffset+80},		size={147,10},									title="Threshold =",									limits={0,inf,0.1},value = Threshold2
	CheckBox	PosDeflection2		labelBack=(60000,60000,60000), pos={165,Ch2PosOffset+80},		size={120,20},		proc=ProcCBDeflection2,	title="Pos."
	CheckBox	NegDeflection2		labelBack=(60000,60000,60000), pos={210,Ch2PosOffset+80},		size={120,20},		proc=ProcCBDeflection2,	title="Neg."
	Execute "CheckBox PosDeflection2 value = CheckPosDeflection2"
	Execute "CheckBox NegDeflection2 value = CheckNegDeflection2"
	Button		DW2				labelBack=(60000,60000,60000), pos={15,Ch2PosOffset+100},		size={52,20},			proc=ButtonDW,			title="Diff",											Disable=2
	Button		Detect2				labelBack=(60000,60000,60000), pos={15,Ch2PosOffset+120},		size={52,20},			proc=ButtonDetect,				title="Detect",										Disable=2
	Button		DiffAndDetect2		labelBack=(60000,60000,60000), pos={69,Ch2PosOffset+100},		size={40,40},			proc=ButtonDandD,				title="D&D"
	ValDisplay	NoEch2				labelBack=(60000,60000,60000), pos={118,Ch1PosOffset+100},		size={136,20},									title="# detected: "
	ValDisplay	FreqEch2			labelBack=(60000,60000,60000), pos={120,Ch1PosOffset+120},		size={134,20},									title="Av Freq (Hz): "
	Execute "ValDisplay DisplayBSLms2 value = root:spacan:BSLms2"
	Execute "ValDisplay DisplayRTms2 value = root:spacan:RTms2"
	Execute "ValDisplay DisplayPms2 value = root:spacan:Pms2"
	Execute "ValDisplay NoEch2 value=root:spacan:NumberofEventsch2"
	Execute "ValDisplay FreqEch2 value=root:spacan:FreqEvents2"
	MainWindowTab1Proc("MainWindowTab",0)
	Variable/G Ch2Detected = 0	// This variable will tell us if the function ButtonD2 has been run.

	// THIRD GROUP: freq/amp plots and correlograms
	//SetDrawLayer UserBack ; SetDrawEnv fstyle= 1, fsize=14 ; DrawText 10,435,"Plots"
	
	TitleBox 		TitleBoxFAvsT1  	frame=0, fsize=12,	fstyle=1, labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+25}, 		size={15, 20},	title="Frequency / Amplitude vs Time"
	TitleBox 		TitleBoxFAvsT2  	frame=0, fsize=11,	fstyle=0, labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+43}, 		size={15, 20},	title="Select channel by clicking on the\rcorresponding graph"
	CheckBox	CheckBoxUseTags		labelBack=(60000,60000,60000), pos={190,Tab2PosOffset+88},	size={20,20},		proc=ProcCheckBoxUseTags, 	title="Use tags", value = UseTags
	Button		ButtonPlot				labelBack=(60000,60000,60000), pos={205,Tab2PosOffset+118},	size={40,40},		proc=ProcButtonPlot, 				title="Plot"
	GroupBox	GBFvsT					labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+72},	size={170,40},fsize=11,title="Frequency vs Time"
	SetVariable 	SetFvsTBin				labelBack=(60000,60000,60000), pos={20,Tab2PosOffset+88},	size={120,20},								title="Bin (s) = ", 		value = FvsTBin
	GroupBox	GBAvsT					labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+112},	size={170,60},fsize=11,title="Amplitude vs Time"
	SetVariable 	SetAvsTBin				labelBack=(60000,60000,60000), pos={20,Tab2PosOffset+128},	size={120,20},								title="Bin (s) = ",		value = AvsTBin
	CheckBox	CBAvsTShowEvents		labelBack=(60000,60000,60000), pos={20,Tab2PosOffset+148},	size={170,20}, title="Show events",proc=ProcCBAvsTShowEvents, value=CheckAvsTShowEvents
	CheckBox	CBAvsTShowAv			labelBack=(60000,60000,60000), pos={115,Tab2PosOffset+148},	size={170,20}, title="av. amp.",proc=ProcCBAvsTShowAv, value=CheckAvsTShowAv
	GroupBox	GBAmpHisto				labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+172},	size={170,40},fsize=11,title="Amplitude Histogram"
	SetVariable 	SetAmpHistoBin			labelBack=(60000,60000,60000), pos={20,Tab2PosOffset+188},	size={120,20}, title="Bin (-) = ", 		value = AmpHistoBin
	GroupBox	GBISIHisto				labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+212},	size={170,40},fsize=11,title="Interevent Histogram"
	SetVariable	SetISIBin					labelBack=(60000,60000,60000), pos={20,Tab2PosOffset+228},	size={120,20}, title="Bin (s) =", value = 	ISIHistoBin	

	Variable/G CorrCh1, CorrCh2, CorrMethod; CorrCh1 = 1; CorrCh2 = 1; CorrMethod = 1
	// Explanations: CorrCh1 takes the values 1 or 2 if "Channel 1" or "Channel 2" are selected, respectively, in the popup menu. Idem for CorrCh2
	// CorrMethod takes the value 1 if "Peak position" is selected, 2 if "Half rise time" is selected.

	TitleBox 		TitleCorr  	frame=0, fsize=12,	fstyle=1, labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+25}, 		size={15, 20},	title="Correlograms"
	// Here I replaced the drop-down menu (to choose the channels to use for the correlogram) by checkboxes, because the drop-down menus not to work with Wine.
	TitleBox 		RefTitleBox  	frame=0, 	labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+91}, 		size={15, 20},											title="Ref:"
	TitleBox 		TestTitleBox  frame=0,	labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+108}, 		size={15, 20},											title="Test:"
	CheckBox 	RefCh1Box 				labelBack=(60000,60000,60000), pos={50,Tab2PosOffset+91}, 		size={20,20},	proc=ProcToggleRefBox,			title="ch1",  				value=RefCh1BoxVar
	CheckBox 	RefCh2Box				labelBack=(60000,60000,60000), pos={94, Tab2PosOffset+91}, 		size={20,20},	proc=ProcToggleRefBox,			title="ch2",				value=RefCh2BoxVar
	CheckBox 	TestCh1Box 				labelBack=(60000,60000,60000), pos={50,Tab2PosOffset+108}, 		size={20,20},	proc=ProcToggleTestBox,		title="ch1",				value=TestCh1BoxVar
	CheckBox 	TestCh2Box				labelBack=(60000,60000,60000), pos={94, Tab2PosOffset+108}, 		size={20,20},	proc=ProcToggleTestBox,		title="ch2",				value=TestCh2BoxVar
	Button		DoCorrelogram			labelBack=(60000,60000,60000), pos={175,Tab2PosOffset+48},		size={40,40},	proc=ButtonDoCorrelogram,		title="Plot"
	SetVariable 	SetCorrTimeWindow		labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+48},		size={150,20},	 										title="Window (s) =",	value=CorrTimeWindow,		limits={0,inf,0.1}	
	SetVariable 	SetCorrBin				labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+68}, 		size={150,20},											title="Bin (s)",			value=CorrBin,					limits={0,inf,0.001}
	CheckBox	CheckBoxUsePeakRef	pos={140,Tab2PosOffset+91},							proc=ProcToggleRef, 				title="peak", 				value = RefUsePeak
	CheckBox	CheckBoxUseHRRef		labelBack=(60000,60000,60000), pos={190,Tab2PosOffset+91},							proc=ProcToggleRef, 				title="half-rise", 			value= RefUseHR
	CheckBox	CheckBoxUsePeakTest	labelBack=(60000,60000,60000), pos={140,Tab2PosOffset+108},							proc=ProcToggleTest, 			title="peak", 				value = TestUsePeak
	CheckBox	CheckBoxUseHRTest		labelBack=(60000,60000,60000), pos={190,Tab2PosOffset+108},							proc=ProcToggleTest, 			title="half-rise", 			value= TestUseHR
	CheckBox	CheckBoxCCTypeRaw		labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+128}, 		size={100,20}, 	proc=ProcToggleCCRaw,			title="Raw",				value=CCTypeRaw
	CheckBox	CheckBoxCCTypeShuff	labelBack=(60000,60000,60000), pos={60,Tab2PosOffset+128}, 	size={100,20}, 	proc=ProcToggleCCShuff,		title="Shuffled",			value=CCTypeShuff
	CheckBox	CheckBoxCCTypeNorm	labelBack=(60000,60000,60000), pos={126,Tab2PosOffset+128}, 	size={100,20}, 	proc=ProcToggleCCNorm,		title="Normalized",		value=CCTypeNorm
	CheckBox	CBUseTagsCorr			labelBack=(60000,60000,60000), pos={15,Tab2PosOffset+148}, 	size={100,20}, 	proc=ProcCBUseTagsCorr,		title="Use tags",		value=CheckUseTagsCorr

	// Tab Rasters
	Variable/G PosSectionRasters = 395
	TitleBox TitleRasters					pos={15,PosSectionRasters},frame=0,fstyle=1,fsize=12,size={80,20},title="Raster plots",labelBack=(60000,60000,60000)
	TitleBox	TitleBoxTickCh				pos={15,PosSectionRasters+20},size={60,20},title="Use ticks for:",frame=0,labelBack=(60000,60000,60000)
	CheckBox CB_Ticks_CH1				labelBack=(60000,60000,60000),pos={90,PosSectionRasters+20},size={60,20},title="Ch1",proc=ProcCB_Ticks_CH1,value=CheckTicksCh1
	CheckBox CB_Ticks_CH2				labelBack=(60000,60000,60000),pos={135,PosSectionRasters+20},size={60,20},title="Ch2",proc=ProcCB_Ticks_CH2,value=CheckTicksCh2
	CheckBox CBRastersUseTags			labelBack=(60000,60000,60000),pos={180,PosSectionRasters+20},size={60,20},title="Use tags",proc=ProcCBRastersUseTags,value=CheckRastersUseTags
	SetVariable SetRowsTicks				labelBack=(60000,60000,60000),pos={15,PosSectionRasters+40},size={160,20},title="Number of rows:",value=TicksNumberRows
	Button ButtonEventsAsTicks				pos={185,PosSectionRasters+40}, size={40,40},title="Plot", proc= ProcButtonPlotTicks
	SetVariable SetRowsSpacing			labelBack=(60000,60000,60000),pos={15,PosSectionRasters+60},size={160,20},title="Rows spacing (%)",value=RowsSpacing

	// Tab TTLs
	Variable/G PosTabTTLs = 395
	TitleBox	TitleTTLDetection				pos={15,PosTabTTLs},frame=0, fsize=12,	fstyle=1,size={80,20},title="TTLs",labelBack=(60000,60000,60000)

	GroupBox	GBTTLsIn				labelBack=(60000,60000,60000), pos={15,PosTabTTLs+18},	size={230,40},fsize=11,title="TTLs in..."
//	TitleBox TitleChooseTTLch				labelBack=(60000,60000,60000),pos={15,PosTabTTLs+20},frame=0,size={80,20},title="TTLs in:"
	CheckBox CB_TTL_CH1				labelBack=(60000,60000,60000),pos={18,PosTabTTLs+35},size={60,20},title="Ch1",proc=ProcCB_TTL_CH1,value=CheckTTLch1
	CheckBox CB_TTL_CH2				labelBack=(60000,60000,60000),pos={60,PosTabTTLs+35},size={60,20},title="Ch2",proc=ProcCB_TTL_CH2,value=CheckTTLch2

	GroupBox	GBTTLsAsBoxes			labelBack=(60000,60000,60000), pos={15,PosTabTTLs+58},	size={230,40},fsize=11,title="TTls as boxes"
//	TitleBox	TitleBoxTickPosition			pos={15,PosTabTTLs+40},size={60,20},title="Ticks position:",frame=0,labelBack=(60000,60000,60000)
	CheckBox CheckTicksAbove			pos={18,PosTabTTLs+75}, size={100,20},title="above", value=TickAbove, proc=ToggleTicksAbove, labelBack=(60000,60000,60000)
	CheckBox CheckTicksBelow			pos={72,PosTabTTLs+75}, size={50,20},title="below",value=TickBelow, proc=ToggleTicksBelow, labelBack=(60000,60000,60000)
	CheckBox CheckTicksAcross			pos={125,PosTabTTLs+75},size={50,20},title="across", value=TickAcross, proc=ToggleTicksAcross, labelBack=(60000,60000,60000)
	CheckBox CheckFillTicks				pos={180,PosTabTTLs+75},size={80,20},title="Fill ticks",value=FillTicks_WindowTTLs,proc=ToggleCBFillTicks, labelBack=(60000,60000,60000)

	GroupBox	GBTTLsRange			labelBack=(60000,60000,60000), pos={15,PosTabTTLs+100},	size={230,80},fsize=11,title="Range of TTLs"
	CheckBox CheckUseRangeTTLs			pos={18,PosTabTTLs+118},size={80,20},title="Use range:",value=CheckRangeTTLs,proc=ToggleUseTagRange, labelBack=(60000,60000,60000)
	CheckBox CBPSTHUseTags			pos={105,PosTabTTLs+35},size={80,20},title="Use tags",value=CheckPSTHUseTags,proc=ProcCBPSTHUseTags, labelBack=(60000,60000,60000)
	SetVariable SetTagStartTTLs			pos={100,PosTabTTLs+117},size={55,20},title=" ", value = RangeStartTTLs, labelBack=(60000,60000,60000)
	SetVariable SetTagEndTTLs			pos={160,PosTabTTLs+118},size={68,20},title="to", value = RangeEndTTLs, labelBack=(60000,60000,60000)
	SetVariable SetBeforeTTLs				pos={20,PosTabTTLs+139},size={110,20},title="Left (s):", value = BeforeTTLs, labelBack=(60000,60000,60000)
	SetVariable SetAfterTTLs				pos={137,PosTabTTLs+139},size={95,20},title="Right:", value = AfterTTLs, labelBack=(60000,60000,60000)
	Button ButtonPlotLightPulses			pos={190,PosTabTTLs+33}, size={40,20},title="Plot", proc= ProcButtonPlotLightPulses
	CheckBox	CBShowPulseNumber		pos={20,PosTabTTLs+159},size={80,20},title="Show Pulse Number",value=CheckShowPulseNumber	,proc=ProcCBShowPulseNumber	, labelBack=(60000,60000,60000)

	GroupBox	GBTTLsPSTH			labelBack=(60000,60000,60000), pos={15,PosTabTTLs+183},	size={230,43},fsize=11,title="PSTH"	
	Button ButtonMakePSTH				pos={20,PosTabTTLs+200}, size={40,20},title="PSTH", proc= ProcButtonMakePSTH
	SetVariable SetPSTHbin				pos={65,PosTabTTLs+201},size={100,20},title="Bin (s):", value = BinSizePSTH, labelBack=(60000,60000,60000), limits={0,inf,0.01}
	CheckBox CBNormPSTH				pos={170,PosTabTTLs+202},size={100,20},title="Norm",value=CheckNormPSTH, labelBack=(60000,60000,60000), proc=ProcCBNormPSTH
	
	if (CheckRangeTTLs == 0)
		CheckBox CBNormPSTH			disable=2
		CheckBox CBShowPulseNumber 	disable=2
		CheckBox CBPSTHUseTags		disable=2
		SetVariable SetPSTHbin			disable=2
		SetVariable SetTagStartTTLs		disable=2
		SetVariable SetTagEndTTLs		disable=2
		SetVariable SetBeforeTTLs		disable=2
		SetVariable SetAfterTTLs			disable=2
		Button ButtonMakePSTH			disable=2
	endif
	if (CheckRangeTTLs == 1)
		CheckBox CBNormPSTH			disable=0
		CheckBox CBShowPulseNumber 	disable=0
		CheckBox CBPSTHUseTags		disable=0
		SetVariable SetPSTHbin			disable=0
		SetVariable SetTagStartTTLs		disable=0
		SetVariable SetTagEndTTLs		disable=0
		SetVariable SetBeforeTTLs		disable=0
		SetVariable SetAfterTTLs			disable=0
		Button ButtonMakePSTH			disable=0
	endif	

	MainWindowTab2Proc("MainWindowTab2",0)
	
	MainWindowTab1Proc("", 0) //Necessary since the TabControl tb, value=0 doesn't seem to work
END

// FUNCTION THAT SYNCHRONIZES ONE CHANNEL ON TO THE OTHER (BUTTON "SYNC" IN THE MAIN WINDOW)
FUNCTION ButtonSyncProc(ctrlName) : ButtonControl
	String ctrlName
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		SetAxis/W=MainWindow#Channel2 bottom V_min,V_max
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		SetAxis/W=MainWindow#Channel1 bottom V_min,V_max
	endif
END

// FUNCTION CALLED BY THE "FLIP" BUTTON IN THE MAIN WINDOW
FUNCTION ButtonFlip(ctrlName) : ButtonControl
	String ctrlName
	getwindow kwTopWin activeSW
	Svar Filename1, Filename2
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		execute Filename1 + "= -" + Filename1
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		execute Filename2 + "= -" + Filename2
	endif
END

// FUNCTION CALLED BY THE BUTTON GRIDS IN THE MAIN WINDOW
FUNCTION ButtonGridsMW(ctrlName) : ButtonControl
	String ctrlName
	Svar Filename1, Filename2
	Nvar GridState1, GridState2
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GridState1 = 1 - GridState1
		ModifyGraph grid=GridState1,gridRGB=(0,0,0)
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GridState2 = 1 - GridState2
		ModifyGraph grid=GridState2,gridRGB=(0,0,0)
	endif
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "CheckBoxLoadDisk" AND "CheckBoxLoadLocal" IN THE MAIN WINDOW
FUNCTION ProcCBLoad(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckLoadLocal, CheckLoadDisk
	CheckLoadLocal = 1 - CheckLoadLocal
	CheckLoadDisk = 1 - CheckLoadDisk
	Execute "CheckBox CheckBoxLoadDisk value=CheckLoadDisk"
	Execute "CheckBox CheckBoxLoadLocal value=CheckLoadLocal"	
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "PosDeflection1" and "NegDeflection1" IN THE MAIN WINDOW (DETECTION PARAMS CHANNEL 1)
FUNCTION ProcCBDeflection1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckPosDeflection1, CheckNegDeflection1
	CheckPosDeflection1 = 1 - CheckPosDeflection1
	CheckNegDeflection1 = 1 - CheckNegDeflection1
	Execute "CheckBox PosDeflection1 value=CheckPosDeflection1"
	Execute "CheckBox NegDeflection1 value=CheckNegDeflection1"	
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "PosDeflection2" and "NegDeflection2" IN THE MAIN WINDOW (DETECTION PARAMS CHANNEL 2)
FUNCTION ProcCBDeflection2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckPosDeflection2, CheckNegDeflection2
	CheckPosDeflection2 = 1 - CheckPosDeflection2
	CheckNegDeflection2 = 1 - CheckNegDeflection2
	Execute "CheckBox PosDeflection2 value=CheckPosDeflection2"
	Execute "CheckBox NegDeflection2 value=CheckNegDeflection2"	
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "RefCh1Box" and "RefCh2Box" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleRefBox(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar RefCh1BoxVar, RefCh2BoxVar
	RefCh1BoxVar = 1 - RefCh1BoxVar
	RefCh2BoxVar = 1 - RefCh1BoxVar
	Execute "CheckBox RefCh1Box value=RefCh1BoxVar"
	Execute "CheckBox RefCh2Box value=RefCh2BoxVar"
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "TestCh1Box" and "TestCh2Box" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleTestBox(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar TestCh1BoxVar, TestCh2BoxVar
	TestCh1BoxVar = 1 - TestCh1BoxVar
	TestCh2BoxVar = 1 - TestCh1BoxVar
	Execute "CheckBox TestCh1Box value=TestCh1BoxVar"
	Execute "CheckBox TestCh2Box value=TestCh2BoxVar"
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "CheckBoxUsePeakRef" and "CheckBoxUseHRRef" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleRef(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar RefUsePeak, RefUseHR
	RefUsePeak = 1 - RefUsePeak
	RefUseHR = 1 - RefUsePeak
	Execute "CheckBox CheckBoxUsePeakRef value=RefUsePeak"
	Execute "CheckBox CheckBoxUseHRRef value=RefUseHR"
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "CheckBoxUsePeakTest" and "CheckBoxUseHRTest" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleTest(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar TestUsePeak, TestUseHR
	TestUsePeak = 1 - TestUsePeak
	TestUseHR = 1 - TestUsePeak
	Execute "CheckBox CheckBoxUsePeakTest value=TestUsePeak"
	Execute "CheckBox CheckBoxUseHRTest value=TestUseHR"
END

// FUNCTION TRIGGERED BY THE CHECKBOXE "CheckBoxCCTypeRaw" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleCCRaw(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	CCTypeRaw = 1 - CCTypeRaw
	if (CCTypeRaw == 1)
		CCTypeShuff = 0
		CCTypeNorm = 0
	endif
	Execute "CheckBox CheckBoxCCTypeRaw value=CCTypeRaw"
	Execute "CheckBox CheckBoxCCTypeShuff value=CCTypeShuff"
	Execute "CheckBox CheckBoxCCTypeNorm value=CCTypeNorm"
END

// FUNCTION TRIGGERED BY THE CHECKBOXE "CheckBoxCCTypeShuff" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleCCShuff(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	CCTypeShuff = 1 - CCTypeShuff
	if (CCTypeShuff == 1)
		CCTypeRaw = 0
		CCTypeNorm = 0
	endif
	Execute "CheckBox CheckBoxCCTypeRaw value=CCTypeRaw"
	Execute "CheckBox CheckBoxCCTypeShuff value=CCTypeShuff"
	Execute "CheckBox CheckBoxCCTypeNorm value=CCTypeNorm"
END

// FUNCTION TRIGGERED BY THE CHECKBOXE "CheckBoxCCTypeNorm" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleCCNorm(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	CCTypeNorm = 1 - CCTypeNorm
	if (CCTypeNorm == 1)
		CCTypeRaw = 0
		CCTypeShuff = 0
	endif
	Execute "CheckBox CheckBoxCCTypeRaw value=CCTypeRaw"
	Execute "CheckBox CheckBoxCCTypeShuff value=CCTypeShuff"
	Execute "CheckBox CheckBoxCCTypeNorm value=CCTypeNorm"
END

FUNCTION ProcCBUseTagsCorr(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckUseTagsCorr
	CheckUseTagsCorr = checked
END

FUNCTION ProcCBAvsTShowEvents(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked	
	Nvar CheckAvsTShowEvents, CheckAvsTShowAv
	CheckAvsTShowEvents = checked
	if (CheckAvsTShowEvents == 0 && CheckAvsTShowAv == 0)
		CheckAvsTShowEvents = 1
		Execute "CheckBox CBAvsTShowEvents value=CheckAvsTShowEvents"
	endif
END

FUNCTION ProcCBAvsTShowAv(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked	
	Nvar CheckAvsTShowAv, CheckAvsTShowEvents
	CheckAvsTShowAv = checked
	if (CheckAvsTShowAv == 0 && CheckAvsTShowEvents == 0)
		CheckAvsTShowAv = 1
		Execute "CheckBox CBAvsTShowAv value=CheckAvsTShowAv"
	endif
END

// FUNCTION CALLED BY THE CHECKBOX "USE TAGS" IN THE SECTION "Common plots" OF THE MAIN WINDOW
FUNCTION ProcCheckBoxUseTags(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar UseTags
	UseTags = checked
END

// FUNCTION CALLED BY THE BUTTON "Plot" IN THE SECTION "Common plots" OF THE MAIN WINDOW
FUNCTION ProcButtonPlot (ctrlName) : ButtonControl
	String ctrlName
	Nvar FvsTBin, AvsTBin, AmpHistoBin, ISIHistoBin
	Wave/T Params
	
	Nvar C1E, C2E
	if (C1E == 0 && C2E == 0)
		Abort "No channels loaded!"
	endif
	
	Nvar Ch1Detected, Ch2Detected
	
	if (Ch1Detected)
		Wave Tags1
	endif
	if (Ch2Detected)
		Wave Tags2
	endif	

	Nvar UseTags
	Nvar CheckAvsTShowAv, CheckAvsTShowEvents

	Params[0][3] = num2str(FvsTBin)
	Params[1][3] = num2str(AvsTBin)
	Params[4][3] = num2str(AmpHistoBin)
	Params[20][3] = num2str(UseTags)
	Params[3][3] = num2str(CheckAvsTShowAv)
	Params[2][3] = num2str(CheckAvsTShowEvents)
	Params[5][3] = num2str(ISIHistoBin)
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)	// If MainWindow#Channel1 is on top
		if (Ch1Detected == 0)
			Abort "Detect events in this channel 1 first!"
		else
			WaveStats/Q Tags1
			if (UseTags == 1 && V_Sum == 0)
				Abort "No events tagged in channel 1!"
			else
				PlotFAvsT_ch1()
			endif
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)	// If MainWindow#Channel2 is on top
		if (Ch2Detected == 0)
			Abort "Detect events in this channel 2 first!"
		else
			WaveStats/Q Tags2
			if (UseTags == 1 && V_Sum == 0)
				Abort "No events tagged in channel 2!"
			else
				PlotFAvsT_ch2()
			endif
		endif
	endif
END

// FUNCTION CALLED BY THE ABOVE FUNCTION (ProcButtonPlot)
FUNCTION PlotFAvsT_ch1()
	Wave Occurrence1, realamp1, Tags1
	Nvar Threshold1, UseTags
	if (UseTags == 0)
		FAvsT(Occurrence1,Realamp1,Threshold1)
	endif
	if (UseTags == 1)
		if (sum(Tags1,0,numpnts(Tags1)-1) == 0)
			Abort "No tagged events. Uncheck the box \"Use tags\" or tag events."
		endif
		Duplicate/o Occurrence1, Occurrence1_BIS
		Duplicate/o Realamp1, Realamp1_BIS
		Occurrence1_BIS = Occurrence1_BIS * Tags1
		Realamp1_BIS = Realamp1_BIS * Tags1
		FAvsT(Occurrence1_BIS,Realamp1_BIS,Threshold1)
		KillWaves Occurrence1_BIS,Realamp1_BIS
	endif
END

// FUNCTION CALLED BY THE ABOVE FUNCTION (ProcButtonPlot)
FUNCTION PlotFAvsT_ch2()
	Nvar UseTags
	Wave Occurrence2, realamp2, Tags2
	Nvar Threshold2
	if (UseTags == 0)
		FAvsT(Occurrence2,Realamp2,Threshold2)
	endif
	if (UseTags == 1)
		Duplicate/o Occurrence2, Occurrence2_BIS
		Duplicate/o Realamp2, Realamp2_BIS
		Occurrence2_BIS = Occurrence2_BIS * Tags2
		Realamp2_BIS = Realamp2_BIS * Tags2
		FAvsT(Occurrence2_BIS,Realamp2_BIS,Threshold2)
		KillWaves Occurrence2_BIS,Realamp2_BIS
	endif
END

// FUNCTION THAT DISPLAYS THE BEGINNING OF THE TWO WAVES IN MAIN WINDOW
FUNCTION ButtonStart(ctrlName) : ButtonControl
	String ctrlName
	Nvar Ch1Loaded, Ch2Loaded
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = V_max-V_min; SetAxis/W=MainWindow#Channel1 bottom 0, Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel2 bottom 0, Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = V_max-V_min; SetAxis/W=MainWindow#Channel2 bottom 0, Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel1 bottom 0, Mi
		endif
	endif
END

// FUNCTION THAT DISPLAYS THE END OF THE TWO WAVES IN MAIN WINDOW
FUNCTION ButtonEnd(ctrlName) : ButtonControl
	String ctrlName
	Nvar Length, Ch1Loaded, Ch2Loaded
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = V_max-V_min; SetAxis/W=MainWindow#Channel1 bottom Length - Mi, Length
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel2 bottom Length - Mi, Length
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = V_max-V_min; SetAxis/W=MainWindow#Channel2 bottom Length - Mi, Length
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel1 bottom Length - Mi, Length
		endif
	endif
END

// SCROLLING BACKWARD IN MAIN WINDOW
FUNCTION ButtonLeft(ctrlName) : ButtonControl
	String ctrlName
	Nvar Ch1Loaded, Ch2Loaded
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel1 bottom V_min-Mi,V_max-Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel2 bottom V_min-Mi,V_max-Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel2 bottom V_min-Mi,V_max-Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel1 bottom V_min-Mi,V_max-Mi
		endif
	endif
END

// SCROLLING FORWARD IN MAIN WINDOW
FUNCTION ButtonRight(ctrlName) : ButtonControl
	String ctrlName
	Nvar Ch1Loaded, Ch2Loaded
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel1 bottom V_min+Mi,V_max+Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel2 bottom V_min+Mi, V_max+Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel2 bottom V_min+Mi,V_max+Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel1 bottom V_min+Mi, V_max+Mi
		endif
	endif
END

// FUNCTION THAT EXPANDS THE TWO TRACES IN MAIN WINDOW
FUNCTION ButtonHexpand(ctrlName) : ButtonControl
	String ctrlName
	Nvar Ch1Loaded, Ch2Loaded
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = (V_max-V_min)/5; SetAxis/W=MainWindow#Channel1 bottom V_min,V_max-3*Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel2 bottom V_min,V_max-3*Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = (V_max-V_min)/5; SetAxis/W=MainWindow#Channel2 bottom V_min,V_max-3*Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel1 bottom V_min,V_max-3*Mi
		endif
	endif
END

// FUNCTION THAT SHRINKS THE TWO TRACES IN MAIN WINDOW
FUNCTION ButtonHshrink(ctrlName) : ButtonControl
	String ctrlName
	Nvar Ch1Loaded, Ch2Loaded
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel1 bottom V_min,V_max+3*Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel2 bottom V_min,V_max+3*Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel2 bottom V_min,V_max+3*Mi
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel1 bottom V_min,V_max+3*Mi
		endif
	endif
END

// FULL SCALE IN MAIN WINDOW
FUNCTION ButtonFullScale(ctrlName) : ButtonControl
	String ctrlName
	Nvar Ch1Loaded, Ch2Loaded
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		SetAxis/W=MainWindow#Channel1/A
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel2/A
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		SetAxis/W=MainWindow#Channel2/A
		if ((Ch1Loaded == 1) & (Ch2Loaded == 1))
			SetAxis/W=MainWindow#Channel1/A
		endif
	endif
END


FUNCTION ButtonLoadWave1(ctrlName) : ButtonControl
	String ctrlName
	
	Nvar CheckLoadLocal
	Wave/T Params
	Params[0][1] = num2str(CheckLoadLocal)
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	
	FunctionLoadWave(1)
END

FUNCTION ButtonLoadWave2(ctrlName) : ButtonControl
	String ctrlName

	Nvar CheckLoadLocal
	Wave/T Params
	Params[0][1] = num2str(CheckLoadLocal)
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	
	FunctionLoadWave(2)
END

// BRANCHING POINT OF THE LOADWAVE1 PROCEDURE
FUNCTION FunctionLoadWave(ChNumber)
	Variable ChNumber
	SetDataFolder root:spacan

	Nvar CheckLoadLocal	// Tells us if waves are loaded from the disk of from Igor data folder.

	Nvar LoadingChannelA = $"LoadingChannel" + num2str(ChNumber)
	Nvar LoadingChannelB = $"LoadingChannel" + num2str(3 - ChNumber)

	LoadingChannelA = 1
	LoadingChannelB = 0

	String/G $"Filename" + num2str(ChNumber)

	if (CheckLoadLocal)
		MakeWaveSelectorPanel(WMWS_Waves,ChNumber)
	else
		LoadWaveFinal(ChNumber)
	endif
	
END

// LOAD AND DISPLAY TRACE 1 (CHANNEL 1) IN MAIN WINDOW
FUNCTION LoadWaveFinal(ChNumber)
	Variable ChNumber

	// Global variables
	Nvar CheckLoadLocal
	Nvar FDC = $"FDC" + num2str(ChNumber)
	Nvar SIa = $"SI" + num2str(ChNumber)
	Nvar ChALoaded = $"Ch" + num2str(ChNumber) + "Loaded"
	Nvar ChBLoaded = $"Ch" + num2str(3 - ChNumber) + "Loaded"
	Nvar BSLpnts = $"BSLpnts" + num2str(ChNumber)
	Nvar BSLms = $"BSLms" + num2str(ChNumber)
	Nvar RTpnts = $"RTpnts" + num2str(ChNumber)
	Nvar RTms = $"RTms" + num2str(ChNumber)
	Nvar Ppnts = $"Ppnts" + num2str(ChNumber)
	Nvar Pms = $"Pms" + num2str(ChNumber)
	Nvar CaE = $"C" + num2str(ChNumber) + "E"
	Nvar CbE = $"C" + num2str(3 - ChNumber) + "E"
	Nvar ChBDetected = $"Ch" + num2str(3 - ChNumber) + "Detected"
	Nvar Length
	// Global strings
	Svar Filename = $"Filename" + num2str(ChNumber)
	
	if (CbE)	// Other channel loaded
		Svar FilenameB = $"Filename" + num2str(3 - ChNumber)
	endif
	
	if (CheckLoadLocal == 0)		// means load from disk
		Nvar RefNum ; open/Z=2/R/T="IGBW"/M="Choose wave" RefNum
		// Flag /Z removes error message if user presses cancel in the dialog.
		// Can not be used with /D flag
		if (V_flag == -1)
			Abort
		endif
		LoadWave/o S_Filename
		FileName = S_WaveNames[0,strlen(S_WaveNames)-2]
		Close RefNum
		
		SIa = DeltaX($FileName)*1e6
		SetScale/P x 0,DeltaX($FileName),"s",$FileName	// This is important to reset the beginning of the wave to zero in case it is not
	else							// means load from data folder
		SIa = DeltaX($FileName)*1e6
	endif

	BSLms = (BSLpnts-1)*SIa*1e-3
	RTms = (RTpnts+1)*SIa*1e-3
	Pms = (Ppnts-1)*SIa*1e-3
	
	String Channel = "Channel" + num2str(ChNumber)
	String ChannelB = "Channel" + num2str(3 - ChNumber)
	
	if (FDC)
		KillWindow MainWindow#$Channel
	else
		FDC = 1
	endif
	
	if (ChNumber == 1)
		if (CbE == 0)
			Display/HOST=MainWindow/W=(0.27,0,1,1)/N=$Channel/L=Left1 $FileName
		else
			KillWindow MainWindow#Channel2
			Display/HOST=MainWindow/W=(0.27,0,1,0.5)/N=$Channel/L=Left1 $FileName
			Display/HOST=MainWindow/W=(0.27,0.5,1,1)/N=$ChannelB/L=Left2 $FileNameB
		endif
		ModifyGraph/W=MainWindow#$Channel freePos(Left1)=0
		if (CbE == 1)
			ModifyGraph/W=MainWindow#$ChannelB freePos(Left2)=0
		endif
	endif
	if (ChNumber == 2)
		if (CbE == 0)
			Display/HOST=MainWindow/W=(0.27,0,1,1)/N=$Channel/L=Left2 $FileName
		else
			KillWindow MainWindow#Channel1
			Display/HOST=MainWindow/W=(0.27,0.5,1,1)/N=$Channel/L=Left2 $FileName
			Display/HOST=MainWindow/W=(0.27,0,1,0.5)/N=$ChannelB/L=Left1 $FileNameB
		endif		
		ModifyGraph/W=MainWindow#$Channel freePos(Left2)=0
		if (CbE == 1)
			ModifyGraph/W=MainWindow#$ChannelB freePos(Left1)=0
		endif
	endif
	
	if (ChNumber == 1)
		ModifyGraph/W=MainWindow#$Channel rgb=(0,0,65535)
		if (CbE == 1)
			ModifyGraph/W=MainWindow#$ChannelB rgb=(65535,0,0)
		endif
	endif
	if (ChNumber == 2)
		ModifyGraph/W=MainWindow#$Channel rgb=(65535,0,0)
		if (CbE == 1)
			ModifyGraph/W=MainWindow#$ChannelB rgb=(0,0,65535)
		endif
	endif
	ModifyGraph/W=MainWindow#$Channel margin(Left)=0
	ModifyGraph/W=MainWindow#$Channel margin(right)=10
	ModifyGraph/W=MainWindow#$Channel margin(bottom)=28
	ModifyGraph/W=MainWindow#$Channel margin(top)=30
	ModifyGraph/W=MainWindow#$Channel StandOff=0
	ModifyGraph/W=MainWindow#$Channel axOffset(bottom)=0
	if (CbE == 1)
		ModifyGraph/W=MainWindow#$ChannelB margin(Left)=0
		ModifyGraph/W=MainWindow#$ChannelB margin(right)=10
		ModifyGraph/W=MainWindow#$ChannelB margin(bottom)=28
		ModifyGraph/W=MainWindow#$ChannelB margin(top)=30
		ModifyGraph/W=MainWindow#$ChannelB StandOff=0
		ModifyGraph/W=MainWindow#$ChannelB axOffset(bottom)=0			
	endif
	if (ChNumber == 1)
		SetDrawEnv/W=MainWindow#$Channel xcoord=abs, ycoord=abs, textrgb=(0,0,65535), fstyle=1, fsize=12
		DrawText/W=MainWindow#$Channel 10,20, "Ch1: " + Filename
		if (CbE == 1)
			SetDrawEnv/W=MainWindow#$ChannelB xcoord=abs, ycoord=abs, textrgb=(65535,0,0), fstyle=1, fsize=12
			DrawText/W=MainWindow#$ChannelB 10,20, "Ch2: " + FilenameB
		endif
	endif
	if (ChNumber == 2)
		SetDrawEnv/W=MainWindow#$Channel xcoord=abs, ycoord=abs, textrgb=(65535,0,0), fstyle=1, fsize=12
		DrawText/W=MainWindow#$Channel 10,20, "Ch2: " + Filename
		if (CbE == 1)
			SetDrawEnv/W=MainWindow#$ChannelB xcoord=abs, ycoord=abs, textrgb=(0,0,65535), fstyle=1, fsize=12
			DrawText/W=MainWindow#$ChannelB 10,20, "Ch1: " + FilenameB
		endif		
	endif
	
	Variable/G $"Length" + num2str(ChNumber)
	Nvar LengthA = $"Length" + num2str(ChNumber)
	LengthA = (numpnts($FileName)-1)*DeltaX($FileName)

	Length = LengthA
	
	ChALoaded = 1

	String DW = "DW" + num2str(ChNumber)

	Button $DW Disable=0, win=MainWindow#MainPanel
	Button FileStart Disable=0, win=MainWindow#MainPanel
	Button Leftward Disable=0, win=MainWindow#MainPanel
	Button Rightward Disable=0, win=MainWindow#MainPanel
	Button FileEnd Disable=0, win=MainWindow#MainPanel
	Button FullScale Disable=0, win=MainWindow#MainPanel
	Button Hexpand Disable=0, win=MainWindow#MainPanel
	Button Hshrink Disable=0, win=MainWindow#MainPanel
	Button FlipWave Disable=0, win=MainWindow#MainPanel
	Button GridsMW Disable=0, win=MainWindow#MainPanel
	
	if (ChALoaded == 1 && ChBLoaded == 1)
		Button ButtonSync Disable=0, win=MainWindow#MainPanel
	endif
	
	CaE = 1	// Channel a exists

	// Charly this aims at solving a known bug of the MainWindow tabs that are set to the value of the last loaded channel
	If (ChBDetected) // If channel 2 has not already been detected
		DoWindow/F MainWindow
		if (ChNumber == 1)
			MainWindowTab1Proc("", 0)
		endif
		if (ChNumber == 2)
			MainWindowTab2Proc("", 0)
		endif
	Endif
	// End
END

// HERE IS THE FUNCTION THAT GENERATES THE LISTBOX CONTROL TO LOAD WAVES FROM DATA FOLDER
// COPIED FROM IGOR PRO DOCUMENTATION
Function MakeWaveSelectorPanel(showWhat,ChNumber)
	Variable showWhat
	Variable ChNumber
	
	String panelName = "Select"
	// figure out what to show in the Wave Selector, and make an appropriate name for the panel
	switch (showWhat)
		case WMWS_Waves:
			panelName+="Waves"
			break;
		case WMWS_NVars:
			panelName+="Vars"
			break;
		case WMWS_Strings:
			panelName+="Strs"
			break;
		case WMWS_DataFolders:
			panelName+="DFs"
			break;
	endswitch
	
	if (WinType(panelName) == 7)
		// if the panel already exists, show it
		DoWindow/F $panelName
	else
		// doesn't exist, make it
		NewPanel/N=$panelName/W=(181,179,471,530) as "Select wave"
		// list box control doesn't have any attributes set on it
		ListBox WaveSelectorList,pos={9,13},size={273,241},mode=4
		// This function does all the work of making the listbox control into a
		// Wave Selector widget. Note the optional parameter that says what type of objects to
		// display in the list. 
		MakeListIntoWaveSelector(panelName, "WaveSelectorList", content = showWhat)

		// This is an extra bonus- you can create your own function to be notified of certain events,
		// such as a change in the selection in the list.
		WS_SetNotificationProc(panelName, "WaveSelectorList", "ExtExampleNotification", isExtendedProc=1)

		String/G $"UserWaveName" + num2str(ChNumber)
		Svar UserWaveName = $"UserWaveName" + num2str(ChNumber)
		UserWaveName = "enter wave name here"
		
		Variable/G CheckUseDefaultName = 1
		Variable/G CheckUseCustomName = 0
		
		TitleBox 	TitleBoxLoad1  	frame=0, fsize=11,	fstyle=3, pos={9,260}, 	size={15, 20}, title="Select one wave or more."
		TitleBox 	TitleBoxLoad2  	frame=0, fsize=11,	fstyle=3, pos={9,275}, 	size={15, 20}, title="Multiple waves will be concatenated."
		
		
		CheckBox CBUseDefaultName		pos={9,295}, size={150,20},title="Use default wave name",value=1,proc=ProcCBUseDefaultName
		CheckBox CBUseCustomName		pos={9,320}, size={100,20},title="Or",proc = ProcCBUseCustomName
		SetVariable EnterWaveName		pos={40,319}, size={160,20}, frame=0,value=UserWaveName,title=" "

		Button ButtonConcAndLoad		pos={210,295},	size={60,20},	proc=ProcButtonConcAndLoad,		title="Load"
		Button ButtonLoadWaveCancel		pos={210,320},	size={60,20},	proc=ProcButtonLoadWaveCancel,		title="Cancel"
	endif
End

Function ProcCBUseDefaultName(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	Nvar CheckUseDefaultName, CheckUseCustomName
	CheckUseDefaultName = checked

	if (CheckUseDefaultName == 1)
		CheckUseCustomName = 0
	endif
	if (CheckUseDefaultName == 0)
		CheckUseCustomName = 1
	endif
	Execute "CheckBox CBUseDefaultName value=CheckUseDefaultName"
	Execute "CheckBox CBUseCustomName	value=CheckUseCustomName"
End

Function ProcCBUseCustomName(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	Nvar CheckUseDefaultName, CheckUseCustomName
	CheckUseCustomName = checked

	if (CheckUseCustomName == 1)
		CheckUseDefaultName = 0
	endif
	if (CheckUseCustomName == 0)
		CheckUseDefaultName = 1
	endif
	Execute "CheckBox CBUseDefaultName value=CheckUseDefaultName"
	Execute "CheckBox CBUseCustomName	value=CheckUseCustomName"
End

FUNCTION ProcButtonLoadWaveCancel (ctrlName) : ButtonControl
	String ctrlName
	String panelName = "SelectWaves"
	DoWindow/K $panelName
	setdatafolder root:spacan
END

FUNCTION ProcButtonConcAndLoad(ctrlName) : ButtonControl
	String ctrlName
	
	SetDataFolder root:spacan
	
	// Global variables
	Nvar LoadingChannel1
	Nvar LoadingChannel2
	Nvar CheckUseDefaultName
	Nvar CheckUseCustomName
	
	Variable ChNumber
	if (LoadingChannel1)
		ChNumber = 1
	endif
	if (LoadingChannel2)
		ChNumber = 2
	endif
	
	// Global Strings
	Svar UserWaveName = $"UserWaveName" + num2str(ChNumber)
	Svar Filename = $"Filename" + num2str(ChNumber)

	GetSelectedWaveList()
	
	Setdatafolder root:spacan
	
	Variable i
	
	// Check if more than one wave selected
	Wave WaveNameList, AbsoluteWaveNameList
	if (numpnts(WaveNameList) == 0)
		Abort "No wave selected!"
	endif
	
	String/G TempString
	Execute "TempString = AbsoluteWaveNameList[0]"
	Execute "Duplicate/o  " + TempString + " Trace"

	// Then loop if more than one wave selected
	// Here I changed the logic of the concatenation process
	// Reason: the string containing the wave names might become too big
	if (numpnts(WaveNameList) > 1)
		TempString = "{"
		For (i = 1; i < numpnts(WaveNameList); i += 1)
			Execute "TempString = AbsoluteWaveNameList[" + num2str(i) + "]"
			Execute "Concatenate/NP/o {Trace," + TempString + "}, Trace"
		Endfor
	endif
	
	KillStrings TempString
	Wave Trace

	if (CheckUseDefaultName)
		Duplicate/o Trace $"Trace" + num2str(ChNumber)
		Filename = "Trace" + num2str(ChNumber)
	endif
	if (CheckUseCustomName)
		Duplicate/o Trace $UserWaveName
		Filename = UserWaveName
	endif
		
	String panelName = "SelectWaves"
	DoWindow/K $panelName
	setdatafolder root:spacan
	
	LoadWaveFinal(ChNumber)

END

// FUNCTION THAT BUILDS A LIST A WAVES SELECTED IN LISTBOX
FUNCTION GetSelectedWaveList()
	
	SetDataFolder root:Packages:WM_WaveSelectorList:WaveSelectorInfo0
	Wave ListWave, SelWave
	Make/n=0/o/t root:spacan:AbsoluteWaveNameList
	Make/n=0/o/t root:spacan:WaveNameList
	String/G temp1, temp2, temp3
	String/G folderpathname, wavenameinlist
	Variable i, j, counter, HierarchyCounter
	counter = 0
	// One challenge here is to figure out the hierarchy
	// There is no clue from the waves generated by the widget
	// Let's fix this by generating a wave containing the hierarchy numbers
	Make/N=0/o HierarchyIndex
	For (i = 0; i < numpnts(SelWave)/2; i += 1)
		execute "temp1 = ListWave[" + num2str(i) + "][1][0]"	// // Item name with the indentation containing information on rank in folder hierarchy
		execute "temp2 = ReplaceString(\" \",\"" + temp1 + "\", \"\")"	// Item name without indentation.
		// strlen(temp1) - strlen(temp2) is the length of the indentation
		// 0 is root
		// 4 means it is under root:
		// 8 means it is under root + 1
		// 12 means it is under root + 2
		// etc...
		InsertPoints i, 1, HierarchyIndex
		execute "HierarchyIndex[i] =  (strlen(temp1) - strlen(temp2))/4"
		// Now the code is:
		// 0 is root
		// 1 means it is under root:
		// 2 means it is under root + 1
		// 3 means it is under root + 2
		// etc...
	Endfor
	// Now let's get the list of waves that have been selected in the listbox
	For (i = 0; i < numpnts(SelWave)/2; i += 1)
		folderpathname = ""
		if (((SelWave[i][1] == 1) || (SelWave[i][1] == 8))& (SelWave[i][0] == 0))	// means wave selected
			// Let's save the wave's name
			execute "temp1 = ListWave[" + num2str(i) + "][1][0]"
			execute "temp2 = ReplaceString(\" \",\"" + temp1 + "\", \"\")"	// Wave name without indentation
			execute "wavenameinlist = \"" + temp2 + "\""
			InsertPoints counter, 1, root:spacan:WaveNameList
			execute "root:spacan:WaveNameList[" + num2str(i) + "] = \"" + folderpathname + "\" + \"" + wavenameinlist + "\""
			// Move up in the folders to find the absolute path leading to this wave
			// The string folderpathname will keep track of this
			if (i > 0)
				HierarchyCounter = HierarchyIndex[i]; print HierarchyCounter
				For (j = i; j > -1; j -= 1)
					if (HierarchyIndex[j] < HierarchyCounter)
						execute "temp1 = ListWave[" + num2str(j) + "][1][0]"
						execute "temp1 = temp1 + \":\""
						execute "temp2 = ReplaceString(\" \",\"" + temp1 + "\", \"\")"
						execute "folderpathname  = \"" + temp2 + "\" + folderpathname"
						HierarchyCounter -= 1
					endif
				Endfor
			else
				Abort "No root folder!"
			endif
		InsertPoints counter, 1, root:spacan:AbsoluteWaveNameList
		execute "root:spacan:AbsoluteWaveNameList[" + num2str(counter) + "] = \"" + folderpathname + "\" + \"" + wavenameinlist + "\""
		counter += 1
		endif
	Endfor
	KillStrings temp1, temp2, temp3, folderpathname, wavenameinlist
	SetDataFolder root:spacan
END

// FUNCTION CALLED BY BUTTON "DoCorrelogram" IN MAIN WINDOW
FUNCTION ButtonDoCorrelogram (ctrlName) : ButtonControl
	String ctrlName
	//Nvar CorrCh1, CorrCh2, CorrMethod
	Nvar RefCh1BoxVar, RefCh2BoxVar, TestCh1BoxVar, TestCh2BoxVar
	Nvar RefUsePeak, RefUseHR, TestUsePeak, TestUseHR
	Nvar RealPeakPosition1, RealPeakPosition2, Occurrence1, Occurrence2, Tags1, Tags2, Length1, Length2
	Nvar CorrTimeWindow, CorrBin
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	Nvar CheckUseTagsCorr
	
	Wave/T Params
	Params[6][3] = num2str(CorrTimeWindow )
	Params[7][3] = num2str(CorrBin)
	Params[8][3] = num2str(RefCh1BoxVar)
	Params[9][3] = num2str(TestCh1BoxVar)
	Params[10][3] = num2str(RefCh2BoxVar)
	Params[11][3] = num2str(TestCh2BoxVar)
	Params[12][3] = num2str(RefUsePeak)
	Params[13][3] = num2str(RefUseHR)
	Params[14][3] = num2str(TestUsePeak)
	Params[15][3] = num2str(TestUseHR)
	Params[16][3] = num2str(CCTypeRaw)
	Params[17][3] = num2str(CCTypeShuff)
	Params[18][3] = num2str(CCTypeNorm)
	Params[19][3] = num2str(CheckUseTagsCorr)
	
	Nvar CheckUseTagsCorr
	Nvar Ch1Detected, Ch2Detected
	if (Ch1Detected == 0 && Ch2Detected == 0)
		Abort "Detect events first!"
	endif
	if ((Ch1Detected == 0 && Ch2Detected == 1) && (RefCh1BoxVar == 1 || TestCh1BoxVar == 1))
		Abort "No events detected in channel 1!"
	endif
	if ((Ch2Detected == 0 && Ch1Detected == 1) && (RefCh2BoxVar == 1 || TestCh2BoxVar == 1))
		Abort "No events detected in channel 2!"
	endif	
	if (CheckUseTagsCorr)
		if (RefCh1BoxVar == 1 || TestCh1BoxVar == 1)
			WaveStats/Q Tags1
			Variable SumTags1 = V_sum
		endif
		if (RefCh2BoxVar == 1 || TestCh2BoxVar == 1)
			WaveStats/Q Tags2
			Variable SumTags2 = V_sum
		endif		
		if (SumTags1 == 0 && SumTags2 == 0)
			if (RefCh1BoxVar == 1 && TestCh1BoxVar == 1)
				Abort "No events tagged in channel 1!"
			endif
			if (RefCh2BoxVar == 1 && TestCh2BoxVar == 1)
				Abort "No events tagged in channel 2!"
			endif
			if ((RefCh1BoxVar && TestCh2BoxVar) || (RefCh2BoxVar && TestCh1BoxVar))
				Abort "No events tagged in either channel!"
			endif
		endif
		if ((SumTags1 == 0 && SumTags2 != 0) && (RefCh1BoxVar == 1 || TestCh1BoxVar == 1))
			Abort "No events tagged in channel 1!"
		endif
		if ((SumTags2 == 0 && SumTags1 != 0) && (RefCh2BoxVar == 1 || TestCh2BoxVar == 1))
			Abort "No events tagged in channel 2!"
		endif		
	endif
	
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"

	if (CheckUseTagsCorr == 0)
		if (RefCh1BoxVar == 1 || TestCh1BoxVar == 1)
			Duplicate/o Tags1 Tags1Corr
			Tags1Corr = 1
		endif
		if (RefCh2BoxVar == 1 || TestCh2BoxVar == 1)
			Duplicate/o Tags2 Tags2Corr
			Tags2Corr = 1
		endif
	endif
	
	if (CheckUseTagsCorr)
		if (RefCh1BoxVar == 1 || TestCh1BoxVar == 1)
			Duplicate/o Tags1 Tags1Corr
		endif
		if (RefCh2BoxVar == 1 || TestCh2BoxVar == 1)
			Duplicate/o Tags2 Tags2Corr
		endif		
	endif
	
	// First case: ref = Ch1 and test = Ch1
	if (RefCh1BoxVar == 1 && RefCh2BoxVar == 0 && TestCh1BoxVar == 1 && TestCh2BoxVar == 0)
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(RealPeakPosition1, Tags1Corr, RealPeakPosition1, Tags1Corr, Length1, Length1)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(Occurrence1, Tags1Corr, Occurrence1, Tags1Corr, Length1, Length1)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(Occurrence1, Tags1Corr, RealPeakPosition1, Tags1Corr, Length1, Length1)
		endif
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(RealPeakPosition1, Tags1Corr, Occurrence1, Tags1Corr, Length1, Length1)
		endif
	endif
	// Second case: ref = Ch2 and test = Ch2
	if (RefCh1BoxVar == 0 && RefCh2BoxVar == 1 && TestCh1BoxVar == 0 && TestCh2BoxVar == 1)
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(RealPeakPosition2, Tags2Corr, RealPeakPosition2, Tags2Corr, Length2, Length2)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(Occurrence2, Tags2Corr, Occurrence2, Tags2Corr, Length2, Length2)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(Occurrence2, Tags2Corr, RealPeakPosition2, Tags2Corr, Length2, Length2)
		endif
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(RealPeakPosition2, Tags2Corr, Occurrence2, Tags2Corr, Length2, Length2)
		endif
	endif
	// Third case: ref = Ch2 and test = Ch1
	if (RefCh1BoxVar == 0 && RefCh2BoxVar == 1 && TestCh1BoxVar == 1 && TestCh2BoxVar == 0)
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(RealPeakPosition2, Tags2, RealPeakPosition1, Tags1Corr, Length2, Length1)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(Occurrence2, Tags2Corr, Occurrence1, Tags1Corr, Length2, Length1)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(Occurrence2, Tags2Corr, RealPeakPosition1, Tags1Corr, Length2, Length1)
		endif
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(RealPeakPosition2, Tags2Corr, Occurrence1, Tags1Corr, Length2, Length1)
		endif
	endif
	// Fourth case: ref = Ch1 and test = Ch2
	if (RefCh1BoxVar == 1 && RefCh2BoxVar == 0 && TestCh1BoxVar == 0 && TestCh2BoxVar == 1)
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(RealPeakPosition1, Tags1Corr, RealPeakPosition2, Tags2Corr, Length1, Length2)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(Occurrence1, Tags1Corr, Occurrence2, Tags2Corr, Length1, Length2)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(Occurrence1, Tags1Corr, RealPeakPosition2, Tags2Corr, Length1, Length2)
		endif
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(RealPeakPosition1, Tags1Corr, Occurrence2, Tags2Corr, Length1, Length2)
		endif
	endif
END

// The value of the baseline is set by the user in points; this procedure calculates the corresponding value in ms
// This function is triggered when the user enters a value for the variable BSLpnts1(2)
FUNCTION SetBSLms (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName ; Variable varNum ; String varStr ; String varName
	Nvar NumChannels
	if (NumChannels == 2)
		Nvar BSLpnts1, BSLms1, SI1; BSLms1 = (BSLpnts1-1)*SI1*1e-3
		Nvar BSLpnts2, BSLms2, SI2; BSLms2 = (BSLpnts2-1)*SI2*1e-3
	endif
END

// The value of the rise time is set by the user in points; this procedure calculates the corresponding value in ms
// This function is triggered when the user enters a value for the variable RTpnts1(2)
FUNCTION SetRTms (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName ; Variable varNum ; String varStr ; String varName
	Nvar NumChannels
	if (NumChannels == 2)
		Nvar RTpnts1, RTms1, SI1; RTms1 = (RTpnts1+1)*SI1*1e-3
		Nvar RTpnts2, RTms2, SI2; RTms2 = (RTpnts2+1)*SI2*1e-3
	endif
END

// The value of the peak is set by the user in points; this procedure calculates the corresponding value in ms
// This function is triggered when the user enters a value for the variable Ppnts1(2)
FUNCTION SetPms (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName ; Variable varNum ; String varStr ; String varName
	Nvar NumChannels
	if (NumChannels == 2)
		Nvar Ppnts1, Pms1, SI1; Pms1 = (Ppnts1-1)*SI1*1e-3
		Nvar Ppnts2, Pms2, SI2; Pms2 = (Ppnts2-1)*SI2*1e-3
	endif
END

FUNCTION ButtonDandD (ctrlName) : ButtonControl
	String ctrlName
	Nvar MainWindowTab1
	Variable ChNumber = MainWindowTab1 + 1	// Channel number
	String ControlName1 = "DW" + num2str(ChNumber)
	String ControlName2 = "Detect" + num2str(ChNumber)
	
	ButtonDW(ControlName1)
	ButtonDetect(ControlName2)
END

// FUNCTION TRIGGERED BY BUTTON "ButtonDW1" (DIFFERENTIATE WAVE 1)
FUNCTION ButtonDW (ctrlName) : ButtonControl
	String ctrlName
	
	// Global variables
	Nvar MainWindowTab1
	Variable ChNumber = MainWindowTab1 + 1	// Channel number
	Nvar BSLpnts = $"BSLpnts" + num2str(ChNumber)
	Nvar RTpnts = $"RTpnts" + num2str(ChNumber)
	Nvar Ppnts = $"Ppnts" + num2str(ChNumber)
	Nvar Threshold = $"Threshold" + num2str(ChNumber)
	Nvar SIa = $"SI" + num2str(ChNumber)
	// Global strings
	Svar FileNameA = $"Filename" + num2str(ChNumber)
	// Waves
	Wave/T Params
	// Before launching the function "Build", writes the detection parameters on the disc in a wave called "Params"
	if (ChNumber == 1)
		Params[1][1] = num2str(BSLpnts) ; Params[2][1] = num2str(RTpnts) ; Params[3][1] = num2str(Ppnts)
	endif
	if (ChNumber == 2)
		Params[7][1] = num2str(BSLpnts) ; Params[8][1] = num2str(RTpnts) ; Params[9][1] = num2str(Ppnts)
	endif
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	BUILD(BSLpnts, RTpnts, Ppnts, $FileNameA, SIa, FileNameA)
	KillWaves BASELINE
	Duplicate/o DIFF $"DIFF" + num2str(ChNumber); KillWaves DIFF
	Duplicate/o PEAK $"PEAK" + num2str(ChNumber); KillWaves PEAK
	
	Variable/G $"Diff" + num2str(ChNumber) + "Clicked"
	Nvar DiffClicked = $"Diff" + num2str(ChNumber) + "Clicked"
	DiffClicked = 1	// Tells that the diff has been performed at least once

	String ButtonDetect = "Detect" + num2str(ChNumber)
	Button $ButtonDetect Disable=0, win=MainWindow#MainPanel
END

// FUNCTION TRIGGERED BY BUTTON "ButtonD1" AND "ButtonD2"
FUNCTION ButtonDetect(ctrlName) : ButtonControl
	String ctrlName
	
	// Global variables
	Nvar MainWindowTab1; Variable ChNumber = MainWindowTab1 + 1	// Channel number
	Nvar BSLms = $"BSLms" + num2str(ChNumber)
	Nvar RTms = $"RTms" + num2str(ChNumber)
	Nvar Pms = $"Pms" + num2str(ChNumber)
	Nvar BSLpnts = $"BSLpnts" + num2str(ChNumber)
	Nvar RTpnts = $"RTpnts" + num2str(ChNumber)
	Nvar Ppnts = $"Ppnts" + num2str(ChNumber)
	Nvar Threshold = $"Threshold" + num2str(ChNumber)
	Nvar SIa = $"SI" + num2str(ChNumber)
	Nvar ChADetected = $"Ch" + num2str(ChNumber) + "Detected"
	ChADetected = 1
	Nvar ChBDetected = $"Ch" + num2str(3 - ChNumber) + "Detected"
	Nvar NumberofEventsch = $"NumberofEventsch" + num2str(ChNumber)
	Nvar FreqEvents = $"FreqEvents" + num2str(ChNumber)
	Nvar Length
	Nvar TWPD = $"TWPD" + num2str(ChNumber)
	Nvar CheckPosDeflection = $"CheckPosDeflection" + num2str(ChNumber)
	// Waves
	Wave DIFF = $"DIFF" + num2str(ChNumber)
	Wave PEAK = $"PEAK" + num2str(ChNumber)
	Wave/T Params
	// Global strings
	Svar FileNameA = $"Filename" + num2str(ChNumber)

	if (ChNumber == 1)
		Params[5][1] = num2str(Threshold)
		Params[4][1] = num2str(TWPD)
		if (CheckPosDeflection == 1)
			Params[6][1] = "1"
		endif
		if (CheckPosDeflection == 0)
			Params[6][1] = "-1"
		endif
	endif
	if (ChNumber == 2)
		Params[11][1] = num2str(Threshold)
		Params[10][1] = num2str(TWPD)
		if (CheckPosDeflection == 1)
			Params[12][1] = "1"
		endif
		if (CheckPosDeflection == 0)
			Params[12][1] = "-1"
		endif
	endif
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	DETECT($FileNameA, DIFF, PEAK, BSLms, RTms, Pms, BSLpnts, RTpnts, Ppnts, Threshold, SIa,TWPD,CheckPosDeflection)
	NumberofEventsch = numpnts(DetectedLevels)
	FreqEvents = NumberofEventsch / Length

	Duplicate/o DetectedLevels, $"DetectedLevels" + num2str(ChNumber); 		KillWaves DetectedLevels
	Duplicate/o PeakY, $"PeakY" + num2str(ChNumber);						KillWaves PeakY
	Duplicate/o PeakX, $"PeakX" + num2str(ChNumber);						KillWaves PeakX
	Duplicate/o BaselineY, $"BaselineY" + num2str(ChNumber);				KillWaves BaselineY
	Duplicate/o BaselineX, $"BaselineX" + num2str(ChNumber);				KillWaves BaselineX
	Duplicate/o Occurrence, $"Occurrence" + num2str(ChNumber);				KillWaves Occurrence
	Duplicate/o RealAmp, $"RealAmp" + num2str(ChNumber);					KillWaves RealAmp
	Duplicate/o RealPeakPosition, $"RealPeakPosition" + num2str(ChNumber);	KillWaves RealPeakPosition
	Duplicate/o RiseTime2080, $"RiseTime2080" + num2str(ChNumber);			KillWaves RiseTime2080
	Duplicate/o RT20, $"RT20" + num2str(ChNumber);						KillWaves RT20
	Duplicate/o RT80, $"RT80" + num2str(ChNumber);						KillWaves RT80
	Duplicate/o EventFootTime $"EventFootTime" + num2str(ChNumber);		KillWaves EventFootTime

	Variable/G $"EventNumber" + num2str(ChNumber)
	Nvar EventNumber = $"EventNumber" + num2str(ChNumber)
	EventNumber = 0

	K16 = 20 ; K17 = 40
	
	Variable/G $"RT2080" + num2str(ChNumber)
	Variable/G $"AMP" + num2str(ChNumber)
	Variable/G $"EventOccurrence" + num2str(ChNumber)
	Variable/G $"EventOccurrenceMin" + num2str(ChNumber)
	Variable/G $"EventOccurrenceSec" + num2str(ChNumber)
	
	Variable/G $"KbdSpeed" + num2str(ChNumber)
	Nvar KbdSpeed = $"KbdSpeed" + num2str(ChNumber)
	KbdSpeed = 3
	
	SHOW(ChNumber,1,0)
END

// FUNCTION THAT CALCULATES THE TIME DERIVATIVE OF THE ORIGINAL TRACE
FUNCTION BUILD (Baseline_pnts, RiseTime_pnts, Peak_pnts, DataWave, SampleInterval, CurrentFileName)
	Variable Baseline_pnts, RiseTime_pnts, Peak_pnts, Sampleinterval
	String CurrentFileName
	Wave DataWave
	// Baseline_pnts, RiseTime_pnts & Peak_pnts must be in points
	// Baseline_pnts & Peak_pnts must be odd numbers
	// The function BUILD uses the user's parameters to build a differentiate wave from the original wave (DataWave).
	// The use of the right number of points accounting for the rise time is critical.	
	Wave DIFF ; KillWaves DIFF
	// ---------- FIRST STEP: DEFINE NUMBER OF POINTS FOR BASELINE, RISE TIME & PEAK ----------
	Variable Baseline_ms ; Baseline_ms = (Baseline_pnts-1)*SampleInterval*1e-3
	Variable RiseTime_ms ; RiseTime_ms = (RiseTime_pnts+1)*SampleInterval*1e-3
	Variable Peak_ms ; Peak_ms = (Peak_pnts-1)*SampleInterval*1e-3
	// --------- SECOND STEP: BUILD THE WAVE THAT WILL BE USED FOR DETECTION ----------
	Duplicate/o DataWave BASELINE
	Smooth/B Baseline_pnts, BASELINE
	Duplicate/o DataWave PEAK
	Smooth/B Peak_pnts, PEAK
	Duplicate/o BASELINE DIFF
	DIFF = BASELINE[p] - PEAK[p+floor(Baseline_pnts/2)+RiseTime_pnts+floor(Peak_pnts/2)+1]
END	

// FUNCTION THAT USES THE TIME DERIVATIVE OF THE SIGNAL TO PERFORM THE THRESHOLD DETECTION
FUNCTION DETECT(DataWave, DifferentiateWave, WavePeak, Baseline_ms, RiseTime_ms, Peak_ms, Baseline_pnts, RiseTime_pnts, Peak_pnts, Thrshld, SampleInterval, TWPD, checkposdeflection)
	Wave DataWave, DifferentiateWave, WavePeak
	Variable Baseline_ms, RiseTime_ms, Peak_ms, Baseline_pnts, RiseTime_pnts, Peak_pnts, Thrshld, SampleInterval, TWPD
	Variable checkposdeflection // if = 1, means detect positive deflections
	String CurrentFileName
	// --------- FIRST STEP: DETECT EVENTS IN DIFFERENTIATE WAVE ----------
	// Charly **************************Here I just use my own findLevels function to be able to load more heavy waves (slower but less memory use)
	if (checkposdeflection == 1)
		My_find_levels(DifferentiateWave, "DetectedLevels", -1*Thrshld)
	else
		My_find_levels(DifferentiateWave, "DetectedLevels", Thrshld)
	endif
	Wave DetectedLevels
	//***************************Original method*************
//	FindLevels/Q DifferentiateWave, Thrshld
//	Wave W_FindLevels ; Duplicate/o W_FindLevels DetectedLevels; KillWaves W_FindLevels
	// Charly End of my changes*******************************************
	
	//Nvar NumberOfEvents ; NumberOfEvents = numpnts(DetectedLevels)
	Variable i
	For ( i = (NumPnts(DetectedLevels)-1) ; i >= 1 ; i-=1 )
		if (checkposdeflection == 1)
			if ( DifferentiateWave[ floor(DetectedLevels[i] / (SampleInterval*1e-6)) + 1 ] - DifferentiateWave[ floor(DetectedLevels[i] / (SampleInterval*1e-6)) ] > 0 || DetectedLevels[i] - DetectedLevels[i-1] <= .001)
				DeletePoints i, 1, DetectedLevels
			endif			
		else
			if ( DifferentiateWave[ floor(DetectedLevels[i] / (SampleInterval*1e-6)) + 1 ] - DifferentiateWave[ floor(DetectedLevels[i] / (SampleInterval*1e-6)) ] < 0 || DetectedLevels[i] - DetectedLevels[i-1] <= .001)
				DeletePoints i, 1, DetectedLevels
			endif
		endif
	endfor
	if (checkposdeflection == 1)
		if ( DifferentiateWave[ floor(DetectedLevels[0] / (SampleInterval*1e-6)) + 1 ] - DifferentiateWave[ floor(DetectedLevels[0] / (SampleInterval*1e-6)) ] > 0 )
			DeletePoints 0, 1, DetectedLevels
		endif
	else
		if ( DifferentiateWave[ floor(DetectedLevels[0] / (SampleInterval*1e-6)) + 1 ] - DifferentiateWave[ floor(DetectedLevels[0] / (SampleInterval*1e-6)) ] < 0 )
			DeletePoints 0, 1, DetectedLevels
		endif
	endif
	// ---------- SECOND STEP: SAVE AMPLITUDE OF DETECTED EVENTS ----------
	
	// Charly********************************Here I use duplicate to avoid putting values in the loop
//	Make/o/N=(numpnts(DetectedLevels)) BaselineX, BaselineY, PeakX, PeakY
	Make/o/N=(numpnts(DetectedLevels)) BaselineY, PeakX, PeakY
	Duplicate/O DetectedLevels, BaselineX
	// Charly End of my changes*******************************************
	// Make/o/N=(numpnts(DetectedLevels)) Amplitudes
	For ( i = 0; i < numpnts(DetectedLevels); i += 1)
		// Two ways of choosing the baseline: either we trust the detection settings, especially the rise time and so we calculate the baseline corresponding
		// to the local maximum of the differentiate wave (DIFF), as follow:
		// Wavestats/Q/R=(W_FindLevels[i], W_FindLevels[i] + .006) DifferentiateWave
		//Amplitudes[i] = -V_max
		//BaselineX[i] = V_maxLoc
		// Second way: we consider that the baseline corresponding to the point crossing the threshold will be correct, and so:
		// Charly**************************** Now useless
//		BaselineX[i] = DetectedLevels[i]
		// Charly End of my changes*******************************************
		//Wave DataWave
		BaselineY[i] = mean(DataWave, BaselineX[i] - Baseline_ms*.001/2, BaselineX[i] + Baseline_ms*.001/2)
		//PeakX[i] = BaselineX[i] + Baseline_ms*.001/2 + RiseTime_ms*.001 + Peak_ms*.001/2
		//PeakY[i] = Amplitudes[i] + BaselineY[i]
	Endfor
	// ---------- THIRD STEP: CALCULATING THE TIME OF OCCURRENCE OF DETECTED EVENTS ----------
	
	// Charly *******************************Here I use duplicate to be sure wave have same number of points and are double precision (usefull for long recordings)
	Duplicate/O DetectedLevels, Occurrence, RealPeakPosition, RiseTime2080, RT20, RT80
//	Make/o/N=(numpnts(DetectedLevels)) Occurrence
	//****************This one normaly doesn't need to be double precision	
	Make/o/N=(numpnts(DetectedLevels)) RealAmp
//	Make/o/N=(numpnts(DetectedLevels)) RealPeakPosition
//	Make/o/N=(numpnts(DetectedLevels)) RiseTime2080
//	Make/o/N=(numpnts(DetectedLevels)) RT20
//	Make/o/N=(numpnts(DetectedLevels)) RT80
	// Charly End of my changes*******************************************

	// Guillaume Jan 10, 2010 - adding a wave to store a different time of occurrence:
	// the intersection between the baseline and the 20-80% rise time line (the "foot" of the event).
	Duplicate/o DetectedLevels EventFootTime
	// End of Guillaume Jan 10, 2010

	For (i = 0; i < numpnts(DetectedLevels); i += 1)
		// Measure the value of the peak and the peak location
		Wavestats/Q/R=(BaselineX[i], BaselineX[i] + Baseline_ms*.001/2 + RiseTime_ms*.001 + Peak_ms*.001/2 + TWPD*0.001) WavePeak
		// Added when introducing the checkboxes to select detection of positive or negative deflections
		if (checkposdeflection == 1)
			RealAmp[i] =  V_max - BaselineY[i]
			RealPeakPosition[i] = V_maxloc
		else
			RealAmp[i] = V_min - BaselineY[i]
			RealPeakPosition[i] = V_minloc
		endif
		// Measure the position of the half-rise
		FindLevel/Q/R=(BaselineX[i], RealPeakPosition[i]) WavePeak (BaselineY[i] + RealAmp[i]/2)
		Occurrence[i] = V_LevelX
		// Measure the position of the 80% rise time
		FindLevel/Q/R=(Occurrence[i], Occurrence[i] + (RiseTime_pnts+floor(Peak_pnts/2)+20)*SampleInterval*1e-6) DataWave (BaselineY[i] + RealAmp[i]*0.8)
		RT80[i] = V_LevelX
		// Measure the position of the 20% rise time
		Duplicate/o/R=(Occurrence[i]-.01, Occurrence[i]) DataWave TEMPWAVE
		TEMPWAVE = TEMPWAVE[numpnts(TEMPWAVE)-1-p]
		SetScale/P x 0,SampleInterval*1e-6, "s", TEMPWAVE
		FindLevel/Q TEMPWAVE (BaselineY[i] + RealAmp[i]*0.2)
		RT20[i] = Occurrence[i] - V_LevelX
		// 20-80% rise time
		RiseTime2080[i] = RT80[i] - RT20[i]
		// Guillaume Jan 10, 2010
		EventFootTime[i] = RT20[i] - (1/3)*(RT80[i] - RT20[i])
		// end of Guillaume Jan 10, 2010
	Endfor
	KillWaves TEMPWAVE
	// ---------- END OF THIRD STEP ----------
END	

Function Show1TabProc(CtrlName, TabNum) : TabControl
	String CtrlName
	Variable TabNum
	
	Variable isTab0= tabNum==0
	Variable isTab1= tabNum==1
	Variable isTab2= tabNum==2
	Variable isTab3= tabNum==3
	Variable isTab4= tabNum==4
	Variable isTab5= tabNum==5
	
	// note: disable=0 means "show", disable=1 means "hide"

	// This line may be in use later 
	SetActiveSubWindow DetectedEvents1#DetectedEvents1Panel
	
	// *************************************For Average Tab***************************************
	ModifyControl CheckRiseTime1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetRTinf1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetRTsup1 disable= !isTab0	// hide if not Tab 0
	ModifyControl CheckBaselineSD1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBslnSDinf1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBslnSDsup1 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonShowRT2080histo1 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonShowBslnSDHisto1 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonAverage1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBaseline1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetDecay1 disable= !isTab0	// hide if not Tab 0
	ModifyControl NumEventsAveraged1 disable= !isTab0	// hide if not Tab 0
	ModifyControl FiltersDisplayText1 disable= !isTab0
	ModifyControl TitleBoxAlign1 disable= !isTab0
	ModifyControl CheckBoxAlign1Foot disable= !isTab0
	ModifyControl CheckBoxAlign1Occ disable= !isTab0
	ModifyControl CheckBoxAlign1Peak disable= !isTab0
//	ModifyControl ShowSD1 disable= !isTab0
	ModifyControl TitleAvTab1 disable= !isTab0
	
	//****************************************for Sup Tab*************************************
	ModifyControl TitleSupTab1 disable= !isTab2
	ModifyControl ButtonSupEvents1 disable= !isTab2
	ModifyControl CheckBoxIncludeAv1 disable= !isTab2
	ModifyControl CBShowIndEvents1 disable= !isTab2
	ModifyControl CheckBoxIncludeAv1 disable= !isTab2
	ModifyControl CBIncludeSD1 disable= !isTab2
	ModifyControl TitleBoxSupTabAlign1 disable= !isTab2
	ModifyControl CBAlign1Foot disable= !isTab2
	ModifyControl CBAlign1Occ disable= !isTab2
	ModifyControl CBAlign1Peak disable= !isTab2
	ModifyControl SetVarBaseline1 disable= !isTab2
	ModifyControl SetVarDecay1 disable= !isTab2
	ModifyControl CBNormalize1 disable= !isTab2
	ModifyControl SetVarSupNormLeft1 disable= !isTab2
	ModifyControl SetVarSupNormRight1 disable= !isTab2
	ModifyControl CBShowCurs1 disable= !isTab2
	ModifyControl CBShowRefLine1 disable= !isTab2
	ModifyControl SetVarRefLine1 disable= !isTab2
	ModifyControl CBUseRndColors1 disable= !isTab2
	ModifyControl CBRemoveAxes1 disable= !isTab2
	ModifyControl ListBoxColors1 disable= !isTab2
	ModifyControl CBOverlayMode1 disable= !isTab2
	
	//****************************************for Kinetics Tab*************************************
	ModifyControl CFPE1 disable= !isTab4	// hide if not Tab 1
	ModifyControl CFAE1 disable= !isTab4	// hide if not Tab 1
	ModifyControl FreeOffset1 disable= !isTab4	// hide if not Tab 1
	ModifyControl Fit2Peak1 disable= !isTab4	// hide if not Tab 1
	ModifyControl FitSingleExp1 disable= !isTab4	// hide if not Tab 1
	ModifyControl FitdbExp1 disable= !isTab4	// hide if not Tab 1
	ModifyControl BFitsigdbexp1 disable= !isTab4	// hide if not Tab 1
	ModifyControl BFitsigmoid1 disable= !isTab4	// hide if not Tab 1
	ModifyControl BFitAllEventsdbExp1 disable= !isTab4	// hide if not Tab 1
	ModifyControl Start_Fit1 disable= !isTab4	// hide if not Tab 1
	ModifyControl End_Fit1 disable= !isTab4 // hide if not Tab 1
	ModifyControl TauFast1 disable= !isTab4	// hide if not Tab 1
	ModifyControl TauSlow1 disable= !isTab4	// hide if not Tab 1
	ModifyControl FastPercentage1 disable= !isTab4	// hide if not Tab 1
	ModifyControl SlowPercentage1 disable= !isTab4	// hide if not Tab 1
	ModifyControl ValOffset1 disable= !isTab4	// hide if not Tab 1
	ModifyControl KineticIndex1 disable= !isTab4	// hide if not Tab 1
	
/// Find out howto fix
	ModifyControl CheckBFitSelectedChan11 disable= !isTab4
	ModifyControl CheckBFitSelectedChan12 disable= !isTab4
	//ModifyControl FitDisplayText1 disable= !isTab1
	
	ModifyControl TitleFitsTab1 disable= !isTab4



	// *********************************For Pairs Tab ***************************************

/// Find out howto fix	
	ModifyControl ButtonSync1 disable= !isTab5		// hide if not Tab 2
	ModifyControl ButtonMAS1 disable= !isTab5	// hide if not Tab 2
	
	ModifyControl SetSyncWinStart1 disable= !isTab5 	// hide if not Tab 2
	ModifyControl SetSyncWinEnd1 disable= !isTab5		// hide if not Tab 2
	
	ModifyControl SetMASPeakWinWidth1 disable= !isTab5		// hide if not Tab 2
	ModifyControl SetMASBslWidth1 disable= !isTab5		// hide if not Tab 2
	ModifyControl SetMASAPRiseTime1 disable= !isTab5		// hide if not Tab 2
	ModifyControl SetMASLagToPeak1 disable= !isTab5		// hide if not Tab 2
	ModifyControl SetMASTrueHistbinSize1 disable= !isTab5		// hide if not Tab 2
	ModifyControl SetMASNoiseHistBinSize1 disable= 1		// hide if not Tab 2
	ModifyControl SetMASHistMin1 disable= 1		// always hide
	ModifyControl SetMASHistMax1 disable= 1		// always hide	
	ModifyControl TitlePairsTab1 disable= !isTab5
	ModifyControl SetMASStart1 disable= !isTab5
	ModifyControl SetMAsEnd1 disable= !isTab5
	ModifyControl SetMASpharmacoN1 disable =!isTab5

	//*************************************** TAB "TAGS" ***********************************
	ModifyControl SetIsolateEventsWinStart1 disable= !isTab1		// hide if not Tab 3
	ModifyControl SetIsolateEventsWinEnd1 disable= !isTab1
	ModifyControl ButtonisolateEvents1 disable= !isTab1
	ModifyControl ButtonLowPass1 disable= !isTab1
	ModifyControl SetLowPassThrshld1 disable= !isTab1
	ModifyControl ButtonTagRangeOfEvents1 disable= !isTab1
	ModifyControl ButtonUntagRangeOfEvents1 disable= !isTab1
	ModifyControl SetBegin1 disable= !isTab1
	ModifyControl SetEnd1	 disable= !isTab1
	ModifyControl ButtonTagRangeOfEvents1_pnts disable= !isTab1
	ModifyControl ButtonUntagRangeOfEvents1_pnts disable= !isTab1
	ModifyControl SetBegin1_pnts disable= !isTab1
	ModifyControl SetEnd1_pnts disable= !isTab1
	ModifyControl BtnSaveTags1 disable= !isTab1
	ModifyControl BtnDeleteNaN1 disable= !isTab1
//	ModifyControl PopupOriginOfTags1 disable= !isTab1
	ModifyControl TitleTagsTab1 disable= !isTab1
	
	//******************************************TAB "Integrals"*******************************
	ModifyControl CheckBIntSelectedChan11 disable = !isTab3
	ModifyControl CheckBIntSelectedChan12 disable = !isTab3
	ModifyControl CheckBIntEvent1 disable = !isTab3
	ModifyControl CheckBIntAvg1 disable = !isTab3
	ModifyControl SetIntegralStart1 disable = !isTab3
	ModifyControl SetIntegralEnd1 disable = !isTab3
	ModifyControl SetIntAvgWidth1 disable = !isTab3
	ModifyControl ChkboxIntegralOffset1 disable = !isTab3
	ModifyControl Integrate1 disable = !isTab3
	ModifyControl DisplayArea1 disable = !isTab3
	ModifyControl TitleIntTab1 disable= !isTab3
//	
	// ***********************************Add your tab here***********************************
	
	//************************************************************************************
	Nvar CurrentTab1
	CurrentTab1 = TabNum
	return 0

End

// THIS FUNCTION BUILDS OR UPDATES THE DETECTED EVENTS WINDOW
// This function is "generalist" (shared by channel 1 and 2)
// Therefore extensive use of strings and $ in the code
FUNCTION SHOW(ChNumber, Number, EventNumber)
	Variable ChNumber	// says if function called by channel 1 or 2
	Variable Number		// says if function used for the first time (=1) or not (=2)
	Variable EventNumber	// number of the event to display

	// ----- Programming tip -----
	// To create a new wave using $
	// Make/o $"wave name"
	// Wave wavename = $"wave name"
	// ... use wavename as regular wave
	// Same for global variables etc

	// Global variables
	Nvar KbdSpeed = $"KbdSpeed" + num2str(ChNumber)
	Nvar CheckPosDeflection = $"CheckPosDeflection" + num2str(ChNumber)
	Nvar BSLms = $"BSLms" + num2str(ChNumber)
	Nvar Pms = $"Pms" + num2str(ChNumber)
	Nvar SIa = $"SI" + num2str(ChNumber)
	Nvar SIb = $"SI" + num2str(3 - ChNumber)
	Nvar NumberofEventsch = $"NumberofEventsch" + num2str(ChNumber)
	Nvar RT2080 = $"RT2080" + num2str(ChNumber)
	Nvar AMP = $"AMP" + num2str(ChNumber)
	Nvar EventOccurrence = $"EventOccurrence" + num2str(ChNumber)
	Nvar EventOccurrenceMin = $"EventOccurrenceMin" + num2str(ChNumber)
	Nvar EventOccurrenceSec = $"EventOccurrenceSec" + num2str(ChNumber)
	Nvar KbdSpeed = $"KbdSpeed" + num2str(ChNumber)
	Nvar ToOHR = $"ToOHR_" + num2str(ChNumber)
	Nvar ToOPeak = $"ToOPeak_" + num2str(ChNumber)
	Nvar Length
	Nvar CaE = $"C" + num2str(ChNumber) + "E"
	Nvar CbE = $"C" + num2str(3 - ChNumber) + "E"
	// Waves
	Wave DetectedLevels = $"DetectedLevels" + num2str(ChNumber)
	Wave Tags = $"Tags" + num2str(ChNumber)
	Wave Occurrence = $"Occurrence" + num2str(ChNumber)
	Wave BaselineY = $"BaselineY" + num2str(ChNumber)
	Wave BaselineX = $"BaselineX" + num2str(ChNumber)
	Wave PeakY = $"PeakY" + num2str(ChNumber)
	Wave PeakX = $"PeakX" + num2str(ChNumber)
	Wave RealAmp = $"RealAmp" + num2str(ChNumber)
	Wave RealPeakPosition = $"RealPeakPosition" + num2str(ChNumber)
	Wave RiseTime2080 = $"RiseTime2080" + num2str(ChNumber)
	Wave RT20 = $"RT20" + num2str(ChNumber)
	Wave RT80 = $"RT80" + num2str(ChNumber)
	Wave EventFootTime = $"EventFootTime" + num2str(ChNumber)
	// Global Strings
	Svar FileNameA = $"FileName" + num2str(ChNumber)
	Svar FileNameB = $"FileName" + num2str(3 - ChNumber)

	// Time window limits
	if (Number == 1)
		Variable/G $"msBefore" + num2str(ChNumber)
		Nvar msBefore = $"msBefore" + num2str(ChNumber)
		msBefore = 20
	
		Variable/G $"msAfter" + num2str(ChNumber)
		Nvar msAfter = $"msAfter" + num2str(ChNumber)
		msAfter = 40	
	endif
	if (Number == 2)
		Nvar msBefore = $"msBefore" + num2str(ChNumber)
		Nvar msAfter = $"msAfter" + num2str(ChNumber)
	endif

	// Save the event in the wave TEMPEVENT#
	Duplicate/o/R=(Occurrence[EventNumber] - msBefore*.001, Occurrence[EventNumber] + msAfter*.001) $FileNameA TEMPWAVE
	TEMPWAVE = TEMPWAVE - BaselineY[EventNumber]
	SetScale/P x -msBefore*.001, (SIa*1e-6), "s", TEMPWAVE
	String strTEMPEVENT = "TEMPEVENT" + num2str(ChNumber)
	Duplicate/o TEMPWAVE $strTEMPEVENT; KillWaves TEMPWAVE
	Wave TEMPEVENT = $"TEMPEVENT" + num2str(ChNumber)

	// ----------------- CURSORS -----------------
	
	// Line cursors
	Make/o/N=2 $"YcsrA" + num2str(ChNumber)
	Wave YcsrA = $"YcsrA" + num2str(ChNumber)
	Make/o/N=2 $"XcsrA" + num2str(ChNumber)
	Wave XcsrA = $"XcsrA" + num2str(ChNumber)	
	Make/o/N=2 $"YcsrB" + num2str(ChNumber)
	Wave YcsrB = $"YcsrB" + num2str(ChNumber)
	Make/o/N=2 $"XcsrB" + num2str(ChNumber)
	Wave XcsrB = $"XcsrB" + num2str(ChNumber)
	Make/o/N=2 $"YcsrC" + num2str(ChNumber)
	Wave YcsrC = $"YcsrC" + num2str(ChNumber)
	Make/o/N=2 $"XcsrC" + num2str(ChNumber)
	Wave XcsrC = $"XcsrC" + num2str(ChNumber)
	
	// Point cursors
	Make/o/N=1 $"YcsrD" + num2str(ChNumber)
	Wave YcsrD = $"YcsrD" + num2str(ChNumber)
	Make/o/N=1 $"XcsrD" + num2str(ChNumber)
	Wave XcsrD = $"XcsrD" + num2str(ChNumber)	
	Make/o/N=1 $"YcsrE" + num2str(ChNumber)
	Wave YcsrE = $"YcsrE" + num2str(ChNumber)
	Make/o/N=1 $"XcsrE" + num2str(ChNumber)
	Wave XcsrE = $"XcsrE" + num2str(ChNumber)
	Make/o/N=1 $"YcsrF" + num2str(ChNumber)
	Wave YcsrF = $"YcsrF" + num2str(ChNumber)
	Make/o/N=1 $"XcsrF" + num2str(ChNumber)
	Wave XcsrF = $"XcsrF" + num2str(ChNumber)

	// Cursor A represents the baseline
	YcsrA = 0
	XcsrA[0] = BaselineX[EventNumber] - BSLms*.001/2 - Occurrence[EventNumber]
	XcsrA[1] = BaselineX[EventNumber] + BSLms*.001/2 - Occurrence[EventNumber]
	// Cursor C represents the real event amplitude
	YcsrC = RealAmp[EventNumber]
	XcsrC[0] = RealPeakPosition[EventNumber] - Pms*.001/2 - Occurrence[EventNumber]
	XcsrC[1] = RealPeakPosition[EventNumber] + Pms*.001/2 - Occurrence[EventNumber]
	// Cursor D marks RT20%
	YcsrD = RealAmp[EventNumber] * 0.2
	XcsrD = RT20[EventNumber] - Occurrence[EventNumber]
	// Cursor E marks RT80%
	YcsrE = RealAmp[EventNumber] * 0.8
	XcsrE = RT80[EventNumber] - Occurrence[EventNumber]
	RT2080 = 1000*RiseTime2080[EventNumber]
	AMP = RealAmp[EventNumber]
	// Cursor F marks the foot of the event
	YcsrF = 0
	XcsrF = EventFootTime[EventNumber] - Occurrence[EventNumber]

	// Prepare strings that define the name of windows and tabs and tab proc
	String DetectedEvents = "DetectedEvents" + num2str(ChNumber)
	String DetectedEventsPanel = "DetectedEvents" + num2str(ChNumber) + "Panel"
	String EventChannelA = "EventChannel" + num2str(ChNumber)
	String EventChannelAB = "EventChannel" + num2str(ChNumber) + num2str(3 - ChNumber)
	String ShowTab = "Show" + num2str(ChNumber) + "Tab"
	String ShowTabProc = "Show" + num2str(ChNumber) + "TabProc"
	String CheckBoxTag = "CheckBoxTag" + num2str(ChNumber)

	If (Number == 1) // IF FUNCTION RAN FOR THE FIRST TIME
	
		Make/o/N=(numpnts(DetectedLevels)) $"Tags" + num2str(ChNumber)
		Wave Tags = $"Tags" + num2str(ChNumber)
		Tags = 0		// INITIALIZE TAGS

		DoWindow/K $DetectedEvents; Display/W=(220,0,900,450)/N=$DetectedEvents; DoWindow/C $DetectedEvents	// INITIALIZE WINDOW
		
		// BUILDING LEFT PANEL
		NewPanel/K=1/N=$DetectedEventsPanel/HOST=$DetectedEvents/FG=(FL, FT, gRight, FB )/W=(0,0,275,450) ; ModifyPanel cbRGB=(65500,65500,65500)
		SetDrawLayer/W=$DetectedEvents#$DetectedEventsPanel UserBack
		SetDrawEnv/W=$DetectedEvents#$DetectedEventsPanel fstyle= 3, fsize=16
		if (ChNumber == 1)
			SetDrawEnv/W=$DetectedEvents#$DetectedEventsPanel textrgb=(0,0,65535)
		endif
		if (ChNumber == 2)
			SetDrawEnv/W=$DetectedEvents#$DetectedEventsPanel textrgb=(65535,0,0)
		endif
		DrawText 70,25,"Detected events - Channel " + num2str(ChNumber)
		
		// USE THIS VARIABLE TO MOVE BLOCKS OF CONTROLS
		Variable TabPosOffset=-7
		Variable ControlsPosOffset = 77
		
		Variable/G $"CurrentTab" + num2str(ChNumber)
		Nvar CurrentTab = $"CurrentTab" + num2str(ChNumber)
		CurrentTab = 0
		
		TabControl $ShowTab,pos={8,TabPosOffset+335},size={350,260}, fstyle=1,proc=$ShowTabProc, win=$DetectedEvents#$DetectedEventsPanel, value=CurrentTab
		// The following three lines create the different tabs of the tab menu
		TabControl $ShowTab,tabLabel(0)="Av.",value= 0, labelBack=(60000,60000,60000)
		TabControl $ShowTab,tabLabel(4)="Kin.",value= 0, labelBack=(60000,60000,60000)
		TabControl $ShowTab,tabLabel(5)="Pairs",value= 0, labelBack=(60000,60000,60000)
		TabControl $ShowTab,tabLabel(1)="Tags",value= 0, labelBack=(60000,60000,60000)
		TabControl $ShowTab,tabLabel(3)="Int.",value=0, labelBack=(60000,60000,60000)
		TabControl $ShowTab,tabLabel(2)="Sup.",value=0, labelBack=(60000,60000,60000)

		// FIRST GROUP: controls
		// Define names of controls
		String CloseWindow = "close_window" + num2str(ChNumber)
		String SetEventNumber = "Set_Event_Number" + num2str(ChNumber)
		String Kbd = "Kbd" + num2str(ChNumber)
		String ButtonNavigate = "ButtonNavigate" + num2str(ChNumber)
		String NavToEvent = "NavToEvent" + num2str(ChNumber)
		String SetKbdSpeed = "SetKbdSpeed" + num2str(ChNumber)
		String msBeforeEvent = "msBeforeEvent" + num2str(ChNumber)
		String msAfterEvent = "msAfterEvent" + num2str(ChNumber)
		String Grids = "Grids" + num2str(ChNumber)
		String CheckShowOtherChannel = "CheckShowChannel" + num2str(3 - ChNumber)
		String strBTDE = "BTDE" + num2str(ChNumber)
		String strBUDE = "BUDE" + num2str(ChNumber)
		String DisplayTaggedEvents = "DisplayTaggedEvents" + num2str(ChNumber)
		String CheckBoxTagAll = "CheckBoxTagAll" + num2str(ChNumber)
		String TagDisplay = "TagDisplay" + num2str(ChNumber)

		String GBControls = "GBControls" + num2str(ChNumber)

		GroupBox	$GBControls					labelBack=(60000,60000,60000), pos={8,ControlsPosOffset-40},	size={345,165},fsize=12,fstyle=1,title="Controls"

//		SetDrawLayer/W=$DetectedEvents#$DetectedEventsPanel UserBack
//		SetDrawEnv/W=$DetectedEvents#$DetectedEventsPanel fstyle= 1, fsize=14
//		DrawText 10,54,"Controls"
		
		Variable/G $"CheckGridOn" + num2str(ChNumber)
		Nvar CheckGridOn = $"CheckGridOn" + num2str(ChNumber)
		CheckGridOn = 0
		
		Button $CloseWindow 			pos={310,8},						size={40,20},			proc=ButtonCloseDetectedEvents,title="Close"

		String TBKbdMode1 = "TBKbdMode1" + num2str(ChNumber)
		String TBKbdMode2 = "TBKbdMode2" + num2str(ChNumber)
		String TBKbdMode3 = "TBKbdMode3" + num2str(ChNumber)

		TitleBox $TBKbdMode1 	frame=0, fsize=11,	fstyle=0, labelBack=(60000,60000,60000), pos={15,ControlsPosOffset-20}, 		size={15, 20},	title="Keyboard mode:"
		TitleBox $TBKbdMode2 	frame=0, fsize=11,	fstyle=0, labelBack=(60000,60000,60000), pos={110,ControlsPosOffset-20}, 		size={15, 20},	title="next/previous event = right/left arrow key"
		TitleBox $TBKbdMode3 	frame=0, fsize=11,	fstyle=0, labelBack=(60000,60000,60000), pos={110,ControlsPosOffset-6}, 		size={15, 20},	title="Tag/untag event = up arrow key - Cancel = \"C\""

		SetVariable $SetEventNumber 		labelBack=(60000,60000,60000), pos={15,ControlsPosOffset+18},		size={120,20},		proc=SetEvent,size={133,20}, title="Event #:",limits={0,(numpnts(DetectedLevels)-1),1}
		Execute "SetVariable Set_Event_Number" + num2str(ChNumber) + " value= EventNumber" + num2str(ChNumber)
		Button $Kbd 						labelBack=(60000,60000,60000), pos={142,ControlsPosOffset+17},	size={100,20},		proc=ButtonKbd,title="keyboard mode"
		SetVariable $SetKbdSpeed		labelBack=(60000,60000,60000), pos={250,ControlsPosOffset+18},	size={85,20},			title="Speed:", value=KbdSpeed, limits={1,inf,1}

		Variable/G $"NavToSeconds" + num2str(ChNumber)
		Nvar NavToSeconds = $"NavToSeconds" + num2str(ChNumber)
		NavToSeconds = 0		

		String TBShowLeftRight = "TBShowLeftRight" + num2str(ChNumber)

		Button $ButtonNavigate			labelBack=(60000,60000,60000), pos={15,ControlsPosOffset+39},size={60,20},title="Navigate",proc=ProcButtonNavigate
		SetVariable $NavToEvent			labelBack=(60000,60000,60000), pos={80,ControlsPosOffset+38},size={205,20},title="to event closest to (s)", limits={0,inf,1}
		Execute "SetVariable NavToEvent" + num2str(ChNumber) + " value= NavToSeconds" + num2str(ChNumber)
//		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0; DrawText 10,122,"Show:"
		TitleBox $TBShowLeftRight			labelBack=(60000,60000,60000), pos={15,ControlsPosOffset+60},size={40,20},frame=0,title="Show:"
		SetVariable $msBeforeEvent 		labelBack=(60000,60000,60000), pos={58,ControlsPosOffset+59},	size={110,20}, 		proc=SetEvent, title="left (ms):",limits={-Inf,Inf,1}, value= msBefore
		SetVariable $msAfterEvent 		labelBack=(60000,60000,60000), pos={175,ControlsPosOffset+59},	size={110,20}, 		proc=SetEvent, title="right (ms):",limits={-Inf,Inf,1}, value= msAfter
		CheckBox $Grids 				labelBack=(60000,60000,60000), pos={300,ControlsPosOffset+50},	size={45,17},			proc=CheckBoxGrids, title="Grid", value=0

		Variable/G $"ShowCh" + num2str(3 - ChNumber)
		Nvar ShowOtherCh = $"ShowCh" + num2str(3 - ChNumber)
		ShowOtherCh = 0

		String TBBrowsing = "TBBrowsing" + num2str(ChNumber)

		CheckBox $CheckShowOtherChannel			labelBack=(60000,60000,60000), pos={300,ControlsPosOffset+70},	size={54,20},			proc=ShowOtherChannel, title="Ch" + num2str(3 - ChNumber), value=0
//		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0 ;  DrawText 10,142,"Browsing: "
		TitleBox $TBBrowsing				labelBack=(60000,60000,60000), pos={15,ControlsPosOffset+79},size={40,20},frame=0,title="Browsing:"	
		CheckBox $strBTDE				labelBack=(60000,60000,60000), pos={77,ControlsPosOffset+80}, 	size={100,20}, 		title="tagged", proc=ProcBTDE
		CheckBox $strBUDE				labelBack=(60000,60000,60000), pos={155,ControlsPosOffset+80}, 	size={100,20}, 		title="untagged events only.", proc=ProcBUDE
		SetActiveSubwindow $DetectedEvents#$DetectedEventsPanel //Needed for drawEnv after displaying tags
		ToOHR = 1; ToOPeak = 0

		String TBTagUntag = "TBTagUntag" + num2str(ChNumber)

//		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0 ;  DrawText 10,163,"Tag/untag:"
		TitleBox TBTagUntag				labelBack=(60000,60000,60000), pos={15,ControlsPosOffset+100},size={40,20},frame=0,title="Tag/untag:"	
		
		Variable/G $"TaggedEvents" + num2str(ChNumber)	
		Nvar TaggedEvents = $"TaggedEvents" + num2str(ChNumber)	
		TaggedEvents = 0
		
		Variable/G $"CheckedTagAll" + num2str(ChNumber)	
		Nvar CheckedTagAll = $"CheckedTagAll" + num2str(ChNumber)	
		CheckedTagAll = 0

		CheckBox $CheckBoxTag 			labelBack=(60000,60000,60000), pos={77,ControlsPosOffset+101},					proc=ProcCheckBoxTag,size={45,20}, title="this event", value=0
		ValDisplay $DisplayTaggedEvents 		labelBack=(60000,60000,60000), pos={225,ControlsPosOffset+100},size={120,20},title="Tagged:"
		Execute "ValDisplay " + DisplayTaggedEvents + " value = " + num2str(TaggedEvents)	
		CheckBox $CheckBoxTagAll 			labelBack=(60000,60000,60000), pos={155,ControlsPosOffset+101}, 						proc=ProcCheckBoxTagAll, size={45,20}, title="all/none"
	
		// BTDE: Browse through detected events only
		// BUDE: Browse through undetected events only
		
		Variable/G $"BTDE_" + num2str(ChNumber)
		Nvar VarBTDE = $"BTDE_" + num2str(ChNumber)
		VarBTDE = 0

		Variable/G $"BUDE_" + num2str(ChNumber)
		Nvar VarBUDE = $"BUDE_" + num2str(ChNumber)
		VarBUDE = 0

		// DISPLAY TAGS IN A GRAPH EMBEDDED INTO THE PANEL
		
		String GBTags = "GBTags" + num2str(ChNumber)
		
		GroupBox	$GBTags	labelBack=(60000,60000,60000), pos={8,ControlsPosOffset+130},	size={345,110},fsize=12,fstyle=1,title="Tags"

		String strTickNames = "TickNames" + num2str(ChNumber)
		String strTickLoc = "TickLoc" + num2str(ChNumber)
		Make/N=2/o/T TickNames; TickNames[0] = "untagged"; TickNames[1] = "tagged"
		Make/N=2/o TickLoc; TickLoc[0] = 0; TickLoc[1] = 1
		Duplicate/o TickNames $strTickNames
		Duplicate/o TickLoc $strTickLoc
		KillWaves TickNames, TickLoc
//		SetDrawLayer/W=$DetectedEvents#$DetectedEventsPanel UserBack
//		SetDrawEnv/W=$DetectedEvents#$DetectedEventsPanel fstyle= 1, fsize=14
//		DrawText 10,186,"Tags"
		Display/HOST=$DetectedEvents#$DetectedEventsPanel/N=$TagDisplay/W=(12,ControlsPosOffset+147,345,ControlsPosOffset+236) Tags
		Label bottom "Event #"
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay manTick=0,userticks(left)={$strTickLoc,$strTickNames}
		if (ChNumber == 1)
			ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay rgb($"Tags" + num2str(ChNumber))=(0,0,65535)
		endif
		if (ChNumber == 2)
			ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay rgb($"Tags" + num2str(ChNumber))=(65535,0,0)
		endif
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay margin(bottom)=43
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay gbRGB=(60000,60000,60000)
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay wbRGB=(60000,60000,60000)
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay standoff(left)=0
		SetAxis/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay left 0,1
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay manTick(left)={0,1,0,0},manMinor(left)={0,0}
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay axOffset(left)=-3
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay axOffset(bottom)=0
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay margin(right)=-0.6

		// Cursors to mark the current event
		Make/o/N=1 $"CursorTags" + num2str(ChNumber) + "_X"
		Wave CursorTags_X = $"CursorTags" + num2str(ChNumber) + "_X"
		CursorTags_X = 0

		Make/o/N=1 $"CursorTags" + num2str(ChNumber) + "_Y"
		Wave CursorTags_Y = $"CursorTags" + num2str(ChNumber) + "_Y"
		CursorTags_Y = 0

		Appendtograph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay CursorTags_Y vs CursorTags_X
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay mode($"CursorTags" + num2str(ChNumber) + "_Y")=3,marker($"CursorTags" + num2str(ChNumber) + "_Y")=10
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay rgb($"CursorTags" + num2str(ChNumber) + "_Y")=(0,0,0)
		ModifyGraph/W=$DetectedEvents#$DetectedEventsPanel#$TagDisplay msize($"CursorTags" + num2str(ChNumber) + "_Y")=5,mrkThick($"CursorTags" + num2str(ChNumber) + "_Y")=5
		
		// Tab "Calculate Average

		// Names of controls
		String TitleAvTab = "TitleAvTab" + num2str(ChNumber)
		String FiltersDisplayText = "FiltersDisplayText" + num2str(ChNumber)
		String CheckRiseTime = "CheckRiseTime" + num2str(ChNumber)
		String ProcCheckRiseTime = "ProcCheckRiseTime" + num2str(ChNumber)
		String SetRTinf = "SetRTinf" + num2str(ChNumber)
		String SetRTsup = "SetRTsup" + num2str(ChNumber)

		TitleBox $TitleAvTab  frame=0,	fsize=12, fstyle=1, labelBack=(60000,60000,60000), pos={15,TabPosOffset+360}, size={15, 80},Title="Calculate average"
		TitleBox $FiltersDisplayText  frame=0,	labelBack=(60000,60000,60000), pos={15,TabPosOffset+380}, size={15, 20},Title="Filters (enter range):"

		// Variables related to the rise time filter

		Variable/G $"CheckRT2080" + num2str(ChNumber)
		Nvar CheckRT2080 = $"CheckRT2080" + num2str(ChNumber)
		CheckRT2080 = 0

		Variable/G $"RTinf" + num2str(ChNumber)
		Nvar RTinf = $"RTinf" + num2str(ChNumber)
		RTinf = 0
		
		Variable/G $"RTsup" + num2str(ChNumber)
		Nvar RTsup = $"RTsup" + num2str(ChNumber)
		RTsup = 2

		CheckBox $CheckRiseTime 			labelBack=(60000,60000,60000), pos={15,TabPosOffset+400},	size={100,20},	title="20-80% rise time (ms)", value=0, 	proc=$ProcCheckRiseTime
		SetVariable $SetRTinf 				labelBack=(60000,60000,60000), pos={160,TabPosOffset+400},	size={60,20},	title=" ",limits={0,inf,0.1}
		Execute "SetVariable " + SetRTinf + " value= RTinf" + num2str(ChNumber)
		SetVariable $SetRTsup 				labelBack=(60000,60000,60000), pos={225,TabPosOffset+400},	size={70,20},	title="-",limits={0,inf,0.1}
		Execute "SetVariable " + SetRTsup + " value=RTsup" + num2str(ChNumber)
		
		Variable/G $"CheckBslnSD" + num2str(ChNumber)
		Nvar CheckBslnSD = $"CheckBslnSD" + num2str(ChNumber)
		CheckBslnSD = 0
		
		Variable/G $"BslnSDinf" + num2str(ChNumber)
		Nvar BslnSDinf = $"BslnSDinf" + num2str(ChNumber)
		BslnSDinf = 0
		
		Variable/G $"BslnSDsup" + num2str(ChNumber)
		Nvar BslnSDsup = $"BslnSDsup" + num2str(ChNumber)
		BslnSDsup = 3
		
		String CheckBaselineSD = "CheckBaselineSD" + num2str(ChNumber)
		String ProcCheckBaselineSD = "ProcCheckBaselineSD" + num2str(ChNumber)
		String SetBslnSDinf = "SetBslnSDinf" + num2str(ChNumber)
		String SetBslnSDsup = "SetBslnSDsup" + num2str(ChNumber)
		String ButtonShowRT2080histo = "ButtonShowRT2080histo" + num2str(ChNumber)
		String ProcButtonShowRT2080histo = "ProcButtonShowRT2080histo" + num2str(ChNumber)
		String ButtonShowBslnSDHisto = "ButtonShowBslnSDHisto" + num2str(ChNumber)
		String ProcButtonShowBslnSDHisto = "ProcButtonShowBslnSDHisto" + num2str(ChNumber)
		String ButtonAverage = "ButtonAverage" + num2str(ChNumber)
			
		CheckBox $CheckBaselineSD	 		labelBack=(60000,60000,60000), pos={15,TabPosOffset+420},	size={100,20},	title="Baseline SD (xx)", value=0, 	proc=$ProcCheckBaselineSD
		SetVariable $SetBslnSDinf 				labelBack=(60000,60000,60000), pos={160,TabPosOffset+420},	size={60,20},	title=" ",limits={0,inf,0.1}
		Execute "SetVariable SetBslnSDinf" + num2str(ChNumber) + " value=BslnSDinf" + num2str(ChNumber)
		SetVariable $SetBslnSDsup 			labelBack=(60000,60000,60000), pos={225,TabPosOffset+420},	size={70,20},	title="-",limits={0,inf,0.1}
		Execute "SetVariable SetBslnSDsup" + num2str(ChNumber) + " value=BslnSDsup" + num2str(ChNumber)
		Button $ButtonShowRT2080histo 		labelBack=(60000,60000,60000), pos={300,TabPosOffset+400},	size={52,20},	title="Histo", 							proc=$ProcButtonShowRT2080histo

		Button $ButtonShowBslnSDHisto 		labelBack=(60000,60000,60000), pos={300,TabPosOffset+420},	size={52,20},	title="Histo", 						proc=$ProcButtonShowBslnSDHisto
		Button $ButtonAverage 				labelBack=(60000,60000,60000), pos={15,TabPosOffset+440},												proc=ProcButtonAverage, size={52,20},title="Average"

		Variable/G $"Baseline" + num2str(ChNumber)
		Nvar Baseline = $"Baseline" + num2str(ChNumber)
		Baseline = 20
		
		Variable/G $"Decay" + num2str(ChNumber)
		Nvar Decay = $"Decay" + num2str(ChNumber)
		Decay = 40		

		String SetBaseline = "SetBaseline" + num2str(ChNumber)
		String SetDecay = "SetDecay" + num2str(ChNumber)
	
		SetVariable $SetBaseline 				labelBack=(60000,60000,60000), pos={75,TabPosOffset+440},	size={75,20},	title="Left:",limits={0,inf,1}, value=Baseline
		SetVariable $SetDecay 				labelBack=(60000,60000,60000), pos={155,TabPosOffset+440},	size={113,20},	title="Right (ms):",limits={0,inf,1}, value=Decay

		Variable/G $"NumberOfAveragedEvents" + num2str(ChNumber)
		Nvar NumberOfAveragedEvents = $"NumberOfAveragedEvents" + num2str(ChNumber)
		NumberOfAveragedEvents = 0

		String NumEventsAveraged = "NumEventsAveraged" + num2str(ChNumber)

		Valdisplay $NumEventsAveraged		labelBack=(60000,60000,60000), pos={275,TabPosOffset+440},	size={80,20},	title="#:"
		Execute "Valdisplay NumEventsAveraged" + num2str(ChNumber) + " value=" + num2str(NumberOfAveragedEvents)

		Variable/G $"CheckShowSD" + num2str(ChNumber)
		Nvar CheckShowSD = $"CheckShowSD" + num2str(ChNumber)
		CheckShowSD = 0

//		String ShowSD = "ShowSD" + num2str(ChNumber)

//		CheckBox $ShowSD					labelBack=(60000,60000,60000), pos={15,TabPosOffset+468},	size={100,20}, title="Show SD",	proc=ProcCheckShowSD
//		Execute "CheckBox ShowSD" + num2str(ChNumber) + " value=CheckShowSD" + num2str(ChNumber)

		String TitleBoxAlign = "TitleBoxAlign" + num2str(ChNumber)

		TitleBox $TitleBoxAlign  	frame=0, 	labelBack=(60000,60000,60000), pos={15,TabPosOffset+469}, 		size={15, 20},											title="Align on:"

		Variable/G $"CheckAlignAv" + num2str(ChNumber) + "Foot"
		Nvar CheckAlignAvFoot = $"CheckAlignAv" + num2str(ChNumber) + "Foot"
		CheckAlignAvFoot = 0
		
		Variable/G $"CheckAlignAv" + num2str(ChNumber) + "Occ"
		Nvar CheckAlignAvOcc = $"CheckAlignAv" + num2str(ChNumber) + "Occ"
		CheckAlignAvOcc = 1

		Variable/G $"CheckAlignAv" + num2str(ChNumber) + "Peak"
		Nvar CheckAlignAvPeak = $"CheckAlignAv" + num2str(ChNumber) + "Peak"
		CheckAlignAvPeak = 0

		String CheckBoxAlignFoot = "CheckBoxAlign" + num2str(ChNumber) + "Foot"
		String CheckBoxAlignOcc = "CheckBoxAlign" + num2str(ChNumber) + "Occ"
		String CheckBoxAlignPeak = "CheckBoxAlign" + num2str(ChNumber) + "Peak"

		CheckBox $CheckBoxAlignFoot			labelBack=(60000,60000,60000), pos={65,TabPosOffset+468},size={40,20},title="Foot", proc=procToggleAlignFoot_av
		CheckBox $CheckBoxAlignOcc			labelBack=(60000,60000,60000), pos={115,TabPosOffset+468},size={40,20},title="Half-rise", proc=procToggleAlignOcc_av
		CheckBox $CheckBoxAlignPeak		labelBack=(60000,60000,60000), pos={180,TabPosOffset+468},size={40,20},title="Peak", proc=procToggleAlignPeak_av
		Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Occ value=1"		
		
		// SUPERIMPOSE EVENTS
		
		String TitleSupTab = "TitleSupTab" + num2str(ChNumber)
		
		TitleBox $TitleSupTab  frame=0,	labelBack=(60000,60000,60000), fstyle=1,fsize=12, pos={15,TabPosOffset+360}, size={15, 120},Title="Superimpose tagged events"

		Variable/G $"CheckIncludeAv" + num2str(ChNumber)	
		Nvar CheckIncludeAv = $"CheckIncludeAv" + num2str(ChNumber)	
		CheckIncludeAv = 0
		
		Variable/G $"CheckShowIndEvents" + num2str(ChNumber)
		Nvar CheckShowIndEvents = $"CheckShowIndEvents" + num2str(ChNumber)
		CheckShowIndEvents = 1

		String CBShowIndEvents = "CBShowIndEvents" + num2str(ChNumber)
		String CheckBoxIncludeAv = "CheckBoxIncludeAv" + num2str(ChNumber)
		String ProcCBShowIndEvents = "ProcCBShowIndEvents" + num2str(ChNumber)
		String ProcCBIncludeAv = "ProcCBIncludeAv" + num2str(ChNumber)

		CheckBox $CBShowIndEvents		labelBack=(60000,60000,60000), pos={15,TabPosOffset+380},size={100,20},title="Show Individual events",		proc=$ProcCBShowIndEvents, value = 1
		CheckBox $CheckBoxIncludeAv		labelBack=(60000,60000,60000), pos={170,TabPosOffset+380},size={90,20},title="Include average", 			proc=$ProcCBIncludeAv, value=0

		Variable/G $"CheckSupShowSD" + num2str(ChNumber)
		Nvar CheckSupShowSD = $"CheckSupShowSD" + num2str(ChNumber)
		CheckSupShowSD = 0
		
		Variable/G $"CheckAlignSup" + num2str(ChNumber) + "Foot"
		Nvar CheckAlignSupFoot = $"CheckAlignSup" + num2str(ChNumber) + "Foot"
		CheckAlignSupFoot = 0
		
		Variable/G $"CheckAlignSup" + num2str(ChNumber) + "Occ"
		Nvar CheckAlignSupOcc = $"CheckAlignSup" + num2str(ChNumber) + "Occ"
		CheckAlignSupOcc = 1

		Variable/G $"CheckAlignSup" + num2str(ChNumber) + "Peak"
		Nvar CheckAlignSupPeak = $"CheckAlignSup" + num2str(ChNumber) + "Peak"
		CheckAlignSupPeak = 0
		
		String CBIncludeSD = "CBIncludeSD" + num2str(ChNumber)
		String TitleBoxSupTabAlign = "TitleBoxSupTabAlign" + num2str(ChNumber)
		String CBAlignFoot = "CBAlign" + num2str(ChNumber) + "Foot"
		String CBAlignOcc = "CBAlign" + num2str(ChNumber) + "Occ"
		String CBAlignPeak = "CBAlign" + num2str(ChNumber) + "Peak"
		String ProcSupCheckShowSD = "ProcSupCheckShowSD" + num2str(ChNumber)
		
		CheckBox $CBIncludeSD			labelBack=(60000,60000,60000), pos={285,TabPosOffset+380},size={20,20},title="SD", 					proc=$ProcSupCheckShowSD, value=0
		TitleBox 	$TitleBoxSupTabAlign  	labelBack=(60000,60000,60000), frame=0, 		pos={15,TabPosOffset+400}, 		size={15, 20},											title="Align on:"
		CheckBox $CBAlignFoot			labelBack=(60000,60000,60000), pos={65,TabPosOffset+400},size={40,20},title="Foot", proc=procToggleAlignFoot_sup
		CheckBox $CBAlignOcc			labelBack=(60000,60000,60000), pos={115,TabPosOffset+400},size={40,20},title="Half-rise", proc=procToggleAlignOcc_sup
		CheckBox $CBAlignPeak			labelBack=(60000,60000,60000), pos={180,TabPosOffset+400},size={40,20},title="Peak", proc=procToggleAlignPeak_sup
		Execute "CheckBox CBAlign" + num2str(ChNumber) + "Occ value=1"
		
		Variable/G $"CheckSupNormalize" + num2str(ChNumber)
		Nvar CheckSupNormalize = $"CheckSupNormalize" + num2str(ChNumber)
		CheckSupNormalize = 0
		
		Variable/G $"SupNormLeft" + num2str(ChNumber)
		Nvar SupNormLeft = $"SupNormLeft" + num2str(ChNumber)
		SupNormLeft = 5
		
		Variable/G $"SupNormRight" + num2str(ChNumber)
		Nvar SupNormRight = $"SupNormRight" + num2str(ChNumber)
		SupNormRight = 2
		
		String CBNormalize = "CBNormalize" + num2str(ChNumber)
		String ProcCBNormalize = "ProcCBNormalize" + num2str(ChNumber)
		String SetVarSupNormRight = "SetVarSupNormRight" + num2str(ChNumber)
		String SetVarSupNormLeft = "SetVarSupNormLeft" + num2str(ChNumber)
		String ButtonSupEvents = "ButtonSupEvents" + num2str(ChNumber)
		String ProcButtonSupEvents = "ProcButtonSupEvents" + num2str(ChNumber)
		
		CheckBox $CBNormalize			labelBack=(60000,60000,60000), pos={15,TabPosOffset+440},size={40,20},title="Normalize",proc=$ProcCBNormalize,value=0
		SetVariable $SetVarSupNormRight	labelBack=(60000,60000,60000), pos={125,TabPosOffset+460},size={100,20},title="Right (ms):",limits={-inf,SupNormLeft,1}
		Execute "SetVariable SetVarSupNormRight" + num2str(ChNumber) + " value=SupNormRight" + num2str(ChNumber)
		SetVariable $SetVarSupNormLeft	labelBack=(60000,60000,60000), pos={15,TabPosOffset+460},size={100,20},title="Left (ms):",limits={SupNormRight,inf,1}
		Execute "SetVariable SetVarSupNormLeft" + num2str(ChNumber) + " value=SupNormLeft" + num2str(ChNumber)
		Button $ButtonSupEvents			labelBack=(60000,60000,60000), pos={15,TabPosOffset+560},size={100,20},title="Display graph", 			proc=$ProcButtonSupEvents

		Variable/G $"SupBaseline" + num2str(ChNumber)
		Nvar SupBaseline = $"SupBaseline" + num2str(ChNumber)
		SupBaseline = 20

		Variable/G $"SupDecay" + num2str(ChNumber)
		Nvar SupDecay = $"SupDecay" + num2str(ChNumber)
		SupDecay = 40	

		String SetVarBaseline = "SetVarBaseline" + num2str(ChNumber)
		String SetVarDecay = "SetVarDecay" + num2str(ChNumber)

		SetVariable $SetVarBaseline 		labelBack=(60000,60000,60000), pos={15,TabPosOffset+420},	size={75,20},	title="Left:",limits={0,inf,1}
		Execute "SetVariable SetVarBaseline" + num2str(ChNumber) + " value=SupBaseline" + num2str(ChNumber)
		SetVariable $SetVarDecay			labelBack=(60000,60000,60000), pos={95,TabPosOffset+420},	size={113,20},	title="Right (ms):",limits={0,inf,1}
		Execute "SetVariable SetVarDecay" + num2str(ChNumber) + " value=SupDecay" + num2str(ChNumber)

		Variable/G $"CheckShowCurs" + num2str(ChNumber)
		Nvar CheckShowCurs = $"CheckShowCurs" + num2str(ChNumber)
		CheckShowCurs = 0

		String CBShowCurs = "CBShowCurs" + num2str(ChNumber)
		String ProcCBShowCurs = "ProcCBShowCurs" + num2str(ChNumber)

		CheckBox $CBShowCurs			labelBack=(60000,60000,60000), pos={15,TabPosOffset+480}, size={100,20},title="Show normalization window",value=0,proc=$ProcCBShowCurs

		Variable/G $"CheckShowRefLine" + num2str(ChNumber)
		Nvar CheckShowRefLine = $"CheckShowRefLine" + num2str(ChNumber)
		CheckShowRefLine = 0

		String CBShowRefLine = "CBShowRefLine" + num2str(ChNumber)
		String ProcCBShowRefLine = "ProcCBShowRefLine" + num2str(ChNumber)

		CheckBox $CBShowRefLine		labelBack=(60000,60000,60000), pos={15,TabPosOffset+500}, size={100,20},title="Show reference line:",value=0,proc=$ProcCBShowRefLine

		Variable/G $"RefLineValue" + num2str(ChNumber)
		Nvar RefLineValue = $"RefLineValue" + num2str(ChNumber)
		RefLineValue = 0

		Variable/G $"CheckUseRndColors" + num2str(ChNumber)
		Nvar CheckUseRndColors = $"CheckUseRndColors" + num2str(ChNumber)
		CheckUseRndColors = 0

		Variable/G $"CheckRemoveAxes" + num2str(ChNumber)
		Nvar CheckRemoveAxes = $"CheckRemoveAxes" + num2str(ChNumber)
		CheckRemoveAxes = 0
		
		Variable/G $"CheckOverlayMode" + num2str(ChNumber)
		Nvar CheckOverlayMode = $"CheckOverlayMode" + num2str(ChNumber)
		CheckOverlayMode = 0

		Variable/G $"WinSupEvents" + num2str(ChNumber)
		Nvar WinSupEvents = $"WinSupEvents" + num2str(ChNumber)
		WinSupEvents = 0
		
		Variable/G $"OverlayCounter" + num2str(ChNumber)
		Nvar OverlayCounter = $"OverlayCounter" + num2str(ChNumber)
		OverlayCounter = 0

		String SetVarRefLine = "SetVarRefLine" + num2str(ChNumber)
		String CBUseRndColors = "CBUseRndColors" + num2str(ChNumber)
		String CBRemoveAxes = "CBRemoveAxes" + num2str(ChNumber)
		String CBOverlayMode = "CBOverlayMode" + num2str(ChNumber)

		String ProcCBUseRndColors = "ProcCBUseRndColors" + num2str(ChNumber)
		String ProcCBRemoveAxes = "ProcCBRemoveAxes" + num2str(ChNumber)
		String ProcCBOverlayMode = "ProcCBOverlayMode" + num2str(ChNumber)

		SetVariable $SetVarRefLine		labelBack=(60000,60000,60000), pos={150,TabPosOffset+500}, size={75,20},title=" ",value=RefLineValue	
		CheckBox $CBUseRndColors		labelBack=(60000,60000,60000), pos={15,TabPosOffset+520}, size={100,20},title="Use random colors",value=0,Proc=$ProcCBUseRndColors
		CheckBox $CBRemoveAxes		labelBack=(60000,60000,60000), pos={15,TabPosOffset+540}, size={100,20},title="Remove axes",value=0,Proc=$ProcCBRemoveAxes
		CheckBox $CBOverlayMode		labelBack=(60000,60000,60000), pos={130,TabPosOffset+540},size={100,20},title="Overlay mode",value=0,Proc=$ProcCBOverlayMode
		
		// **************************** LISTBOX FOR COLOR SELECTION ******************************
		Variable NumColors = 7
		Make/o/T/N=(NumColors,1) tw					// NumColors row, 1 col text wave (2D)
		tw[0] = "Black"
		tw[1] = "Red"
		tw[2] = "Orange"
		tw[3] = "Yellow"
		tw[4] = "Green"
		tw[5] = "Blue"
		tw[6] = "Purple"
		Make/o/B/U/N=(NumColors,1,2) sw				// NumColors row, 1 col, 2 plane byte wave
		Make/o/W/U/N=(3,NumColors) myColors
		myColors[][0] = {0,0,0}			// Black; SD box will be (21760,21760,21760)
		myColors[][1] = {65535,0,0}		// red; SD box will be (65280,32768,32768)
		myColors[][2] = {65280,43520,0}	// Orange; SD box will be (65280,54528,32768)
		myColors[][3] = {65280,65280,0}	// Yellow; SD box will be (65280,65280,32768)
		myColors[][4] = {0,65535,0}		// green; SD box will be (32768,65280,32768)
		myColors[][5] = {0,0,65535}		// blue; SD box will be (32768,40704,65280)
		myColors[][6] = {52224,0,41728}	// Purple; SD box will be (65280,32768,58880)
		MatrixTranspose myColors		// above was easier to enter as 3 rows, NumColors cols		
		String ListBoxColors = "ListBoxColors" + num2str(ChNumber)
		ListBox $ListBoxColors labelBack=(60000,60000,60000), pos={240,TabPosOffset+400},size={100,150},frame=0,mode=1,editStyle=0,listWave=tw,selWave=sw,colorWave=myColors
		// mode=1: one or zero selection allowed.
		sw[][][1]= p
		SetDimLabel 2,1,backColors,sw
		// **************************** END OF LISTBOX FOR COLOR SELECTION ******************************
		
		// FITS
		
		String TitleFitsTab = "TitleFitsTab" + num2str(ChNumber)
		
		TitleBox $TitleFitsTab  frame=0,		labelBack=(60000,60000,60000), fstyle=1,fsize=12, pos={15,TabPosOffset+360}, size={15, 80},Title="Decay kinetics"

		Variable/G $"CheckFit" + num2str(ChNumber)
		Nvar CheckFit = $"CheckFit" + num2str(ChNumber)
		CheckFit = 0		// When CheckFit = 0, check box CFPE is on; when CheckFit = 1, check box CFAE is on.
		
		Variable/G $"CheckOffset" + num2str(ChNumber)
		Nvar CheckOffset = $"CheckOffset" + num2str(ChNumber)
		CheckOffset = 0
		
		
		String CheckBFitSelectedChanAA = "CheckBFitSelectedChan" + num2str(ChNumber) + num2str(ChNumber)
		String CheckBFitSelectedChanAB = "CheckBFitSelectedChan" + num2str(ChNumber) + num2str(3 - ChNumber)
		String fit_chan_select = "fit_chan_select" + num2str(ChNumber)
	
		CheckBox $CheckBFitSelectedChanAA 	labelBack=(60000,60000,60000), pos={15,TabPosOffset+380},size={100,20},title="Channel" + num2str(ChNumber), 	proc=$fit_chan_select, 	value = 1
		CheckBox $CheckBFitSelectedChanAB		labelBack=(60000,60000,60000), pos={15,TabPosOffset+400},size={100,20},title="Channel" + num2str(3 - ChNumber), 	proc=$fit_chan_select, 	value = 0	//, disable=2 // CFAE : Check Fit Average Event

		Variable/G $"Fit_chan_select_nb" + num2str(ChNumber)		// Seems needed to avoid bug on first run
		Nvar Fit_chan_select_nb = $"Fit_chan_select_nb" + num2str(ChNumber)
		Fit_chan_select_nb = ChNumber

		String CFPE = "CFPE" + num2str(ChNumber)
		String PCFPE = "PCFPE" + num2str(ChNumber)
		String CFAE = "CFAE" + num2str(ChNumber)
		String PCFAE = "PCFAE" + num2str(ChNumber)
		String FreeOffset = "FreeOffset" + num2str(ChNumber)
		String Fit2Peak = "Fit2Peak" + num2str(chNumber)
//		String CheckBoxFreeOffset = "CheckBoxFreeOffset" + num2str(ChNumber)
		Variable/G $"CheckFit2Peak" + num2str(ChNumber)
		NVar checkFit2Peak = $"CheckFit2Peak" + num2str(ChNumber)
		checkFit2Peak = 0
		
		CheckBox $CFPE					labelBack=(60000,60000,60000), pos={90,TabPosOffset+380},size={100,20},title="event", 	proc=$PCFPE, 	value = 1 - CheckFit // CFPE : Check Fit Present Event
		CheckBox $CFAE					labelBack=(60000,60000,60000), pos={90,TabPosOffset+400},size={100,20},title="average", 	proc=$PCFAE, 	value = CheckFit, disable=2 // CFAE : Check Fit Average Event
		CheckBox $FreeOffset				labelBack=(60000,60000,60000), pos={150,TabPosOffset+380},size={100,20},title="free offset", proc=CheckBoxFreeOffset, value = CheckOffset
		checkBox $Fit2Peak					labelBack=(60000,60000,60000), pos={150,TabPosOffset+400},size={100,20},title="extrapolate to peak (exp & biExp)", proc=Fit2PeakProc, value = CheckFit2Peak
		Variable/G $"StartFit" + num2str(ChNumber)
		Nvar StartFit = $"StartFit" + num2str(ChNumber)
		StartFit = 10
		
		Variable/G $"EndFit" + num2str(ChNumber)
		Nvar EndFit = $"EndFit" + num2str(ChNumber)
		EndFit = Decay

		String FitSingleExp = "FitSingleExp" + num2str(ChNumber)
		String FitSingleExp_select = "FitSingleExp_select" + num2str(ChNumber)
		String Fitdbexp = "Fitdbexp" + num2str(ChNumber)
		String Fitdbexp_select = "Fitdbexp_select" + num2str(ChNumber)
		String BFitsigdbexp = "BFitsigdbexp" + num2str(ChNumber)
		String FitSigDbExp_select = "FitSigDbExp_select" + num2str(ChNumber)
		String BFitsigmoid = "BFitsigmoid" + num2str(ChNumber)
		String FitSigmoid_select = "FitSigmoid_select" + num2str(ChNumber)
		String BFitAllEventsdbexp = "BFitAllEventsdbexp" + num2str(ChNumber)
		//String FitAllEventsdbexp = "FitAllEventsdbexp" + num2str(ChNumber)

		Button $FitSingleExp			labelBack=(60000,60000,60000), pos={15,TabPosOffset+420},size={80,20},		proc=$FitSingleExp_select,		title="Single exp."
		Button $Fitdbexp				labelBack=(60000,60000,60000), pos={15,TabPosOffset+440},size={80,20},		proc=$Fitdbexp_select,		title="double exp."
		Button $BFitsigdbexp 			labelBack=(60000,60000,60000), pos={15,TabPosOffset+460}, size={80,20},  	proc=$FitSigDbExp_select, 	title="sigm.+dbexp"
		Button $BFitsigmoid 			labelBack=(60000,60000,60000), pos={15,TabPosOffset+480}, size={80,20},  	proc=$FitSigmoid_select, 		title="sigmoid"
		Button $BFitAllEventsdbexp	labelBack=(60000,60000,60000), pos={15,TabPosOffset+500}, size={80,20}, 	proc=FitAllEventsdbexp, 		title="Fit all events"

		Variable/G $"OffsetFitdbexp" + num2str(ChNumber)
		Nvar OffsetFitdbexp = $"OffsetFitdbexp" + num2str(ChNumber)
		OffsetFitdbexp = 0
		
		String Start_Fit = "Start_Fit" + num2str(ChNumber)
		String End_Fit = "End_Fit" + num2str(ChNumber)
		String TauFast = "TauFast" + num2str(ChNumber)
		String TauSlow = "TauSlow" + num2str(ChNumber)
		String FastPercentage = "FastPercentage" + num2str(ChNumber)
		String SlowPercentage = "SlowPercentage" + num2str(ChNumber)
		String ValOffset = "ValOffset" + num2str(ChNumber)
		String KineticIndex = "KineticIndex" + num2str(ChNumber)
		
		SetVariable $Start_Fit 		labelBack=(60000,60000,60000), pos={100,TabPosOffset+420},size={115,20}, 	title="Start (%):",limits={0,100,1}, value= StartFit
		SetVariable $End_Fit 			labelBack=(60000,60000,60000), pos={227,TabPosOffset+420}, size={115,20}, 	title="End (ms):", limits={0,Decay,1}, value=EndFit
		ValDisplay $TauFast 			labelBack=(60000,60000,60000), pos={100,TabPosOffset+440},size={115,20},	title="Tau(fast):",value=0
		ValDisplay $TauSlow			labelBack=(60000,60000,60000), pos={227,TabPosOffset+440},size={115,20},	title="Tau(slow):",value=0
		ValDisplay $FastPercentage 	labelBack=(60000,60000,60000), pos={100,TabPosOffset+460}, size={115,20}, 	title="% fast:", value=0
		ValDisplay $SlowPercentage 	labelBack=(60000,60000,60000), pos={227,TabPosOffset+460}, size={115,20}, 	title="% slow:", value=0
		ValDisplay $ValOffset 			labelBack=(60000,60000,60000), pos={100,TabPosOffset+480}, size={115,20}, 	title="Offset =",value=0
		Valdisplay $KineticIndex 		labelBack=(60000,60000,60000), pos={227,TabPosOffset+480}, size={115,20}, value=0, title="Kinetic Index :"
		
		// PAIRED RECORDINGS
		
		String TitlePairsTab = "TitlePairsTab" + num2str(ChNumber)
		String ButtonSync = "ButtonSync" + num2str(ChNumber)
		String SyncChan = "SyncChan" + num2str(ChNumber) + "From" + num2str(3 - ChNumber)
		String ButtonMAS = "ButtonMAS" + num2str(ChNumber)
		
		TitleBox $TitlePairsTab  frame=0,	fstyle=1,fsize=12, pos={15,TabPosOffset+360}, size={15, 80},Title="Paired recordings"
		Button $ButtonSync			labelBack=(60000,60000,60000), pos={15,TabPosOffset+380},size={160,20},	proc=$SyncChan,	title="Synchronise ch" + num2str(ChNumber) + " from ch" + num2str(3 - ChNumber)	
		Button $ButtonMAS			labelBack=(60000,60000,60000), pos={15,TabPosOffset+450},size={160,20},	proc=MASproc,	title="Amplitude distribution"
		
		// Variables for Sync

		Variable/G $"SyncWinStart" + num2str(ChNumber)
		Nvar SyncWinStart = $"SyncWinStart" + num2str(ChNumber)
		
		Variable/G $"SyncWinEnd" + num2str(ChNumber)
		Nvar SyncWinEnd = $"SyncWinEnd" + num2str(ChNumber)

		String SetSyncWinStart = "SetSyncWinStart" + num2str(ChNumber)
		String SetSyncWinEnd = "SetSyncWinEnd" + num2str(ChNumber)

		SetVariable $SetSyncWinStart 	labelBack=(60000,60000,60000), pos={15,TabPosOffset+405},size={203,20}, 	title="Sync window start (ms):",limits={-100,100,1}, value= SyncWinStart
		SetVariable $SetSyncWinEnd 	labelBack=(60000,60000,60000), pos={223,TabPosOffset+405},size={122,20}, 	title="end (ms):",limits={-100,100,1}, value= SyncWinEnd
		
		// variables for MAS
		
		Variable/G $"MASPeakWinWidth" + num2str(ChNumber)
		Nvar MASPeakWinWidth = $"MASPeakWinWidth" + num2str(ChNumber)
		MASPeakWinWidth = 1
		
		Variable/G $"MASBslWidth" + num2str(ChNumber)
		Nvar MASBslWidth = $"MASBslWidth" + num2str(ChNumber)
		MASBslWidth = 3
		
		Variable/G $"MASAPRiseTime" + num2str(ChNumber)
		Nvar MASAPRiseTime = $"MASAPRiseTime" + num2str(ChNumber)
		
		Variable/G $"MASLagToPeak" + num2str(ChNumber)
		Nvar MASLagToPeak = $"MASLagToPeak" + num2str(ChNumber)
		
		Variable/G $"MASTrueHistbinSize" + num2str(ChNumber)
		Nvar MASTrueHistbinSize = $"MASTrueHistbinSize" + num2str(ChNumber)
		MASTrueHistbinSize = 10
		
		Variable/G $"MASNoiseHistBinSize" + num2str(ChNumber)
		Nvar MASNoiseHistBinSize = $"MASNoiseHistBinSize" + num2str(ChNumber)
		MASNoiseHistBinSize = 10
		
		Variable/G $"MASHistMin" + num2str(ChNumber)
		Nvar MASHistMin = $"MASHistMin" + num2str(ChNumber)
		MASHistMin = -500
		
		Variable/G $"MASHistMax" + num2str(ChNumber)
		Nvar MASHistMax = $"MASHistMax" + num2str(ChNumber)
		MASHistMax = 300
		
		Variable/G $("MASStart" + num2str(ChNumber))
		NVAR MASStart = $("MASStart" + num2str(ChNumber))
		MASStart = 0
		
		Variable/G $("MASEnd" + num2str(ChNumber))
		NVAR MASEnd = $("MASEnd" + num2str(ChNumber))
		wave recording = $(FilenameA)
		MASEnd = pnt2x(recording, (numpnts(recording) - 1))
		
		String/G $("MASpharmaco" + num2str(ChNumber))
		SVar MASPharmaco = $("MASpharmaco" + num2str(ChNumber))
		MASPharmaco = "Ctrl"
		
		String SetMASPeakWinWidth = "SetMASPeakWinWidth" + num2str(ChNumber)
		String SetMASBslWidth = "SetMASBslWidth" + num2str(ChNumber)
		String SetMASAPRiseTime = "SetMASAPRiseTime" + num2str(ChNumber)
		String SetMASLagToPeak = "SetMASLagToPeak" + num2str(ChNumber)
		String SetMASTrueHistBinSize = "SetMASTrueHistBinSize" + num2str(ChNumber)
		String SetMASNoiseHistBinSize = "SetMASNoiseHistBinSize" + num2str(ChNumber)
		String SetMASHistMin = "SetMASHistMin" + num2str(ChNumber)
		String SetMASHistMax = "SetMASHistMax" + num2str(ChNumber)
		string SetMASStart = "SetMASStart" + num2str(ChNumber)
		string SetMASEnd = "SetMASEnd" + num2str(ChNumber)
		string SetMASpharmacoN = "SetMASpharmacoN" + num2str(ChNumber)
		
		SetVariable $SetMASPeakWinWidth 	labelBack=(60000,60000,60000), pos={15,TabPosOffset+475},size={155,20}, 	title="Peak window (ms):",limits={0,100,1}, value= MASPeakWinWidth
		SetVariable $SetMASBslWidth			labelBack=(60000,60000,60000), pos={175,TabPosOffset+475},size={170,20}, 	title="Baseline window (ms):",limits={0,100,1}, value= MASBslWidth
		SetVariable $SetMASAPRiseTime	 	labelBack=(60000,60000,60000), pos={15,TabPosOffset+495},size={155,20}, 	title="AP Rise Time (ms) :",limits={0,100,1}, value= MASAPRiseTime
		SetVariable $SetMASLagToPeak 		labelBack=(60000,60000,60000), pos={175,TabPosOffset+495},size={170,20}, 	title="Lag to event peak(ms):",limits={0,100,1}, value= MASLagToPeak
		SetVariable $SetMASTrueHistBinSize 	labelBack=(60000,60000,60000), pos={15,TabPosOffset+515},size={155,20}, 	title="Bin size (pA):",limits={0,100,1}, value= MASTrueHistBinSize
		SetVariable $SetMASNoiseHistBinSize	labelBack=(60000,60000,60000), pos={175,TabPosOffset+515},size={170,20}, 	title="Bin size (noise) (pA):",limits={0,100,1}, value= MASNoiseHistBinSize, disable = 1
		SetVariable $SetMASpharmacoN 	labelBack=(60000,60000,60000), pos={15,TabPosOffset+515},size={155,20}, 	title="Pharmaco name:",limits={0,100,1}, value= MASpharmaco
		SetVariable $SetMASHistMin			labelBack=(60000,60000,60000), pos={15,TabPosOffset+535},size={170,20}, 	title="Histogram min (pA):",limits={-1000,0,1}, value= MASHistMin, disable=1
		SetVariable $SetMASHistMax			labelBack=(60000,60000,60000), pos={195,TabPosOffset+535},size={110,20}, 	title="max (pA):",limits={0,1000,1}, value= MASHistMax, disable=1
		SetVariable $SetMASStart				labelBack=(60000,60000,60000), pos={15,TabPosOffset+535},size={155,20}, 	title="Range start (s):",limits={0,1000,1}, value= MASStart
		SetVariable $SetMASEnd				labelBack=(60000,60000,60000), pos={175,TabPosOffset+535},size={170,20}, 	title="end (s):                ",limits={0,1000,1}, value= MASEnd
		// MANAGE TAGS
		
		String TitleTagsTab = "TitleTagsTab" + num2str(ChNumber)
		
		TitleBox $TitleTagsTab  frame=0,	fstyle=1,fsize=12, pos={15,TabPosOffset+360}, size={15, 80},Title="Manage tags"

		Variable/G $"IsolateEventsWinStart" + num2str(ChNumber)
		Nvar IsolateEventsWinStart = $"IsolateEventsWinStart" + num2str(ChNumber)
		IsolateEventsWinStart = 40
		
		Variable/G $"IsolateEventsWinEnd" + num2str(ChNumber)
		Nvar IsolateEventsWinEnd = $"IsolateEventsWinEnd" + num2str(ChNumber)
		IsolateEventsWinEnd = 40
		
		Variable/G $"LowPassThrshld" + num2str(ChNumber)
		Nvar LowPassThrshld = $"LowPassThrshld" + num2str(ChNumber)
		LowPassThrshld = 200

		String SetIsolateEventsWinStart = "SetIsolateEventsWinStart" + num2str(ChNumber)
		String SetIsolateEventsWinEnd = "SetIsolateEventsWinEnd" + num2str(ChNumber)
		String ButtonIsolateEvents = "ButtonIsolateEvents" + num2str(ChNumber)
		String SetLowPassThrshld = "SetLowPassThrshld" + num2str(ChNumber)
		String ButtonLowPass = "ButtonLowPass" + num2str(ChNumber)

		SetVariable $SetIsolateEventsWinStart	labelBack=(60000,60000,60000), pos={15,TabPosOffset+420},	size={185,20}, 	title="No events before (ms):",limits={0,1000,1}, value= IsolateEventsWinStart
		SetVariable $SetIsolateEventsWinEnd	labelBack=(60000,60000,60000), pos={205,TabPosOffset+420},	size={86,20}, 	title="after:",limits={0,1000,1}, value= IsolateEventsWinEnd
		Button $ButtonIsolateEvents			labelBack=(60000,60000,60000), pos={300,TabPosOffset+419},	size={40,20},		title="apply",		proc=ProcIsolateEvents
		SetVariable $SetLowPassThrshld		labelBack=(60000,60000,60000), pos={15,TabPosOffset+440},	size={185,20}, 	title="No events below (pA):",limits={0,2000,1}, value= LowPassThrshld
		Button $ButtonLowPass				labelBack=(60000,60000,60000), pos={300,TabPosOffset+439},	size={40,20}, 	title="apply", 		proc=ProcLowPass

		Variable /G $"LimitBegin" + num2str(ChNumber)
		Nvar LimitBegin = $"LimitBegin" + num2str(ChNumber)
		LimitBegin = 0
		
		Variable/G $"LimitEnd" + num2str(ChNumber)
		Nvar LimitEnd = $"LimitEnd" + num2str(ChNumber)
		LimitEnd = 0

		String ButtonTagRangeOfEvents = "ButtonTagRangeOfEvents" + num2str(ChNumber)
		String ProcTagRange = "ProcTagRange" + num2str(ChNumber)
		String ButtonUntagRangeOfEvents = "ButtonUntagRangeOfEvents" + num2str(ChNumber)
		String ProcUntagRange = "ProcUntagRange" + num2str(ChNumber)
		String SetBegin = "SetBegin" + num2str(ChNumber)
		String SetEnd = "SetEnd" + num2str(ChNumber)

		Button $ButtonTagRangeOfEvents		labelBack=(60000,60000,60000), pos={15,TabPosOffset+380},	size={40,20},		title="Tag", 		proc=$ProcTagRange
		Button $ButtonUntagRangeOfEvents		labelBack=(60000,60000,60000), pos={55,TabPosOffset+380},	size={40,20},		title="Untag", 	proc=$ProcUntagRange
		SetVariable $SetBegin					labelBack=(60000,60000,60000), pos={100,TabPosOffset+379},	size={165,20},	title="events between (s)", value = LimitBegin, limits={0,Length,1}
		SetVariable $SetEnd					labelBack=(60000,60000,60000), pos={270,TabPosOffset+379},	size={73,20},		title="&", value = LimitEnd, limits={LimitBegin,Length,1}

		Variable/G $"LimitBegin" + num2str(ChNumber) + "_pnts"
		Nvar LimitBegin_pnts = $"LimitBegin" + num2str(ChNumber) + "_pnts"
		LimitBegin_pnts = 0
		
		Variable/G $"LimitEnd" + num2str(ChNumber) + "_pnts"
		Nvar LimitEnd_pnts = $"LimitEnd" + num2str(ChNumber) + "_pnts"
		LimitEnd_pnts = 0

		String ButtonTagRangeOfEvents_pnts = "ButtonTagRangeOfEvents" + num2str(ChNumber) + "_pnts"
		String ProcTagRange_pnts = "ProcTagRange" + num2str(ChNumber) + "_pnts"
		String ButtonUntagRangeOfEvents_pnts = "ButtonUntagRangeOfEvents" + num2str(ChNumber) + "_pnts"
		String ProcUntagRange_pnts = "ProcUntagRange" + num2str(ChNumber) + "_pnts"
		String SetBegin_pnts = "SetBegin" + num2str(ChNumber) + "_pnts"
		String SetEnd_pnts = "SetEnd" + num2str(ChNumber) + "_pnts"
		String BtnSaveTags = "BtnSaveTags" + num2str(ChNumber)
		String BtnDeleteNaN = "BtnDeleteNaN" + num2str(ChNumber)
		String ProcSaveTags = "ProcSaveTags" + num2str(ChNumber)

		Button $ButtonTagRangeOfEvents_pnts		labelBack=(60000,60000,60000), pos={15,TabPosOffset+400},	size={40,20},		title="Tag", 		proc=$ProcTagRange_pnts
		Button $ButtonUntagRangeOfEvents_pnts	labelBack=(60000,60000,60000), pos={55,TabPosOffset+400},	size={40,20},		title="Untag", 	proc=$ProcUntagRange_pnts
		SetVariable $SetBegin_pnts				labelBack=(60000,60000,60000), pos={100,TabPosOffset+399},	size={165,20},	title="events between #", value = LimitBegin_pnts, limits={0,NumberofEventsch-1,1}
		SetVariable $SetEnd_pnts					labelBack=(60000,60000,60000), pos={270,TabPosOffset+399},	size={73,20},		title="&", value = LimitEnd_pnts, limits={LimitBegin_pnts,NumberofEventsch-1,1}
		Button $BtnSaveTags						labelBack=(60000,60000,60000), pos={90,TabPosOffset+460},	size={80,20},						proc=$ProcSaveTags,title="Save Tags"
		Button $BtnDeleteNaN						labelBack=(60000,60000,60000), pos={15,TabPosOffset+460}, 	size={70,20},						proc=ProcDeleteNaN,title="Untag NaNs"

//		Variable/G $"Tags" + num2str(ChNumber) + "_origin"
//		Nvar Tags_origin = $"Tags" + num2str(ChNumber) + "_origin"

//		String PopupOriginOfTags = "PopupOriginOfTags" + num2str(ChNumber)
//		String OriginOfTags = "OriginOfTags" + num2str(ChNumber)

//		REMOVED BECAUSE POPUPS DON'T WORK WITH WINE
//		PopupMenu	$PopupOriginOfTags			labelBack=(60000,60000,60000), pos={10,TabPosOffset+485}, 	title="Select origin of Tags1 : ",size={20,20},proc=$OriginOfTags, 	value="Tags (true);Tags synchronised",	Disable=0

		// INTEGRALS

		String TitleIntTab = "TitleIntTab" + num2str(ChNumber)
		
		TitleBox $TitleIntTab  frame=0,	labelBack=(60000,60000,60000), fstyle=1,fsize=12, pos={15,TabPosOffset+360}, size={15, 80},Title="Calculate integrals"

		Variable/G $"IntegralStart" + num2str(ChNumber) 
		Nvar IntegralStart = $"IntegralStart" + num2str(ChNumber) 
		IntegralStart = -Baseline
		
		Variable/G $"IntegralEnd" + num2str(ChNumber)
		Nvar IntegralEnd = $"IntegralEnd" + num2str(ChNumber)
		IntegralEnd = Decay
		
		Variable/G $"CheckIntegralOffset" + num2str(ChNumber)
		Nvar CheckIntegralOffset = $"CheckIntegralOffset" + num2str(ChNumber)
		CheckIntegralOffset = 1
		
		Variable/G $"IntAvgWidth" + num2str(ChNumber)
		Nvar IntAvgWidth = $"IntAvgWidth" + num2str(ChNumber)
		IntAvgWidth = 3
		
		Variable/G $"IntegrateChanSelectNb" + num2str(ChNumber)
		Nvar IntegrateChanSelectNb = $"IntegrateChanSelectNb" + num2str(ChNumber)
		IntegrateChanSelectNb = ChNumber	// IntegrateChanSelectNb tells channel to integrate (active channel by default
		
		Variable/G $"Area" + num2str(ChNumber)
		Nvar AreaVar = $"Area" + num2str(ChNumber)
		AreaVar = 0
		
		String CheckBIntSelectedChanAA = "CheckBIntSelectedChan" + num2str(ChNumber) + num2str(ChNumber)
		String CheckBIntSelectedChanAB = "CheckBIntSelectedChan" + num2str(ChNumber) + num2str(3 - ChNumber)
		String IntegrateChanSelect = "IntegrateChanSelect" + num2str(ChNumber)

		CheckBox $CheckBIntSelectedChanAA 	labelBack=(60000,60000,60000), pos={15,TabPosOffset+380},size={100,20},title=("Channel" + num2str(ChNumber)), 	proc=$IntegrateChanSelect, 	value = 1
		CheckBox $CheckBIntSelectedChanAB	labelBack=(60000,60000,60000), pos={15,TabPosOffset+400},size={100,20},title=("Channel" + num2str(3 - ChNumber)), 	proc=$IntegrateChanSelect, 	value = 0

		Variable/G $"CheckIntegrate" + num2str(ChNumber)
		Nvar CheckIntegrate = $"CheckIntegrate" + num2str(ChNumber)
		CheckIntegrate = 1	// Variable that tells to integrate current evnt or avg
		
		String CheckBIntEvent = "CheckBIntEvent" + num2str(ChNumber)
		String IntegrateEvent = "IntegrateEvent" + num2str(ChNumber)
		String CheckBIntAvg = "CheckBIntAvg" + num2str(ChNumber)
		String IntegrateAvg = "IntegrateAvg" + num2str(ChNumber)
		String SetIntegralStart = "SetIntegralStart" + num2str(ChNumber)
		String SetIntegralEnd = "SetIntegralEnd" + num2str(ChNumber)
		String SetIntAvgWidth = "SetIntAvgWidth" + num2str(ChNumber)
		String ChkboxIntegralOffset = "ChkboxIntegralOffset" + num2str(ChNumber)
		String ButtonIntegrate = "Integrate" + num2str(ChNumber)
		String DisplayArea = "DisplayArea" + num2str(ChNumber)

		CheckBox $CheckBIntEvent		labelBack=(60000,60000,60000), pos={140,TabPosOffset+380},size={100,20},	title="event", 		proc=$IntegrateEvent, 	value = CheckIntegrate
		CheckBox $CheckBIntAvg			labelBack=(60000,60000,60000), pos={140,TabPosOffset+400},size={100,20},	title="average", 	proc=$IntegrateAvg, 	value = (1 - CheckIntegrate)
		SetVariable $SetIntegralStart		labelBack=(60000,60000,60000), pos={15,TabPosOffset+420},size={165,20},	title="Integrate from : ", value = IntegralStart, limits={(-Baseline),0,1}
		SetVariable $SetIntegralEnd		labelBack=(60000,60000,60000), pos={185,TabPosOffset+420},size={73,20},	title=" to : ", value = IntegralEnd, limits={0,decay,1}
		SetVariable $SetIntAvgWidth		labelBack=(60000,60000,60000), pos={15,TabPosOffset+450},size={200,20},	title="Average window width : ", value = IntAvgWidth, limits={0,(Baseline + Decay),1}
		CheckBox $ChkboxIntegralOffset	labelBack=(60000,60000,60000), pos={245,TabPosOffset+380},size={100,20},	title="Offset trace", proc=SetCheckIntegralOffset, value = CheckIntegralOffset
		Button $ButtonIntegrate			labelBack=(60000,60000,60000), pos={15,TabPosOffset+480},size={80,20},		title="Integrate !", 	proc=SpAcAnIntegrate
		Valdisplay $DisplayArea 			labelBack=(60000,60000,60000), pos={227,TabPosOffset+480}, size={115,20}, value=0, title="Area :"

		// *******************************************************Right panels**********************************

		// DISPLAY EVENT

		Display/HOST=$DetectedEvents/N=$EventChannelA/W=(0.4,0.01,0.69,0.49) TEMPEVENT
		
		AppendToGraph/W=$DetectedEvents#$EventChannelA YcsrA vs XcsrA
		ModifyGraph/W=$DetectedEvents#$EventChannelA lsize($"YcsrA" + num2str(ChNumber))=5
		ModifyGraph/W=$DetectedEvents#$EventChannelA rgb($"YcsrA" + num2str(ChNumber))=(65280,0,0)
		
		AppendToGraph/W=$DetectedEvents#$EventChannelA YcsrC vs XcsrC
		ModifyGraph/W=$DetectedEvents#$EventChannelA lsize($"YcsrC" + num2str(ChNumber))=5
		ModifyGraph/W=$DetectedEvents#$EventChannelA rgb($"YcsrC" + num2str(ChNumber))=(65280,0,0)
		
		AppendToGraph/W=$DetectedEvents#$EventChannelA YcsrD vs XcsrD
		ModifyGraph/W=$DetectedEvents#$EventChannelA mode($"YcsrD" + num2str(ChNumber))=3,mrkThick($"YcsrD" + num2str(ChNumber))=2
		ModifyGraph/W=$DetectedEvents#$EventChannelA rgb($"YcsrD" + num2str(ChNumber))=(0,52224,0)
		
		AppendToGraph/W=$DetectedEvents#$EventChannelA YcsrE vs XcsrE
		ModifyGraph/W=$DetectedEvents#$EventChannelA mode($"YcsrE" + num2str(ChNumber))=3,mrkThick($"YcsrE" + num2str(ChNumber))=2
		ModifyGraph/W=$DetectedEvents#$EventChannelA rgb($"YcsrE" + num2str(ChNumber))=(0,52224,0)
		
		AppendToGraph/W=$DetectedEvents#$EventChannelA YcsrF vs XcsrF
		ModifyGraph/W=$DetectedEvents#$EventChannelA mode($"YcsrF" + num2str(ChNumber))=2,lsize($"YcsrF" + num2str(ChNumber))=5
		ModifyGraph/W=$DetectedEvents#$EventChannelA rgb($"YcsrF" + num2str(ChNumber))=(52224,0,41728)

		if (ChNumber == 1)
			Cursor/F/H=1/A=1/W=MainWindow#Channel1 A $FileNameA Occurrence[EventNumber], BaselineY[EventNumber]
			if (CbE == 1)		// Show cursor on other channel if exists.
				Cursor/F/H=1/A=1/W=MainWindow#Channel2 A $FileNameB Occurrence[EventNumber], BaselineY[EventNumber]
			endif
		endif
		if (ChNumber == 2)
			Cursor/F/H=1/A=1/W=MainWindow#Channel2 A $FileNameA Occurrence[EventNumber], BaselineY[EventNumber]
			if (CbE == 1)		// Show cursor on other channel if exists.
				Cursor/F/H=1/A=1/W=MainWindow#Channel1 A $FileNameB Occurrence[EventNumber], BaselineY[EventNumber]
			endif
		endif
		
		ModifyGraph/W=$DetectedEvents#$EventChannelA margin(left)=40
		ModifyGraph/W=$DetectedEvents#$EventChannelA margin(top)=1
		ModifyGraph/W=$DetectedEvents#$EventChannelA margin(right)=1
		ModifyGraph/W=$DetectedEvents#$EventChannelA margin(bottom)=28
		ModifyGraph/W=$DetectedEvents#$EventChannelA axOffset(left)=-2
		ModifyGraph/W=$DetectedEvents#$EventChannelA axOffset(bottom)=0
		
		if (ChNumber == 1)
			ModifyGraph/W=$DetectedEvents#$EventChannelA rgb($strTEMPEVENT)=(0,0,65535)
		endif
		if (ChNumber == 2)
			ModifyGraph/W=$DetectedEvents#$EventChannelA rgb($strTEMPEVENT)=(65535,0,0)
		endif
		
		ModifyGraph/W=$DetectedEvents#$EventChannelA StandOff = 0
		SetDrawEnv/W=$DetectedEvents#$EventChannelA ycoord = left, linefgc = (47872,47872,47872), dash = 1
		DrawLine/W=$DetectedEvents#$EventChannelA 0, 0, 1, 0
		
		Variable/G $"ACFT" + num2str(ChNumber)
		Nvar ACFT = $"ACFT" + num2str(ChNumber)
		ACFT = 0	//  says if DetectedEvents1#Average1 exists

		Variable/G $"ACFT" + num2str(ChNumber) + "ch" + num2str(3 - ChNumber)
		Nvar ACFTaChb = $"ACFT" + num2str(ChNumber) + "ch" + num2str(3 - ChNumber)
		ACFTaChb = 0	// says is DetectedEvents1#Average12 exists.
		
		// Prepare the waves which will be used for the cursors of the fit
		String Curseur_debut_fit_X = "Curseur_debut_fit_X" + num2str(ChNumber)
		String Curseur_debut_fit_X_Av = "Curseur_debut_fit_X" + num2str(ChNumber) + "_Av"
		String Curseur_debut_fit_Y = "Curseur_debut_fit_Y" + num2str(ChNumber)
		String Curseur_debut_fit_Y_Av = "Curseur_debut_fit_Y" + num2str(ChNumber) + "_Av"
		String Curseur_fin_fit_X = "Curseur_fin_fit_X" + num2str(ChNumber)
		String Curseur_fin_fit_X_Av = "Curseur_fin_fit_X" + num2str(ChNumber) + "_Av"
		String Curseur_fin_fit_Y = "Curseur_fin_fit_Y" + num2str(ChNumber)
		String Curseur_fin_fit_Y_Av = "Curseur_fin_fit_Y" + num2str(ChNumber) + "_Av"
		
		Make/o/N=2 $Curseur_debut_fit_X
		Make/o/N=2 $Curseur_debut_fit_X_Av
		Make/o/N=2 $Curseur_debut_fit_Y
		Make/o/N=2 $Curseur_debut_fit_Y_Av
		Make/o/N=2 $Curseur_fin_fit_X
		Make/o/N=2 $Curseur_fin_fit_X_Av
		Make/o/N=2 $Curseur_fin_fit_Y
		Make/o/N=2 $Curseur_fin_fit_Y_Av
		
		Variable/G $"PreviousEvent" + num2str(ChNumber)	// important to keep memory of previous event, used in "browsing through tagged events only"
		Nvar PreviousEvent = $"PreviousEvent" + num2str(ChNumber)
		PreviousEvent = 0
		
		Execute "TabControl Show" + num2str(ChNumber) + "Tab value=CurrentTab" + num2str(ChNumber)
		Execute "Show" + num2str(ChNumber) + "TabProc(\"Show" + num2str(ChNumber) + "Tab\", CurrentTab" + num2str(ChNumber) + ")"
	Else

		CheckBox $CheckBoxTag value=Tags[EventNumber]

		if (ChNumber == 1)
			Cursor/F/H=1/A=1/W=MainWindow#Channel1 A $FileNameA Occurrence[EventNumber], BaselineY[EventNumber]
			if (CbE == 1)
				Cursor/F/H=1/A=1/W=MainWindow#Channel2 A $FileNameB Occurrence[EventNumber], BaselineY[EventNumber]
			endif
		endif
		if (ChNumber == 2)
			Cursor/F/H=1/A=1/W=MainWindow#Channel2 A $FileNameA Occurrence[EventNumber], BaselineY[EventNumber]
			if (CbE == 1)
				Cursor/F/H=1/A=1/W=MainWindow#Channel1 A $FileNameB Occurrence[EventNumber], BaselineY[EventNumber]
			endif
		endif
	
		Wave CursorTags_X = $"CursorTags" + num2str(ChNumber) + "_X"
		Wave CursorTags_Y = $"CursorTags" + num2str(ChNumber) + "_Y"
		CursorTags_X = EventNumber
		CursorTags_Y = Tags[EventNumber]

//		Execute "TabControl Show" + num2str(ChNumber) + "Tab value=CurrentTab" + num2str(ChNumber)
//		Execute "Show" + num2str(ChNumber) + "TabProc(\"Show" + num2str(ChNumber) + "Tab\", CurrentTab" + num2str(ChNumber) + ")"

	Endif
	
	if (CheckPosDeflection)
		SetAxis/W=$DetectedEvents#$EventChannelA left -0.4*YcsrC[0], 1.2*YcsrC[0]
	else	
		SetAxis/W=$DetectedEvents#$EventChannelA left 1.2*YcsrC[0], -.4*YcsrC[0]
	endif

	Nvar ShowOtherCh = $"ShowCh" + num2str(3 - ChNumber)

	if (ShowOtherCh)

		Wave TEMPEVENTchB = $"TEMPEVENT" + num2str(ChNumber) + "_CH" + num2str(3 - ChNumber)

		String StrTEMPEVENTchB = "TEMPEVENT" + num2str(ChNumber) + "_CH" + num2str(3 - ChNumber)

		Duplicate/o/R=(Occurrence[EventNumber] - msBefore*.001 , Occurrence[EventNumber] + msAfter*.001 ) $FileNameB TEMPEVENTchB
		SetScale/P x -msBefore*.001, (SIb*1e-6), "s", TEMPEVENTchB
		
		if (ChNumber == 1)
			ModifyGraph/W=$DetectedEvents#$EventChannelAB rgb($StrTEMPEVENTchB)=(65535,0,0)
		endif
		if (ChNumber == 2)
			ModifyGraph/W=$DetectedEvents#$EventChannelAB rgb($StrTEMPEVENTchB)=(0,0,65535)
		endif

		Variable substracter = mean(TEMPEVENTchB, -(7 * 0.001), -(4 * 0.001))
		TEMPEVENTchB -= substracter
		
		Wavestats/Q TEMPEVENTchB
		
		SetAxis/W=$DetectedEvents#$EventChannelAB left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2

	endif
	
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlignFoot" IN DETECTEDEVENTS, TAB AVERAGE
FUNCTION procToggleAlignFoot_av(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar CheckAlignAvFoot = $"CheckAlignAv" + num2str(ChNumber) + "Foot"
	Nvar CheckAlignAvOcc = $"CheckAlignAv" + num2str(ChNumber) + "Occ"
	Nvar CheckAlignAvPeak = $"CheckAlignAv" + num2str(ChNumber) + "Peak"

	CheckAlignAvFoot = 1 - CheckAlignAvFoot

	if (CheckAlignAvFoot == 1)
		CheckAlignAvOcc = 0
		CheckAlignAvPeak = 0
	endif
	if (CheckAlignAvFoot == 0 && CheckAlignAvOcc == 0 && CheckAlignAvPeak == 0)
		CheckAlignAvFoot = 1
		CheckAlignAvOcc = 0
		CheckAlignAvPeak = 0
	endif
	// CheckBoxes of the Average tab
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Foot value=CheckAlignAv" + num2str(ChNumber) + "Foot"
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Occ value=CheckAlignAv" + num2str(ChNumber) + "Occ"
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Peak value=CheckAlignAv" + num2str(ChNumber) + "Peak"	
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlign#Occ" IN THE DETECTED EVENTS WINDOW
FUNCTION procToggleAlignOcc_av(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar CheckAlignAvFoot = $"CheckAlignAv" + num2str(ChNumber) + "Foot"
	Nvar CheckAlignAvOcc = $"CheckAlignAv" + num2str(ChNumber) + "Occ"
	Nvar CheckAlignAvPeak = $"CheckAlignAv" + num2str(ChNumber) + "Peak"
	
	CheckAlignAvOcc = 1 - CheckAlignAvOcc
	
	if (CheckAlignAvOcc == 1)
		CheckAlignAvFoot = 0
		CheckAlignAvPeak = 0
	endif
	if (CheckAlignAvFoot == 0 && CheckAlignAvOcc == 0 && CheckAlignAvPeak == 0)
		CheckAlignAvFoot = 0
		CheckAlignAvOcc = 1
		CheckAlignAvPeak = 0
	endif
	// CheckBoxes of the Average tab
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Foot value=CheckAlignAv" + num2str(ChNumber) + "Foot"
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Occ value=CheckAlignAv" + num2str(ChNumber) + "Occ"
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Peak value=CheckAlignAv" + num2str(ChNumber) + "Peak"
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlign#Peak" IN THE DETECTED EVENTS WINDOW
FUNCTION procToggleAlignPeak_av(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar CheckAlignAvFoot = $"CheckAlignAv" + num2str(ChNumber) + "Foot"
	Nvar CheckAlignAvOcc = $"CheckAlignAv" + num2str(ChNumber) + "Occ"
	Nvar CheckAlignAvPeak = $"CheckAlignAv" + num2str(ChNumber) + "Peak"
	
	CheckAlignAvPeak = 1 - CheckAlignAvPeak
	if (CheckAlignAvPeak == 1)
		CheckAlignAvFoot = 0
		CheckAlignAvOcc = 0
	endif
	if (CheckAlignAvFoot == 0 && CheckAlignAvOcc == 0 && CheckAlignAvPeak == 0)
		CheckAlignAvFoot = 0
		CheckAlignAvOcc = 0
		CheckAlignAvPeak = 1
	endif
	// CheckBoxes of the Average tab
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Foot value=CheckAlignAv" + num2str(ChNumber) + "Foot"
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Occ value=CheckAlignAv" + num2str(ChNumber) + "Occ"
	Execute "CheckBox CheckBoxAlign" + num2str(ChNumber) + "Peak value=CheckAlignAv" + num2str(ChNumber) + "Peak"
END

FUNCTION procToggleAlignFoot_sup(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar CheckAlignSupFoot = $"CheckAlignSup" + num2str(ChNumber) + "Foot"
	Nvar CheckAlignSupOcc = $"CheckAlignSup" + num2str(ChNumber) + "Occ"
	Nvar CheckAlignSupPeak = $"CheckAlignSup" + num2str(ChNumber) + "Peak"

	CheckAlignSupFoot = 1 - CheckAlignSupFoot

	if (CheckAlignSupFoot == 1)
		CheckAlignSupOcc = 0
		CheckAlignSupPeak = 0
	endif
	if (CheckAlignSupFoot == 0 && CheckAlignSupOcc == 0 && CheckAlignSupPeak == 0)
		CheckAlignSupFoot = 1
		CheckAlignSupOcc = 0
		CheckAlignSupPeak = 0
	endif
	// CheckBoxes of the Average tab	
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Foot value=CheckAlignSup" + num2str(ChNumber) + "Foot"
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Occ value=CheckAlignSup" + num2str(ChNumber) + "Occ"
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Peak value=CheckAlignSup" + num2str(ChNumber) + "Peak"	
END

FUNCTION procToggleAlignOcc_sup(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar CheckAlignSupFoot = $"CheckAlignSup" + num2str(ChNumber) + "Foot"
	Nvar CheckAlignSupOcc = $"CheckAlignSup" + num2str(ChNumber) + "Occ"
	Nvar CheckAlignSupPeak = $"CheckAlignSup" + num2str(ChNumber) + "Peak"
	
	CheckAlignSupOcc = 1 - CheckAlignSupOcc
	
	if (CheckAlignSupOcc == 1)
		CheckAlignSupFoot = 0
		CheckAlignSupPeak = 0
	endif
	if (CheckAlignSupFoot == 0 && CheckAlignSupOcc == 0 && CheckAlignSupPeak == 0)
		CheckAlignSupFoot = 0
		CheckAlignSupOcc = 1
		CheckAlignSupPeak = 0
	endif
	// CheckBoxes of the Average tab	
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Foot value=CheckAlignSup" + num2str(ChNumber) + "Foot"
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Occ value=CheckAlignSup" + num2str(ChNumber) + "Occ"
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Peak value=CheckAlignSup" + num2str(ChNumber) + "Peak"
END

FUNCTION procToggleAlignPeak_sup(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar CheckAlignSupFoot = $"CheckAlignSup" + num2str(ChNumber) + "Foot"
	Nvar CheckAlignSupOcc = $"CheckAlignSup" + num2str(ChNumber) + "Occ"
	Nvar CheckAlignSupPeak = $"CheckAlignSup" + num2str(ChNumber) + "Peak"
	
	CheckAlignSupPeak = 1 - CheckAlignSupPeak
	if (CheckAlignSupPeak == 1)
		CheckAlignSupFoot = 0
		CheckAlignSupOcc = 0
	endif
	if (CheckAlignSupFoot == 0 && CheckAlignSupOcc == 0 && CheckAlignSupPeak == 0)
		CheckAlignSupFoot = 0
		CheckAlignSupOcc = 0
		CheckAlignSupPeak = 1
	endif
	// CheckBoxes of the Average tab	
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Foot value=CheckAlignSup" + num2str(ChNumber) + "Foot"
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Occ value=CheckAlignSup" + num2str(ChNumber) + "Occ"
	Execute "CheckBox CBAlign" + num2str(ChNumber) + "Peak value=CheckAlignSup" + num2str(ChNumber) + "Peak"
END


Function Show2TabProc(CtrlName, TabNum) : TabControl
	String CtrlName
	Variable TabNum
	
	Variable isTab0= tabNum==0
	Variable isTab1= tabNum==1
	Variable isTab2= tabNum==2
	Variable isTab3= tabNum==3
	Variable isTab4= tabNum==4
	Variable isTab5= tabNum==5
	
	// Template if you want to add a new tab
	//Variable isTab5= tabNum==5
	
	// note: disable=0 means "show", disable=1 means "hide"
	
	// This line may be in use later 
	SetActiveSubWindow DetectedEvents2#DetectedEvents2Panel
	
	// *************************************For Average Tab***************************************
	ModifyControl CheckRiseTime2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetRTinf2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetRTsup2 disable= !isTab0	// hide if not Tab 0
	ModifyControl CheckBaselineSD2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBslnSDinf2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBslnSDsup2 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonShowRT2080histo2 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonShowBslnSDHisto2 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonAverage2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBaseline2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetDecay2 disable= !isTab0	// hide if not Tab 0
	ModifyControl NumEventsAveraged2 disable= !isTab0	// hide if not Tab 0
	ModifyControl FiltersDisplayText2 disable= !isTab0
	ModifyControl ButtonSupEvents2 disable= !isTab0
	ModifyControl CheckBoxIncludeAv2 disable= !isTab0
	ModifyControl TitleBoxAlign2 disable= !isTab0
	ModifyControl CheckBoxAlign2Foot disable= !isTab0
	ModifyControl CheckBoxAlign2Occ disable= !isTab0
	ModifyControl CheckBoxAlign2Peak disable= !isTab0
//	ModifyControl ShowSD2 disable= !isTab0
	ModifyControl TitleAvTab2 disable= !isTab0
		
	//****************************************for Sup Tab*************************************
	ModifyControl TitleSupTab2 disable= !isTab2
	ModifyControl ButtonSupEvents2 disable= !isTab2
	ModifyControl CheckBoxIncludeAv2 disable= !isTab2
	ModifyControl CBShowIndEvents2 disable= !isTab2
	ModifyControl CheckBoxIncludeAv2 disable= !isTab2
	ModifyControl CBIncludeSD2 disable= !isTab2
	ModifyControl TitleBoxSupTabAlign2 disable= !isTab2
	ModifyControl CBAlign2Foot disable= !isTab2
	ModifyControl CBAlign2Occ disable= !isTab2
	ModifyControl CBAlign2Peak disable= !isTab2
	ModifyControl SetVarBaseline2 disable= !isTab2
	ModifyControl SetVarDecay2 disable= !isTab2
	ModifyControl CBNormalize2 disable= !isTab2
	ModifyControl SetVarSupNormLeft2 disable= !isTab2
	ModifyControl SetVarSupNormRight2 disable= !isTab2
	ModifyControl CBShowCurs2 disable= !isTab2
	ModifyControl CBShowRefLine2 disable= !isTab2
	ModifyControl SetVarRefLine2 disable= !isTab2
	ModifyControl CBUseRndColors2 disable= !isTab2
	ModifyControl CBRemoveAxes2 disable= !isTab2
	ModifyControl ListBoxColors2 disable= !isTab2
	ModifyControl CBOverlayMode2 disable= !isTab2
		
	//****************************************for Kinetics Tab*************************************
	ModifyControl CFPE2 disable= !isTab4	// hide if not Tab 4
	ModifyControl CFAE2 disable= !isTab4	
	ModifyControl FreeOffset2 disable= !isTab4	
	ModifyControl Fit2Peak2 disable= !isTab4
	ModifyControl FitSingleExp2 disable= !isTab4	
	ModifyControl FitdbExp2 disable= !isTab4 
	ModifyControl BFitsigdbexp2 disable= !isTab4	
	ModifyControl BFitsigmoid2 disable= !isTab4	
	ModifyControl BFitAllEventsdbExp2 disable= !isTab4	
	ModifyControl Start_Fit2 disable= !isTab4	
	ModifyControl End_Fit2 disable= !isTab4
	ModifyControl TauFast2 disable= !isTab4	
	ModifyControl TauSlow2 disable= !isTab4	
	ModifyControl FastPercentage2 disable= !isTab4	
	ModifyControl SlowPercentage2 disable= !isTab4	
	ModifyControl ValOffset2 disable= !isTab4	
	ModifyControl KineticIndex2 disable= !isTab4
	ModifyControl TitleFitsTab2 disable= !isTab4

/// Find out  howto modify here

	ModifyControl CheckBFitSelectedChan21 disable= !isTab4
	ModifyControl CheckBFitSelectedChan22 disable= !isTab4
	//ModifyControl FitDisplayText2 disable= !isTab1

	// *********************************For Pairs Tab ***************************************
	
	ModifyControl ButtonSync2 disable= !isTab5		// hide if not Tab 5
	ModifyControl ButtonMAS2 disable= !isTab5
	
	ModifyControl SetSyncWinStart2 disable= !isTab5
	ModifyControl SetSyncWinEnd2 disable= !isTab5
	
	ModifyControl SetMASPeakWinWidth2 disable= !isTab5
	ModifyControl SetMASBslWidth2 disable= !isTab5
	ModifyControl SetMASAPRiseTime2 disable= !isTab5
	ModifyControl SetMASLagToPeak2 disable= !isTab5
	ModifyControl SetMASTrueHistbinSize2 disable= !isTab5
	ModifyControl SetMASNoiseHistBinSize2 disable= 1
	ModifyControl SetMASHistMin2 disable= 1
	ModifyControl SetMASHistMax2 disable= 1
	ModifyControl TitlePairsTab2 disable= !isTab5
	ModifyControl SetMASStart2 disable= !isTab5
	ModifyControl SetMAsEnd2 disable= !isTab5
	ModifyControl SetMASPharmacoN2 disable=!isTab5
	//*************************************** TAB "MANAGE TAGS" ***********************************
	ModifyControl SetIsolateEventsWinStart2 disable= !isTab1		// hide if not Tab 1
	ModifyControl SetIsolateEventsWinEnd2 disable= !isTab1
	ModifyControl ButtonisolateEvents2 disable= !isTab1
	ModifyControl ButtonLowPass2 disable= !isTab1
	ModifyControl SetLowPassThrshld2 disable= !isTab1
	ModifyControl ButtonTagRangeOfEvents2 disable= !isTab1
	ModifyControl ButtonUntagRangeOfEvents2 disable= !isTab1
	ModifyControl SetBegin2 disable= !isTab1
	ModifyControl SetEnd2	 disable= !isTab1
	ModifyControl ButtonTagRangeOfEvents2_pnts disable= !isTab1
	ModifyControl ButtonUntagRangeOfEvents2_pnts disable= !isTab1
	ModifyControl SetBegin2_pnts disable= !isTab1
	ModifyControl SetEnd2_pnts disable= !isTab1
	ModifyControl BtnSaveTags2 disable= !isTab1
	ModifyControl BtnDeleteNaN2 disable= !isTab1
//	ModifyControl PopupOriginOfTags2 disable= !isTab1
	ModifyControl TitleTagsTab2 disable= !isTab1
	
	//******************************************TAB "Integrals"*******************************
	ModifyControl CheckBIntSelectedChan21 disable = !isTab3
	ModifyControl CheckBIntSelectedChan22 disable = !isTab3
	ModifyControl CheckBIntEvent2 disable = !isTab3
	ModifyControl CheckBIntAvg2 disable = !isTab3
	ModifyControl SetIntegralStart2 disable = !isTab3
	ModifyControl SetIntegralEnd2 disable = !isTab3
	ModifyControl SetIntAvgWidth2 disable = !isTab3
	ModifyControl ChkboxIntegralOffset2 disable = !isTab3
	ModifyControl Integrate2 disable = !isTab3
	ModifyControl DisplayArea2 disable = !isTab3
	ModifyControl TitleIntTab2 disable = !isTab3
	// ***********************************Add your tab here***********************************
	
	//************************************************************************************
	Nvar CurrentTab2
	CurrentTab2 = TabNum
	return 0
End

Function ProcOffsetFitdbexp1(ctrlName,checked) : CheckBoxControl
		String ctrlName
		Variable checked
		Nvar OffsetFPEdbexp1
		OffsetFPEdbexp1 = 1 - OffsetFPEdbexp1
End

// Charly now useless
//Function ProcOffsetFitdbexp2(ctrlName,checked) : CheckBoxControl
//		String ctrlName
//		Variable checked
//		Nvar OffsetFPEdbexp2
//		OffsetFPEdbexp2 = 1 - OffsetFPEdbexp2
//End
// Charly end

Function PCFPE1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckFit1
	CheckFit1 = 1 - CheckFit1
	CheckBox CFAE1 value = CheckFit1
End

Function PCFAE1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckFit1
	CheckFit1 = 1 - CheckFit1
	CheckBox CFPE1 value = 1 - CheckFit1
End

Function PCFPE2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckFit2
	CheckFit2 = 1 - CheckFit2
	CheckBox CFAE2 value = CheckFit2
End

Function PCFAE2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckFit2
	CheckFit2 = 1 - CheckFit2
	CheckBox CFPE2 value = 1 - CheckFit2
End

// THE FUNCTION TRIGGERED BY THE "Set_Event_Number#" CONTROL
// THIS VERSION OF SETEVENT IS GENERALIZED
Function SetEvent(ctrlName,varNum,varStr,varName) : SetVariableControl
		String ctrlName
		Variable varNum
		String varStr
		String varName
		
		// Get ChNumber from the name of the window
		Variable ChNumber
		getwindow kwTopWin activeSW
		ChNumber = str2num(S_Value[14])
		
		// Global variables
		Nvar EventNumber = $"EventNumber" + num2str(ChNumber) 
		Nvar Baselinems = $"Baselinems" + num2str(ChNumber) 
		Nvar Peakms = $"Peakms" + num2str(ChNumber) 
		Nvar BTDE = $"BTDE_" + num2str(ChNumber) 
		Nvar BUDE = $"BUDE_" + num2str(ChNumber) 
		Nvar NumberofEventsch = $"NumberofEventsch" + num2str(ChNumber) 
		Nvar PreviousEvent = $"PreviousEvent" + num2str(ChNumber) 
		
		// waves
		Wave DetectedLevels = $"DetectedLevels" + num2str(ChNumber) 
		Wave Tags = $"Tags" + num2str(ChNumber) 
		Wave Occurrence = $"Occurrence" + num2str(ChNumber) 
		Wave BaselineY = $"BaselineY" + num2str(ChNumber) 
		Wave BaselineX = $"BaselineX" + num2str(ChNumber)
		Wave PeakY = $"PeakY" + num2str(ChNumber)
		Wave PeakX = $"PeakX" + num2str(ChNumber)
		Wave RealAmp = $"RealAmp" + num2str(ChNumber)
		Wave RealPeakPosition = $"RealPeakPosition" + num2str(ChNumber)
		Wave RiseTime2080 = $"RiseTime2080" + num2str(ChNumber)
		Wave RT20 = $"RT20" + num2str(ChNumber)
		Wave RT80 = $"RT80" + num2str(ChNumber)

		// Erase all waves on graph
		String DetectedEvents = "DetectedEvents" + num2str(ChNumber)
		String EventChannelA = "EventChannel" + num2str(ChNumber)
		GetWindow $DetectedEvents#$EventChannelA, WaveList
		Wave/T W_WaveList
		Variable i
		String TEMPSTR
		for (i=1; i < numpnts(W_WaveList)/3; i+=1)
			TEMPSTR = W_WaveList[i][0]
			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
				RemoveFromGraph/W=$DetectedEvents#$EventChannelA $W_WaveList[i][0]
			elseif (cmpstr(TEMPSTR[0,2], "UC_" ) == 0)
				RemoveFromGraph/W=$DetectedEvents#$EventChannelA $W_WaveList[i][0]
			elseif (cmpstr(TEMPSTR[0,2], "LC_" ) == 0)
				RemoveFromGraph/W=$DetectedEvents#$EventChannelA $W_WaveList[i][0]
			endif
			
			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
				RemoveFromGraph/W=$DetectedEvents#$EventChannelA $W_WaveList[i][0]
			endif
			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
				RemoveFromGraph/W=$DetectedEvents#$EventChannelA $W_WaveList[i][0]
			endif
		endfor
		
		if (BTDE)
			if (Tags[EventNumber] == 0)
				if (PreviousEvent < EventNumber)	// if we go right
					FindLevel/P/Q/R=[EventNumber,NumberofEventsch-1] Tags, 1	// Find the next tagged event
					if (V_flag == 0)
						EventNumber = V_LevelX
						PreviousEvent = EventNumber
					else
						EventNumber = PreviousEvent
					endif
				endif
				if (PreviousEvent > EventNumber)	// if we go left
					FindLevel/P/Q/R=[EventNumber,0] Tags, 1	// Find the previous tagged event
					if (V_flag == 0)
						EventNumber = V_LevelX
						PreviousEvent = EventNumber
					else
						EventNumber = PreviousEvent
					endif
				endif
			endif
		endif
		if (BUDE)
			if (Tags[EventNumber] == 1)
				if (PreviousEvent < EventNumber)	// if we go right
					FindLevel/P/Q/R=[EventNumber,NumberofEventsch-1] Tags, 0	// Find the next untagged event
					if (V_flag == 0)
						EventNumber = V_LevelX
						PreviousEvent = EventNumber
					else
						EventNumber = PreviousEvent
					endif
				endif
				if (PreviousEvent > EventNumber)	// if we go left
					FindLevel/P/Q/R=[EventNumber,0] Tags, 0	// Find the previous tagged event
					if (V_flag == 0)
						EventNumber = V_LevelX
						PreviousEvent = EventNumber
					else
						EventNumber = PreviousEvent
					endif
				endif
			endif
		endif
		if (BTDE == 0 && BUDE == 0)
			PreviousEvent = EventNumber
		endif
		SHOW(ChNumber,2,EventNumber)
End

Function ButtonCloseDetectedEvents(ctrlName) : ButtonControl
	String ctrlName
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	String DetectedEvents = "DetectedEvents" + num2str(ChNumber)
	DoWindow/K $DetectedEvents
End

Function CheckBoxGrids(ctrlName,Checked) : CheckBoxControl
	String ctrlName
	Variable Checked
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar CheckGridOn = $"CheckGridOn" + num2str(ChNumber)
	Nvar ShowOtherCh = $"ShowCh" + num2str(3 - ChNumber)
	Nvar ACFTa = $"ACFT" + num2str(ChNumber)
	Nvar ACFTaChb = $"ACFT" + num2str(ChNumber) + "Ch" + num2str(3 - ChNumber)
	
	String DetectedEvents = "DetectedEvents" + num2str(ChNumber)
	String EventChannelA = "EventChannel" + num2str(ChNumber)
	String EventChannelAB = "EventChannel" + num2str(ChNumber) + num2str(3 - ChNumber)
	String AverageA = "Average" + num2str(ChNumber)
	String AverageAB = "Average" + num2str(ChNumber) + num2str(3 - ChNumber)
	
	CheckGridOn =1 - CheckGridOn
	
	if (Checked)
		ModifyGraph/W=$DetectedEvents#$EventChannelA grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=$DetectedEvents#$EventChannelA grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		if (ShowOtherCh)
			ModifyGraph/W=$DetectedEvents#$EventChannelAB grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=$DetectedEvents#$EventChannelAB grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
		if (ACFTa)
			ModifyGraph/W=$DetectedEvents#$AverageA grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=$DetectedEvents#$AverageA grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
		if (ACFTaChb)
			ModifyGraph/W=$DetectedEvents#$AverageAB grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=$DetectedEvents#$AverageAB grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
	else
		ModifyGraph/W=$DetectedEvents#$EventChannelA grid(left)=0
		ModifyGraph/W=$DetectedEvents#$EventChannelA grid(bottom)=0
		if (ShowOtherCh)
			ModifyGraph/W=$DetectedEvents#$EventChannelAB grid(left)=0
			ModifyGraph/W=$DetectedEvents#$EventChannelAB grid(bottom)=0
		endif
		if (ACFTa)
			ModifyGraph/W=$DetectedEvents#$AverageA grid(left)=0
			ModifyGraph/W=$DetectedEvents#$AverageA  grid(bottom)=0
		endif
		if (ACFTaChb)
			ModifyGraph/W=$DetectedEvents#$AverageAB grid(left)=0
			ModifyGraph/W=$DetectedEvents#$AverageAB grid(bottom)=0
		endif
	endif
End

Function CheckBoxFreeOffset(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	Nvar CheckOffset = $"CheckOffset" + num2str(ChNumber)
	CheckOffset = 1-CheckOffset
End

Function Fit2PeakProc(ctrlName, checked) : checkBoxControl
	String ctrlName
	Variable checked
	
	string channelNb = ctrlName[strlen(ctrlName) - 1] // take the last caracter
	NVar CheckFit2Peak = $("checkFit2Peak" + channelNb)
	checkFit2Peak = 1 - checkFit2Peak
End

// Charly
Function KbdCtrlWinCloser(CtrlName) : ButtonControl
	String CtrlName
	DoWindow/K ControlsPanel // Self killing
End
// End

Function ProcButtonNavigate(ctrlName) : ButtonControl
	String ctrlName
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])

	// Global variables
	Nvar NavToSeconds = $"NavToSeconds" + num2str(ChNumber)
	Nvar EventNumber = $"EventNumber" + num2str(ChNumber) 
	// Wave
	Wave Occurrence = $"Occurrence" + num2str(ChNumber)
	
	FindLevel/Q/P Occurrence NavToSeconds
	
	EventNumber = round(V_LevelX)
	
	if (V_LevelX < 0)
		EventNumber = 0
	endif
	if (V_LevelX > numpnts(Occurrence)-1)
		EventNumber = numpnts(Occurrence)-1
	endif
	
	SHOW(ChNumber, 2, EventNumber)
	
End

Function ButtonKbd(ctrlName) : ButtonControl
	String ctrlName
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar KbdBrowseChecked
//	If (! Nvar_exists(KbdBrowseChecked)) // If it has not already been used
//		NewPanel/W=(200, 200, 400, 400)/N=ControlsPanel/k=1 as "Controls"
//		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,20,"Previous event : Left arrow"
//		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,50,"Next event : Right arrow"
//		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,80,"Tag/Untag event : top arrow"
//		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,110,"Stop : \"C\""
//		Button ClosingButton, pos ={50, 140}, size={100, 20}, proc=KbdCtrlWinCloser, Title="Close"
//		PauseForUser ControlsPanel
//		Variable/G KbdBrowseChecked = 0
//	EndIf
	
	String DetectedEvents = "DetectedEvents" + num2str(ChNumber)
	String EventChannelA = "EventChannel" + num2str(ChNumber)
	
	GetWindow $DetectedEvents#$EventChannelA, WaveList
	Wave/T W_WaveList

	Variable i
	String TEMPSTR

	for (i=1; i < numpnts(W_WaveList)/3; I+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
			RemoveFromGraph/W=$DetectedEvents#$EventChannelA $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
			RemoveFromGraph/W=$DetectedEvents#$EventChannelA $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
			RemoveFromGraph/W=$DetectedEvents#$EventChannelA $W_WaveList[i][0]
		endif
	endfor
	DoUpdate
	WaitForKey(ChNumber)
End

// CHECKBOX THAT TAGS/UNTAGS SINGLE EVENTS
Function ProcCheckBoxTag(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	// Global variables
	Nvar EventNumber = $"EventNumber" + num2str(ChNumber)
	Nvar TaggedEvents = $"TaggedEvents" + num2str(ChNumber)
	Nvar NumberofEventsch = $"NumberofEventsch" + num2str(ChNumber)
	Nvar BTDE = $"BTDE_" + num2str(ChNumber)
	Nvar BUDE = $"BUDE_" + num2str(ChNumber)
	Nvar CheckedTagAll = $"CheckedTagAll" + num2str(ChNumber)
	// Waves
	Wave Tags = $"Tags" + num2str(ChNumber)
	Wave CursorTags_Y = $"CursorTags" + num2str(ChNumber) + "_Y"
	
	// Names of checkboxes
	String CheckBoxBTDE = "BTDE" + num2str(ChNumber)
	String CheckBoxBUDE = "BUDE" + num2str(ChNumber)
	String CheckBoxTagAll = "CheckBoxTagAll" + num2str(ChNumber)
	
	Tags[EventNumber]=checked
	CursorTags_Y = checked
	TaggedEvents = sum(Tags,0,numpnts(Tags)-1)
	
	Execute "ValDisplay DisplayTaggedEvents" + num2str(ChNumber) + " value=" + num2str(TaggedEvents)

	If ( (sum(Tags,0,numpnts(Tags)-1))==0)
		BTDE = 0; Checkbox $CheckBoxBTDE value = 0
	endif
	if (CheckedTagAll)
		If ( (sum(Tags,0,numpnts(Tags)-1)) < NumberofEventsch)
			CheckedTagAll = 0; CheckBox $CheckBoxTagAll value=0
		endif
	endif
	If ( (sum(Tags,0,numpnts(Tags)-1)) == NumberofEventsch)
		BUDE = 0; Checkbox $CheckBoxBUDE value = 0
		CheckedTagAll = 1; CheckBox $CheckBoxTagAll value=1
	endif
End

// CHECKBOX THAT TAGS/UNTAGS ALL DETECTED EVENTS
Function ProcCheckBoxTagAll(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	// Global variables
	Nvar TaggedEvents = $"TaggedEvents" + num2str(ChNumber)
	Nvar NumberofEventsch = $"NumberofEventsch" + num2str(ChNumber)
	Nvar BTDE = $"BTDE_" + num2str(ChNumber)
	Nvar BUDE = $"BUDE_" + num2str(ChNumber)
	Nvar CheckedTagAll = $"CheckedTagAll" + num2str(ChNumber)
	// Waves
	Wave Tags = $"Tags" + num2str(ChNumber)
	Wave CursorTags_Y = $"CursorTags" + num2str(ChNumber) + "_Y"
	
	// Names of checkboxes
	String CheckBoxTag = "CheckBoxTag" + num2str(ChNumber)
	String CheckBoxBTDE = "BTDE" + num2str(ChNumber)
	String CheckBoxBUDE = "BUDE" + num2str(ChNumber)
	
	if (CheckedTagAll == 0)
		Tags=1
		TaggedEvents = numpnts(Tags)
		Execute "ValDisplay DisplayTaggedEvents" + num2str(ChNumber) + " value=" + num2str(TaggedEvents)
		CheckBox $CheckBoxTag value=1
		CursorTags_Y = 1
	else
		Tags=0
		TaggedEvents=0
		Execute "ValDisplay DisplayTaggedEvents" + num2str(ChNumber) + " value=" + num2str(TaggedEvents)
		CheckBox $CheckBoxTag value=0
		CursorTags_Y = 0
	endif
		CheckedTagAll = 1 - CheckedTagAll
	If ( (sum(Tags,0,numpnts(Tags)-1))==0)
		BTDE = 0; Checkbox $CheckBoxBTDE value = 0
	endif
	If ( (sum(Tags,0,numpnts(Tags)-1))==NumberofEventsch)
		BUDE = 0; Checkbox $CheckBoxBUDE value = 0
	endif
End

Function ProcTagRange1(CtrlName) : ButtonControl
	String CtrlName
	Wave Tags1, Occurrence1, CursorTags1_Y
	Nvar LimitBegin1, LimitEnd1, EventNumber1, TaggedEvents1, CheckedTagAll1, NumberofEventsch1, BUDE_1
	Variable FirstPoint, LastPoint
	// Find the index of points located within the limits
	// First Points
	if (LimitBegin1 < Occurrence1[0])
		FirstPoint = 0
	endif
	if (LimitBegin1 > Occurrence1[numpnts(Occurrence1)-1])
		FirstPoint = numpnts(Occurrence1)-1
	endif
	if ((LimitBegin1 > Occurrence1[0]) & (LimitBegin1 < Occurrence1[numpnts(Occurrence1)-1]))
		FindLevel/P Occurrence1, LimitBegin1
		FirstPoint = floor(V_levelx) + 1
	endif
	// Second Points
	if (LimitEnd1 > Occurrence1[numpnts(Occurrence1)-1])
		LastPoint = numpnts(Occurrence1)-1
	endif
	if (LimitEnd1 < Occurrence1[0])
		LastPoint = 0
	endif
	if ((LimitEnd1 < Occurrence1[numpnts(Occurrence1)-1]) & (LimitEnd1 > Occurrence1[0]))
		FindLevel/P Occurrence1, LimitEnd1
		LastPoint = floor(V_levelx)
	endif
	Tags1[FirstPoint,LastPoint] = 1
	// Update the display and checkbox
	CursorTags1_Y = Tags1[EventNumber1]
	CheckBox CheckBoxTag1 value=Tags1[EventNumber1]
	TaggedEvents1 = sum(Tags1,0,numpnts(Tags1)-1)
	Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
	// Check if all events tagged
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==numpnts(Occurrence1))
		CheckedTagAll1 = 1; CheckBox CheckBoxTagAll1 value=1
		If (BUDE_1 == 1)	// Then impossible to browse though undetected events only
			BUDE_1 = 0; Checkbox BUDE1 value = 0
		endif
	endif
End

Function ProcTagRange2(CtrlName) : ButtonControl
	String CtrlName
	Wave Tags2, Occurrence2, CursorTags2_Y
	Nvar LimitBegin2, LimitEnd2, EventNumber2, TaggedEvents2, CheckedTagAll2, NumberofEventsch2, BUDE_2
	Variable FirstPoint, LastPoint
	// Find the index of points located within the limits
	// First Points
	if (LimitBegin2 < Occurrence2[0])
		FirstPoint = 0
	endif
	if (LimitBegin2 > Occurrence2[numpnts(Occurrence2)-1])
		FirstPoint = numpnts(Occurrence2)-1
	endif
	if ((LimitBegin2 > Occurrence2[0]) & (LimitBegin2 < Occurrence2[numpnts(Occurrence2)-1]))
		FindLevel/P Occurrence2, LimitBegin2
		FirstPoint = floor(V_levelx) + 1
	endif
	// Second Points
	if (LimitEnd2 > Occurrence2[numpnts(Occurrence2)-1])
		LastPoint = numpnts(Occurrence2)-1
	endif
	if (LimitEnd2 < Occurrence2[0])
		LastPoint = 0
	endif
	if ((LimitEnd2 < Occurrence2[numpnts(Occurrence2)-1]) & (LimitEnd2 > Occurrence2[0]))
		FindLevel/P Occurrence2, LimitEnd2
		LastPoint = floor(V_levelx)
	endif
	Tags2[FirstPoint,LastPoint] = 1
	// Update the display and checkbox
	CursorTags2_Y = Tags2[EventNumber2]
	CheckBox CheckBoxTag2 value=Tags2[EventNumber2]
	TaggedEvents2 = sum(Tags2,0,numpnts(Tags2)-1)
	Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
	// Check if all events tagged
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==numpnts(Occurrence2))
		CheckedTagAll2 = 1; CheckBox CheckBoxTagAll2 value=1
		If (BUDE_2 == 1)	// Then impossible to browse though undetected events only
			BUDE_2 = 0; Checkbox BUDE2 value = 0
		endif
	endif
End

Function ProcUntagRange1(CtrlName) : ButtonControl
	String CtrlName
	Wave Tags1, Occurrence1, CursorTags1_Y
	Nvar LimitBegin1, LimitEnd1, EventNumber1, TaggedEvents1, CheckedTagAll1, NumberofEventsch1, BTDE_1
	Variable FirstPoint, LastPoint
	// Find the index of points located within the limits
	// First Points
	if (LimitBegin1 < Occurrence1[0])
		FirstPoint = 0
	endif
	if (LimitBegin1 > Occurrence1[numpnts(Occurrence1)-1])
		FirstPoint = numpnts(Occurrence1)-1
	endif
	if ((LimitBegin1 > Occurrence1[0]) & (LimitBegin1 < Occurrence1[numpnts(Occurrence1)-1]))
		FindLevel/P Occurrence1, LimitBegin1
		FirstPoint = floor(V_levelx) + 1
	endif
	// Second Points
	if (LimitEnd1 > Occurrence1[numpnts(Occurrence1)-1])
		LastPoint = numpnts(Occurrence1)-1
	endif
	if (LimitEnd1 < Occurrence1[0])
		LastPoint = 0
	endif
	if ((LimitEnd1 < Occurrence1[numpnts(Occurrence1)-1]) & (LimitEnd1 > Occurrence1[0]))
		FindLevel/P Occurrence1, LimitEnd1
		LastPoint = floor(V_levelx)
	endif
	Tags1[FirstPoint,LastPoint] = 0
	// Update the display and checkbox
	CursorTags1_Y = Tags1[EventNumber1]
	CheckBox CheckBoxTag1 value=Tags1[EventNumber1]
	TaggedEvents1 = sum(Tags1,0,numpnts(Tags1)-1)
	Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
	// Check if no events tagged
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==0)
		CheckedTagAll1 = 0; CheckBox CheckBoxTagAll1 value=0
		If (BTDE_1 == 1)	// Then impossible to browse though detected events only
			BTDE_1 = 0; Checkbox BTDE1 value = 0
		endif
	endif
End

Function ProcUntagRange2(CtrlName) : ButtonControl
	String CtrlName
	Wave Tags2, Occurrence2, CursorTags2_Y
	Nvar LimitBegin2, LimitEnd2, EventNumber2, TaggedEvents2, CheckedTagAll2, NumberofEventsch2, BTDE_2
	Variable FirstPoint, LastPoint
	// Find the index of points located within the limits
	// First Points
	if (LimitBegin2 < Occurrence2[0])
		FirstPoint = 0
	endif
	if (LimitBegin2 > Occurrence2[numpnts(Occurrence2)-1])
		FirstPoint = numpnts(Occurrence2)-1
	endif
	if ((LimitBegin2 > Occurrence2[0]) & (LimitBegin2 < Occurrence2[numpnts(Occurrence2)-1]))
		FindLevel/P Occurrence2, LimitBegin2
		FirstPoint = floor(V_levelx) + 1
	endif
	// Second Points
	if (LimitEnd2 > Occurrence2[numpnts(Occurrence2)-1])
		LastPoint = numpnts(Occurrence2)-1
	endif
	if (LimitEnd2 < Occurrence2[0])
		LastPoint = 0
	endif
	if ((LimitEnd2 < Occurrence2[numpnts(Occurrence2)-1]) & (LimitEnd2 > Occurrence2[0]))
		FindLevel/P Occurrence2, LimitEnd2
		LastPoint = floor(V_levelx)
	endif
	Tags2[FirstPoint,LastPoint] = 0
	// Update the display and checkbox
	CursorTags2_Y = Tags2[EventNumber2]
	CheckBox CheckBoxTag2 value=Tags2[EventNumber2]
	TaggedEvents2 = sum(Tags2,0,numpnts(Tags2)-1)
	Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
	// Check if no events tagged
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==0)
		CheckedTagAll2 = 0; CheckBox CheckBoxTagAll2 value=0
		If (BTDE_2 == 1)	// Then impossible to browse though detected events only
			BTDE_2 = 0; Checkbox BTDE2 value = 0
		endif
	endif
End

// TAG ALL EVENTS WITHIN A RANGE OF NUMBERS (Channel 1)
Function ProcTagRange1_pnts(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags1, Occurrence1, CursorTags1_Y
	Nvar LimitBegin1_pnts, LimitEnd1_pnts, EventNumber1, TaggedEvents1, CheckedTagAll1, NumberofEventsch1, BUDE_1
	// Tagging the events
	Tags1[LimitBegin1_pnts,LimitEnd1_pnts] = 1
	// Update the display and checkbox
	CursorTags1_Y = Tags1[EventNumber1]
	CheckBox CheckBoxTag1 value=Tags1[EventNumber1]
	TaggedEvents1 = sum(Tags1,0,numpnts(Tags1)-1)
	Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
	// Check if all events tagged
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==numpnts(Occurrence1))
		CheckedTagAll1 = 1; CheckBox CheckBoxTagAll1 value=1
		If (BUDE_1 == 1)	// Then impossible to browse though undetected events only
			BUDE_1 = 0; Checkbox BUDE1 value = 0
		endif
	endif
End

// TAG ALL EVENTS WITHIN A RANGE OF NUMBERS (Channel 2)
Function ProcTagRange2_pnts(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags2, Occurrence2, CursorTags2_Y
	Nvar LimitBegin2_pnts, LimitEnd2_pnts, EventNumber2, TaggedEvents2, CheckedTagAll2, NumberofEventsch2, BUDE_2
	// Tagging the events
	Tags2[LimitBegin2_pnts,LimitEnd2_pnts] = 1
	// Update the display and checkbox
	CursorTags2_Y = Tags2[EventNumber2]
	CheckBox CheckBoxTag2 value=Tags2[EventNumber2]
	TaggedEvents2 = sum(Tags2,0,numpnts(Tags2)-1)
	Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
	// Check if all events tagged
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==numpnts(Occurrence2))
		CheckedTagAll2 = 1; CheckBox CheckBoxTagAll2 value=1
		If (BUDE_2 == 1)	// Then impossible to browse though undetected events only
			BUDE_2 = 0; Checkbox BUDE2 value = 0
		endif
	endif
End

// UNTAG ALL EVENTS WITHIN A RANGE OF NUMBERS (Channel 1)
Function ProcUntagRange1_pnts(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags1, Occurrence1, CursorTags1_Y
	Nvar LimitBegin1_pnts, LimitEnd1_pnts, EventNumber1, TaggedEvents1, CheckedTagAll1, NumberofEventsch1, BTDE_1
	// Untagging the events
	Tags1[LimitBegin1_pnts,LimitEnd1_pnts] = 0
	CursorTags1_Y = Tags1[EventNumber1]
	CheckBox CheckBoxTag1 value=Tags1[EventNumber1]
	TaggedEvents1 = sum(Tags1,0,numpnts(Tags1)-1)
	Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
	// Check if no events tagged
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==0)
		CheckedTagAll1 = 0; CheckBox CheckBoxTagAll1 value=0
		If (BTDE_1 == 1)	// Then impossible to browse though detected events only
			BTDE_1 = 0; Checkbox BTDE1 value = 0
		endif
	endif
End

// UNTAG ALL EVENTS WITHIN A RANGE OF NUMBERS (Channel 2)
Function ProcUntagRange2_pnts(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags2, Occurrence2, CursorTags2_Y
	Nvar LimitBegin2_pnts, LimitEnd2_pnts, EventNumber2, TaggedEvents2, CheckedTagAll2, NumberofEventsch2, BTDE_2
	Tags2[LimitBegin2_pnts,LimitEnd2_pnts] = 0
	CursorTags2_Y = Tags2[EventNumber2]
	CheckBox CheckBoxTag2 value=Tags2[EventNumber2]
	TaggedEvents2 = sum(Tags2,0,numpnts(Tags2)-1)
	Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
	// Check if no events tagged
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==0)
		CheckedTagAll2 = 0; CheckBox CheckBoxTagAll2 value=0
		If (BTDE_2 == 1)	// Then impossible to browse though detected events only
			BTDE_2 = 0; Checkbox BTDE2 value = 0
		endif
	endif
End

// SWITCH ON/OFF OF THE FILTER FOR RISE TIME - FOR CALCULATION OF THE AVERAGE (Channel 1)
Function ProcCheckRiseTime1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckRT20801
	CheckRT20801 = checked
End

// SWITCH ON/OFF OF THE FILTER FOR RISE TIME - FOR CALCULATION OF THE AVERAGE (Channel 2)
Function ProcCheckRiseTime2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckRT20802
	CheckRT20802 = checked
End


// SWITCH ON/OFF OF THE FILTER FOR BASELINE SD - FOR CALCULATION OF THE AVERAGE (Channel 1)
Function ProcCheckBaselineSD1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckBslnSD1
	CheckBslnSD1 = checked
End

// SWITCH ON/OFF OF THE FILTER FOR BASELINE SD - FOR CALCULATION OF THE AVERAGE (Channel 2)
Function ProcCheckBaselineSD2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckBslnSD2
	CheckBslnSD2 = checked
End

// DRAWS THE RISE TIME HISTOGRAM (TAGGED EVENTS ONLY - Channel 1)
Function ProcButtonShowRT2080histo1(ctrlName) : ButtonControl
	String ctrlName
	Wave RiseTime20801, Tags1
	if (sum(Tags1, 0, numpnts(tags1)-1) == 0 )
		Abort "Please tag events first. This function only displays the rise time histogram of tagged events."
	endif
	FindLevels/P/Q Tags1 1
	Wave W_FindLevels
	Make/o/N=0 RTvalues1
	Variable i
	for (i=0 ; i < numpnts(W_FindLevels) ; i+=1)
		InsertPoints i, 1, RTvalues1
		RTvalues1[i] = RiseTime20801[W_FindLevels[i]]
	endfor
	Duplicate/o RTvalues1 RiseTime2080Histo1
	Histogram/B=1 RTvalues1,RiseTime2080Histo1
	Killwaves RTvalues1
	DoWindow/K Window_RT_Histo1
	Display RiseTime2080Histo1 as "20-80% Rise Time Histogram"; ModifyGraph mode=5
	DoWindow/C Window_RT_Histo1; ModifyGraph standoff=0; ModifyGraph rgb=(0,12800,52224)
	ControlBar/W=Window_RT_Histo1 35
	Button CloseWindow_RT1 pos={480,10},size={40,20},title="close", proc=procCloseWindow_RT1
	GetAxis/Q bottom ; SetAxis bottom V_min - ((V_min+V_max)/2 - V_min), V_max + ((V_min+V_max)/2 - V_min)
End

// DRAWS THE RISE TIME HISTOGRAM (TAGGED EVENTS ONLY - Channel 2)
Function ProcButtonShowRT2080histo2(ctrlName) : ButtonControl
	String ctrlName
	Wave RiseTime20802, Tags2
	if (sum(Tags2, 0, numpnts(tags2)-1) == 0 )
		Abort "Please tag events first. This function only displays the rise time histogram of tagged events."
	endif
	FindLevels/P/Q Tags2 1
	Wave W_FindLevels
	Make/o/N=0 RTvalues2
	Variable i
	for (i=0 ; i < numpnts(W_FindLevels) ; i+=1)
		InsertPoints i, 1, RTvalues2
		RTvalues2[i] = RiseTime20802[W_FindLevels[i]]
	endfor
	Duplicate/o RTvalues2 RiseTime2080Histo2
	Histogram/B=1 RTvalues2,RiseTime2080Histo2
	Killwaves RTvalues2
	DoWindow/K Window_RT_Histo2
	Display RiseTime2080Histo2 as "20-80% Rise Time Histogram"; ModifyGraph mode=5
	DoWindow/C Window_RT_Histo2; ModifyGraph standoff=0; ModifyGraph rgb=(0,12800,52224)
	ControlBar/W=Window_RT_Histo2 35
	Button CloseWindow_RT2 pos={480,10},size={40,20},title="close", proc=procCloseWindow_RT2
	GetAxis/Q bottom ; SetAxis bottom V_min - ((V_min+V_max)/2 - V_min), V_max + ((V_min+V_max)/2 - V_min)
End

// DRAWS THE BASELINE SD HISTOGRAM (TAGGED EVENTS ONLY - Channel 1)
Function ProcButtonShowBslnSDHisto1(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags1, DetectedLevels1
	if (sum(Tags1, 0, numpnts(tags1)-1) == 0 )
		Abort "Please tag events first. This function only displays the baseline SD histogram of tagged events."
	endif
	FindLevels/Q Tags1 1
	Wave W_FindLevels
	Nvar Baseline1
	Svar FileName1
	Variable i
	Make/o/N=0 BaselineSD1
	For (i=0 ; i < numpnts(W_FindLevels) ; i+=1)
		WaveStats/Q/R=(DetectedLevels1[W_FindLevels[i]] - Baseline1*.001, DetectedLevels1[W_FindLevels[i]]) $FileName1
		InsertPoints i, 1, BaselineSD1
		BaselineSD1[i] = V_sdev
	Endfor
	Duplicate/o BaselineSD1, BaselineSDHisto1 ; Histogram/B=1 BaselineSD1,BaselineSDHisto1
	DoWindow/K Window_BslnSD_histo1
	Display BaselineSDHisto1 as "Baseline SD Histogram"; ModifyGraph mode=5
	DoWindow/C Window_BslnSD_histo1; ModifyGraph standoff=0; ModifyGraph rgb=(0,12800,52224)
	ControlBar/W=Window_BslnSD_histo1 35
	Button CloseWindow_BslnSD1 pos={480,10},size={40,20},title="close", proc=procCloseWindow_BslnSD1
	GetAxis/Q bottom ; SetAxis bottom V_min - ((V_min+V_max)/2 - V_min), V_max + ((V_min+V_max)/2 - V_min)
	ShowInfo
End

// DRAWS THE BASELINE SD HISTOGRAM (TAGGED EVENTS ONLY - Channel 2)
Function ProcButtonShowBslnSDHisto2(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags2, DetectedLevels2
	if (sum(Tags2, 0, numpnts(tags2)-1) == 0 )
		Abort "Please tag events first. This function only displays the baseline SD histogram of tagged events."
	endif
	FindLevels/Q Tags2 1
	Wave W_FindLevels
	Nvar Baseline2
	Svar FileName2
	Variable i
	Make/o/N=0 BaselineSD2
	For (i=0 ; i < numpnts(W_FindLevels) ; i+=1)
		WaveStats/Q/R=(DetectedLevels2[W_FindLevels[i]] - Baseline2*.001, DetectedLevels2[W_FindLevels[i]]) $FileName2
		InsertPoints i, 1, BaselineSD2
		BaselineSD2[i] = V_sdev
	Endfor
	Duplicate/o BaselineSD2, BaselineSDHisto2 ; Histogram/B=1 BaselineSD2,BaselineSDHisto2
	DoWindow/K Window_BslnSD_histo2
	Display BaselineSDHisto2 as "Baseline SD Histogram"; ModifyGraph mode=5
	DoWindow/C Window_BslnSD_histo2; ModifyGraph standoff=0; ModifyGraph rgb=(0,12800,52224)
	ControlBar/W=Window_BslnSD_histo2 35
	Button CloseWindow_BslnSD2 pos={480,10},size={40,20},title="close", proc=procCloseWindow_BslnSD2
	GetAxis/Q bottom ; SetAxis bottom V_min - ((V_min+V_max)/2 - V_min), V_max + ((V_min+V_max)/2 - V_min)
	ShowInfo
End

Function procCloseWindow_RT1(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K Window_RT_Histo1
end

Function procCloseWindow_RT2(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K Window_RT_Histo2
end

Function procCloseWindow_BslnSD1(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K Window_BslnSD_histo1
end

Function procCloseWindow_BslnSD2(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K Window_BslnSD_histo2
end

// FUNCTION CALLED BY BUTTON "ButtonSupEvents" IN DETECTED EVENTS 1
Function ProcButtonSupEvents1(ctrlName) : ButtonControl
	String ctrlName
	SuperimposeEvents(1)
end

// FUNCTION CALLED BY BUTTON "ButtonSupEvents" IN DETECTED EVENTS 2
Function ProcButtonSupEvents2(ctrlName) : ButtonControl
	String ctrlName
	SuperimposeEvents(2)
end

// New function SuperimposeEvents; this function is generalist (i.e. can be used for both channels)
Function SuperimposeEvents(ChNumber)
	Variable ChNumber
	
	// TODO
	// Create folder root:spacan:supevents1 and supevents2 and put there subfolders created by the function.
	// Put scale management variables in root:spacan:supevents1 or 2
	// Don't use subwindows for 2 channels display but move axis inside the same graph.
	// Then test the scale bar management functions.
	// KillDataFolder when closing SupEvents window

	// 1. Get the names of some waves, global variables and strings
	// Waves
	Wave TagWave = $"root:spacan:Tags" + num2str(ChNumber)					// Wave containing tags
	Wave HalfRiseTime = $"root:spacan:Occurrence" + num2str(ChNumber)			// Half rise time of detected events
	Wave EventFootTime = $"EventFootTime" + num2str(ChNumber)				// Foot time of detected events
	Wave RealPeakPosition = $"RealPeakPosition" + num2str(ChNumber)			// Peak time of detected events
	Wave RiseTime2080 = $"root:spacan:RiseTime2080" + num2str(ChNumber)		// Rise time of detected events
	Wave DetectedLevels = $"DetectedLevels" + num2str(ChNumber)				// ?
	// Global variables
	Nvar Baseline = $"SupBaseline" + num2str(ChNumber)
	Nvar Decay = $"SupDecay" + num2str(ChNumber)
	Nvar CheckRT2080 = $"CheckRT2080" + num2str(ChNumber)
	Nvar RTinf = $"RTinf" + num2str(ChNumber)
	Nvar RTsup = $"RTinf" + num2str(ChNumber)
	Nvar CheckBslnSD = $"CheckBslnSD" + num2str(ChNumber)
	Nvar BslnSDinf = $"BslnSDinf" + num2str(ChNumber)
	Nvar BslnSDsup = $"BslnSDsup" + num2str(ChNumber)
	Nvar CheckOverlayMode = $"CheckOverlayMode" + num2str(ChNumber)
	Nvar WinSupEvents = $"WinSupEvents" + num2str(ChNumber)
	Nvar ShowOtherCh = $"ShowCh" + num2str(3 - ChNumber)
	Nvar OverlayCounter = $"OverlayCounter" + num2str(ChNumber)
	Nvar NumTaggedEvents = $"TaggedEvents" + num2str(ChNumber)				// Number of tagged events
	Nvar CheckAlignSupOcc = $"CheckAlignSup" + num2str(ChNumber) + "Occ"
	Nvar CheckAlignSupPeak = $"CheckAlignSup" + num2str(ChNumber) + "Peak"
	Nvar CheckAlignSupFoot = $"CheckAlignSup" + num2str(ChNumber) + "Foot"
	Nvar SI = $"SI" + num2str(ChNumber)
	Nvar SIb = $"SI" + num2str(3 - ChNumber)
	Nvar CheckSupNormalize = $"CheckSupNormalize" + num2str(ChNumber)
	Nvar SupNormLeft = $"SupNormLeft" + num2str(ChNumber)
	Nvar SupNormRight = $"SupNormRight" + num2str(ChNumber)
	Nvar CheckShowIndEvents =  $"CheckShowIndEvents" + num2str(ChNumber)
	Nvar CheckUseRndColors = $"CheckUseRndColors" + num2str(ChNumber)
	Nvar NumberOfAveragedEvents = $"NumberOfAveragedEvents" + num2str(ChNumber)
	Nvar CheckIncludeAv = $"CheckIncludeAv" + num2str(ChNumber)
	Nvar CheckSupShowSD = $"CheckSupShowSD" + num2str(ChNumber)
	Nvar CheckShowCurs = $"CheckShowCurs" + num2str(ChNumber)
	Nvar CheckRemoveAxes = $"CheckRemoveAxes" + num2str(ChNumber)
	Nvar CheckShowRefLine = $"CheckShowRefLine" + num2str(ChNumber)
	Nvar RefLineValue = $"RefLineValue" + num2str(ChNumber)
	Nvar CaE = $"C" + num2str(ChNumber) + "E"			// 1 if ChNumber loaded, 0 if not
	Nvar CbE = $"C" + num2str(3 - ChNumber) + "E"			// 1 if other channel loaded, 0 if not
	// Global strings
	Svar FilenameA = $"Filename" + num2str(ChNumber)
	if (CbE)
		Svar FilenameB = $"Filename" + num2str(3 - ChNumber)
	endif

	// 2. Dealing with tags
	If (sum(TagWave,0,numpnts(TagWave)-1) == 0)
		Abort "No tagged events in channel " + num2str(ChNumber)
	Endif
	Duplicate/o TagWave TagWaveModified
	InsertPoints numpnts(TagWaveModified), 1, TagWaveModified	// Here is a trick to overcome a problem of FindLevels (dealing with detecting the last 1 in a wave).
	FindLevels/Q TagWaveModified 1
	Duplicate/o W_FindLevels TaggedEvents
	Variable i, j
	// Here a loop to clean up tags and apply filters (baseline SD and rise time)
	For (i = numpnts(TaggedEvents)-1 ; i > -1 ; i-=1)
		if (Numtype(HalfRiseTime[TagWaveModified[i]]) == 2)		// Removing bad events (NaN)
			TagWaveModified[TaggedEvents[i]] = 0
		endif
		if (HalfRiseTime[TaggedEvents[i]] < Baseline*0.001 || HalfRiseTime[TaggedEvents[i]] > pnt2x($FilenameA,numpnts($FilenameA)-1) - Decay*0.001)
			TagWaveModified[TaggedEvents[i]] = 0
		endif
		if (CheckRT2080)		// Filter for rise time
			if (RiseTime2080[TaggedEvents[i]] < RTinf*.001 || RiseTime2080[TaggedEvents[i]] > RTsup*.001)
				TagWaveModified[TaggedEvents[i]] = 0		
			endif
		endif
		if (CheckBslnSD)			// Filter for baseline
			WaveStats/Q/R=(DetectedLevels[TaggedEvents[i]] - Baseline*0.001, DetectedLevels[TaggedEvents[i]]) $FileNameA
			if (V_sdev < BslnSDinf || V_sdev > BslnSDsup)		
				TagWaveModified[TaggedEvents[i]] = 0
			endif
		endif
	Endfor
	// Get the numbers of events that remain after cleanup (tag = 1)
	FindLevels/Q TagWaveModified 1
	if (V_flag == 2)
		Abort "No event compatible with filter(s) range(s)."
	endif

	// 3. Create graphs, data folders, control bar etc
	String MainDataFolder = "SupEvents" + num2str(ChNumber)
	KillDataFolder/Z root:spacan:$MainDataFolder
	NewDataFolder/o root:spacan:$MainDataFolder
	Execute "NewDataFolder/o root:spacan:" + MainDataFolder + ":ScaleBars"
	SetDataFolder root:spacan
	String WindowName = "SupEvents" + num2str(ChNumber)
	String TempFolderName
	If (CheckOverlayMode == 0 || WinSupEvents == 0)	// If no overlay or window does not exist yet
		String DispNumAvEvents = "DispNumAvEvents" + num2str(ChNumber)
		String DispNumTaggedEvents = "DispNumTaggedEvents" + num2str(ChNumber)
		String ButtonCloseSupEvents = "ButtonCloseSupEvents" + num2str(ChNumber)
		String ProcButtonCloseSupEvents = "ProcButtonCloseSupEvents" + num2str(ChNumber)
		String ButtonSupEventsSync = "ButtonSupEvents" + num2str(ChNumber) + "Sync"
		String ProcButtonSupEventsSync = "ProcButtonSupEvents" + num2str(ChNumber) + "Sync"
		String CheckBoxSupEventsGrid = "CheckBoxSupEvents" + num2str(ChNumber) + "Grid"
		String ProcSupEventsGrid = "ProcSupEvents" + num2str(ChNumber) + "Grid"
		String ButtonFullScale = "ButtonFullScale" + num2str(ChNumber)
		String ProcButtonFullScale = "ProcButtonFullScale" + num2str(ChNumber)
		String ButtonSaveSup = "ButtonSaveSup" + num2str(ChNumber)
		String ProcButtonSaveSup = "ProcButtonSaveSup" + num2str(ChNumber)
		DoWindow/K $WindowName
		// Create an empty graph
		Display/W=(210,50,700,500) as "Superimposed tagged events, Channel " + num2str(ChNumber)
		DoWindow/C $WindowName
		if (ShowOtherCh)
			Make/N=1/o DummyXa; Make/N=1/o DummyYa; Make/N=1/o DummyXb; Make/N=1/o DummyYb
			String LeftA = "left" + num2str(ChNumber)
			AppendToGraph/W=$WindowName/L=$LeftA DummyYa vs DummyXa
			ModifyGraph/W=$WindowName mode(DummyYa)=3,marker(DummyYa)=8,rgb(DummyYa)=(65535,65535,65535)
			ModifyGraph/W=$WindowName axisEnab($leftA)={0.51,1}
			ModifyGraph/W=$WindowName freePos($leftA)=0
			String LeftB = "left" + num2str(3 - ChNumber)
			AppendToGraph/W=$WindowName/L=$LeftB DummyYb vs DummyXb
			ModifyGraph/W=$WindowName mode(DummyYb)=3,marker(DummyYb)=8,rgb(DummyYb)=(65535,65535,65535)
			ModifyGraph/W=$WindowName axisEnab($leftB)={0,0.49}
			ModifyGraph/W=$WindowName freePos($leftB)=0
		else
			Make/N=1/o DummyX; Make/N=1/o DummyY
			AppendToGraph/W=$WindowName DummyY vs DummyX
			ModifyGraph/W=$WindowName mode(DummyY)=3,marker(DummyY)=8,rgb(DummyY)=(65535,65535,65535)
		endif
		ModifyGraph/W=$WindowName margin(left) = 50
		// Clear up previous folders and reinitialize OverlayCounter
		If (OverlayCounter > 0)
			For (i = 0; i < OverlayCounter+1; i += 1)
				TempFolderName = "root:spacan:SupEvents" + num2str(ChNumber) + ":EventsCh" + num2str(ChNumber) + "Series" + num2str(i)
				if (DataFolderExists(TempFolderName))
					KillDataFolder/Z TempFolderName
				endif
			Endfor
			OverlayCounter = 0
		endif
		if (CheckOverlayMode && OverlayCounter == 0)
			TempFolderName = "root:spacan:SupEvents" + num2str(ChNumber) + ":EventsCh" + num2str(ChNumber) + "Series" + num2str(i)
			if (DataFolderExists(TempFolderName))
				KillDataFolder/Z TempFolderName
			endif
		endif
	Endif
	TempFolderName = "root:spacan:SupEvents" + num2str(ChNumber) + ":EventsCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
	Newdatafolder/O $TempFolderName	// where the waves of individual events will be stored
	if (CheckOverlayMode == 0)
		Execute "ValDisplay " + DispNumTaggedEvents + " value=" + num2str(NumTaggedEvents)
		Execute "ValDisplay " + DispNumAvEvents + " value=" + num2str(sum(TagWaveModified,0,numpnts(TagWaveModified)))
	endif

	// 4. Copy events into waves (in root:spacan:events) and superimpose the events
	String TmpWaveNameA, TmpWaveNameB, TmpWaveNameAplot, TmpWaveNameBplot
	String DetectedEvents = "DetectedEvents" + num2str(ChNumber)
	String DetectedEventsPanel = "DetectedEvents" + num2str(ChNumber) + "Panel"
	String ListBoxColors = "ListBoxColors" + num2str(ChNumber)
	if (CheckShowIndEvents)
		for (i = 0 ; i < Numpnts(TaggedEvents) ; i+=1)
			if (TagWaveModified[TaggedEvents[i]] == 1)
				TmpWaveNameA = TempFolderName + ":Ch" + num2str(ChNumber) + "Event" + num2str(TaggedEvents[i]) + "Series" + num2str(OverlayCounter)
				if (CheckAlignSupOcc)	// means align on half-rise
					Duplicate/o/R=(HalfRiseTime[TaggedEvents[i]] - Baseline*.001, HalfRiseTime[TaggedEvents[i]] + Decay*.001) $FileNameA TEMPEVENT
				endif
				if (CheckAlignSupFoot)	// means align on foot
					Duplicate/o/R=(EventFootTime[TaggedEvents[i]] - Baseline*.001, EventFootTime[TaggedEvents[i]] + Decay*.001) $FileNameA TEMPEVENT
				endif
				if (CheckAlignSupPeak)	// means align on peak time
					Duplicate/o/R=(RealPeakPosition[TaggedEvents[i]] - Baseline*.001, RealPeakPosition[TaggedEvents[i]] + Decay*.001) $FileNameA TEMPEVENT
				endif
				SetScale/P x -Baseline*.001, SI*1e-6, "s", TEMPEVENT
				if (CheckSupNormalize)
					Wavestats/Q/R=(-SupNormLeft*.001, -SupNormRight*.001) TEMPEVENT
					TEMPEVENT = TEMPEVENT - V_avg
				endif
				Duplicate/o TEMPEVENT $TmpWaveNameA
				if (ShowOtherCh)

					TmpWaveNameB = TempFolderName + ":Ch" + num2str(3 - ChNumber) + "Event" + num2str(TaggedEvents[i]) + "Series" + num2str(OverlayCounter)
					if (CheckAlignSupOcc)
						Duplicate/o/R=(HalfRiseTime[TaggedEvents[i]] - Baseline*.001, HalfRiseTime[TaggedEvents[i]] + Decay*.001) $FileNameB TEMPEVENT
					endif
					if (CheckAlignSupFoot)
						Duplicate/o/R=(EventFootTime[TaggedEvents[i]] - Baseline*.001, EventFootTime[TaggedEvents[i]] + Decay*.001) $FileNameB TEMPEVENT
					endif
					if (CheckAlignSupPeak)
						Duplicate/o/R=(RealPeakPosition[TaggedEvents[i]] - Baseline*.001, RealPeakPosition[TaggedEvents[i]] + Decay*.001) $FileNameB TEMPEVENT
					endif
					SetScale/P x -Baseline*.001, SIb*1e-6, "s", TEMPEVENT
					if (CheckSupNormalize)
						Wavestats/Q/R=(-SupNormLeft*.001, -SupNormRight*.001) TEMPEVENT
						TEMPEVENT = TEMPEVENT - V_avg
					endif					
					Duplicate/o TEMPEVENT $TmpWaveNameB
					
					Appendtograph/W=$WindowName/L=$LeftA $TmpWaveNameA					
					Appendtograph/W=$WindowName/L=$LeftB $TmpWaveNameB
					
					// Dealing with colors
					TmpWaveNameAplot = "Ch" + num2str(ChNumber) + "Event" + num2str(TaggedEvents[i]) + "Series" + num2str(OverlayCounter)
					TmpWaveNameBplot = "Ch" + num2str(3 - ChNumber) + "Event" + num2str(TaggedEvents[i]) + "Series" + num2str(OverlayCounter)
					// Case 1: the user asked for random colors
					if (CheckUseRndColors)
						Variable Col1 = abs(enoise(65535))
						Variable Col2 = abs(enoise(65535))
						Variable Col3 = abs(enoise(65535))
						ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(Col1,Col2,Col3)
						ModifyGraph/W=$WindowName rgb($TmpWaveNameBplot)=(Col1,Col2,Col3)
					else
					// Case 2: take the value in LisBoxColors	
						ControlInfo/W=$DetectedEvents#$DetectedEventsPanel $ListBoxColors
						if (V_value == 0)	// means black
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(0,0,0)
							ModifyGraph/W=$WindowName rgb($TmpWaveNameBplot)=(0,0,0)
						endif
						if (V_value == 1)	// means red
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(65535,0,0)
							ModifyGraph/W=$WindowName rgb($TmpWaveNameBplot)=(65535,0,0)
						endif
						if (V_value == 2)	// means orange
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(65280,43520,0)
							ModifyGraph/W=$WindowName rgb($TmpWaveNameBplot)=(65280,43520,0)
						endif		
						if (V_value == 3)	// means yellow
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(65280,65280,0)
							ModifyGraph/W=$WindowName rgb($TmpWaveNameBplot)=(65280,65280,0)
						endif
						if (V_value == 4)	// means green
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(0,65535,0)
							ModifyGraph/W=$WindowName rgb($TmpWaveNameBplot)=(0,65535,0)
						endif	
						if (V_value == 5)	// means blue
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(0,0,65535)
							ModifyGraph/W=$WindowName rgb($TmpWaveNameBplot)=(0,0,65535)
						endif	
						if (V_value == 6)	// means purple
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(52224,0,41728)
							ModifyGraph/W=$WindowName rgb($TmpWaveNameBplot)=(52224,0,41728)
						endif																						
					endif
				else
					Appendtograph/W=$WindowName $TmpWaveNameA
					TmpWaveNameAplot = "Ch" + num2str(ChNumber) + "Event" + num2str(TaggedEvents[i]) + "Series" + num2str(OverlayCounter)
					// Case 1: the user asked for random colors
					if (CheckUseRndColors)
						ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(abs(enoise(65535)),abs(enoise(65535)),abs(enoise(65535)))
					else
					// Case 2: take the value in LisBoxColors		
						ControlInfo/W=$DetectedEvents#$DetectedEventsPanel $ListBoxColors
						if (V_value == 0)	// means black
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(0,0,0)
						endif
						if (V_value == 1)	// means red
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(65535,0,0)
						endif
						if (V_value == 2)	// means orange
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(65280,43520,0)
						endif
						if (V_value == 3)	// means yellow
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(65280,65280,0)
						endif		
						if (V_value == 4)	// means green
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(0,65535,0)
						endif
						if (V_value == 5)	// means blue
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(0,0,65535)
						endif	
						if (V_value == 6)	// means purple
							ModifyGraph/W=$WindowName rgb($TmpWaveNameAplot)=(52224,0,41728)
						endif
					endif	
				endif
			endif
		endfor
	endif

	Killwaves/Z TEMPEVENT

	NumberOfAveragedEvents = sum(TagWaveModified,0,numpnts(TagWaveModified)-1)

	// New method for average, in order to enable easy calculation of the SD.
	// We will use an array to have quick assess to every point of every wave.
	if (CheckIncludeAv)
		// First get size of event (segment size)
		// ...Channel A
		if (CheckAlignSupOcc)
			Duplicate/o/R=(HalfRiseTime[TaggedEvents[0]] - Baseline*.001, HalfRiseTime[TaggedEvents[0]] + Decay*.001) $FileNameA TEMPWAVE_A
		endif
		if (CheckAlignSupFoot)
			Duplicate/o/R=(EventFootTime[TaggedEvents[0]] - Baseline*.001, EventFootTime[TaggedEvents[0]] + Decay*.001) $FileNameA TEMPWAVE_A
		endif
		if (CheckAlignSupPeak)
			Duplicate/o/R=(RealPeakPosition[TaggedEvents[0]] - Baseline*.001, RealPeakPosition[TaggedEvents[0]] + Decay*.001) $FileNameA TEMPWAVE_A
		endif
		Variable	segmentsizeA = numpnts(TEMPWAVE_A)
		// ...Channel B
		if (ShowOtherCh)
			if (CheckAlignSupOcc)
				Duplicate/o/R=(HalfRiseTime[TaggedEvents[0]] - Baseline*.001, HalfRiseTime[TaggedEvents[0]] + Decay*.001) $FileNameB TEMPWAVE_B
			endif
			if (CheckAlignSupFoot)
				Duplicate/o/R=(EventFootTime[TaggedEvents[0]] - Baseline*.001, EventFootTime[TaggedEvents[0]] + Decay*.001) $FileNameB TEMPWAVE_B
			endif
			if (CheckAlignSupPeak)
				Duplicate/o/R=(RealPeakPosition[TaggedEvents[0]] - Baseline*.001, RealPeakPosition[TaggedEvents[0]] + Decay*.001) $FileNameB TEMPWAVE_B
			endif
			Variable	segmentsizeB = numpnts(TEMPWAVE_B)
		endif
		
		Make/o/N=(NumberOfAveragedEvents,segmentsizeA) array_a
		if (ShowOtherCh)
			Make/o/N=(NumberOfAveragedEvents,segmentsizeB) array_b
		endif
		For (i = 0 ; i < Numpnts(TaggedEvents) ; i+=1)
		// Filling up the array with individual detected events
	//	if (TagWaveModified[TaggedEvents[i]] == 1)
			if (CheckAlignSupOcc)
				Duplicate/o/R=(HalfRiseTime[TaggedEvents[i]] - Baseline*.001, HalfRiseTime[TaggedEvents[i]] + Decay*.001) $FileNameA TEMPWAVE_A
			endif
			if (CheckAlignSupFoot)
				Duplicate/o/R=(EventFootTime[TaggedEvents[i]] - Baseline*.001, EventFootTime[TaggedEvents[i]] + Decay*.001) $FileNameA TEMPWAVE_A
			endif
			if (CheckAlignSupPeak)
				Duplicate/o/R=(RealPeakPosition[TaggedEvents[i]] - Baseline*.001, RealPeakPosition[TaggedEvents[i]] + Decay*.001) $FileNameA TEMPWAVE_A
			endif
			SetScale/P x -Baseline*.001, SI*1e-6, "s", TEMPWAVE_A
			if (CheckSupNormalize)
				Wavestats/Q/R=(-SupNormLeft*.001, -SupNormRight*.001) TEMPWAVE_A
				TEMPWAVE_A = TEMPWAVE_A - V_avg
			endif
			For (j = 0; j < segmentsizeA+1; j += 1)
				array_a[i][j] = TEMPWAVE_A[j]
			Endfor
			if (ShowOtherCh)
				if (CheckAlignSupOcc)
					Duplicate/o/R=(HalfRiseTime[TaggedEvents[i]] - Baseline*.001, HalfRiseTime[TaggedEvents[i]] + Decay*.001) $FileNameB TEMPWAVE_B
				endif
				if (CheckAlignSupFoot)
					Duplicate/o/R=(EventFootTime[TaggedEvents[i]] - Baseline*.001, EventFootTime[TaggedEvents[i]] + Decay*.001) $FileNameB TEMPWAVE_B
				endif
				if (CheckAlignSupPeak)
					Duplicate/o/R=(RealPeakPosition[TaggedEvents[i]] - Baseline*.001, RealPeakPosition[TaggedEvents[i]] + Decay*.001) $FileNameB TEMPWAVE_B
				endif
				SetScale/P x -Baseline*.001, SIb*1e-6, "s", TEMPWAVE_B
				if (CheckSupNormalize)
					Wavestats/Q/R=(-SupNormLeft*.001, -SupNormRight*.001) TEMPWAVE_B
					TEMPWAVE_B = TEMPWAVE_B - V_avg
				endif
				For (j = 0; j < segmentsizeB+1; j += 1)
					array_b[i][j] = TEMPWAVE_B[j]
				Endfor
			endif
		Endfor
		Make/N=(segmentsizeA)/o TMP_AVERAGE_A		// for average
		Make/N=(segmentsizeA)/o TMP_SD_A			// for standard deviation
		if (ShowOtherCh)
			Make/N=(segmentsizeB)/o TMP_AVERAGE_B		// for average
			Make/N=(segmentsizeB)/o TMP_SD_B		// for standard deviation
		endif
		For (i = 0 ;  i <segmentsizeA+1 ; i+=1)
			Make/o/N=(NumberOfAveragedEvents) TMP_PointAverage_a
			TMP_PointAverage_a = array_a[p][i]; Wavestats/Q TMP_PointAverage_a
			TMP_AVERAGE_A[i] = V_avg
			TMP_SD_A[i] = V_sdev
		Endfor
		if (ShowOtherCh)
			For (i = 0 ;  i <segmentsizeB+1 ; i+=1)
				Make/o/N=(NumberOfAveragedEvents) TMP_PointAverage_b
				TMP_PointAverage_b = array_b[p][i]; Wavestats/Q TMP_PointAverage_b
				TMP_AVERAGE_B[i] = V_avg
				TMP_SD_B[i] = V_sdev
			Endfor
		endif
		
		String AvEventA = TempFolderName + ":AverageCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		String AvEventAPlot = "AverageCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		SetScale/P x -Baseline*.001, SI*1e-6, "s", TMP_AVERAGE_A
		Duplicate/o TMP_AVERAGE_A $AvEventA
		String SDEventA = TempFolderName + ":SDCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		String SDEventAplot = "SDCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		SetScale/P x -Baseline*.001, SI*1e-6, "s", TMP_SD_A
		Duplicate/o TMP_SD_A $SDEventA
		if (ShowOtherCh)
			String AvEventB = TempFolderName + ":AverageCh" + num2str(3 - ChNumber) + "Series" + num2str(OverlayCounter)
			String AvEventBplot = "AverageCh" + num2str(3 - ChNumber) + "Series" + num2str(OverlayCounter)
			SetScale/P x -Baseline*.001, SIb*1e-6, "s", TMP_AVERAGE_B
			Duplicate/o TMP_AVERAGE_B $AvEventB
			String SDEventB = TempFolderName + ":SDCh" + num2str(3 - ChNumber) + "Series" + num2str(OverlayCounter)
			String SDEventBplot = "SDCh" + num2str(3 - ChNumber) + "Series" + num2str(OverlayCounter)
			SetScale/P x -Baseline*.001, SIb*1e-6, "s", TMP_SD_B
			Duplicate/o TMP_SD_B $SDEventB
		endif

		if (CheckSupShowSD)
		
			Duplicate/o TMP_SD_A, SDEventA_top; SDEventA_top = TMP_AVERAGE_A + TMP_SD_A
			reverse SDEventA_top
			Duplicate/o SDEventA_top, SDEventA_top_x
			SDEventA_top_x = p*SI*1e-6 - Baseline*.001
			reverse SDEventA_top_x
			Duplicate/o TMP_SD_A, SDEventA_bottom; SDEventA_bottom = TMP_AVERAGE_A - TMP_SD_A
			Duplicate/o SDEventA_bottom, SDEventA_bottom_x
			SDEventA_bottom_x = p*SI*1e-6 - Baseline*.001
			Concatenate/NP/o {SDEventA_bottom, SDEventA_top}, SDEventA_box
			Concatenate/NP/o {SDEventA_bottom_x, SDEventA_top_x}, SDEventA_box_x
			InsertPoints numpnts(SDEventA_box), 1, SDEventA_box; SDEventA_box[numpnts(SDEventA_box)-1] = SDEventA_box[0]
			InsertPoints numpnts(SDEventA_box_x), 1, SDEventA_box_x; SDEventA_box_x[numpnts(SDEventA_box_x)-1] = SDEventA_box_x[0]				
			String SDEventAbox_y = TempFolderName + ":SDCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter) + "_y"
			String SDEventAbox_yplot = "SDCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter) + "_y"
			Duplicate/o SDEventA_box $SDEventAbox_y
			String SDEventAbox_x = TempFolderName + ":SDCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter) + "_x"
			String SDEventAbox_xplot = "SDCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter) + "_x"
			Duplicate/o SDEventA_box_x $SDEventAbox_x
			
			if (ShowOtherCh == 0)
				Appendtograph/W=$WindowName $SDEventAbox_y vs $SDEventAbox_x
				// Here choose colors
				ControlInfo/W=$DetectedEvents#$DetectedEventsPanel $ListBoxColors
				if (CheckShowIndEvents)		// if events shown, use color that can be seen above superimposed events
					if (V_value == 0)	// Use pale red if black selected
						ModifyGraph/W=$WindowName lsize($SDEventAbox_yplot)=2,rgb($SDEventAbox_yplot)=(65280,32768,32768)
					else		// Otherwise use grey
						ModifyGraph/W=$WindowName lsize($SDEventAbox_yplot)=2,rgb($SDEventAbox_yplot)=(21760,21760,21760)
					endif
				else		// else use color indicated by the user (but paler)
					//SetDrawLayer UserBack
					if (V_value == 0)	// means black
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(21760,21760,21760)
						//SetDrawEnv/W=$WindowName xcoord=bottom, ycoord=left, fillpat=3, fillfgc=(21760,21760,21760), linefgc=(65535,65535,65535)
					endif
					if (V_value == 1)	// means red
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(65280,32768,32768)
						//SetDrawEnv/W=$WindowName xcoord=bottom, ycoord=left, fillpat=3, fillfgc=(65280,32768,32768), linefgc=(65535,65535,65535)
					endif
					if (V_value == 2)	// means orange
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(65280,54528,32768)
						//SetDrawEnv/W=$WindowName xcoord=bottom, ycoord=left, fillpat=3, fillfgc=(65280,54528,32768), linefgc=(65535,65535,65535)
					endif
					if (V_value == 3)	// means yellow
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(65280,65280,32768)
						//SetDrawEnv/W=$WindowName xcoord=bottom, ycoord=left, fillpat=3, fillfgc=(65280,65280,32768), linefgc=(65535,65535,65535)
					endif		
					if (V_value == 4)	// means green
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(32768,65280,32768)
						//SetDrawEnv/W=$WindowName xcoord=bottom, ycoord=left, fillpat=3, fillfgc=(32768,65280,32768), linefgc=(65535,65535,65535)
					endif
					if (V_value == 5)	// means blue
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(32768,40704,65280)
						//SetDrawEnv/W=$WindowName xcoord=bottom, ycoord=left, fillpat=3, fillfgc=(32768,40704,65280), linefgc=(65535,65535,65535)
					endif	
					if (V_value == 6)	// means purple
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(65280,32768,58880)
						//SetDrawEnv/W=$WindowName xcoord=bottom, ycoord=left, fillpat=3, fillfgc=(65280,32768,58880), linefgc=(65535,65535,65535)
					endif
					//DrawPoly SDEventA_box_x[0], SDEventA_box[0], 1, 1, SDEventA_box_x, SDEventA_box
				endif
				
			else
			
				Appendtograph/W=$WindowName/L=$LeftA $SDEventAbox_y vs $SDEventAbox_x

				Duplicate/o TMP_SD_B, SDEventB_top; SDEventB_top = TMP_AVERAGE_B + TMP_SD_B
				Reverse SDEventB_top
				Duplicate/o SDEventB_top, SDEventB_top_x
				SDEventB_top_x = p*SIb*1e-6 - Baseline*.001
				Reverse SDEventB_top_x
				Duplicate/o TMP_SD_B, SDEventB_bottom; SDEventB_bottom = TMP_AVERAGE_B - TMP_SD_B
				Duplicate/o SDEventB_bottom, SDEventB_bottom_x
				SDEventB_bottom_x = p*SIb*1e-6 - Baseline*.001
				Concatenate/NP/o {SDEventB_bottom, SDEventB_top}, SDEventB_box
				Concatenate/NP/o {SDEventB_bottom_x, SDEventB_top_x}, SDEventB_box_x
				InsertPoints numpnts(SDEventB_box), 1, SDEventB_box; SDEventB_box[numpnts(SDEventB_box)-1] = SDEventB_box[0]
				InsertPoints numpnts(SDEventB_box_x), 1, SDEventB_box_x; SDEventB_box_x[numpnts(SDEventB_box_x)-1] = SDEventB_box_x[0]				
				String SDEventBbox_y = TempFolderName + ":SDCh" + num2str(3 - ChNumber) + "Series" + num2str(OverlayCounter) + "_y"
				String SDEventBbox_yplot = "SDCh" + num2str(3 - ChNumber) + "Series" + num2str(OverlayCounter) + "_y"
				Duplicate/o SDEventB_box $SDEventBbox_y
				String SDEventBbox_x = TempFolderName + ":SDCh" + num2str(3 - ChNumber) + "Series" + num2str(OverlayCounter) + "_x"
				String SDEventBbox_xplot = "SDCh" + num2str(3 - ChNumber) + "Series" + num2str(OverlayCounter) + "_x"
				Duplicate/o SDEventB_box_x $SDEventBbox_x
				
				Appendtograph/W=$WindowName/L=$LeftB $SDEventBbox_y vs $SDEventBbox_x

				// Here choose colors
				ControlInfo/W=$DetectedEvents#$DetectedEventsPanel $ListBoxColors
				if (CheckShowIndEvents)		// if events shown, use color that can be seen above superimposed events
					if (V_value == 0)	// Use pale red if black selected
						ModifyGraph/W=$WindowName lsize($SDEventAbox_yplot)=2,rgb($SDEventAbox_yplot)=(65280,32768,32768)
						ModifyGraph/W=$WindowName lsize($SDEventBbox_yplot)=2,rgb($SDEventBbox_yplot)=(65280,32768,32768)
					else		// Otherwise use grey
						ModifyGraph/W=$WindowName lsize($SDEventAbox_yplot)=2,rgb($SDEventAbox_yplot)=(21760,21760,21760)
						ModifyGraph/W=$WindowName lsize($SDEventBbox_yplot)=2,rgb($SDEventBbox_yplot)=(21760,21760,21760)
					endif
				else		// else use color indicated by the user (but paler)
					if (V_value == 0)	// means black
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(21760,21760,21760)
						ModifyGraph/W=$WindowName rgb($SDEventBbox_yplot)=(21760,21760,21760)
					endif
					if (V_value == 1)	// means red
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(65280,32768,32768)
						ModifyGraph/W=$WindowName rgb($SDEventBbox_yplot)=(65280,32768,32768)
					endif
					if (V_value == 2)	// means orange
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(65280,54528,32768)
						ModifyGraph/W=$WindowName rgb($SDEventBbox_yplot)=(65280,54528,32768)
					endif
					if (V_value == 3)	// means yellow
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(65280,65280,32768)
						ModifyGraph/W=$WindowName rgb($SDEventBbox_yplot)=(65280,65280,32768)
					endif		
					if (V_value == 4)	// means green
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(32768,65280,32768)
						ModifyGraph/W=$WindowName rgb($SDEventBbox_yplot)=(32768,65280,32768)
					endif
					if (V_value == 5)	// means blue
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(32768,40704,65280)
						ModifyGraph/W=$WindowName rgb($SDEventBbox_yplot)=(32768,40704,65280)
					endif	
					if (V_value == 6)	// means purple
						ModifyGraph/W=$WindowName rgb($SDEventAbox_yplot)=(65280,32768,58880)
						ModifyGraph/W=$WindowName rgb($SDEventBbox_yplot)=(65280,32768,58880)
					endif		
				endif
			endif
		endif

		if (ShowOtherCh)
			Appendtograph/W=$WindowName/L=$LeftA $AvEventA
			Appendtograph/W=$WindowName/L=$LeftB $AvEventB

			ControlInfo/W=$DetectedEvents#$DetectedEventsPanel $ListBoxColors
			if (CheckShowIndEvents)		// if events shown, use color that can be seen above superimposed events
				if (V_value == 0)	// Use pale red if black selected
					ModifyGraph/W=$WindowName lsize($AvEventAplot)=2,rgb($AvEventAplot)=(65280,32768,32768)
					ModifyGraph/W=$WindowName lsize($AvEventBplot)=2,rgb($AvEventBplot)=(65280,32768,32768)
				else		// Otherwise use grey
					ModifyGraph/W=$WindowName lsize($AvEventAplot)=2,rgb($AvEventAplot)=(21760,21760,21760)
					ModifyGraph/W=$WindowName lsize($AvEventBplot)=2,rgb($AvEventBplot)=(21760,21760,21760)
				endif
			else		// else use color indicated by the user (but paler)
				if (V_value == 0)	// means black
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(21760,21760,21760)
					ModifyGraph/W=$WindowName rgb($AvEventBplot)=(21760,21760,21760)
				endif
				if (V_value == 1)	// means red
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(65280,32768,32768)
					ModifyGraph/W=$WindowName rgb($AvEventBplot)=(65280,32768,32768)
				endif
				if (V_value == 2)	// means orange
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(65280,54528,32768)
					ModifyGraph/W=$WindowName rgb($AvEventBplot)=(65280,54528,32768)
				endif
				if (V_value == 3)	// means yellow
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(65280,65280,32768)
					ModifyGraph/W=$WindowName rgb($AvEventBplot)=(65280,65280,32768)
				endif		
				if (V_value == 4)	// means green
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(32768,65280,32768)
					ModifyGraph/W=$WindowName rgb($AvEventBplot)=(32768,65280,32768)
				endif
				if (V_value == 5)	// means blue
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(32768,40704,65280)
					ModifyGraph/W=$WindowName rgb($AvEventBplot)=(32768,40704,65280)
				endif	
				if (V_value == 6)	// means purple
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(65280,32768,58880)
					ModifyGraph/W=$WindowName rgb($AvEventBplot)=(65280,32768,58880)
				endif		
			endif
			
		else
		
			Appendtograph/W=$WindowName $AvEventA; ModifyGraph/W=$WindowName lsize($AvEventAplot)=2
			ControlInfo/W=$DetectedEvents#$DetectedEventsPanel $ListBoxColors
			if (CheckShowIndEvents == 0)		// If no individual events shown; use colors from ListBoxColors for the average
				if (V_value == 0)	// means black
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(0,0,0)
				endif
				if (V_value == 1)	// means red
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(65535,0,0)
				endif
				if (V_value == 2)	// means orange
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(65280,43520,0)
				endif
				if (V_value == 3)	// means yellow
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(65280,65280,0)
				endif		
				if (V_value == 4)	// means green
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(0,65535,0)
				endif
				if (V_value == 5)	// means blue
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(0,0,65535)
				endif	
				if (V_value == 6)	// means purple
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(52224,0,41728)
				endif
			else		// else use a color that can be seen above the color chosen for the events
				if (V_value == 0)	// Use red if black selected
					ModifyGraph/W=$WindowName rgb($AvEventAplot)=(65535,0,0)
				else		// Otherwise use black
					ModifyGraph/W=$WindowName lsize($AvEventAplot)=2,rgb($AvEventAplot)=(0,0,0)
				endif
			endif
		endif
	endif
	
	// Cleanup
	KillWaves/Z TEMPWAVE_A, TMP_AVERAGE_A, TMP_SD_A, SDEventA_top, SDEventA_top_x, TMP_PointAverage_a, array_a
	KillWaves/Z SDEventA_bottom, SDEventA_bottom_x, SDEventA_box, SDEventA_box_x
	KillWaves/Z TEMPWAVE_B, TMP_AVERAGE_B, TMP_SD_B, SDEventB_top, SDEventB_top_x, TMP_PointAverage_b, array_b
	KillWaves/Z SDEventB_bottom, SDEventB_bottom_x, SDEventB_box, SDEventB_box_x

	// Cursor showing the normalization window
	DoUpdate
	
	if (CheckSupNormalize && CheckShowCurs)
		Make/N=4/o NormCursorLeftX; Make/N=4/o NormCursorLeftY
		Make/N=4/o NormCursorRightX; Make/N=4/o NormCursorRightY
		// If average displayed, align the cursor on the average
		if (ShowOtherCh)
			GetAxis/Q/W=$WindowName $LeftA
		else
			GetAxis/Q/W=$WindowName left
		endif
		NormCursorLeftY[0] = (V_max - V_min)*0.05
		NormCursorLeftY[1] = NormCursorLeftY[0]
		NormCursorLeftY[2] = - (V_max - V_min)*0.05
		NormCursorLeftY[3] = NormCursorLeftY[2]
			
		NormCursorLeftX[0] = -SupNormLeft*.001 + (Baseline + Decay)*.001*0.01
		NormCursorLeftX[1] = -SupNormLeft*.001
		NormCursorLeftX[2] = -SupNormLeft*.001
		NormCursorLeftX[3] = -SupNormLeft*.001 + (Baseline + Decay)*.001*0.01
		
		NormCursorRightY[0] = (V_max - V_min)*0.05
		NormCursorRightY[1] = NormCursorRightY[0]
		NormCursorRightY[2] = - (V_max - V_min)*0.05
		NormCursorRightY[3] = NormCursorRightY[2]
		
		NormCursorRightX[0] = -SupNormRight*.001 - (Baseline + Decay)*.001*0.01
		NormCursorRightX[1] = -SupNormRight*.001
		NormCursorRightX[2] = -SupNormRight*.001
		NormCursorRightX[3] = -SupNormRight*.001 - (Baseline + Decay)*.001*0.01
		
		String NormCsrLeft_Y = TempFolderName + ":CsrLeftYCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		String NormCsrLeft_Yplot = "CsrLeftYCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		String NormCsrLeft_X = TempFolderName + ":CsrLeftXCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		String NormCsrLeft_Xplot = "CsrLeftXCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		Duplicate/o NormCursorLeftY $NormCsrLeft_Y; KillWaves NormCursorLeftY
		Duplicate/o NormCursorLeftX $NormCsrLeft_X; KillWaves NormCursorLeftX
		
		String NormCsrRight_Y = TempFolderName + ":CsrRightYCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		String NormCsrRight_Yplot = "CsrRightYCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		String NormCsrRight_X = TempFolderName + ":CsrRightXCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		String NormCsrRight_Xplot = "CsrRightXCh" + num2str(ChNumber) + "Series" + num2str(OverlayCounter)
		Duplicate/o NormCursorRightY $NormCsrRight_Y; KillWaves NormCursorRightY
		Duplicate/o NormCursorRightX $NormCsrRight_X; KillWaves NormCursorRightX
		
		ControlInfo/W=$DetectedEvents#$DetectedEventsPanel $ListBoxColors
		if (ShowOtherCh)
			AppendToGraph/W=$WindowName/L=$LeftA $NormCsrLeft_Y vs $NormCsrLeft_X
			AppendToGraph/W=$WindowName/L=$LeftA $NormCsrRight_Y vs $NormCsrRight_X
			if (V_value == 4)	// means events/average green
				ModifyGraph/W=$WindowName lsize($NormCsrLeft_Yplot)=2,rgb($NormCsrLeft_Yplot)=(65535,0,0)
				ModifyGraph/W=$WindowName lsize($NormCsrRight_Yplot)=2,rgb($NormCsrRight_Yplot)=(65535,0,0)
			else
				ModifyGraph/W=$WindowName lsize($NormCsrLeft_Yplot)=2,rgb($NormCsrLeft_Yplot)=(0,65280,0)
				ModifyGraph/W=$WindowName lsize($NormCsrRight_Yplot)=2,rgb($NormCsrRight_Yplot)=(0,65280,0)
			endif
			SetAxis/W=$WindowName bottom -baseline*0.001, decay*0.001
		else
			AppendToGraph/W=$WindowName $NormCsrLeft_Y vs $NormCsrLeft_X
			AppendToGraph/W=$WindowName $NormCsrRight_Y vs $NormCsrRight_X

			SetAxis/W=$WindowName bottom -baseline*0.001, decay*0.001
		endif
	endif

	If (CheckOverlayMode == 0 || WinSupEvents == 0)
		ModifyGraph/W=$WindowName standoff=0
	endif

	
	if (CheckRemoveAxes == 0 && ShowOtherCh)
		GetAxis/Q/W=$WindowName $LeftA
		SetAxis/W=$WindowName $LeftA (V_min - (V_max - V_min)*0.05), (V_max + (V_max - V_min)*0.05)
		GetAxis/Q/W=$WindowName $LeftB
		SetAxis/W=$WindowName $LeftB (V_min - (V_max - V_min)*0.05), (V_max + (V_max - V_min)*0.05)
		ModifyGraph lblPosMode($leftA)=2
		ModifyGraph lblPosMode($leftB)=2
	endif
	if (CheckRemoveAxes == 0 && ShowOtherCh == 0)
		GetAxis/Q/W=$WindowName Left
		SetAxis/W=$WindowName left (V_min - (V_max - V_min)*0.05), (V_max + (V_max - V_min)*0.05)
		SetAxis/W=$WindowName left (V_min - (V_max - V_min)*0.05), (V_max + (V_max - V_min)*0.05)
		ModifyGraph lblPosMode(left)=2
	endif	

	if (ShowOtherCh)
		ModifyGraph/W=$WindowName axOffset($LeftA)=-2
		ModifyGraph/W=$WindowName axOffset(bottom)=0
		ModifyGraph/W=$WindowName axOffset($LeftB)=-2
	endif
	if (ShowOtherCh == 0)
		ModifyGraph/W=$WindowName axOffset(left)=-2
		ModifyGraph/W=$WindowName axOffset(bottom)=0
	endif

	Make/N=2/o RefLine_y; Make/N=2/o RefLine_x
	if (CheckShowRefLine)
		RefLine_y = RefLineValue
		RefLine_x[0] = -Baseline*.001
		RefLine_x[1] = Decay*0.001
		if (ShowOtherCh)
			AppendToGraph/W=$WindowName/L=$LeftA RefLine_y vs RefLine_x
			ModifyGraph/W=$WindowName lstyle(RefLine_y)=2,rgb(RefLine_y)=(0,0,0)
			ModifyGraph/W=$WindowName margin(left)=57
			SetDrawEnv xcoord = bottom, ycoord = $LeftA
			DrawText -Baseline*.001, RefLineValue, num2str(RefLineValue)
		else
			AppendToGraph/W=$WindowName RefLine_y vs RefLine_x
			ModifyGraph/W=$WindowName lstyle(RefLine_y)=2,rgb(RefLine_y)=(0,0,0)
			ModifyGraph/W=$WindowName margin(left)=57
			SetDrawEnv xcoord = bottom, ycoord = left
			DrawText -Baseline*.001, RefLineValue, num2str(RefLineValue)			
		endif
	endif

	if (CheckRemoveAxes && ShowOtherCh)
		ModifyGraph/W=$WindowName noLabel=2,axThick=0
		ModifyGraph/W=$WindowName margin=28
	endif
	
	WinSupEvents = 1

	if (CheckOverlayMode)
		OverlayCounter += 1
	endif
	
	// CONTROL BAR
	
	ControlBar/W=$WindowName 120
	ModifyGraph/W=$WindowName cbRGB=(60000,60000,60000)
	CheckBox $CheckBoxSupEventsGrid	labelback=(60000,60000,60000), pos={600,29},size={40,20},	title="Grid", 	proc=$ProcSupEventsGrid
	Button $ButtonFullScale				labelback=(60000,60000,60000), pos={530,27},size={60,20},	title="Full Scale", 	proc=$ProcButtonFullScale
	Button $ButtonSaveSup				labelback=(60000,60000,60000), pos={530,3},size={60,20},	title="Save fig", proc=$ProcButtonSaveSup
	ValDisplay $DispNumTaggedEvents		labelback=(60000,60000,60000), pos={515,57},	size={120,20},	title="# tagged: "
	ValDisplay $DispNumAvEvents			labelback=(60000,60000,60000), pos={515,80},	size={130,20},	title="# displayed: "
	Button $ButtonCloseSupEvents			labelback=(60000,60000,60000), pos={600,3},size={40,20},	title="Close", proc=$ProcButtonCloseSupEvents

	// SCALE BAR MANAGEMENT
	String GBScalesSupEvents = "GBScalesSupEvents" + num2str(ChNumber)
	String ScaleBarTitleRasters1 = "ScaleBarTitleRasters1" + num2str(ChNumber)
	String ScaleBarTitleRasters2 = "ScaleBarTitleRasters2" + num2str(ChNumber)
	String SetScaleNumCtrl = "SetScaleNum_" + WindowName
	String SetXScaleCtrl = "SetXScaleB_" + WindowName
	String SetYScaleCtrl = "SetYScaleB_" + WindowName
	String ButtonDoneScaleBars = "ButtonDoneScaleBars" + num2str(ChNumber)
	String ButtonEraseScaleBars = "ButtonEraseScaleBars" + num2str(ChNumber)
	
	GroupBox $GBScalesSupEvents			pos={5,2},size={500,110},title="Scale bars",fsize=12,fstyle=1
	
	Execute "SetDataFolder root:spacan:" + MainDataFolder + ":ScaleBars"
		
	Variable/G $"ScaleNum_" + WindowName
	Nvar ScaleNumVar = $"ScaleNum_" + WindowName
	ScaleNumVar = NaN
		
	Variable/G $"YScaleBar_" + WindowName
	Nvar YScaleBarVar = $"YScaleBar_" + WindowName
	YScaleBarVar = NaN		
		
	Variable/G $"XScaleBar_" + WindowName
	Nvar XScaleBarVar = $"XScaleBar_" + WindowName
	XScaleBarVar = NaN		

	TitleBox $ScaleBarTitleRasters1			frame=0, fsize=11,	fstyle=1,pos={175,22},size={100,20},title="Use graph marquee to draw scale bars",frame=0,labelBack=(60000,60000,60000)
	TitleBox $ScaleBarTitleRasters2			frame=0, fsize=11,	fstyle=1,pos={175,38},size={100,20},title="(click and drag diagonally, and select DrawScale)",frame=0,labelBack=(60000,60000,60000)
	SetVariable $SetScaleNumCtrl	pos={175,57},size={85,20},labelback=(60000,60000,60000),title="Bar #:",value=ScaleNumVar,proc=ProcSetScaleNum,limits={1,inf,1}
	SetVariable $SetYScaleCtrl 	pos={265,57},size={110,20},labelback=(60000,60000,60000),title="Y:",value=YScaleBarVar,proc=ProcSetScaleB
	SetVariable $SetXScaleCtrl 	pos={380,57},size={110,20},labelback=(60000,60000,60000),title="X:",value=XScaleBarVar,proc=ProcSetScaleB
	Button $ButtonDoneScaleBars			pos={175,80},size={140,20},labelback=(60000,60000,60000),title="Done editing scale bars",proc=ProcDoneScaleBars
	Button $ButtonEraseScaleBars			pos={320,80},size={110,20},labelback=(60000,60000,60000),title="Delete scale bars",proc=ProcEraseScaleBars
		
	// Here must use AxisList to get list of axis
	String AxisListOutput = AxisList("SupEvents" + num2str(ChNumber))
	print AxisListOutput
	Make/N=0/o/T VertAxisList		// Will contain all axis starting with left
	Make/N=0/o/T HorizAxisList		// Will contain all axis starting with right
	Variable VertCounter = 0
	Variable HorizCounter = 0
	Variable CharCounter = 0
	String TempString = ""
	For (i = 0; i < strlen(AxisListOutput)+1; i += 1)
		if (stringmatch(AxisListOutput[i],";") == 0)		// If character is not ";"
			TempString[CharCounter] = AxisListOutput[i]
			CharCounter += 1
		else
			if (stringmatch(TempString[0],"l") == 1 || stringmatch(TempString[0],"r") == 1)
				InsertPoints VertCounter, 1, VertAxisList
				VertAxisList[VertCounter] = TempString
			endif
			if (stringmatch(TempString[0],"b"))
				InsertPoints HorizCounter, 1, HorizAxisList
				HorizAxisList[HorizCounter] = TempString
			endif			
			TempString = ""
		endif
	Endfor
	// The lists of vertical and horizontal axis are in VertAxisList and HorizAxisList
	
	Make/N=(numpnts(VertAxisList))/o VertAxisSelWave = p
	Make/N=(numpnts(HorizAxisList))/o HorizAxisSelWave = p
	
	Make/N=(numpnts(VertAxisList))/o/T LBVertAxis_tw
	LBVertAxis_tw[0] = "Ch" + num2str(3 - ChNumber)
	LBVertAxis_tw[1] = "Ch" + num2str(ChNumber)
	
	String TBVertAxisSupEvents = "TBVertAxisSupEvents" + num2str(ChNumber)
	String LBVertAxisCtrl = "LBVertAxis_" + WindowName
	String TBHorizAxisSupEvents = "TBHorizAxisSupEvents" + num2str(ChNumber)
	String LBHorizAxisCtrl = "LBHorizAxis_" + WindowName
	
	TitleBox $TBVertAxisSupEvents			pos={15,22},size={60,40},title="Vert. axis",frame=0,labelBack=(60000,60000,60000)
	ListBox $LBVertAxisCtrl 				labelBack=(60000,60000,60000), pos={15,44},size={70,40},frame=0,mode=1,editStyle=0,listWave=LBVertAxis_tw,selWave=VertAxisSelWave
	TitleBox $TBHorizAxisSupEvents		pos={90,22},size={60,40},title="Horiz. axis",frame=0,labelBack=(60000,60000,60000)
	ListBox $LBHorizAxisCtrl 				labelBack=(60000,60000,60000), pos={90,44},size={70,40},frame=0,mode=1,editStyle=0,listWave=HorizAxisList,selWave=HorizAxisSelWave

	// END OF SCALE BAR MANAGEMENT		

	SetDataFolder root:spacan

END

FUNCTION ProcButtonFullScale1(ctrlName) : ButtonControl
	String ctrlName
	SetAxis/W=SupEvents1/A
END

FUNCTION ProcButtonFullScale2(ctrlName) : ButtonControl
	String ctrlName
	SetAxis/W=SupEvents2/A
END

FUNCTION ProcButtonSaveSup1(ctrlName) : ButtonControl
	String ctrlName
	SavePICT/EF=1/E=-3
END

FUNCTION ProcButtonSaveSup2(ctrlName) : ButtonControl
	String ctrlName
	SavePICT/EF=1/E=-3
END

Function ProcSupEvents1Grid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ShowCh2
	if (checked == 1)
		if (ShowCh2 == 1)
			ModifyGraph/W=SupEvents1 grid(left1)=1,gridRGB(left1)=(0,0,0)
			ModifyGraph/W=SupEvents1 grid(left2)=1,gridRGB(left2)=(0,0,0)
			ModifyGraph/W=SupEvents1 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		else
			ModifyGraph/W=SupEvents1 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=SupEvents1 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif		
	else
		if (ShowCh2 == 1)
			ModifyGraph/W=SupEvents1 grid(left1)=0
			ModifyGraph/W=SupEvents1 grid(left2)=0
			ModifyGraph/W=SupEvents1 grid(bottom)=0
		else
			ModifyGraph/W=SupEvents1 grid(left)=0
			ModifyGraph/W=SupEvents1 grid(bottom)=0
		endif
	endif	
End

Function ProcSupEvents2Grid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ShowCh1
	if (checked == 1)
		if (ShowCh1 == 1)
			ModifyGraph/W=SupEvents2 grid(left1)=1,gridRGB(left1)=(0,0,0)
			ModifyGraph/W=SupEvents2 grid(left2)=1,gridRGB(left2)=(0,0,0)
			ModifyGraph/W=SupEvents2 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		else
			ModifyGraph/W=SupEvents2 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=SupEvents2 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif		
	else
		if (ShowCh1 == 1)
			ModifyGraph/W=SupEvents2 grid(left1)=0
			ModifyGraph/W=SupEvents2 grid(left2)=0
			ModifyGraph/W=SupEvents2 grid(bottom)=0
		else
			ModifyGraph/W=SupEvents2 grid(left)=0
			ModifyGraph/W=SupEvents2 grid(bottom)=0
		endif
	endif	
End

//Function ProcButtonSupEvents1Sync(ctrlName) : ButtonControl
//	String ctrlName
//	getwindow kwTopWin activeSW
//	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
//		GetAxis/Q/W=SupEvents1#SupEvents11 bottom
//		SetAxis/W=SupEvents1#SupEvents12 bottom V_min,V_max
//	endif
//	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
//		GetAxis/Q/W=SupEvents1#SupEvents12 bottom
//		SetAxis/W=SupEvents1#SupEvents11 bottom V_min,V_max
//	endif
//end

//Function ProcButtonSupEvents2Sync(ctrlName) : ButtonControl
//	String ctrlName
//	getwindow kwTopWin activeSW
//	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
//		GetAxis/Q/W=SupEvents2#SupEvents22 bottom
//		SetAxis/W=SupEvents2#SupEvents21 bottom V_min,V_max
//	endif
//	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
//		GetAxis/Q/W=SupEvents2#SupEvents21 bottom
//		SetAxis/W=SupEvents2#SupEvents22 bottom V_min,V_max
//	endif
//end

Function ProcButtonCloseSupEvents1(ctrlName) : ButtonControl
	String ctrlName
	Nvar WinSupEvents1
	DoWindow/K SupEvents1
	WinSupEvents1 = 0
	KillDataFolder/Z root:spacan:SupEvents1
end

Function ProcButtonCloseSupEvents2(ctrlName) : ButtonControl
	String ctrlName
	Nvar WinSupEvents2
	DoWindow/K SupEvents2
	WinSupEvents2 = 0
	KillDataFolder/Z root:spacan:SupEvents2
end

Function ProcCBShowIndEvents1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckShowIndEvents1, CheckIncludeAv1
	CheckShowIndEvents1 = checked
	if (CheckShowIndEvents1 == 0)
		CheckIncludeAv1 = 1
	endif
	Execute "CheckBox CheckBoxIncludeAv1 value=CheckIncludeAv1"
End

Function ProcCBShowIndEvents2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckShowIndEvents2, CheckIncludeAv2
	CheckShowIndEvents2 = checked
	if (CheckShowIndEvents2 == 0)
		CheckIncludeAv2 = 1
	endif
	Execute "CheckBox CheckBoxIncludeAv2 value=CheckIncludeAv2"
End

Function ProcCBIncludeAv1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIncludeAv1, CheckSupShowSD1, CheckShowIndEvents1
	CheckIncludeAv1 = checked
	if (CheckIncludeAv1 == 0)
		CheckSupShowSD1 = 0
		CheckShowIndEvents1 = 1
		Execute "CheckBox CBIncludeSD1 value=CheckSupShowSD1"
		Execute "CheckBox CBShowIndEvents1	value=CheckShowIndEvents1"
	endif
End

Function ProcCBIncludeAv2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIncludeAv2, CheckSupShowSD2, CheckShowIndEvents2
	CheckIncludeAv2 = checked
	if (CheckIncludeAv2 == 0)
		CheckSupShowSD2 = 0
		CheckShowIndEvents2 = 1
		Execute "CheckBox CBIncludeSD2 value=CheckSupShowSD2"
		Execute "CheckBox CBShowIndEvents2	value=CheckShowIndEvents2"
	endif
End

Function ProcCBNormalize1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckSupNormalize1, CheckShowCurs1
	CheckSupNormalize1 = checked
	if (CheckSupNormalize1 == 0)
		CheckShowCurs1 = 0
		Execute "CheckBox CBShowCurs1 value=CheckShowCurs1"
	endif
End

Function ProcCBNormalize2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckSupNormalize2, CheckShowCurs2
	CheckSupNormalize2 = checked
	if (CheckSupNormalize2 == 0)
		CheckShowCurs2 = 0
		Execute "CheckBox CBShowCurs2 value=CheckShowCurs2"
	endif
End	

// Show SD for average tab
Function ProcCheckShowSD(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	Nvar CheckShowSD = $"CheckShowSD" + num2str(ChNumber)
	CheckShowSD = checked
	Execute "CheckBox ShowSD" + num2str(ChNumber) + " value=CheckShowSD" + num2str(ChNumber)
End

// Show SD for Superimpose events tab
Function ProcSupCheckShowSD1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckSupShowSD1, CheckIncludeAv1
	CheckSupShowSD1 = checked
	if (CheckIncludeAv1 == 0)
		CheckSupShowSD1 = 0
	endif
	Execute "CheckBox CBIncludeSD1 value=CheckSupShowSD1"
End

Function ProcSupCheckShowSD2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckSupShowSD2, CheckIncludeAv2
	CheckSupShowSD2 = checked
	if (CheckIncludeAv2 == 0)
		CheckSupShowSD2 = 0
	endif
	Execute "CheckBox CBIncludeSD2 value=CheckSupShowSD2"
End

Function ProcCBShowCurs1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckShowCurs1, CheckSupNormalize1
	CheckShowCurs1 = checked
	if (CheckSupNormalize1 == 0)
		CheckShowCurs1 = 0
	endif
	Execute "CheckBox CBShowCurs1 value=CheckShowCurs1"
End

Function ProcCBShowCurs2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckShowCurs2, CheckSupNormalize2
	CheckShowCurs2 = checked
	if (CheckSupNormalize2 == 0)
		CheckShowCurs2 = 0
	endif
	Execute "CheckBox CBShowCurs2 value=CheckShowCurs2"
End

Function ProcCBShowRefLine1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckShowRefLine1
	CheckShowRefLine1 = checked
End

Function ProcCBShowRefLine2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckShowRefLine2
	CheckShowRefLine2 = checked
End

Function ProcCBUseRndColors1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckUseRndColors1
	CheckUseRndColors1 = checked
End

Function ProcCBUseRndColors2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckUseRndColors2
	CheckUseRndColors2 = checked
End

Function ProcCBRemoveAxes1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckRemoveAxes1
	CheckRemoveAxes1 = checked
End

Function ProcCBRemoveAxes2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckRemoveAxes2
	CheckRemoveAxes2 = checked
End

Function ProcCBOverlayMode1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckOverlayMode1
	CheckOverlayMode1 = checked
End		

Function ProcCBOverlayMode2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckOverlayMode2
	CheckOverlayMode2 = checked
End	

// FUNCTION THAT CALCULATES THE AVERAGE EVENT (Channel 1), CALLED BY THE BUTTON "AVERAGE"
Function ProcButtonAverage(ctrlName) : ButtonControl
	String ctrlName
	
	// Get ChNumber from the name of the window
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	// Global variables
	Nvar CheckRT2080 = $"CheckRT2080" + num2str(ChNumber)
	Nvar CheckBslnSD = $"CheckBslnSD" + num2str(ChNumber)
	Nvar RTinf = $"RTinf" + num2str(ChNumber)
	Nvar RTSup = $"RTSup" + num2str(ChNumber)
	Nvar BslnSDinf = $"BslnSDinf" + num2str(ChNumber)
	Nvar BslnSDsup = $"BslnSDsup" + num2str(ChNumber)
	Nvar ShowOtherCh = $"ShowCh" + num2str(3 - ChNumber)
	Nvar Baseline = $"Baseline" + num2str(ChNumber)
	Nvar Decay = $"Decay" + num2str(ChNumber)
	Nvar SIa = $"SI" + num2str(ChNumber)
	Nvar SIb = $"SI" + num2str(3 - ChNumber)
	Nvar ACFTa = $"ACFT" + num2str(ChNumber)
	Nvar ACFTaChb = $"ACFT" + num2str(ChNumber) + "Ch" + num2str(3 - ChNumber)
	Nvar CheckPosDeflection = $"CheckPosDeflection" + num2str(ChNumber)
	Nvar CheckShowSD = $"CheckShowSD" + num2str(ChNumber)
	Nvar CheckAlignAvOcc = $"CheckAlignAv" + num2str(ChNumber) + "Occ"
	Nvar CheckAlignAvFoot = $"CheckAlignAv" + num2str(ChNumber) + "Foot"
	Nvar CheckAlignAvPeak = $"CheckAlignAv" + num2str(ChNumber) + "Peak"
	Nvar NumberOfAveragedEvents = $"NumberOfAveragedEvents" + num2str(ChNumber)
	Nvar CheckGridOn = $"CheckGridOn" + num2str(ChNumber)
	// Global strings
	Svar FileNameA = $"Filename" + num2str(ChNumber)
	Svar FileNameB = $"Filename" + num2str(3 - ChNumber)
	// Waves
	Wave Tags = $"Tags" + num2str(ChNumber)
	Wave RiseTime2080 = $"RiseTime2080" + num2str(ChNumber)
	Wave DetectedLevels = $"DetectedLevels" + num2str(ChNumber)
	Wave Occurrence = $"Occurrence" + num2str(ChNumber)
	Wave EventFootTime = $"EventFootTime" + num2str(ChNumber)
	Wave RealPeakPosition = $"RealPeakPosition" + num2str(ChNumber)
	Wave BaselineY = $"BaselineY" + num2str(ChNumber)
	
	String DetectedEvents = "DetectedEvents" + num2str(ChNumber)
	String AverageA = "Average" + num2str(ChNumber)
	String AverageAB = "Average" + num2str(ChNumber) + num2str(3 - ChNumber)
	
	Variable i, j
	if (sum(Tags,0,numpnts(Tags)-1) == 0)
		Abort "No tagged events. The average is calculated from tagged events only."
	endif
	
	Duplicate/o Tags Tags_modified
	// Here is a trick to overcome a problem of FindLevels (dealing with detecting the last 1 in a wave).
	InsertPoints numpnts(Tags_modified), 1, Tags_modified
	FindLevels/Q Tags_modified 1; Wave W_FindLevels

	For (i = numpnts(W_FindLevels)-1 ; i > -1 ; i-=1)
		if (Numtype(Occurrence[W_FindLevels[i]]) == 2)		// Removing bad events (NaN)
			Tags_modified[W_FindLevels[i]] = 0
		endif
		if (Occurrence[W_FindLevels[i]] < Baseline*0.001 || Occurrence[W_FindLevels[i]] > pnt2x($FilenameA,numpnts($FilenameA)-1) - Decay*0.001)
			Tags_modified[W_FindLevels[i]] = 0
		endif
		if (CheckRT2080)
			if (RiseTime2080[W_FindLevels[i]] < RTinf*.001 || RiseTime2080[W_FindLevels[i]] > RTsup*.001)
				Tags_modified[W_FindLevels[i]] = 0		// Filter for rise time
			endif
		endif
		if (CheckBslnSD)
			WaveStats/Q/R=(DetectedLevels[W_FindLevels[i]] - Baseline*0.001, DetectedLevels[W_FindLevels[i]]) $FileNameA
			if (V_sdev < BslnSDinf || V_sdev > BslnSDsup)		// Filter for baseline
				Tags_modified[W_FindLevels[i]] = 0
			endif
		endif
	Endfor
	FindLevels/Q Tags_modified 1
	if (V_flag == 2)
		Abort "No event compatible with filter(s) range(s)."
	endif
	
	NumberOfAveragedEvents = sum(Tags_modified,0,numpnts(Tags_modified))
	
	// --- MODIFICATION OF FEB 01, 2010 --- GUILLAUME
	// New method for average, in order to enable easy calculation of the SD.
	// We will use an array to have quick assess to every point of every wave.
	
	// First get size of event (segment size)
	// ...Channel A
	if (CheckAlignAvOcc)	// means align on half-rise
		Duplicate/o/R=(Occurrence[W_FindLevels[0]] - Baseline*.001, Occurrence[W_FindLevels[0]] + Decay*.001) $FileNameA TEMPWAVEa
	endif
	if (CheckAlignAvFoot)	// means align on foot
		Duplicate/o/R=(EventFootTime[W_FindLevels[0]] - Baseline*.001, EventFootTime[W_FindLevels[0]] + Decay*.001) $FileNameA TEMPWAVEa
	endif
	if (CheckAlignAvPeak)	// means align on peak time
		Duplicate/o/R=(RealPeakPosition[W_FindLevels[0]] - Baseline*.001, RealPeakPosition[W_FindLevels[0]] + Decay*.001) $FileNameA TEMPWAVEa
	endif
	Variable	segmentsizeA = numpnts(TEMPWAVEa)
	if (ShowOtherCh)
		// ...Channel B
		if (CheckAlignAvOcc)	// means align on half-rise
			Duplicate/o/R=(Occurrence[W_FindLevels[0]] - Baseline*.001, Occurrence[W_FindLevels[0]] + Decay*.001) $FileNameB TEMPWAVEab
		endif
		if (CheckAlignAvFoot)	// means align on foot
			Duplicate/o/R=(EventFootTime[W_FindLevels[0]] - Baseline*.001, EventFootTime[W_FindLevels[0]] + Decay*.001) $FileNameB TEMPWAVEab
		endif
		if (CheckAlignAvPeak)	// means align on peak time
			Duplicate/o/R=(RealPeakPosition[W_FindLevels[0]] - Baseline*.001, RealPeakPosition[W_FindLevels[0]] + Decay*.001) $FileNameB TEMPWAVEab
		endif
		Variable	segmentsizeAB = numpnts(TEMPWAVEab)	
	endif
	// Then calculate the average
	Make/o/N=(NumberOfAveragedEvents,segmentsizeA) arrayA
	if (ShowOtherCh)
		Make/o/N=(NumberOfAveragedEvents,segmentsizeAB) arrayAB
	endif
	For (i = 0 ; i < Numpnts(W_FindLevels) ; i+=1)
		// Filling up the array with individual detected events
		if (CheckAlignAvOcc)	// means align on half-rise
			Duplicate/o/R=(Occurrence[W_FindLevels[i]] - Baseline*.001, Occurrence[W_FindLevels[i]] + Decay*.001) $FileNameA TEMPWAVEa
		endif
		if (CheckAlignAvFoot)	// means align on foot
			Duplicate/o/R=(EventFootTime[W_FindLevels[i]] - Baseline*.001, EventFootTime[W_FindLevels[i]] + Decay*.001) $FileNameA TEMPWAVEa
		endif
		if (CheckAlignAvPeak == 1)	// means align on peak time
			Duplicate/o/R=(RealPeakPosition[W_FindLevels[i]] - Baseline*.001, RealPeakPosition[W_FindLevels[i]] + Decay*.001) $FileNameA TEMPWAVEa
		endif
		TEMPWAVEa = TEMPWAVEa - BaselineY[W_FindLevels[i]]
		For (j = 0; j < segmentsizeA+1; j += 1)
			arrayA[i][j] = TEMPWAVEa[j]
		Endfor
		if (ShowOtherCh)
			if (CheckAlignAvOcc)	// means align on half-rise
				Duplicate/o/R=(Occurrence[W_FindLevels[i]] - Baseline*.001, Occurrence[W_FindLevels[i]] + Decay*.001) $FileNameB TEMPWAVEab
			endif
			if (CheckAlignAvFoot)	// means align on foot
				Duplicate/o/R=(EventFootTime[W_FindLevels[i]] - Baseline*.001, EventFootTime[W_FindLevels[i]] + Decay*.001) $FileNameB TEMPWAVEab
			endif
			if (CheckAlignAvPeak)	// means align on peak time
				Duplicate/o/R=(RealPeakPosition[W_FindLevels[i]] - Baseline*.001, RealPeakPosition[W_FindLevels[i]] + Decay*.001) $FileNameB TEMPWAVEab
			endif
			For (j = 0; j < segmentsizeAB+1; j += 1)
				arrayAB[i][j] = TEMPWAVEab[j]
			Endfor
		endif
	endfor
	Make/N=(segmentsizeA)/o TMP_AVERAGEa	// for average
	Make/N=(segmentsizeA)/o TMP_SDa		// for standard deviation
	if (ShowOtherCh)
		Make/N=(segmentsizeAB)/o TMP_AVERAGEab	// for average
		Make/N=(segmentsizeAB)/o TMP_SDab		// for standard deviation
	endif
	For (i = 0 ;  i <segmentsizeA+1 ; i+=1)
		Make/o/N=(NumberOfAveragedEvents) TMP_PointAverageA
		TMP_PointAverageA = arrayA[p][i]; Wavestats/Q TMP_PointAverageA
		TMP_AVERAGEa[i] = V_avg
		TMP_SDa[i] = V_sdev
	Endfor
	if (ShowOtherCh)
		For (i = 0 ;  i <segmentsizeAB+1 ; i+=1)
			Make/o/N=(NumberOfAveragedEvents) TMP_PointAverageAB
			TMP_PointAverageAB = arrayAB[p][i]; Wavestats/Q TMP_PointAverageAB
			TMP_AVERAGEab[i] = V_avg
			TMP_SDab[i] = V_sdev
		Endfor
	endif
	Duplicate/o TMP_AVERAGEa, AverageEventA; KillWaves TMP_AVERAGEa
	SetScale/P x -Baseline*.001, SIa*1e-6, "s", AverageEventA
	Duplicate/o TMP_SDa, SDEventA; KillWaves TMP_SDa
	SetScale/P x -Baseline*.001, SIa*1e-6, "s", SDEventA
	if (ShowOtherCh)
		Duplicate/o TMP_AVERAGEab, AverageEventAchB; KillWaves TMP_AVERAGEab
		SetScale/P x -Baseline*.001, SIb*1e-6, "s", AverageEventAchB
		Duplicate/o TMP_SDab, SDAchB; KillWaves TMP_SDab
		SetScale/P x -Baseline*.001, SIb*1e-6, "s", SDAchB
		//Charly **************************************************** Here I substract baseline from AverageEvent1ch2
		Variable substracter = mean(AverageEventAchB, -(7 * 0.001), -(4 * 0.001))
		AverageEventAchB -= substracter			
		//Charly End of changes***********************************************
	endif
	// --- END OF MODIFICATION OF FEB 01, 2010 --- GUILLAUME

	if (ACFTa)	// if $DetectedEvents#$Average exists
		KillWindow $DetectedEvents#$AverageA
	else
		ACFTa = 1
	endif
	
	Duplicate/o AverageEventA $"AverageEvent" + num2str(ChNumber); KillWaves AverageEventA
	Wave AverageEventA = $"AverageEvent" + num2str(ChNumber)
	Display/HOST=$DetectedEvents/N=$AverageA/W=(0.71,0,0.99,0.49) AverageEventA
	SetScale/P x -Baseline*.001,SIa*1e-6,"s", AverageEventA
	ModifyGraph/W=$DetectedEvents#$AverageA margin(left)=40
	ModifyGraph/W=$DetectedEvents#$AverageA margin(top)=1
	ModifyGraph/W=$DetectedEvents#$AverageA margin(right)=1
	ModifyGraph/W=$DetectedEvents#$AverageA margin(bottom)=28
	ModifyGraph/W=$DetectedEvents#$AverageA axOffset(left)=-2
	ModifyGraph/W=$DetectedEvents#$AverageA axOffset(bottom)=0
	if (ChNumber == 1)
		ModifyGraph/W=$DetectedEvents#$AverageA rgb($"AverageEvent" + num2str(ChNumber))=(0,0,65535)
	endif
	if (ChNumber == 2)
		ModifyGraph/W=$DetectedEvents#$AverageA rgb($"AverageEvent" + num2str(ChNumber))=(65535,0,0)
	endif
	ModifyGraph/W=$DetectedEvents#$AverageA StandOff = 0
	Wavestats/Q AverageEventA
	if (CheckPosDeflection)
		SetAxis/W=$DetectedEvents#$AverageA left -0.4*V_max, 1.2*V_max
	else	
		SetAxis/W=$DetectedEvents#$AverageA left 1.2*V_min, -.4*V_min
	endif
	
	if (CheckGridOn)
		ModifyGraph/W=$DetectedEvents#$AverageA grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=$DetectedEvents#$AverageA grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	endif

	// Here add standard deviation
	if (CheckShowSD)
		Duplicate/o SDEventA, SDEventA_top; SDEventA_top = AverageEventA + SDEventA
		reverse SDEventA_top
		Duplicate/o SDEventA_top, SDEventA_top_x
		SDEventA_top_x = p*SIa*1e-6 - Baseline*.001
		reverse SDEventA_top_x
		Duplicate/o SDEventA, SDEventA_bottom; SDEventA_bottom = AverageEventA - SDEventA
		Duplicate/o SDEventA_bottom, SDEventA_bottom_x
		SDEventA_bottom_x = p*SIa*1e-6 - Baseline*.001
		concatenate/NP/o {SDEventA_bottom, SDEventA_top}, SDEventA_box
		concatenate/NP/o {SDEventA_bottom_x, SDEventA_top_x}, SDEventA_box_x
		Duplicate/o SDEventA_box $"SDEvent" + num2str(ChNumber) + "_box"; KillWaves SDEventA_box
		Duplicate/o SDEventA_box_x $"SDEvent" + num2str(ChNumber) + "_box_x"; KillWaves SDEventA_box_x
		Appendtograph/W=$DetectedEvents#$AverageA $"SDEvent" + num2str(ChNumber) + "_box" vs $"SDEvent" + num2str(ChNumber) + "_box_x"
		ModifyGraph rgb($"SDEvent" + num2str(ChNumber) + "_box")=(34816,34816,34816)
	endif

	SetDrawEnv ycoord = left, linefgc = (47872,47872,47872), dash = 1
	DrawLine 0, 0, 1, 0
	ShowInfo
	
	if (ShowOtherCh)
		if (ACFTaChb)	// if DetectedEvents1#Average12 exists
			KillWindow $DetectedEvents#$AverageAB
		else
			ACFTaChb = 1
		endif
		
		Duplicate/o AverageEventAchB $"AverageEvent" + num2str(ChNumber) + "ch" + num2str(3 - ChNumber)
		KillWaves AverageEventAchB
		Wave AverageEventAchB = $"AverageEvent" + num2str(ChNumber) + "ch" + num2str(3 - ChNumber)
		Display/HOST=$DetectedEvents/N=$AverageAB/W=(0.71,0.51,0.99,0.99) AverageEventAchB
		SetScale/P x -Baseline*.001,SIb*1e-6,"s", AverageEventAchB
		ModifyGraph/W=$DetectedEvents#$AverageAB margin(left)=40
		ModifyGraph/W=$DetectedEvents#$AverageAB margin(top)=1
		ModifyGraph/W=$DetectedEvents#$AverageAB margin(right)=1
		ModifyGraph/W=$DetectedEvents#$AverageAB margin(bottom)=28
		ModifyGraph/W=$DetectedEvents#$AverageAB axOffset(left)=-2
		ModifyGraph/W=$DetectedEvents#$AverageAB axOffset(bottom)=0
		if (ChNumber == 1)
			ModifyGraph/W=$DetectedEvents#$AverageAB rgb($"AverageEvent" + num2str(ChNumber) + "ch" + num2str(3 - ChNumber))=(65535,0,0)
		endif
		if (ChNumber == 2)
			ModifyGraph/W=$DetectedEvents#$AverageAB rgb($"AverageEvent" + num2str(ChNumber) + "ch" + num2str(3 - ChNumber))=(0,0,65535)
		endif
		ModifyGraph/W=$DetectedEvents#$AverageAB StandOff = 0
		Wavestats/Q AverageEventAchB
		SetAxis/W=$DetectedEvents#$AverageAB left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2
		
		if (CheckGridOn)
			ModifyGraph/W=$DetectedEvents#$AverageAB grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=$DetectedEvents#$AverageAB grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
		
		if (CheckShowSD)
			Duplicate/o SDAchB, SDEventAB_top; SDEventAB_top = AverageEventAchB + SDAchB
			reverse SDEventAB_top
			Duplicate/o SDEventAB_top, SDEventAB_top_x
			SDEventAB_top_x = p*SIb*1e-6 - Baseline*.001
			reverse SDEventAB_top_x
			Duplicate/o SDAchB, SDEventAB_bottom; SDEventAB_bottom = AverageEventAchB - SDAchB
			Duplicate/o SDEventAB_bottom, SDEventAB_bottom_x
			SDEventAB_bottom_x = p*SIb*1e-6 - Baseline*.001
			concatenate/NP/o {SDEventAB_bottom, SDEventAB_top}, SDEventAB_box
			concatenate/NP/o {SDEventAB_bottom_x, SDEventAB_top_x}, SDEventAB_box_x
			Duplicate/o SDEventAB_box $"SDEvent" + num2str(ChNumber) + num2str(3 - ChNumber) + "_box"
			KillWaves SDEventAB_box
			Duplicate/o SDEventAB_box_x $"SDEvent" + num2str(ChNumber) + num2str(3 - ChNumber) + "_box_x"
			KillWaves SDEventAB_box_x
			Appendtograph/W=$DetectedEvents#$AverageAB $"SDEvent" + num2str(ChNumber) + num2str(3 - ChNumber) + "_box" vs $"SDEvent" + num2str(ChNumber) + num2str(3 - ChNumber) + "_box_x"
			ModifyGraph/W=$DetectedEvents#$AverageAB rgb($"SDEvent" + num2str(ChNumber) + num2str(3 - ChNumber) + "_box")=(34816,34816,34816)
		endif
		
	endif
	Execute "Valdisplay NumEventsAveraged" + num2str(ChNumber) + " value=" + num2str(NumberOfAveragedEvents)
	String EndFit = "End_Fit" + num2str(ChNumber)
	SetVariable $EndFit limits={0, Decay, 1}
End

Function WaitForKey(ChNumber)
	Variable ChNumber
		// Le boutton "Keyboard" permet de prendre le contrle de la fentre "SingleEvents"  l'aide du clavier
		// Le corps de la fonction WaitForKey est constitu d'une boucle Do... While(1) 
		// Pour sortir de la boucle, taper "C"
	
	// Global variables
	Nvar EventNumber = $"EventNumber" + num2str(ChNumber)
	Nvar Baselinems = $"Baselinems" + num2str(ChNumber)
	Nvar Peakms = $"Peakms" + num2str(ChNumber)
	Nvar KbdSpeed = $"KbdSpeed" + num2str(ChNumber)
	Nvar BTDE = $"BTDE_" + num2str(ChNumber)
	Nvar BUDE = $"BUDE_" + num2str(ChNumber)
	Nvar NumberofEventsch = $"NumberofEventsch" + num2str(ChNumber)
	Nvar PreviousEvent = $"PreviousEvent" + num2str(ChNumber)
	Nvar CheckedTagAll = $"CheckedTagAll" + num2str(ChNumber)
	Nvar TaggedEvents = $"TaggedEvents" + num2str(ChNumber)
	// Waves
	Wave DetectedLevels = $"DetectedLevels" + num2str(ChNumber)
	Wave Tags = $"Tags" + num2str(ChNumber)
	Wave Occurrence = $"Occurrence" + num2str(ChNumber)
	Wave BaselineY = $"BaselineY" + num2str(ChNumber)
	Wave BaselineX = $"BaselineX" + num2str(ChNumber)
	Wave PeakY = $"PeakY" + num2str(ChNumber)
	Wave PeakX = $"PeakX" + num2str(ChNumber)
	Wave RealAmp = $"RealAmp" + num2str(ChNumber)
	Wave RealPeakPosition = $"RealPeakPosition" + num2str(ChNumber)
	Wave RiseTime2080 = $"RiseTime2080" + num2str(ChNumber)
	Wave RT20 = $"RT20" + num2str(ChNumber)
	Wave RT80 = $"RT80" + num2str(ChNumber)
	Wave CursorTags_X = $"CursorTags" + num2str(ChNumber) + "_X"
	Wave CursorTags_Y = $"CursorTags" + num2str(ChNumber) + "_Y"
	
	String CheckBoxTag = "CheckBoxTag" + num2str(ChNumber)
	String CheckBoxBTDE = "BTDE" + num2str(ChNumber)
	String CheckBoxTagAll = "CheckBoxTagAll" + num2str(ChNumber)
	String CheckBoxBUDE = "BUDE" + num2str(ChNumber)
	
	String state
	Variable i

	Do
		state = KeyboardState("")

		If (char2num(state[9])==67)	// Stopper la function  l'aide de la touche "C"
			Break
		Endif

		If (str2num(state[5])==1)		// Afficher l'vnement prcdent
			if (EventNumber>0)
				if (BTDE)
					if (Tags[EventNumber-1] == 0)
						FindLevel/P/Q/R=[EventNumber-1,0] Tags, 1	// Find the previous tagged event
						if (V_flag == 0)
							EventNumber = V_LevelX
							PreviousEvent = EventNumber
						endif
					else
						EventNumber -= 1
					endif
				endif
				if (BUDE)
					if (Tags[EventNumber-1] == 1)
						FindLevel/P/Q/R=[EventNumber-1,0] Tags, 0	// Find the previous untagged event
						if (V_flag == 0)
							EventNumber = V_LevelX
							PreviousEvent = EventNumber
						endif
					else
						EventNumber -= 1
					endif
				endif
				if (BTDE == 0 && BUDE == 0)
					EventNumber -= 1
					PreviousEvent = EventNumber
				endif
				SHOW(ChNumber,2,EventNumber)
				DoUpdate
				i=0
				Do
					i+=1
				While (i<400000/(1+exp(KbdSpeed/3)))
			endif
		Endif
		
		If (str2num(state[6])==1)		// Afficher l'vnement suivant
			if (EventNumber<(numpnts(DetectedLevels)-1))
				if (BTDE)
					if (Tags[EventNumber+1] == 0)
						FindLevel/P/Q/R=[EventNumber+1,NumberofEventsch-1] Tags, 1	// Find the next tagged event
						if (V_flag == 0)
							EventNumber = V_LevelX
							PreviousEvent = EventNumber
						endif
					else
						EventNumber += 1
					endif
				endif
				if (BUDE)
					if (Tags[EventNumber+1] == 1)
						FindLevel/P/Q/R=[EventNumber+1,NumberofEventsch-1] Tags, 0	// Find the next untagged event
						if (V_flag == 0)
							EventNumber = V_LevelX
							PreviousEvent = EventNumber
						endif
					else
						EventNumber += 1
					endif
				endif
				if (BTDE == 0 && BUDE == 0)
					EventNumber += 1
					PreviousEvent = EventNumber
				endif
				SHOW(ChNumber,2,EventNumber)
				DoUpdate
				i=0
				Do
					i+=1
				While (i<400000/(1+exp(KbdSpeed/3)))
			endif
		Endif
		
		If (str2num(state[7])==1)		// Bascule Checkbox "Tag"
			Tags[EventNumber]=1-Tags[EventNumber]
			CheckBox $CheckBoxTag value=(Tags[EventNumber])
			TaggedEvents = sum(Tags,0,numpnts(Tags)-1)
			CursorTags_Y = Tags[EventNumber]
			Execute "ValDisplay DisplayTaggedEvents" + num2str(ChNumber) + " value=" + num2str(TaggedEvents)
			If ( (sum(Tags,0,numpnts(Tags)-1))==0)
				BTDE = 0; Checkbox $CheckBoxBTDE value = 0
			endif
			if (CheckedTagAll == 1)
				If ( (sum(Tags,0,numpnts(Tags)-1)) < NumberofEventsch)
					CheckedTagAll = 0; CheckBox $CheckBoxTagAll value=0
				endif
			endif
			If ( (sum(Tags,0,numpnts(Tags)-1))==NumberofEventsch)
				BUDE = 0; Checkbox $CheckBoxBUDE value = 0
				CheckedTagAll = 1; CheckBox $CheckBoxTagAll value=1
			endif		
			DoUpdate
			Do
			 	state = KeyboardState("")
			While (str2num(state[7])==1)
		Endif
		
	While(1)
End

// CHECKBOX TO SELECT TIME OF OCCURRENCE AS THE HALF-RISE (Channel 2)
Function ProcToOHR2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ToOHR_2, ToOPeak_2, EventNumber2
	ToOHR_2 = 1-ToOHR_2
	ToOPeak_2 = 1-ToOPeak_2
	CheckBox ToOPeak2 value=ToOPeak_2
	if (ToOHR_2 == 1)
		Execute "ValDisplay DisplayOccurrence2 value=root:spacan:Occurrence2[" + num2str(EventNumber2) + "]"
	else
		Execute "ValDisplay DisplayOccurrence2 value=root:spacan:realpeakposition2[" + num2str(EventNumber2) + "]"
	endif
End

// CHECKBOX TO SELECT TIME OF OCCURRENCE AS THE TIME OF THE PEAK (Channel 2)
Function ProcToOPeak2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ToOHR_2, ToOPeak_2, EventNumber2
	ToOHR_2 = 1-ToOHR_2
	ToOPeak_2 = 1-ToOPeak_2
	CheckBox ToOHR2 value=ToOHR_2
	if (ToOHR_2 == 1)
		Execute "ValDisplay DisplayOccurrence2 value=root:spacan:Occurrence2[" + num2str(EventNumber2) + "]"
	else
		Execute "ValDisplay DisplayOccurrence2 value=root:spacan:realpeakposition2[" + num2str(EventNumber2) + "]"
	endif
End

// CHECKBOX TO CHOSE TO BROWSE THROUGH TAGGED EVENTS ONLY
Function ProcBTDE(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window (DetectedEvents#)
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	// Global variables
	Nvar BTDE = $"BTDE_" + num2str(ChNumber)
	Nvar BUDE = $"BUDE_" + num2str(ChNumber)
	// Waves
	Wave Tags = $"Tags" + num2str(ChNumber)

	String CheckBoxBTDE = "BTDE" + num2str(ChNumber)
	String CheckBoxBUDE = "BUDE" + num2str(ChNumber)

	If ( (sum(Tags,0,numpnts(Tags)-1))==0)
		BTDE = 0
		Checkbox $CheckBoxBTDE value = 0
		Abort "No tagged event."
	else
		BTDE = 1 - BTDE
		BUDE = 0
		Checkbox $CheckBoxBUDE value = 0
	endif
End

// CHECKBOX TO CHOSE TO BROWSE THROUGH UNTAGGED EVENTS ONLY (Channel 1)
Function ProcBUDE(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window (DetectedEvents#)
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	// Global variables
	Nvar BTDE = $"BTDE_" + num2str(ChNumber)
	Nvar BUDE = $"BUDE_" + num2str(ChNumber)
	Nvar NumberofEvents = $"NumberofEventsch" + num2str(ChNumber)
	// Waves
	Wave Tags = $"Tags" + num2str(ChNumber)

	String CheckBoxBTDE = "BTDE" + num2str(ChNumber)
	String CheckBoxBUDE = "BUDE" + num2str(ChNumber)

	If ( (sum(Tags,0,numpnts(Tags)-1))==NumberofEvents)
		BUDE = 0
		Checkbox $CheckBoxBUDE value = 0
		Abort "No untagged event."
	else
		BUDE = 1 - BUDE
		BTDE = 0
		Checkbox $CheckBoxBTDE value = 0
	endif
End

// SHOW CHANNEL (3 - ChNumber) WHEN BROWSING THROUGH DETECTED EVENTS IN CHANNEL ChNumber
Function ShowOtherChannel(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	// Get ChNumber from the name of the window (DetectedEvents#)
	Variable ChNumber
	getwindow kwTopWin activeSW
	ChNumber = str2num(S_Value[14])
	
	// Global variables
	Nvar SIb = $"SI" + num2str(3 - ChNumber)
	Nvar EventNumber = $"EventNumber" + num2str(ChNumber)
	Nvar ACFTa = $"ACFT" + num2str(ChNumber)
	Nvar ACFTaChb = $"ACFT" + num2str(ChNumber) + "ch" + num2str(3 - ChNumber)
	Nvar ShowOtherCh = $"ShowCh" + num2str(3 - ChNumber)
	Nvar CE = $"C" + num2str(3 - ChNumber) + "E"	// CE defined in function "Initialize". C#E = 1 if channel # exists
	Nvar msBefore = $"msBefore" + num2str(ChNumber)
	Nvar msAfter = $"msAfter" + num2str(ChNumber)
	Nvar CheckGridOn = $"CheckGridOn" + num2str(ChNumber)
	// Global strings
	Svar FilenameB = $"Filename" + num2str(3 - ChNumber)
	// Waves
	Wave Occurrence = $"Occurrence" + num2str(ChNumber)
	Wave DetectedLevels = $"DetectedLevels" + num2str(ChNumber)

	String/G $"WindowShowCh" + num2str(3 - ChNumber)
	Svar WindowShowOtherCh = $"WindowShowCh" + num2str(3 - ChNumber)
	WindowShowOtherCh = "Channel " + num2str(3 - ChNumber)

	String CheckShowOtherCh = "CheckShowChannel" + num2str(3 - ChNumber)
	String DetectedEventsA = "DetectedEvents" + num2str(ChNumber)
	String EventChannelAB = "EventChannel" + num2str(ChNumber) + num2str(3 - ChNumber)
	String AverageAB = "Average" + num2str(ChNumber) + num2str(3 - ChNumber)

	if (CE == 0)
		CheckBox $CheckShowOtherCh value=0
		Abort "No wave loaded in channel " + num2str(3 - ChNumber) + " !"
	endif

	if (checked)		// THE CHECKBOX IS ACTIVATED -> display channel (3 - ChNumber)
		Duplicate/o/R=( Occurrence[EventNumber] - msBefore*.001 , Occurrence[EventNumber] + msAfter*.001 ) $FileNameB TEMPEVENT
		SetScale/P x -msBefore*.001, (SIb*1e-6), "s", TEMPEVENT
		Duplicate/o TEMPEVENT $"TEMPEVENT" + num2str(ChNumber) + "_CH" + num2str(3 - ChNumber)
		Wave TEMPEVENTchB = $"TEMPEVENT" + num2str(ChNumber) + "_CH" + num2str(3 - ChNumber)
		Display/HOST=$DetectedEventsA/N=$EventChannelAB/W=(0.4,0.51,0.69,0.99) TEMPEVENTchB as WindowShowOtherCh
		ModifyGraph/W=$DetectedEventsA#$EventChannelAB margin(left)=40
		ModifyGraph/W=$DetectedEventsA#$EventChannelAB margin(top)=1
		ModifyGraph/W=$DetectedEventsA#$EventChannelAB margin(right)=1
		ModifyGraph/W=$DetectedEventsA#$EventChannelAB margin(bottom)=28
		ModifyGraph/W=$DetectedEventsA#$EventChannelAB StandOff = 0
		ModifyGraph/W=$DetectedEventsA#$EventChannelAB axOffset(left)=-2
		ModifyGraph/W=$DetectedEventsA#$EventChannelAB axOffset(bottom)=0
		if (ChNumber == 1)
			ModifyGraph/W=$DetectedEventsA#$EventChannelAB rgb($"TEMPEVENT" + num2str(ChNumber) + "_CH" + num2str(3 - ChNumber))=(65535,0,0)
		endif
		if (ChNumber == 2)
			ModifyGraph/W=$DetectedEventsA#$EventChannelAB rgb($"TEMPEVENT" + num2str(ChNumber) + "_CH" + num2str(3 - ChNumber))=(0,0,65535)
		endif
		Wavestats/Q TEMPEVENTchB
		SetAxis/W=$DetectedEventsA#$EventChannelAB left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2
		
		if (CheckGridOn)
			ModifyGraph/W=$DetectedEventsA#$EventChannelAB grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=$DetectedEventsA#$EventChannelAB grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
		
	else
		KillWindow $DetectedEventsA#$EventChannelAB
		if (ACFTaChb == 1)
			KillWindow $DetectedEventsA#$AverageAB
			ACFTaChb = 0
		endif
	endif
	ShowOtherCh = 1 - ShowOtherCh
End

// =================== BELOW: A SERIES OF FIT FUNCTIONS ===================

// THE FIT FUNCTION TO FIT POSTSYNAPTIC CURRENTS: A SIGMOID TO FIT THE RISE AND A DOUBLE EXPONENTIAL TO FIT THE DECAY
Function Sigmoid_Dbexp(w,x) : FitFunc
	Wave w; Variable x
	return ( w[0] / ( 1 + exp( (w[1] - x) / w[2] )) ) * ( w[3] + w[4]*exp(-w[5]*x) + w[6]*exp(-w[7]*x) )
	// Sigmoid - w[0] : max
	// Sigmoid - w[1] : xhalf
	// Sigmoid - w[2] : rate
	// Dbexp - w[3] : offset
	// Dbexp - w[4] : amplitude exp1
	// Dbexp - w[5] : invtau1
	// Dbexp - w[6] : amplitude exp2
	// Dbexp - w[7] : invtau2
End

// A TRIPLE EXPONENTIAL FUNCTION.
// CAN BE REMOVED FROM SPACAN.
Function Trplexp(w,x) : FitFunc
	Wave w; Variable x
	return w[0] + w[1]*exp(-x/w[2]) + w[3]*exp(-x/w[4]) + w[5]*exp(-x/w[6])
End

// A QUADRUPLE EXPONENTIAL FUNCTION.
// CAN BE REMOVED FROM SPACAN.
Function Qdrplexp(w,x) : FitFunc
	Wave w; Variable x
	return w[0] + w[1]*exp(-x/w[2]) + w[3]*exp(-x/w[4]) + w[5]*exp(-x/w[6]) + w[7]*exp(-x/w[8])
End

// ANOTHER CUSTOM-MADE FIT FUNCTION
// CAN BE REMOVED FROM SPACAN.
Function Loglin(w,x) : FitFunc
	Wave w; Variable x
	return log(w[0] * exp(w[1]*x))
End

// ANOTHER CUSTOM-MADE FIT FUNCTION: THE SUM OF TWO GAUSSIAN CURVES.
// CAN BE REMOVED FROM SPACAN.
Function GaussSum(w,x) : FitFunc
	Wave w; Variable x
	return w[0] + w[1] * exp( - (x - w[2])*(x - w[2]) / (w[3]*w[3])) + w[4] * exp( - (x - w[5])*(x - w[5]) / (w[6]*w[6]))
End

// A FUNCTION USED TO FIT THE CENTRAL PEAK OF SPIKE CROSS-CORRELOGRAMS IN GOLGI CELLS PAIRED RECORDINGS
// CAN BE REMOVED FROM SPACAN.
Function Fit_CC(wave_to_fit)
	Wave wave_to_fit
	Nvar CorrBin
	Make/D/N=7/O W_coef
	Variable Var1, Var2, Var3, Var4, Var5, Var6, Var7
	Var1 = -1; Var2 = 12; Var3 = -0.0014; Var4 = 0.0009; Var5 = 12; Var6 = 0.0011; Var7 = 0.0009
	Prompt Var1, "y0 ?"
	Prompt Var2, "A1 ?"
	Prompt Var3, "x1 ?"
	Prompt Var4, "width1 ?"
	Prompt Var5, "A2 ?"
	Prompt Var6, "x2 ?"
	Prompt Var7, "width2 ?"
	Doprompt "fit parameters", Var1, Var2, Var3, Var4, Var5, Var6, Var7
	If (V_flag == 1)
		Abort
	endif
	W_coef[0] = {Var1,Var2,Var3,Var4,Var5,Var6,Var7}
	FuncFit GaussSum W_coef  Corr /D
	Duplicate/o W_coef dummy
	Make/D/N=7/O W_coef
	W_coef[0] = {dummy[0],dummy[1],dummy[2],dummy[3],dummy[4],dummy[5],dummy[6]}
	Wave Fit_Corr
	//Removefromgraph fit_Corr
	FuncFit GaussSum W_coef  Corr /D
	ModifyGraph offset(fit_Corr)={CorrBin/2,0}
	ModifyGraph rgb(fit_Corr)=(0,12800,52224)
	DrawText 0.06,0.15,"y0 = " + num2str(W_coef[0])
	DrawText 0.06,0.25,"A1 = " + num2str(W_coef[1])
	DrawText 0.06,0.35,"x0 = " + num2str(W_coef[2] + (CorrBin/2))
	DrawText 0.06,0.45,"width = " + num2str(W_coef[3])
	DrawText 0.6,0.25,"A2 = " + num2str(W_coef[4])
	DrawText 0.6,0.35,"x0 = " + num2str(W_coef[5] + (CorrBin/2))
	DrawText 0.6,0.45,"width = " + num2str(W_coef[6])
End

// Charly **********************************************here are some usefull functions for paired recordings

// ***************This function aims at deleting NaNs in the occurrence wave (usefull when the findLevels function didn't work properly to find half rise time)
Function ProcDeleteNaN(CtrlName) : ButtonControl
	String CtrlName
	
	String Tags_waveN, Occurrence_waveN
	Tags_waveN = "Tags" + CtrlName[(strlen(CtrlName) - 1)] // 1 or 2 will be added at the end of the name depending on the button that triggered the procedure
	Occurrence_waveN = "Occurrence" + CtrlName[(strlen(CtrlName) - 1)]
	
		Wave Tags = $Tags_waveN
		Wave Occurrence = $Occurrence_waveN
		
		Variable counter
		For (counter = 0; counter < numpnts(Tags); counter += 1)
			If (numtype(Occurrence[counter]) == 2) // If Point is NaN
				If (Tags[counter] == 1)
					Print "NaN foud at step " + num2str(counter) + " will be removed"
				EndIf
				Tags[counter] = 0 //Untag the event
				//Print "NaN found at step : " + num2str(counter) 
			EndIf
		EndFor
End

// Charly ************************************************ This is the function I use instead of Igor built in FindLevels (Uses more processor so longer but less memory use, usefull for big waves)******
Function My_Find_Levels(src_wave, Dest_waveN, thrshld)
	Wave src_wave
	String Dest_waveN
	Variable thrshld
	
	Duplicate/O src_wave, delta_wave
	Make/O/D/N=0 temp_wave
	Variable TEMPO_delta, sample_interval, Yn_min_1, Yn
	delta_wave -= Thrshld
	
	
	Variable counter, counter2
	counter2 = 0
	
	For(counter = 0; counter < numpnts(delta_wave); counter += 1)
		
		//Insert something
		If (counter == (numpnts(delta_wave) - 1))
			//Insert a behaviour for last point
		ElseIf (((Delta_wave[counter] <= 0) && (Delta_wave[counter + 1] > 0)) || ((Delta_wave[counter] >= 0) && (Delta_wave[counter + 1] < 0)))
			InsertPoints counter2, 1, temp_wave
			sample_interval = deltaX(src_wave)
			Yn_min_1 = (Abs(delta_wave[counter]))
			Yn = (Abs(delta_wave[counter + 1]))
			
			TEMPO_delta = (sample_interval * (Yn_min_1 / (Yn_min_1 + Yn)))
			temp_wave[counter2] = ((pnt2X(delta_wave, counter)) + TEMPO_Delta)
			counter2 += 1
		EndIF
	EndFOr
	Duplicate/O TEmp_wave, $dest_waveN
	KillWaves temp_wave, delta_wave
	
End

// // Charly ****************** this loads another function for paired recordings
//Function Btn_MAS(ctrlName) : ButtonControl
//	String CtrlName
//	If (Cmpstr((CtrlName[StrLen(CtrlName) - 1]), "1") == 0)
//		MAS_prompt(1)
//	ElseIf (Cmpstr((CtrlName[StrLen(CtrlName) - 1]), "2") == 0)
//		MAS_prompt(2)
//	EndIf
//End

// Charly ***************************** Function to select one reference channel to use only tags from target channel that match w/ reference chan.
Function tags_synchroniser(check_win_start, check_win_end, Occurrence_ch1, Occurrence_ch2, True_Tags_ch1, True_Tags_ch2, PeakPositionW_ch2, Tags_ch2_syncN)
	Variable check_win_start, check_win_end //Defines the borders of the window set by the crosscorelogram
	Wave Occurrence_ch1, Occurrence_ch2, True_Tags_ch1, True_Tags_ch2, PeakPositionW_ch2//Defines what is the reference occurrence wave (ch1)
	String Tags_ch2_syncN
	// invert order of arguments if necessary (Same applies for Tags) True means original wave
	
	If (WaveExists(Tags_ch1_save) == 0)//Saves the wave if this is not already the case
		Duplicate/O True_Tags_ch1, Tags_ch1_save
	EndIf
	
	If (WaveExists(Tags_ch2_save) == 0) //Saves the wave if this is not already the case
		Duplicate/O True_tags_ch2, Tags_ch2_save
	EndIf
	
	Duplicate/O True_tags_ch2, Tags_ch2_sync // The temporary output (the wave after synchronisation)
	
	Variable counter, counter_2 = 0
	
	For (counter = 0; (counter < numpnts(Occurrence_ch1)); counter += 1) // Browse through channel 1
			
		// If the occurrence of the point is NaN go to next point untill the next non NaN point
		For ( ; ((numtype(Occurrence_ch1[counter]) == 2) && (counter < numpnts(Occurrence_ch1))); ) 
			counter += 1
		EndFor
	
			If ((True_Tags_ch1[counter]) == 1) // If event selected on chan 1
			
				// If the occurrence of the point is NaN go to next point untill the next non NaN point
				For ( ; ((numtype(Occurrence_ch2[counter_2]) == 2) && (counter_2 < numpnts(Occurrence_ch2))); ) 
					counter_2 += 1
				EndFor
				
				// As long as event on channel 2 is before the end of the window compared to event on channel 1
				For	(; ((Occurrence_ch2[counter_2] < (Occurrence_ch1[counter] + check_win_end)) && (counter_2 < numpnts(Occurrence_ch2))); ) // We use a For/endFor loop to test condition before running the loop	
					
					// If the occurrence of the point is NaN go to next point untill the next non NaN point
					For ( ; ((numtype(Occurrence_ch2[counter_2]) == 2) && (counter_2 < numpnts(Occurrence_ch2))); ) 
						counter_2 += 1
					EndFor
					
					
			 		If ((True_tags_ch2[counter_2]) == 1) // And if event selected on chan 2
						// Check if event on chan 2 corresponds to event on chan 1
			
						//If the event is before the window
						If((Occurrence_ch2[counter_2]) < ((Occurrence_ch1[counter]) + check_win_start))
							Tags_ch2_sync[counter_2] = 0 // untag it (We use untag instead of tag to take into account the previous state of the tags
						Endif	
					
					EndIf
						
						counter_2 += 1	// Update the counter each time an event on chan 2 is checked
				EndFor
			
			EndIf
		
	EndFor
	// Clear potentially remaining tags
	Tags_ch2_sync[counter_2, (Numpnts(Occurrence_ch2) - 1)] = 0
	Duplicate/O Tags_ch2_sync, $Tags_ch2_syncN
	KillWaves Tags_ch2_sync // Kill the temporary output
	
End

// Charly **************** Selects the origin of the channel 1 tags after synch 
Function OriginOfTags1(CtrlName,popNum,popStr) : PopupMenuControl
	String CtrlName
	Variable popNum // Selected item reference number
	String popStr // content of current popup item as string
	Nvar Tags1_origin
	If (popNum == 1)
		If (WaveExists(Tags_ch1_save))
			Duplicate/O Tags_ch1_save, Tags1
			Print "Tags1 reinitialized"
			Tags1_origin = 1
		EndIf
	Elseif (popNum == 2)
		If (WaveExists(Tags1Sync))
			Duplicate/O Tags1Sync, Tags1
			Tags1_origin = 2
		EndIf
	EndIf
End


// Charly **************** Selects the origin of the channel 2 tags after synch 
Function OriginOfTags2(CtrlName,popNum,popStr) : PopupMenuControl
	String CtrlName
	Variable popNum // Selected item reference number
	String popStr // content of current popup item as string
	Nvar Tags2_origin
	If (popNum == 1)
		If (WaveExists(Tags_ch2_save))
			Duplicate/O Tags_ch2_save, Tags2
			Print "Tags2 reinitialized"
			Tags2_origin = 1
		EndIf
	Elseif (popNum == 2)
		If (WaveExists(Tags2Sync))
			Duplicate/O Tags2Sync, Tags2
			Tags2_origin = 2
		EndIf
	EndIf
End

// Charly ******************************Triggers sync of events on chan 1 corresponding to events on chan 2
Function SyncChan1From2(ctrlName) : ButtonControl
	String CtrlName
	
	DoAlert 1, "Do you want to reinitialise tags1"
	
	If (V_flag == 1)
		// To prevent use of false tags1 as ref
		OriginOfTags1("",1,"")
	EndIf
	
	// Borders of the detection window
	NVar SyncWinStart1, SyncWinEnd1
	Variable check_win_start, check_win_end
	check_win_start = SyncWinStart1 * 0.001 // set to secs
	check_win_end = SyncWinEnd1 * 0.001 // set tio secs
	
	// If all the arguments are present
	If (WaveExists(Occurrence1) && WaveExists(Occurrence2) && WaveExists(Tags1) && WaveExists(Tags2))
		Wave Occurrence1, Occurrence2, Tags1, Tags2, RealPeakPosition1
		tags_synchroniser(check_win_start, check_win_end, Occurrence2, Occurrence1, Tags2, Tags1, RealPeakPosition1, "Tags1Sync")
	Else
		Abort "Be sure to tag events on both channels before"
	EndIf
	DoAlert 1, "Do you want to update and overwrite tags1"
	If (V_flag == 1) // If yes is clicked
		OriginOfTags1("",2,"")
	EndIf
End

// Charly ******************************Triggers sync of events on chan 2 corresponding to events on chan 1
Function SyncChan2From1(ctrlName) : ButtonControl
	String CtrlName
	
	DoAlert 1, "Do you want to reinitialise tags2"
	
	If (V_flag == 1)
		// To prevent use of false tags2 as ref
		OriginOfTags2("",1,"")
	EndIf
	
	// Borders of the detection window
	NVar SyncWinStart2, SyncWinEnd2
	Variable check_win_start, check_win_end
	check_win_start = SyncWinStart2 * 0.001 // set to secs
	check_win_end = SyncWinEnd2 * 0.001 // set tio secs
	
	// If all the arguments are present
	If (WaveExists(Occurrence1) && WaveExists(Occurrence2) && WaveExists(Tags1) && WaveExists(Tags2))
		Wave Occurrence1, Occurrence2, Tags1, Tags2, RealPeakPosition2
		tags_synchroniser(check_win_start, check_win_end, Occurrence1, Occurrence2, Tags1, Tags2, RealPeakPosition2, "Tags2Sync")
	Else
		Abort "Be sure to tag events on both channels before"
	EndIf
	DoAlert 1, "Do you want to update and overwrite tags2"
	If (V_flag == 1) // If yes is clicked
		OriginOfTags2("",2,"")
	EndIf
End

// Charly ************************* Two function to save easyly tags as separate waves
Function ProcSaveTags1(ctrlName) : ButtonControl
	String CtrlName
	If (WaveExists(Tags_ch1_save))
		DoAlert 1, "Caution, you are about to save the tags1 backup if you want to save the actual copy click NO"
		If (V_Flag == 1)
			Save/C Tags_ch1_save
		Else
			Save/C Tags1
		EndIf
	Else
		Save/C Tags1
	EndIf
End

Function ProcSaveTags2(ctrlName) : ButtonControl
	String CtrlName
	If (WaveExists(Tags_ch2_save))
		DoAlert 1, "Caution, you are about to save the tags2 backup if you want to save the actual copy click NO"
		If (V_Flag == 2)
			Save/C Tags_ch2_save
		Else
			Save/C Tags2
		EndIf
	Else
		Save/C Tags2
	EndIf
End

// Charly different Fit functions called by the procedure launched by the buttons
Function FitDbExp1()
	SpAcAnGeneralFit(1, 1, "dblexp")
End

Function FitDbExp2()
	SpAcAnGeneralFit(2, 2, "dblexp")
End

Function Fitdbexp_ch1_on_win2()
	SpAcAnGeneralFit(2, 1, "dblexp")
End

Function Fitdbexp_ch2_on_win1()
	SpAcAnGeneralFit(1, 2, "dblexp")
End

Function FitSingleExp1()
	SpAcAnGeneralFit(1, 1, "exp")
End

Function FitSingleExp2()
	SpAcAnGeneralFit(2, 2, "exp")
End

Function FitSingleExp_ch1_on_win2()
	SpAcAnGeneralFit(2, 1, "exp")
End

Function FitSingleExp_ch2_on_win1()
	SpAcAnGeneralFit(1, 2, "exp")
End

Function FitSigDbExp1()
	SpAcAnGeneralFit(1, 1, "SigDbExp")
End

Function FitSigDbExp_ch2_on_win1()
	SpAcAnGeneralFit(1, 2, "SigDbExp")
End

Function FitSigDbExp2()
	SpAcAnGeneralFit(2, 2, "SigDbExp")
End

Function FitSigDbExp_ch1_on_win2()
	SpAcAnGeneralFit(2, 1, "SigDbExp")
End

Function FitSigmoid1()
	SpAcAnGeneralFit(1, 1, "Sigmoid")
End

Function FitSigmoid_ch2_on_win1()
	SpAcAnGeneralFit(1, 2, "Sigmoid")
End

Function FitSigmoid2()
	SpAcAnGeneralFit(2, 2, "Sigmoid")
End

Function FitSigmoid_ch1_on_win2()
	SpAcAnGeneralFit(2, 1, "Sigmoid")
End

// Charly *****************Function to set the role of the fitdbexp button depending on the value of the selected chan popup menu (either fit channel 1 or 2)
Function Fitdbexp_select1(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb1
	
	If (fit_chan_select_nb1 == 1)
		Fitdbexp1()
	ElseIf (fit_chan_select_nb1 == 2)
		Fitdbexp_ch2_on_win1()
	EndIf
End

Function Fitdbexp_select2(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb2
	
	If (fit_chan_select_nb2 == 2)
		Fitdbexp2()
	ElseIf (fit_chan_select_nb2 == 1)
		Fitdbexp_ch1_on_win2()
	EndIf
End

Function FitSingleExp_select1(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb1
	
	If (fit_chan_select_nb1 == 1)
		FitSingleExp1()
	ElseIf (fit_chan_select_nb1 == 2)
		FitSingleExp_ch2_on_win1()
	EndIf
End

Function FitSingleExp_select2(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb2
	
	If (fit_chan_select_nb2 == 2)
		FitSingleExp2()
	ElseIf (fit_chan_select_nb2 == 1)
		FitSingleExp_ch1_on_win2()
	EndIf
End

Function FitSigDbExp_select1(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb1
	
	If (fit_chan_select_nb1 == 1)
		FitSigDbExp1()
	ElseIf (fit_chan_select_nb1 == 2)
		FitSigDbExp_ch2_on_win1()
	EndIf
End

Function FitSigDbExp_select2(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb2
	
	If (fit_chan_select_nb2 == 2)
		FitSigDbExp2()
	ElseIf (fit_chan_select_nb2 == 1)
		FitSigDbExp_ch1_on_win2()
	EndIf
End

Function FitSigmoid_select1(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb1
	
	If (fit_chan_select_nb1 == 1)
		FitSigmoid1()
	ElseIf (fit_chan_select_nb1 == 2)
		FitSigmoid_ch2_on_win1()
	EndIf
End

Function FitSigmoid_select2(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb2
	
	If (fit_chan_select_nb2 == 2)
		FitSigmoid2()
	ElseIf (fit_chan_select_nb2 == 1)
		FitSigmoid_ch1_on_win2()
	EndIf
End

// Function to select the role of the checkboxes menu used to select the channel that has to be fitted
Function fit_chan_select1(Name,CtrlVal)
	String Name
	Variable CtrlVal // Selected item reference number
	
	Variable/G fit_chan_select_nb1
	
	If (cmpstr(Name[strlen(Name) - 1], "1") == 0) // If last caracter of name is 1 (for ch1 checkBox)
		If (CtrlVal == 1) // If box selected
			Fit_chan_select_nb1 = 1 // Then select channel 1
			Checkbox CheckBFitSelectedChan12, value= 0 // Deselect box 2
		ElseIf (CtrlVal == 0) // If box Deselected
			Fit_chan_select_nb1 = 2 // Then select channel 2
			Checkbox CheckBFitSelectedChan12, value= 1 // Select box 2
		EndIf
	ElseIf(cmpstr(Name[strlen(Name) - 1], "2") == 0) // If last caracter of name is 2 (for ch2 checkBox)
		If (CtrlVal == 1) // If box selected
			Fit_chan_select_nb1 = 2 // Then select channel 2
			Checkbox CheckBFitSelectedChan11, value= 0 // Deselect box 1
		ElseIf (CtrlVal == 0) // If box Deselected
			Fit_chan_select_nb1 = 1 // Then select channel 1
			Checkbox CheckBFitSelectedChan11, value= 1 // Select box 1
		EndIf
	EndIf
End

Function fit_chan_select2(Name,CtrlVal)
	String Name
	Variable CtrlVal // Selected item reference number
	
	Variable/G fit_chan_select_nb2
	
	If(cmpstr(Name[strlen(Name) - 1], "2") == 0) // If last caracter of name is 2 (for ch2 checkBox)
		If (CtrlVal == 1) // If box selected
			Fit_chan_select_nb2 = 2 // Then select channel 2
			Checkbox CheckBFitSelectedChan21, value= 0 // Deselect box 1
		ElseIf (CtrlVal == 0) // If box Deselected
			Fit_chan_select_nb2 = 1 // Then select channel 1
			Checkbox CheckBFitSelectedChan21, value= 1 // Select box 1
		EndIf
	ElseIf (cmpstr(Name[strlen(Name) - 1], "1") == 0) // If last caracter of name is 1 (for ch1 checkBox)
		If (CtrlVal == 1) // If box selected
			Fit_chan_select_nb2 = 1 // Then select channel 1
			Checkbox CheckBFitSelectedChan22, value= 0 // Deselect box 2
		ElseIf (CtrlVal == 0) // If box Deselected
			Fit_chan_select_nb2 = 2 // Then select channel 2
			Checkbox CheckBFitSelectedChan22, value= 1 // Select box 2
		EndIf
	
	EndIf
End


Function IntegrateChanSelect1(Name,CtrlVal)
	String Name
	Variable CtrlVal // Selected item reference number
	
	Variable/G IntegrateChanSelectNb1
	
	If (cmpstr(Name[strlen(Name) - 1], "1") == 0) // If last caracter of name is 1 (for ch1 checkBox)
		If (CtrlVal == 1) // If box selected
			IntegrateChanSelectNb1 = 1 // Then select channel 1
			Checkbox CheckBIntSelectedChan12, value= 0 // Deselect box 2
		ElseIf (CtrlVal == 0) // If box Deselected
			IntegrateChanSelectNb1 = 2 // Then select channel 2
			Checkbox CheckBIntSelectedChan12, value= 1 // Select box 2
		EndIf
	ElseIf(cmpstr(Name[strlen(Name) - 1], "2") == 0) // If last caracter of name is 2 (for ch2 checkBox)
		If (CtrlVal == 1) // If box selected
			IntegrateChanSelectNb1 = 2 // Then select channel 2
			Checkbox CheckBIntSelectedChan11, value= 0 // Deselect box 1
		ElseIf (CtrlVal == 0) // If box Deselected
			IntegrateChanSelectNb1 = 1 // Then select channel 1
			Checkbox CheckBIntSelectedChan11, value= 1 // Select box 1
		EndIf
	EndIf
End

Function IntegrateChanSelect2(Name,CtrlVal)
	String Name
	Variable CtrlVal // Selected item reference number
	
	Variable/G IntegrateChanSelectNb2
	
	If(cmpstr(Name[strlen(Name) - 1], "2") == 0) // If last caracter of name is 2 (for ch2 checkBox)
		If (CtrlVal == 1) // If box selected
			IntegrateChanSelectNb2 = 2 // Then select channel 2
			Checkbox CheckBIntSelectedChan21, value= 0 // Deselect box 1
		ElseIf (CtrlVal == 0) // If box Deselected
			IntegrateChanSelectNb2 = 1 // Then select channel 1
			Checkbox CheckBIntSelectedChan21, value= 1 // Select box 1
		EndIf
	ElseIf (cmpstr(Name[strlen(Name) - 1], "1") == 0) // If last caracter of name is 1 (for ch1 checkBox)
		If (CtrlVal == 1) // If box selected
			IntegrateChanSelectNb2 = 1 // Then select channel 1
			Checkbox CheckBIntSelectedChan22, value= 0 // Deselect box 2
		ElseIf (CtrlVal == 0) // If box Deselected
			IntegrateChanSelectNb2 = 2 // Then select channel 2
			Checkbox CheckBIntSelectedChan22, value= 1 // Select box 2
		EndIf

	EndIf
End

Function IntegrateEvent1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIntegrate1
	CheckIntegrate1 = 1 - CheckIntegrate1
	CheckBox CheckBIntEvent1 value = CheckIntegrate1
	CheckBox CheckBIntAvg1 value = 1 - CheckIntegrate1
End

Function IntegrateAvg1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIntegrate1
	CheckIntegrate1 = 1 - CheckIntegrate1
	CheckBox CheckBIntAvg1 value = 1 - CheckIntegrate1
	CheckBox CheckBIntEvent1 value = CheckIntegrate1
End

Function IntegrateEvent2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIntegrate2
	CheckIntegrate2 = 1 - CheckIntegrate2
	CheckBox CheckBIntEvent2 value = CheckIntegrate2
	CheckBox CheckBIntAvg2 value = 1 - CheckIntegrate2
End

Function IntegrateAvg2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIntegrate2
	CheckIntegrate2 = 1 - CheckIntegrate2
	CheckBox CheckBIntAvg2 value = 1 - CheckIntegrate2
	CheckBox CheckBIntEvent2 value = CheckIntegrate2
End

Function ProcLowPass(CtrlName) : ButtonControl
	String CtrlName
	String ChannelNb=CtrlName[Strlen(CtrlName) - 1]
	NVar Thrshld = $("LowPassThrshld" + ChannelNb)
	LowPass(thrshld, ChannelNb)
End

Function ProcBandPass(CtrlName) : ButtonControl
	String CtrlName
	String ChannelNb=CtrlName[Strlen(CtrlName) - 1]
	Nvar LowerAmpl = BandPassLow
	Nvar UpperAmpl = BandPassHigh
	AmplitudeRangeFilter(str2num(ChannelNb), LowerAmpl, UpperAmpl)
End

Function ProcIsolateEvents(CtrlName) : ButtonControl
	String CtrlName
	String ChannelNb=CtrlName[Strlen(CtrlName) - 1]
	Wave TagsW=$("Tags" + ChannelNb)
	Wave OccurrenceW=$("Occurrence" + ChannelNb)
	NVar IsolateEventsWinStart = $("IsolateEventsWinStart" + ChannelNb)
	NVar IsolateEventsWinEnd = $("IsolateEventsWinEnd" + ChannelNb)
	RemoveDoubleEvents(TagsW, OccurrenceW, IsolateEventsWinEnd, IsolateEventsWinStart)
End

Function restoreKinBtns(channelNb)
	variable channelNb
	
	string chNb = num2str(channelNb)

//	ModifyControl $("CFPE" + chNb), disable= 0
//	ModifyControl $("CFAE" + chNb) disable= 0	
//	ModifyControl $("FreeOffset" + chNb) disable= 0	
//	ModifyControl $("Fit2Peak" + chNb) disable= 0
//	ModifyControl $("FitSingleExp" + chNb) disable= 0	
//	ModifyControl $("FitdbExp" + chNb) disable= 0 
//	ModifyControl $("BFitsigdbexp" + chNb) disable= 0	
//	ModifyControl $("BFitsigmoid" + chNb) disable= 0	
//	ModifyControl $("BFitAllEventsdbExp" + chNb) disable= 0
//	ModifyControl $("TitleFitsTab" + chNb) disable= 0	
//	ModifyControl $("Start_Fit" + chNb) disable= 0
//	ModifyControl $("End_Fit" + chNb) disable= 0
//
//	ModifyControl $("CheckBFitSelectedChan" + chNb + "1") disable= 0
//	ModifyControl $("CheckBFitSelectedChan" + chNb + "2") disable= 0
			
	// restore output fields
	ModifyControl $("TauFast" + chNb), title="Tau(fast):",  disable= 0	
	ModifyControl $("TauSlow" + chNb), title="Tau(slow):",  disable= 0	
	ModifyControl $("FastPercentage" + chNb), title="% fast:",  disable= 0	
	ModifyControl $("SlowPercentage" + chNb), title="% slow:",  disable= 0	
	ModifyControl $("ValOffset" + chNb), title="Offset =", disable= 0
	ModifyControl $("KineticIndex" + chNb), title="Kinetic index :", disable= 0


/// Find out  howto modify here



End

Function SpAcAnGeneralFit(WinNb, ChannelNb, FitTypeStr) // FitTypeStr shall be one of the following strings : exp, dbexp, sigmoid, sigdbexp
	Variable WinNb, ChannelNb
	String FitTypeStr
	
	restoreKinBtns(WinNb) // change the state of the kinetics buttons to actiive and default	
	
	//***********************************************Local variables***********************************************
	// Name of the wave that will be fitted and the window where it is displayed
	String WaveToFitN, WindowN, TEMPSTR
	Variable debut, fin, counter

	// The following lines create the constraints for the fit
	String constraint1,  constraint2, constraint3, constraint4, constraint5
	//*************************************************some globals******************************************
	Variable/G V_FitOptions = 4 // V_fitOptions=4 suppresses the fit window (runs faster) (global because used w/ execute
	// Temporary output wave for the fit parameters
	KillWaves/Z TempFitParamW
	Make/O/D/N=10 TEMPFitParamW
	//*********************************** Bring back the necessary global variables and strings******************************
	// Variabel whose name depend on the channel to fit
	NVar Ppnts_V = $("Ppnts" + num2str(ChannelNb))
	NVar SampleInt_V = $("SI" + num2str(ChannelNb))
	
	// Variables depending on the winDow they are accessed from
	NVar CheckOffset_V = $("CheckOffset" + num2str(WinNb))
	NVar Decay_V = $("Decay" + num2str(WinNb))
	NVar CheckFit_V = $("CheckFit" + num2str(WinNb))
	NVar StartFit_V = $("StartFit" + num2str(WinNb))
	NVar EndFit_V = $("EndFit" + num2str(WinNb))
	NVar checkFit2Peak_V = $("checkFit2Peak" + num2str(WinNb))
	
	// Parameters that are only loaded for sigmadbexp fit	
	If (Cmpstr(FitTypeStr, "sigdbexp") == 0) 
		Wave RealAmp_W = $("RealAmp" + num2str(ChannelNb))
		NVar EventNb_V = $("EventNumber" + num2str(ChannelNb))
		SVar FileNameStr = $("FileName" + num2str(ChannelNb))
	EndIf

	//*********************************************Determining wave to fit*********************************
	If (CheckFit_V == 0) // Means fit current event
		If (ChannelNb == WinNb) // If top graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#EventChannel" + num2str(ChannelNb)
			WaveToFitN = "TEMPEVENT" + num2str(WinNb)
		Else // if bottom graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#EventChannel" + num2str(WinNb) + num2str(ChannelNb)
			WaveToFitN = "TEMPEVENT" + num2str(3 - ChannelNb) + "_CH" + num2str(ChannelNb)
		EndIf	
	ElseIf (CheckFit_V == 1) // Means fit average
		If (ChannelNb == WinNb) // If top graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#Average" + num2str(ChannelNb)
			WaveToFitN = "AverageEvent" + num2str(WinNb)
		Else // If bottom graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#Average" + num2str(WinNb) + num2str(ChannelNb)
			WaveToFitN = "AverageEvent" + num2str(WinNb) + "CH" + num2str(ChannelNb)
		EndIf
	EndIf
	
	
	// **************************************Preliminary for sigmoid fit ***********************************
	If (Cmpstr(FitTypeStr[0, 2], "sig") == 0) // For sig or sigdbexp fit
		WaveStats/Q $WaveToFitN
		if (startFit_V  != 0)
			FindLevel/Q/R=(-V_MinLoc,V_MinLoc) $WaveToFitN, V_min*0.01*StartFit_V
			If (V_Flag != 0)
				Abort "No Level Found"
			Endif
			Debut = V_LevelX
		else
			debut = -V_MinLoc
		endIf
	EndIf
	
	// *********************************Common (Fin is where the fit ends)*************************
	Fin = EndFit_V * 0.001	
	
	// **********************The following lines are meant to remove any previous fit wave on the graph*******************
	GetWindow $(WindowN), WaveList
	Wave/T W_WaveList
	
	//Set as active subwindow the window we are going to work in
	DoWindow/F $("DetectedEvents" + num2str(WinNb))
	Execute/Q/Z "SetActiveSubWindow " + WindoWN // Don't know why but it is the only thing that works ($ doesn't)
	
	If (V_Flag != 0) // If execute didn't work
		Abort "Setting active subwindow failed"
	EndIf
	
	For (counter = 1; counter < DimSize(W_WaveList, 0); counter += 1)
		TEMPSTR = W_WaveList[counter] [0]
		If (cmpstr(TEMPSTR[0, 2], "fit") == 0) // If the begginning of the waveN contains fit remove it from the graph
			RemoveFromGraph $W_WaveList[counter][0] // /W not used since active subwindow is supposed to be selected
		ElseIf (cmpstr(TEMPSTR[0, 2], "UC_") == 0) // For the confidence interval waves
			RemoveFromGraph $W_WaveList[counter][0]
		ElseIf (cmpstr(TEMPSTR[0, 2], "LC_") == 0) // For the confidence interval waves
			RemoveFromGraph $W_WaveList[counter][0]
		EndIf
	EndFor
	
	// A trick to prevent the procedure to display the fit by itself (If so the it's a mess , trust me man)
	Make/O $("fit_" + WaveToFitN) = 0
	
	AppendToGraph $("fit_" + WaveToFitN) // /W not used since active subwindow is supposed to be selected
	
	if (Cmpstr(FitTypeStr[StrLen(FitTypeStr) - 3, StrLen(FitTypeStr)], "exp") == 0) // in case of exp (single or double fit
		Make/O $("UC_" + WaveToFitN) = 0
		Make/O $("LC_" + WaveToFitN) = 0
		AppendToGraph $("UC_" + WaveToFitN) // /W not used since active subwindow is supposed to be selected
		AppendToGraph $("LC_" + WaveToFitN) // /W not used since active subwindow is supposed to be selected	
		//changin the display of confidence interval waves to dotted lines
		ModifyGraph lstyle($("UC_" + WaveToFitN))=2
		ModifyGraph lstyle($("LC_" + WaveToFitN))=2
	EndIf
	
	// changing fit color
	If (ChannelNb == 2)
		ModifyGraph rgb($("fit_" + WaveToFitN))=(0, 16000, (2^16 - 1))// Since the curve is red change the fit colour to blue
		if (Cmpstr(FitTypeStr[StrLen(FitTypeStr) - 3, StrLen(FitTypeStr)], "exp") == 0) // for exponential fits
			ModifyGraph rgb($("UC_" + WaveToFitN))=(0, 16000, (2^16 - 1))// Since the curve is red change the fit colour to blue
			ModifyGraph rgb($("LC_" + WaveToFitN))=(0, 16000, (2^16 - 1))// Since the curve is red change the fit colour to blue
		endIf
	EndIf
	
	// temporary wave to be fitted (for naming conveniences
	Duplicate/O $WaveToFitN, TEMPWAVE
	// For the events only (avg doesn't need smoothing)
	If (CheckFit_V == 0)
		Smooth/B Ppnts_V, TEMPWAVE
	EndIf
	
	K0 = 0
	
	//***********************************Following section only required for sigmoid fit************************************************
	If ((Cmpstr(FitTypeStr, "sigdbexp") == 0)) // If we use sigdbexp
		Make/D/o/N=9 SigExp
		WaveStats/Q/R=(0, Decay_V*.001) $WaveToFitN // To find the minimum, i.e. the peak
		// Then, we fit a sigmoid function to the rise phase of the event
		K0 = 0; SigExp[0]=K0		// Sigmoid : base
		CurveFit/Q/N/H="1000" Sigmoid $WaveToFitN (-V_MinLoc,V_MinLoc) /D // Fit sigmoid to the rise (symetrical around half rise)
		SigExp[1, 3] = {K1, K2, K3} // (max, xhalf, rate)
	EndIf
	
	//********************************************Actual fitting commands**************************************************
	// The exponential (or double exponential) fit command
	String ConstantFitParameters // To hold the offset parameter
	If ((Cmpstr(FitTypeStr[StrLen(FitTypeStr) - 3, StrLen(FitTypeStr)], "exp") == 0) && (Cmpstr(FitTypeStr[0,2], "sig") != 0))	// If we fit exponential (single or double)
		
		// First determine the limits of the fit (debut is the beginning)
		WaveStats/Q/R=(0, Decay_V * 0.001) TEMPWAVE // find the X location of the minimum (i.e. the peak)
		FindLevel/Q/R=(V_MinLoc, Decay_V*0.001) TEMPWave V_min*(1-StartFit_V*0.01) // Find, between Peak location and the end of the window, the 1 - StartFit % of the peak value
	
		If (V_Flag != 0) // if FindLevel didn't work
			Abort "No level found"
		EndIf
		
		Debut=V_LevelX // start of the fit in seconds
		
		if (CheckOffset_V==1)
			ConstantFitParameters = "00000"
		else
			ConstantFitParameters = "10000"
		endif
		
		variable X0
		if (checkFit2Peak_V == 1)
			X0=V_MinLoc
		else
			X0=Debut
		endif

		// The following lines pecify the fit constraints
		constraint1 = "K2 < K4" // to have K2 be tau fast and K4 be tau slow
		constraint2 = "K4 < " + num2str(EndFit_V * 0.001) // decay can't be longer than the end of the event
		constraint3 = "K2 < " + num2str(EndFit_V * 0.001) // decay can't be longer than the end of the event
		constraint4 = "K4 > 0.00001" // K4 is positive but 0 gives a bug in Igor
		constraint5 = "K2 > 0.00001" // same for K2
		
		
		If (cmpstr(fitTypeStr, "dblexp") == 0)// for double exp fit
			Make/O/N=2/T fitConstraints={constraint1, constraint2, constraint3, constraint4, constraint5}
		else // For single exp fit
			Make/O/N=2/T fitConstraints={constraint3, constraint5}
		endif

		Execute  "CurveFit/Q/N/K={" + num2str(X0)+ "}/H=\"" + ConstantFitParameters + "\" " + FitTypeStr + "_XOffset " + WaveToFitN + "(" + num2str(debut)  + ", "  + num2str(fin) + ") /D/C=fitConstraints/F={0.99, 1}"
		TEMPFitParamW[0]=(K2 * 1000) // Tau fast (ms)
		TEMPFitParamW[1]=(K4 * 1000) // Tau slow
		TEMPFitParamW[2] = (K1 / (K1 + K3))*100 // % Fast
		TEMPFitParamW[3] = (K3 / (K1 + K3))*100 // % slow
		TEMPFitParamW[8]=K0 // Offset
		
	// The sigmoid double exponential fitting command	
	ElseIf (Cmpstr(FitTypeStr, "sigdbexp") == 0)
		WaveStats/Q/R=(0,Decay_V*.001) $WaveToFitN
		K0 = 0
		if (CheckOffset_V==1)
			ConstantFitParameters = "\"10000\""
		else
			ConstantFitParameters = "\"00000\""
		endif
		
		// Initial guess for the biexp part
		Execute "CurveFit/Q/N/H=" + ConstantFitParameters + " dblexp " + WaveToFitN + "(" + num2str(V_MinLoc + (10*SampleInt_V*1e-6)) + ", " + num2str(fin) + ") /D" // Starts the fit 10 points after the peak
		
		SigExp[4]=K3/(K3+K1)		// Dbexp : amplitude fast
		SigExp[5]=K4			// Dbexp : fast time constant
		SigExp[6]=K1/(K3+K1)		// Dbexp : amplitude slow
		SigExp[7]=K2			// Dbexp : slow time constant
		SigExp[8]=K0			// Dbexp : offset
		
		Make/D/N=8/O W_coef
		W_coef[0] = {SigExp[1],SigExp[2],SigExp[3],0,SigExp[4],SigExp[5],SigExp[6],SigExp[7]}
		// W_coef = {max, xhalf, rate, baseline, amplitude fast, fast time constant, amplitude slow, slow time constant}
	
		if (CheckOffset_V==1)
			Execute "FuncFit/N/H=\"00000000\" Sigmoid_Dbexp W_coef "+ WaveToFitN + "("+ num2str(debut) + ", " + num2str(fin) + ") /D/F={0.99, 1}" // Offset is set on zero (The fit returns to baseline)
		else
			Execute "FuncFit/N/H=\"00010000\" Sigmoid_Dbexp W_coef " + WaveToFitN + "(" + num2str(debut) + ", " + num2str(fin) + ") /D/F={0.99, 1}" // Offset is set on zero (The fit returns to baseline)
		endif
		// The following lines are for displaying the coefficients in the control bar (nicer than the raw result in the command window, isn't it ?) 
		TEMPFitParamW[0] = (1/W_coef[5])*1000	// taufast
		TEMPFitParamW[1] = (1/W_coef[7])*1000	// tauslow
		TEMPFitParamW[2] = (W_coef[4] / ( W_coef[4] + W_coef[6] ))*100	// % fast
		TEMPFitParamW[3] = (W_coef[6] / ( W_coef[4] + W_coef[6] ))*100	// % slow
		TEMPFitParamW[4] = W_coef[3]	// Offset // Usefull ?
		TEMPFitParamW[8] = W_coef[3]	// Offset
	
	// The simple sigmoid fitting command	
	ElseIf (Cmpstr(FitTypeStr, "sigmoid") == 0)
		Make/D/o/N=9 SigExp
		WaveStats/Q/R=(0, Decay_V*.001) $WaveToFitN // To find the minimum, i.e. the peak
		// Then, we fit a sigmoid function to the rise phase of the event
		K0 = 0; SigExp[0]=K0		// Sigmoid : base
		print debut
		CurveFit/Q/N/H="1000" Sigmoid $WaveToFitN (Debut,V_MinLoc) /D // Fit sigmoid to the rise (symetrical around half rise)
		SigExp[1, 3] = {K1, K2, K3} // (max, xhalf, rate)
		TEMPFitParamW[0, 2] = SigExp[p+1] // (max, xhalf, rate)
	EndIf
	
	//**********************************************Displaying the cursors***************************
	//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		getaxis/W=DetectedEvents2#EventChannel2 bottom
//		Curseur_debut_fit_X2[0] = debut - .05*(V_max-V_min)
//		Curseur_debut_fit_X2[1] = debut + .05*(V_max-V_min)
//		Curseur_debut_fit_Y2[0] = K0 + K1*exp(-K2*debut) + K3*exp(-K4*debut)
//		Curseur_debut_fit_Y2[1] = K0 + K1*exp(-K2*debut) + K3*exp(-K4*debut)
//		getaxis/W=DetectedEvents2#EventChannel2 left
//		Curseur_fin_fit_X2[0] = EndFitdbexp2*.001
//		Curseur_fin_fit_X2[1] = EndFitdbexp2*.001
//		Curseur_fin_fit_Y2[0] = K0 + K1*exp(-K2*fin) + K3*exp(-K4*fin) - 0.05*(V_max-V_min)
//		Curseur_fin_fit_Y2[1] = K0 + K1*exp(-K2*fin) + K3*exp(-K4*fin) + 0.05*(V_max-V_min)
//		AppendToGraph/W=DetectedEvents2#EventChannel2 Curseur_debut_fit_Y2 vs Curseur_debut_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_debut_fit_Y2)=(0,0,0)
//		AppendToGraph/W=DetectedEvents2#EventChannel2 Curseur_fin_fit_Y2 vs Curseur_fin_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_fin_fit_Y2)=(0,0,0)
//	endif
	
	
	//**********************************************Displaying the values in the detected events windows***************************

	if ((cmpstr(fitTypeStr, "dblexp") == 0) || (cmpstr(fitTypeStr, "sigdbexp") == 0))// Double exponential fit or sigmoid double exponential fit
		execute "ValDisplay TauFast" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[0])
		execute "ValDisplay TauSlow" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[1])
		execute "ValDisplay FastPercentage" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[2])
		execute "ValDisplay SlowPercentage" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[3])
		execute "ValDisplay ValOffset" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[8])
		Variable KI = (TEMPFitParamW[2]/100)*TEMPFitParamW[0] + (TEMPFitParamW[3]/100)*TEMPFitParamW[1]
		execute "ValDisplay KineticIndex" + num2str(WinNb) + " value=" + num2str(KI)
	elseif (cmpstr(fitTypeStr, "exp") == 0) // Single exopnential fit
		//First hide the displays we are not going to use
		ModifyControl $("TauSlow" + num2str(winNb)), title="Tau(slow):",  disable= 1	
		ModifyControl $("FastPercentage" + num2str(winNb)), title="% fast:",  disable= 1	
		ModifyControl $("SlowPercentage" + num2str(winNb)), title="% slow:",  disable= 1	
		ModifyControl $("KineticIndex" + num2str(winNb)), title="Kinetic index :", disable= 1
		ModifyControl $("TauFast" + num2str(winNb)), title="Tau:"
		execute "ValDisplay ValOffset" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[8])
		execute "ValDisplay TauFast" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[0])
	elseif (cmpstr(fitTypeStr, "sigmoid") == 0)
		ModifyControl $("TauFast" + num2str(winNb)), title="Max :"
		ModifyControl $("TauSlow" + num2str(winNb)), title="XHalf :"	
		ModifyControl $("FastPercentage" + num2str(winNb)), title="Rate :"	
		ModifyControl $("SlowPercentage" + num2str(winNb)), title="% slow:",  disable= 1	
		ModifyControl $("KineticIndex" + num2str(winNb)), title="Kinetic index :", disable= 1
		ModifyControl $("ValOffset" + num2str(WinNb)), disable=1

		execute "ValDisplay TauFast" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[0])
		execute "ValDisplay TauSlow" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[1])
		execute "ValDisplay FastPercentage" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[2])
	elseif (cmpstr(fitTypeStr, "sigdbexp") == 0)
	
	EndIf
	Duplicate/O TEMPFitParamW, $("FitParametersFit" + FitTypeStr + num2str(winNb))
	KillWaves TEMPFitParamW
	Killvariables V_FitOptions
	
//	// Change color and thickness of wave displayed
//	if (WinNb == 1 && ChannelNb == 1 && CheckFit_V == 0)
//		ModifyGraph/W=DetectedEvents1#EventChannel1  lsize(fit_TEMPEVENT1)=2
//	endif
//	if (WinNb == 1 && ChannelNb == 1 && CheckFit_V == 1)
//		ModifyGraph/W=DetectedEvents1#Average1  lsize(fit_AverageEvent1)=2
//	endif
//	if (WinNb == 2 && ChannelNb == 2 && CheckFit_V == 0)
//		ModifyGraph/W=DetectedEvents2#EventChannel2  rgb(fit_TEMPEVENT2)=(0,0,65535)
//		ModifyGraph/W=DetectedEvents2#EventChannel2  lsize(fit_TEMPEVENT2)=2
//	endif
//	if (WinNb == 2 && ChannelNb == 2 && CheckFit_V == 1)
//		ModifyGraph/W=DetectedEvents2#Average2  rgb(fit_AverageEvent2)=(0,0,65535)
//		ModifyGraph/W=DetectedEvents2#Average2  lsize(fit_AverageEvent2)=2
//	endif
//	
//	if (WinNb == 1 && ChannelNb == 2 && CheckFit_V == 0)
//		ModifyGraph/W=DetectedEvents1#EventChannel12  lsize(fit_TEMPEVENT1_CH2)=2
//	endif
//	if (WinNb == 1 && ChannelNb == 2 && CheckFit_V == 1)
//		ModifyGraph/W=DetectedEvents1#Average12  lsize(fit_AverageEvent1Ch2)=2
//	endif
//	if (WinNb == 2 && ChannelNb == 1 && CheckFit_V == 0)
//		//ModifyGraph/W=DetectedEvents2#EventChannel21  rgb(fit_TEMPEVENT2_CH1)=(0,0,65535)
//		ModifyGraph/W=DetectedEvents2#EventChannel21  lsize(fit_TEMPEVENT2_CH1)=2
//	endif
//	if (WinNb == 2 && ChannelNb == 1 && CheckFit_V == 1)
//		//ModifyGraph/W=DetectedEvents2#Average21  rgb(fit_AverageEvent2Ch1)=(0,0,65535)
//		ModifyGraph/W=DetectedEvents2#Average21  lsize(fit_AverageEvent2Ch1)=2
//	endif
End

Function SetCheckIntegralOffset(Ctrlname, Checked) : CheckBoxControl
	String CtrlName
	Variable Checked
	NVar CheckIntegralOffset = $("CheckIntegralOffset" + CtrlName[strlen(CtrlName) - 1]) // Append the last caracter (i.e. channel number)
	CheckIntegralOffset = 1 - CheckIntegralOffset // 1 if 0 or 0 if 1
End

Function SpAcAnGeneralIntegration(WinNb, ChannelNb)
	Variable WinNb, ChannelNb
	
	//***********************************************Local variables***********************************************
	// Name of the wave that will be fitted and the window where it is displayed
	String WaveToIntN, WindowN, TEMPSTR
	Variable debut, fin, AvgWidth, TheArea
	
	//*********************************** Bring back the necessary global variables and strings******************************
	// Variables depending on the winDow they are accessed from
	NVar CheckIntegralOffset_V = $("CheckIntegralOffset" + num2str(WinNb))
	NVar EventOrAvg_V = $("CheckIntegrate" + num2str(WinNb))
	NVar IntegralStart_V = $("IntegralStart" + num2str(WinNb))
	NVar IntegralEnd_V = $("IntegralEnd" + num2str(WinNb))
	NVar IntAvgWidth_V = $("IntAvgWidth" + num2str(WinNb))
	// Convert ms to seconds
	debut = IntegralStart_V * 0.001
	fin = IntegralEnd_V * 0.001
	AvgWidth = IntAvgWidth_V * 0.001
	//*********************************************Determining wave to fit*********************************
	If (EventOrAvg_V == 1) // Means integrate current event
		If (ChannelNb == WinNb) // If top graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#EventChannel" + num2str(ChannelNb)
			WaveToIntN = "TEMPEVENT" + num2str(WinNb)
		Else // if bottom graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#EventChannel" + num2str(WinNb) + num2str(ChannelNb)
			WaveToIntN = "TEMPEVENT" + "_ch" + num2str(ChannelNb)
		EndIf	
	ElseIf (EventOrAvg_V == 0) // Means fit average
		If (ChannelNb == WinNb) // If top graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#Average" + num2str(ChannelNb)
			WaveToIntN = "AverageEvent" + num2str(WinNb)
		Else // If bottom graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#Average" + num2str(WinNb) + num2str(ChannelNb)
			WaveToIntN = "AverageEvent" + num2str(WinNb) + "CH" + num2str(ChannelNb)
		EndIf
	EndIf	
	
	// temporary wave for the integral (for convenience)
	print WaveToIntN
	Duplicate/O/R=(debut, fin) $WaveToIntN, TEMPWAVE	
	
	
	GetWindow $(WindowN), WaveList
	Wave/T W_WaveList
		
	//Set as active subwindow the window we are going to work in
	DoWindow/F $("DetectedEvents" + num2str(WinNb))
	Execute/Q/Z "SetActiveSubWindow " + WindoWN // Don't know why but it is the only thing that works ($ doesn't)
	If (V_Flag != 0) // If execute didn't work
		Abort "Setting active subwindow failed"
	EndIf
	
	Variable counter
	For (counter = 1; counter < DimSize(W_WaveList, 0); counter += 1)
		TEMPSTR = W_WaveList[counter] [0]
		If (cmpstr(TEMPSTR[0, 5], "IntOff") == 0) // If the begginning of the waveN contains fit remove it from the graph
			RemoveFromGraph $W_WaveList[counter][0] // /W not used since active subwindow is supposed to be selected
		EndIf
	EndFor
	
	If (CheckintegralOffset_V == 0) // If no offset
		Duplicate/O/R=(Debut, fin) TempWave, LineAB
		LineAB = 0
	Else // If we offset the trace
		LineOnCurve($WaveToIntN, debut, fin, AvgWidth)
		Wave LineAB
		TempWave[x2pnt(TempWave, debut), x2pnt(TempWave, fin)] -= LineAB // Substract the line to the curve to offset on the appropriate range
	EndIf
	
	Duplicate/O LineAB, $("IntOff" + WaveToIntN)
	KillWaves LineAB
	
	AppendToGraph $("IntOff" + WaveToIntN) // /W not used since active subwindow is supposed to be selected
	If (ChannelNb == 2)
		ModifyGraph rgb($("IntOff" + WaveToIntN))=(0, 16000, (2^16 - 1))// Since the curve is red change the fit colour to blue
	EndIf
	
	// The actual  integration process
	TheArea = Area(TempWave, debut, fin) // caltulate the integral between debut and fin (trapezoidal)
	KillWaves TempWave
	Return TheArea
End

function SpAcAnIntegrate(CtrlName) : buttonControl
	String CtrlName
	
	NVar TheArea = $("Area" + CtrlName[strlen(CtrlName) - 1])
	NVar Channelnb = $("IntegrateChanSelectNb" + CtrlName[strlen(CtrlName) - 1])
	
	Variable WinNb = Str2num(CtrlName[strlen(CtrlName) - 1]) // The last caracter of controlName is the window number
	TheArea = SpAcAnGeneralIntegration(WinNb, ChannelNb)
	
	Execute/Q "ValDisplay DisplayArea" + num2str(WinNb) + " value=" + num2str(TheArea)
End

Function LineOnCurve(TheWave, LineStart, LineEnd, AvgWidth) // Draws the line between two points of a curve
	Wave TheWave
	Variable LineStart, LineEnd, AvgWidth
	
	//Locals
	Variable v_APreMean, v_BPostMean, v_Xpoints, V_Slope
		
	v_APreMean = Mean(TheWave, (LineStart -(AvgWidth/2)), (LineStart +(AvgWidth/2)))   // Variable equal to mean of 5 seconds before burst	
	v_BPostMean = Mean(TheWave, (LineEnd -(AvgWidth/2)), (LineEnd -(AvgWidth/2)))  // Variable equal to to 5 sec after burst
 
	Duplicate/O/R=(LineStart, LineEnd), TheWave, LineAB  ///makes destination waves with correct scaling
	
	v_Xpoints = numpnts(LineAB)									/// Number of points in X wave for calc slope
	v_Slope = (v_BPostMean-v_APreMean)  /  (v_Xpoints) 				/// Cqlc slope Rise/run
	//print v_Slope, "V_Bpostmean" = num2str(v_BpostMean)
	
	LineAB = ((v_APreMean)  + (v_Slope * p))    // p= point value on x wave, multiplied by slope

	//Print "v_Bpostmean" + num2str(v_BpostMean), lineAB[inf]  /// confirms wave arrives at 2nd mean with correct slope
	// Function returns the lineAB wave
END

// Charly End of Changes************************************************************************************************

Function FitAllEventsdbexp(ctrlName) : ButtonControl // Caution can work only for event corresponding to displayed channel (not synchronous events) because of windowN so far
	String ctrlName

	Variable/G V_FitOptions = 4 // V_fitOptions=4 suppresses the fit window (runs faster) (global because used w/ execute)
	// take care w/ wavestats great risk to take other event in the range (use w/ removeDoubleEvents) use prompt to put as option (new window + dowindow/k=)
	// Do also option for Check2080 & CheckBslnSD (Tags_Modified[p]=0)
	String channelNb=ctrlName[strlen(ctrlName) - 1] // Take last caracter as indicator of channel
	
	Nvar ACFTa = $"ACFT" + channelNb	// = 1 if window DetectedEventsX#AverageX exists.
	
	If (ACFTa == 0)
		Abort "Igor needs to guess the fit parameters from the average event. Please calculate the average event first."
	Endif
	
	DoAlert 1, "This function will try to fit a biexponential function to the decay of ALL tagged events. This may take a while. Continue ?"
	if (V_flag == 2)
		Abort
	endif
	
	Wave Tags=$("Tags" + channelNb)
	if (sum(Tags) == 0)
		Abort "No tagged events."
	endif
	
	Svar fileName= $("FileName" + channelNb)
	String TEMPSTR
	Variable Debut, Fin // start and end of fit
	String FitTypeStr = "dblexp_XOffset" // The curveFit keyword (take care with the constants and outputs if using an other.
	String ConstantFitParameters // The parameters to be held constant
	
	// Importing required waves
	Wave DetectedLevels=$("DetectedLevels" + channelNb)
	Wave RiseTime2080=$("RiseTime2080" + channelNb)
	Wave Occurrence=$("Occurrence" + channelNb)
	Wave BaselineY=$("BaselineY" + channelNb)
	Wave Peak=$("Peak" + channelNb)
	Wave avgEvnt=$("AverageEvent" + channelNb)

	//Importing global variables
	Nvar CheckBslnSD=$("CheckBslnSD" + channelNb)
	Nvar Baseline=$("Baseline" + channelNb)
	Nvar Decay=$("Decay" + channelNb)
	Nvar BslnSDinf=$("BslnSDinf" + channelNb)
	Nvar BslnSDsup=$("BslnSDsup" + channelNb)
	Nvar RTinf=$("RTinf" + channelNb)
	Nvar RTsup=$("RTsup" + channelNb)
	Nvar CheckRT2080=$("CheckRT2080" + channelNb)
	Nvar StartFit=$("StartFit" + channelNb)
	Nvar EndFit=$("EndFit" + channelNb)
	Nvar CheckOffset=$("CheckOffset" + channelNb)
	Nvar SI=$("SI" + channelNb)
	Nvar Ppnts=$("Ppnts" + channelNb)
	
	//Creating output matrix
	Make/O/N=(numpnts(tags), 8) tempMatriX
	//Labeling the MatriX
	SetDimLabel 1, 0, TauFast, tempMatriX
	SetDimLabel 1, 1, TauSlow, tempMatriX
	SetDimLabel 1, 2, AmplFast, tempMatriX
	SetDimLabel 1, 3, AmplSlow, tempMatriX
	SetDimLabel 1, 4, Offset, tempMatriX
	SetDimLabel 1, 5, KI, tempMatriX
	SetDimLabel 1, 6, FitStart, tempMatriX
	SetDimLabel 1, 7, FitEnd, tempMatriX

	// The following lines create the constraints for the fit
	String constraint1 = "K4 < K2" // to have K4 be tau fast and K2 be tau slow
	String constraint2 = "K4 < " + num2str(EndFit * 0.001) // decay can't be longer than the end of the event
	String constraint3 = "K2 < " + num2str(EndFit * 0.001) // decay can't be longer than the end of the event
	String constraint4 = "K4 > 0.00001" // K4 is positive but 0 gives a bug in Igor
	String constraint5 = "K2 > 0.00001" // same for K2
	Make/O/N=2/T fitAllEventsConstraints={constraint1, constraint2, constraint3, constraint4, constraint5}
	
	// parameters to be held constant
	K0 = 0			
	if (CheckOffset==1)
		ConstantFitParameters = "00000"
	else
		ConstantFitParameters = "10000"
	endif
	
	// geting first guesses for the fit parameters from the average event (helps a lot with small spontaneous events)
	duplicate/O avgEvnt, tempAvgEvnt
	Wavestats/Q/R=(0, EndFit) tempAvgEvnt
	FindLevel/Q/R=(V_MinLoc, Decay*.001) tempAvgEvnt V_min*(1-StartFit*.01)
	debut = V_LevelX; fin = EndFit*.001
	Execute/Z "CurveFit/Q/N/H=\"" + ConstantFitParameters + "\" " + FitTypeStr + " tempAvgEvnt (" + num2str(debut) + ", "+ num2str(fin) + ") /D/C=fitAllEventsConstraints"
	Variable avgK1=K1, avgK2=K2, avgK3=K3, avgK4=K4 //To keep the parameters for furteher use.
	
	// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
	// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
	String windowN="DetectedEvents" + channelNb + "#EventChannel" + ChannelNb
	String waveToFitN="TEMPEVENT" + channelNb
	GetWindow $WindowN, WaveList
	Wave/T W_WaveList
// probably useless	
	//Set as active subwindow the window we are going to work in
	DoWindow/F $("DetectedEvents" + channelNb)
	Execute/Q/Z "SetActiveSubWindow " + WindoWN // Don't know why but it is the only thing that works ($ doesn't)
	
	If (V_Flag != 0) // If execute didn't work
		Abort "Setting active subwindow failed"
	EndIf
	
	Variable i	
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
			RemoveFromGraph $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
			RemoveFromGraph $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
			RemoveFromGraph $W_WaveList[i][0]
		endif		
	endfor
// end	
//*********************************************** Preliminary steps : selecting events to fit ***********************************	
	
	// Removeing events w/ NaN occurrence if not already done
	ProcDeleteNaN(ctrlName)
	
	Duplicate/o Tags, $("Tags" + channelNb + "_modified")
	wave Tags_modified = $("Tags" + channelNb + "_modified")
	For (i = numpnts(DetectedLevels)-1 ; i > -1 ; i-=1)
		
		
		if (Occurrence[i] < Baseline*0.001 || Occurrence[i] > pnt2x($Filename,numpnts($Filename)-1) - Decay*0.001) // If event not completely in whole trace (for first or last events)
			Tags_modified[i] = 0
		endif
		
//// Select as options
//		if (CheckRT20801 == 1)
//			if (RiseTime20801[i] < RTinf1*.001 || RiseTime20801[i] > RTsup1*.001)
//				Tags1_modified[i] = 0		// Filter for rise time
//			endif
//		endif
//		if (CheckBslnSD1 == 1)
//			WaveStats/Q/R=(DetectedLevels1[i] - Baseline1*0.001, DetectedLevels1[i]) $FileName1
//			if (V_sdev < BslnSDinf1 || V_sdev > BslnSDsup1)		// Filter for baseline
//				Tags1_modified[i] = 0
//			endif
//		endif
//		
////END
	Endfor
	
	
	for (i = 0; i < numpnts(DetectedLevels); i += 1)
		if (Tags_modified[i] == 0) // If event not selected
			tempMatriX[i][] = NaN // All columns set to NaN in current row
		else
			// ----------------- SAVING THE EVENT TO BE FITTED IN A WAVE -----------------
			Duplicate/o/R=(Occurrence[i] - Baseline*.001, Occurrence[i] + Decay*.001) $FileName TEMPWAVE
			TEMPWAVE = TEMPWAVE - BaselineY[i] // Trace offset
			SetScale/P x -Baseline*.001, (SI*1e-6), "s", TEMPWAVE
			Duplicate/O TEMPWAVE, TEMPWAVE2
			

			// Implement supperimpose all fits w/ opssibility to get trace (to remove event that doesn't fit)
			// Voir ce dont Philippe a parle la derniere fois.
			
			
			// Determine the limits of the fit
			Smooth/B Ppnts, TEMPWAVE2
			Wavestats/Q/R=(0, EndFit) TEMPWAVE2
			//FindPeak/N/Q/R=(0, Decay1*.001) TEMPWAVE2
			FindLevel/Q/R=(V_MinLoc, Decay*.001) TEMPWAVE2 V_min*(1-StartFit*.01)
			
			If (V_Flag == 0) // level was found
				debut = V_LevelX; fin = EndFit*.001
			else
				// the beginning point for the fit was not found.
				print "Fitting of event number " + num2str(i) + " was not possible, beginning not found. This may be an artifact."
				tempMatriX[i][]=NaN // Maybe use zero instead in the future to discriminate fits that were not attempted from fits that didn't work
				continue //try next fit
			endif

			// **************************************The fit command**************************
			
			k1=avgK1; K2=avgK2; K3=avgK3; K4=avgK4 // this is to use the initial guess from the avg event.
			Execute/Z "CurveFit/Q/G/N/H=\"" + ConstantFitParameters + "\" " + FitTypeStr + " TEMPWAVE (" + num2str(debut) + ", "+ num2str(fin) + ") /D/C=fitAllEventsConstraints" // The actual fit

			If (V_Flag != 0) // If fit didn't work
				tempMatriX[i][]=NaN // Maybe use zero instead in the future to discriminate fits that were not attempted from fits that didn't work
				continue //try next fit
			elseif ((K4==0)  || (K4==endFit) || (K2==0) || (K2==endFit)) // If coefficients are moved to boundaries
				tempMatriX[i][]=NaN // Maybe use zero instead in the future to discriminate fits that were not attempted from fits that didn't work
				continue //try next fit
			endIf
			
			variable KI // The wheighed decay time constant
			KI= (K2*1000) * (K1/(K1+K3)) + (K4*1000) * (K3/K1+K3)
			
			if ((KI > 0) && (KI < endFit))
				// *************change the following lines if changing the fit type*************************
				tempMatriX[i][%TauFast] = K4 * 1000 // To convert to ms
				tempMatriX[i][%TauSlow] = K2 * 1000 // To convert to ms
				tempMatriX[i][%Offset] = K0
				tempMatriX[i][%AmplFast] = K3
				tempMatriX[i][%AmplSlow] = K1
				tempMatriX[i][%FitStart] = debut
				tempMatriX[i][%FitEnd] = fin
				// *****************************************************************************************
			else
				tempMatriX[i][]=NaN // Maybe use zero instead in the future to discriminate fits that were not attempted from fits that didn't work
				continue //try next fit
			endif
		endif
	endfor

	//creating temporary matriX subRanges as display function input
	Duplicate/O/R=[][0, 0] tempMatriX, fitAllTauFast //caution labels not used
	Duplicate/O/R=[][1, 1] tempMatriX, fitAllTauSlow//caution labels not used
	Duplicate/O/R=[][2, 2] tempMatriX, fitAllPercentFast, tempFitAllAmplFast //caution labels not used
//	fitAllAmplFast = (tempMatriX[][%"AmplFast"] / (tempMatriX[][%"AmplFast"] +tempMatriX[][%"AmplSlow"])) * 100
	Duplicate/O/R=[][3, 3] tempMatriX, fitAllPercentSlow, tempFitAllAmplSlow //caution labels not used
//	fitAllAmplSlow = (tempMatriX[][%"AmplSlow"] / (tempMatriX[][%"AmplFast"] +tempMatriX[][%"AmplSlow"])) * 100
	fitAllPercentFast = (tempFitAllAmplFast / (tempFitAllAmplFast + tempFitAllAmplSlow) * 100)
	fitAllPercentSlow = (tempFitAllAmplSlow / (tempFitAllAmplFast + tempFitAllAmplSlow) * 100)
	KillWaves tempFitAllAmplFast, tempFitAllAmplSlow
	
	Duplicate/O/R=[][4, 4] tempMatriX, fitAllFitOffset//caution labels not used
	duplicate/O FitAllPercentFast, tempKI
	tempKI = (fitAllPercentFast/100) * fitAllTauFAst + (fitAllPercentSlow /  100) * fitAllTauSlow
	tempMatriX[][%KI]=tempKI[p]
	
//	DisplayFitResults(fitAllTauFast, fitAllTauSlow, fitAllAmplFast, fitAllAmplSlow, fitAllFitOffset, str2num(channelNb)) // Normally Takes waves as input (TauFast, TauSlow, AmpFast, AmpSlow, fitOffset
	Duplicate/O tempMatriX, $("fitParamsFitAllEvents" + channelNb)
	DoWindow/K FitResults 
	Edit/K=1/N=FitResultsTable $("fitParamsFitAllEvents" + channelNb).ld as "Fit Results"
	DoWindow/C FitResults 
	KillWaves tempMatriX, fitAllTauFast, FitAllTauSlow, fitAllPercentFast, fitAllPercentSlow, fitAllFitOffset, tempKI
End

Function DisplayFitResults(TauFast, TauSlow, AmpFast, AmpSlow, FitOffset, channel_number)
	Wave TauFast, TauSlow, AmpFast, AmpSlow, FitOffset
	Variable channel_number
	//
	Duplicate/o TauFast, W_HIST_TauFast
	Duplicate/o TauSlow, W_HIST_TauSlow
	Duplicate/o FitOffset, W_HIST_FitOffset
	Duplicate/o AmpFast, W_HIST_AmpFast
	Duplicate/o AmpSlow, W_HIST_AmpSlow
	Duplicate/o TauFast, KI
	KI = 0.01*AmpFast*TauFast + 0.01*AmpSlow*TauSlow
	Duplicate/o KI, W_HIST_KI
	//
	Wavestats/Q TauFast; Histogram/B={0,1,V_max} TauFast,W_HIST_TauFast 
	Wavestats/Q TauSlow; Histogram/B={0,1,V_max} TauSlow,W_HIST_TauSlow
	Wavestats/Q FitOffset; Histogram/B=1 FitOffset,W_HIST_FitOffset
	Wavestats/Q AmpFast; Histogram/B={0,5,V_max/5} AmpFast,W_HIST_AmpFast
	Wavestats/Q AmpSlow; Histogram/B={0,5,V_max/5} AmpSlow,W_HIST_AmpSlow
	Wavestats/Q KI; Histogram/B={0,1,V_max} KI,W_HIST_KI
	//
	DoWindow/K Window_Fits
	Display/W=(50,30,500,500)/L=Left1/B=Bottom1 W_HIST_TauFast as "Fit parameters"; DoWindow/C Window_Fits
	Appendtograph/L=Left1/B=Bottom1 W_HIST_TauSlow
	Appendtograph/L=Left2/B=Bottom2 W_HIST_AmpFast; Appendtograph/L=Left2/B=Bottom2 W_HIST_AmpSlow
	Appendtograph/L=Left3/B=Bottom3 W_HIST_KI
	Appendtograph/L=Left4/B=Bottom4 W_HIST_FitOffset
	//
	ModifyGraph axisEnab(Left1)={0.60,0.99}; 			ModifyGraph axisEnab(Left2)={0.60,0.99}
	ModifyGraph axisEnab(Left3)={0.01,0.45}; 			ModifyGraph axisEnab(Left4)={0.01,0.45}
	ModifyGraph axisEnab(Bottom1)={0.05,0.45}; 		ModifyGraph axisEnab(Bottom2)={0.55,0.95}
	ModifyGraph axisEnab(Bottom3)={0.05,0.45}; 		ModifyGraph axisEnab(Bottom4)={0.55,0.95}
	ModifyGraph freePos={0,kwFraction}; 				ModifyGraph margin(bottom)=43
	ModifyGraph freePos(Left1)={0.03,kwFraction}; 		ModifyGraph freePos(Left2)={0.53,kwFraction}
	ModifyGraph freePos(Left3)={0.03,kwFraction}; 		ModifyGraph freePos(Left4)={0.53,kwFraction}
	ModifyGraph freePos(Bottom1)={0.58,kwFraction}; 	ModifyGraph freePos(Bottom3)={0.0,kwFraction}
	ModifyGraph freePos(Bottom2)={0.58,kwFraction}; 	ModifyGraph freePos(Bottom4)={0.0,kwFraction}
	//
	ModifyGraph wbRGB=(56576,56576,56576); ModifyGraph gbRGB=(56576,56576,56576)
	ModifyGraph rgb(W_HIST_TauFast)=(0,39168,0); ModifyGraph rgb(W_HIST_TauSlow)=(65280,0,52224)
	ModifyGraph rgb(W_HIST_AmpFast)=(0,39168,0); ModifyGraph rgb(W_HIST_AmpSlow)=(65280,0,52224)
	ModifyGraph lsize=1.5
	ModifyGraph rgb(W_HIST_KI)=(0,0,0)
	SetAxis Bottom2 0, 100
	Label Bottom1 "Time constant (ms)";  ModifyGraph lblPosMode=0,lblPos(Bottom1)=43,lblLatPos=0
	Label Bottom2 "Amplitude (%)";  ModifyGraph lblPosMode=0,lblPos(Bottom2)=43,lblLatPos=0
	Label Bottom3 "Kinetic Index (ms)"; ModifyGraph lblPosMode(Bottom3)=1,lblMargin(Bottom3)=1
	Label Bottom4 "Offset (xx)"; ModifyGraph lblPosMode(Bottom4)=1,lblMargin(Bottom3)=1
	SetDrawEnv fstyle= 1, textrgb= (0,39168,0); DrawText 0.3,0.050,"Fast"
	SetDrawEnv fstyle= 1, textrgb= (65280,0,52224); DrawText 0.3,0.080,"Slow"
	ModifyGraph margin(top)=30
	FindLevels/Q Tags1_modified 1
	SetDrawEnv fstyle= 1; DrawText 0.030,-0.03,"Histograms of fit parameters. " + num2str(V_LevelsFound) + " events fitted."
End

Function FitAllEventssigdbexp1(ctrlName) : ButtonControl
	String ctrlName
	
	DoAlert 1, "This function will try to fit the product of a sigmoid and a biexponential function on ALL tagged events. This may take a while. Continue ?"
	if (V_flag == 2)
		Abort
	endif
	Wave Tags1
	if (sum(Tags1, 0, numpnts(Tags1)-1) == 0)
		Abort "No tagged events."
	endif
	Variable i
	Variable V_FitOptions=4
	Svar FileName1
	String TEMPSTR
	Variable Debut, Fin
	Wave DetectedLevels1, RiseTime20801, Occurrence1, BaselineY1, Peak1, RealAmp1
	Nvar CheckBslnSD1, Baseline1, Decay1, BslnSDinf1, BslnSDsup1, RTinf1, RTsup1, CheckRT20801, StartFitsigdbexp1, EndFitsigdbexp1, SI1, Ppnts1, CheckOffset1
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpTauFast
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpTauSlow
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpAmpFast
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpAmpSlow
	Make/N=(numpnts(DetectedLevels1))/o KI1_sigdbexp
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpOffset
	// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
	// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
	GetWindow DetectedEvents1#EventChannel1, WaveList
	Wave/T W_WaveList = W_WaveList
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif		
	endfor
	Duplicate/o Tags1, Tags1_modified
	For (i = numpnts(DetectedLevels)-1 ; i > -1 ; i-=1)
		if (Numtype(Occurrence1[i]) == 2)		// Removing bad events (NaN)
			Tags1_modified[i] = 0
		endif
		if (Occurrence1[i] < Baseline1*0.001 || Occurrence1[i] > pnt2x($Filename1,numpnts($Filename1)-1) - Decay1*0.001)
			Tags1_modified[i] = 0
		endif
		if (CheckRT20801 == 1)
			if (RiseTime20801[i] < RTinf1*.001 || RiseTime20801[i] > RTsup1*.001)
				Tags1_modified[i] = 0		// Filter for rise time
			endif
		endif
		if (CheckBslnSD1 == 1)
			WaveStats/Q/R=(DetectedLevels1[i] - Baseline1*0.001, DetectedLevels1[i]) $FileName1
			if (V_sdev < BslnSDinf1 || V_sdev > BslnSDsup1)		// Filter for baseline
				Tags1_modified[i] = 0
			endif
		endif
		if (Occurrence1[i+1] < Occurrence1[i] + Decay1*0.001)	// Filter for events with next event in decay 
			Tags1_modified[i] = 0
		endif
	Endfor
	Duplicate/o Tags1, Tags1_modified
	for (i = 0; i < numpnts(DetectedLevels1); i += 1)
		if (Tags1_modified[i] == 0)
			FitSigdbexpTauFast[i] = NaN; FitSigdbexpTauSlow[i] = NaN; FitSigdbexpAmpFast[i] = NaN; FitSigdbexpAmpSlow[i] = NaN
		else
			// ----------------- SAVING THE EVENT TO BE FITTED IN A WAVE -----------------
			Duplicate/o/R=(Occurrence1[i] - Baseline1*.001, Occurrence1[i] + Decay1*.001) $FileName1 DUMMY
			DUMMY = DUMMY - BaselineY1[i]
			SetScale/P x -Baseline1*.001, (SI1*1e-6), "s", DUMMY
			// First, determine the limits of the fit
			// Debut is where the fit starts
			if (StartFitsigdbexp1 < 50)	
				Duplicate/o/R=(-.01, 0) DUMMY TEMPWAVE
				TEMPWAVE = TEMPWAVE[numpnts(TEMPWAVE)-1-p]
				SetScale/P x 0,SI1*1e-6, "s", TEMPWAVE
				FindLevel/Q TEMPWAVE RealAmp1[i]*0.01*StartFitsigdbexp1
				Debut = - V_LevelX
			endif
			if (StartFitsigdbexp1 == 50)
				Debut = 0
			endif
			if (StartFitsigdbexp1 > 50)
				FindLevel/Q/R=(0,0.01) DUMMY RealAmp1[i]*0.01*StartFitsigdbexp1
				Debut = V_LevelX
			endif
			// Fin is where the fit ends
			fin = EndFitsigdbexp1*.001
			// The real work starts here.
			// First, I fit a biexponential function to the decay of the average event to give an initial guess of the decay time constants
			//FindPeak/N/Q/R=(0,Decay1*.001) DUMMY
			WaveStats/Q/R=(0,Decay1*.001) DUMMY
			K0 = 0
			if (CheckOffset1==1)
				CurveFit/Q/H="00000" dblexp DUMMY(V_MinLoc + (10*SI1*1e-6), fin) /D // Starts the fit 10 points after the peak
			else
				CurveFit/Q/H="10000" dblexp DUMMY(V_MinLoc + (10*SI1*1e-6), fin) /D // Starts the fit 10 points after the peak
			endif
			Make/o/N=9 SigExp
			SigExp[4]=K3/(K3+K1)	// Dbexp : amplitude fast
			SigExp[5]=K4			// Dbexp : fast time constant
			SigExp[6]=K1/(K3+K1)	// Dbexp : amplitude slow
			SigExp[7]=K2			// Dbexp : slow time constant
			SigExp[8]=K0			// Dbexp : offset
			// Then, we fit a sigmoid function to the rise phase of the average event
			K0 = 0; SigExp[0]=K0		// Sigmoid : base
			CurveFit/Q/H="1000" Sigmoid DUMMY(-V_MinLoc,V_MinLoc) /D
			SigExp[1]=K1			// Sigmoid : max
			SigExp[2]=K2			// Sigmoid : xhalf
			SigExp[3]=K3			// Sigmoid : rate
			// The real fitting procedure
			Make/D/N=8/O W_coef
			W_coef[0] = {SigExp[1],SigExp[2],SigExp[3],0,SigExp[4],SigExp[5],SigExp[6],SigExp[7]}
			// W_coef = {max, xhalf, rate, baseline, amplitude fast, fast time constant, amplitude slow, slow time constant}
			// W_coef[0] = max
			// W_coef[1] = xhalf
			// W_coef[2] = rate
			// W_coef[3] = baseline
			// W_coef[4] = amplitude fast
			// W_coef[5] = fast time constant
			// W_coef[6] = amplitude slow
			// W_coef[7] = slow time constant
			if (CheckOffset1==1)
				FuncFit/H="00000000" Sigmoid_Dbexp W_coef DUMMY(debut,fin) /D	// Free offset
			else
				FuncFit/H="00010000" Sigmoid_Dbexp W_coef DUMMY(debut,fin) /D	// Null offset
			endif
			// The following lines are for displaying the coefficients in the control bar (nicer than the raw result in the command window, isn't it ?) 
			FitSigdbexpTauFast[i] = (1/W_coef[5])*1000	// taufast
			FitSigdbexpTauSlow[i] = (1/W_coef[7])*1000	// tauslow
			FitSigdbexpAmpFast[i] = (W_coef[4] / ( W_coef[4] + W_coef[6] ))*100	// % fast
			FitSigdbexpAmpSlow[i] = (W_coef[6] / ( W_coef[4] + W_coef[6] ))*100	// % slow
			FitSigdbexpOffset[i] = W_coef[3]
		endif
	endfor
	DisplayFitResults(FitSigdbexpTauFast, FitSigdbexpTauSlow, FitSigdbexpAmpFast, FitSigdbexpAmpSlow, FitSigdbexpOffset, 1)
	
	//KI1_sigdbexp = 0.01*FitSigdbexpAmpFast*FitSigdbexpTauFast + 0.01*FitSigdbexpAmpSlow*FitSigdbexpTauSlow
	//Duplicate/o KI1_sigdbexp, W_HIST_KI1_sigdbexp
	///Histogram/B={0,1,50} KI1_sigdbexp,W_HIST_KI1_sigdbexp
	//DoWindow/K Window_KI1_sigdbexp; display W_HIST_KI1_sigdbexp; DoWindow/C Window_KI1_sigdbexp
	//Wavestats/Q KI1_sigdbexp
	//SetAxis bottom -V_sdev, V_avg + 2*V_sdev
	//ModifyGraph standoff=0
	//FindLevels/Q Tags1_modified, 1
	//DrawText 0.04,0.08,"Number of fits: " + num2str(V_LevelsFound)
End

Function FAvsT(WaveOccurrence,WaveAmplitude,VarThrshld)
	Wave WaveOccurrence, WaveAmplitude
	Variable VarThrshld
	String TITLE ; TITLE="Amplitude"
	Nvar Length, FvsTBin, AvsTBin, AmpHistoBin, CheckAvsTShowAv, CheckAvsTShowEvents, ISIHistoBin
	Variable BinF, BinA, BinA2, i, X_start
	String TITLE1, TITLE2, TITLE3, TITLE4

	BinF = FvsTBin ; BinA = AvsTBin ; BinA2 = AmpHistoBin
	
	// Here, I could put some error messages in case the bin values are not good
	
	TITLE1 = "Frequency vs Time, bin = " + num2str(BinF) + " s"
	TITLE2 = "Amplitude vs Time"
	TITLE3 = "Amplitude vs Time, bin = " + num2str(BinA) + " s"
	TITLE4 = "Amplitude Histogram"
	
	Duplicate/o WaveOccurrence WaveOccurrence2
	Duplicate/o WaveAmplitude, WaveAmplitude2
	for (i = (numpnts(WaveOccurrence2)-1) ; i > -1 ; i-=1)
		if (numtype(WaveOccurrence2[i]) == 2 || WaveOccurrence2[i] == 0)
			DeletePoints i, 1, WaveOccurrence2
			DeletePoints i, 1, WaveAmplitude2
		endif
	endfor
	WaveAmplitude2 = - WaveAmplitude2
	
	// ISI Histogram
	Duplicate/o WaveOccurrence WaveOccurrence_TEMP1
	Duplicate/o WaveOccurrence WaveOccurrence_TEMP2
	InsertPoints 0, 1, WaveOccurrence_TEMP1
	InsertPoints numpnts(WaveOccurrence), 1, WaveOccurrence_TEMP2
	Make/N=(numpnts(WaveOccurrence)+1)/o WaveISI_TEMP
	WaveISI_TEMP = WaveOccurrence_TEMP2 - WaveOccurrence_TEMP1
	DeletePoints (numpnts(WaveISI_TEMP)-1), 1, WaveISI_TEMP
	DeletePoints 0, 1, WaveISI_TEMP
	WaveStats/Q WaveISI_TEMP
	Make/N=(floor((V_max/ISIHistoBin)*1.3))/O ISI_Histogram
	Histogram/B={0,ISIHistoBin,floor((V_max/ISIHistoBin)*1.1)} WaveISI_TEMP ISI_Histogram
	KillWaves WaveOccurrence_TEMP1, WaveOccurrence_TEMP2, WaveISI_TEMP
	
	// FIRST, calculate Freq vs Time
	Make/N=0/o TEMP
	Variable X1, X2
	FindLevel/Q WaveOccurrence2, BinF
	if (V_flag == 1)
		if (BinF < WaveOccurrence2[0])
			X2 = 0
		endif
		if (BinF > WaveOccurrence2[numpnts(WaveOccurrence2)-1])
			X2 = numpnts(WaveOccurrence2)
		endif
	else
		X2 = floor(V_LevelX) + 1
	endif
	InsertPoints 0,1,TEMP
	TEMP[0] = X2
	i=1
	For (X_Start = BinF; X_Start < (WaveOccurrence2[numpnts(WaveOccurrence2)-1] - BinF); X_Start += BinF) 
		if (X_start < WaveOccurrence2[0])
			X1 = 0
		else
			FindLevel/Q WaveOccurrence2, X_start
			// permet de trouver le # de l'vnement qui tombe juste avant X_Start
			// Cet vnement est de dernier en dehors de la zone d'intrt (largeur d'un bin)
			X1 = floor(V_LevelX)
		endif
		if ((X_start+BinF) < WaveOccurrence2[0])
			X2 = 0
		else
			Findlevel/Q WaveOccurrence2 (X_Start + BinF)
			//permet de trouver le # de l'vnement qui tombe juste avant la fin du bin
			//Cet vnement est le dernier contenu dans la zone d'intrt
			X2 = floor(V_LevelX)
		endif
		InsertPoints i, 1, TEMP
		TEMP[i] = X2 - X1	// nombre d'vnements  l'intrieur du bin
		i+=1
	Endfor
	
	SetScale/P x, 0,BinF,"s",TEMP
	TEMP = TEMP / BinF
	Duplicate/o TEMP FreqVsTime
	KillWaves TEMP
	
	// => result is wave FreqVsTime
	
	// Plot amplitude (binned) vs. time

	Make/N=0/o TEMP; Make/N=0/o TEMP2
	X1 = NaN
	FindLevel/Q/P WaveOccurrence2, BinA
	// Premier Bin
	if (V_flag == 0)
		X1 = 0
		X2 = floor(V_LevelX)
		InsertPoints 0,1,TEMP
		InsertPoints 0,1,TEMP2
		Wavestats/Q/R=[X1,X2] WaveAmplitude2
		TEMP[i] = V_avg; TEMP2[i] = V_sdev
	else
		X2 = NaN
		InsertPoints 0,1,TEMP
		InsertPoints 0,1,TEMP2
		TEMP[i] = NaN
		TEMP2[i] = NaN
	endif
	// Bins suivants
	i=1
	For (X_Start = BinA; X_Start < (WaveOccurrence2[numpnts(WaveOccurrence2)-1] - BinA); X_Start += BinA)
		FindLevel/Q WaveOccurrence2, X_start
		// permet de trouver le # de l'vnement qui tombe juste avant X_Start
		// Cet vnement est de dernier en dehors de la zone d'intrt (largeur d'un bin)
		if (V_flag == 0)
			if ( WaveOccurrence2[floor(V_LevelX) + 1] < (X_start + BinA) )
				X1 = floor(V_LevelX) + 1
			else
				X1 = NaN
			endif
		else
			FindLevel/Q WaveOccurrence2, (X_start+BinA)
			if (V_flag == 0)
				X1 = 0
			else
				X1 = NaN
			endif
		endif
		FindLevel/Q WaveOccurrence2, (X_start+BinA)
		//permet de trouver le # de l'vnement qui tombe juste avant la fin du bin
		//Cet vnement est le dernier contenu dans la zone d'intrt
		if (V_flag == 0)
			if ( WaveOccurrence2[floor(V_LevelX)] > X_start )
				X2 = floor(V_LevelX)
			else
				X2 = NaN
			endif
		else
			X2 = NaN
		endif
		InsertPoints i, 1, TEMP
		InsertPoints i, 1, TEMP2
		Wavestats/Q/R=[X1,X2] WaveAmplitude2
		TEMP[i] = V_avg; TEMP2[i] = V_sdev
		i+=1
	Endfor
	SetScale/P x, BinA/2,BinA,"s",TEMP
	Duplicate/o TEMP AmpVsTime ; KillWaves TEMP
	Duplicate/o TEMP2 AmpVsTime_sdev1; KillWaves TEMP2
	Duplicate/o AmpVsTime_sdev1, AmpVsTime_sdev2
	//AmpVsTime_sdev2 = -AmpVsTime_sdev1

	// Cumulative amplitude graphs
	
	Duplicate/o WaveAmplitude CumAmp
	CumAmp = - CumAmp
	Sort/R CumAmp CumAmp
	for (i = numpnts(CumAmp)-1; i>-1; i -= 1)
		if (CumAmp[i]==0)
			Deletepoints i,1, CumAmp
		endif
	endfor
	sort CumAmp CumAmp
	Duplicate/o CumAmp WaveXCumAmp
	WaveXCumAmp = p
	WaveXCumAmp = WaveXCumAmp / ((numpnts(WaveXCumAmp)-1))
	
	Variable PlotLimitLeft = 220
	Variable PlotLimitTop = 50
	Variable PlotSize = 200
	Variable PlotInterval = 20
	
	// Plot Freq vs Time
	DoWindow/K Window_FreqvsTime
	Display/W=(PlotLimitLeft,PlotLimitTop,PlotLimitLeft+PlotSize,PlotLimitTop+PlotSize) FreqVsTime as "Frequency vs Time"
	DoWindow/C Window_FreqvsTime; ModifyGraph standoff=0; ModifyGraph mode=0
	Label left "Frequency (Hz)"; Label bottom "Time (s)"
	Wavestats/Q FreqVsTime ; SetAxis left 0,V_max*1.2 ; SetAxis bottom 0,Length
	ModifyGraph mode=6; ModifyGraph rgb=(0,0,0)
	ControlBar 30
	CheckBox CheckBoxFvsTGrid	pos={50,7},size={40,20},title="Grid", proc=ProcCheckBoxFvsTGrid
	Button ButtonSaveFigFvsT		pos={100,5},size={60,20},title="Save Fig.", proc=ProcSaveFig
	Button ButtonFvsTClose		pos={160,5},size={40,20},title="Close", proc=ProcButtonFvsTClose
	Button ButtonFvsTCloseAll		pos={200,5},size={60,20},title="Close All", proc=ProcButtonFAvsTCloseAll
	
	// Plot Amp vs Occ & Amp vs Time
	DoWindow/K Window_AmpvsTime
	Display/W=(PlotLimitLeft + PlotSize + PlotInterval,PlotLimitTop,PlotLimitLeft+2*PlotSize + PlotInterval,PlotLimitTop+PlotSize) as "Amplitude vs Time"
	DoWindow/C Window_AmpvsTime
	if (CheckAvsTShowEvents)
		Appendtograph/W=Window_AmpvsTime WaveAmplitude2 vs WaveOccurrence2
		ModifyGraph mode(WaveAmplitude2)=3,marker(WaveAmplitude2)=19,msize(WaveAmplitude2)=1 ; ModifyGraph rgb(WaveAmplitude2)=(0,0,0)
	endif
	if (CheckAvsTShowAv)
		Appendtograph/W=Window_AmpvsTime AmpVsTime
		ModifyGraph mode(AmpVsTime)=3, marker(AmpVsTime)=19
		ModifyGraph msize(AmpVsTime)=3,opaque(AmpVsTime)=1,rgb(AmpVsTime)=(0,43520,65280)
		ModifyGraph useMrkStrokeRGB(AmpVsTime)=1
		ErrorBars AmpVsTime Y,wave=(AmpVsTime_sdev1,AmpVsTime_sdev2)
	endif
	SetAxis bottom 0, Length ; ModifyGraph standoff=0
	Wavestats/Q WaveAmplitude2 ; SetAxis Left 0, 1.2*V_max; SetAxis bottom 0,Length	
	
	Make/o/N=2 WaveXAvsT, WaveYAvsT
	WaveXAvsT[0] = 0 ; WaveXAvsT[1] = Length ; WaveYAvsT = VarThrshld
	AppendToGraph WaveYAvsT vs WaveXAvsT ; ModifyGraph lstyle(WaveYAvsT)=3
	Label left "Amplitude (xx)" ; Label bottom "Time (s)"
	
	ControlBar 30
	CheckBox CheckBoxAvsTGrid	pos={50,7},size={40,20},title="Grid", proc=ProcCheckBoxAvsTGrid
	Button ButtonSaveFigAvsT		pos={100,5},size={60,20},title="Save Fig.", proc=ProcSaveFig
	Button ButtonAvsTClose		pos={160,5},size={40,20},title="Close", proc=ProcButtonAvsTClose
	Button ButtonAvsTCloseAll		pos={200,5},size={60,20},title="Close All", proc=ProcButtonFAvsTCloseAll
	
	// Plot Amplitude histogram
	Duplicate/o WaveAmplitude2 AmplitudeHistogram; Wavestats/Q WaveAmplitude2
	Histogram/B={0,BinA2,(V_max/binA2)*1.3} WaveAmplitude2 AmplitudeHistogram
	DoWindow/K Window_AmpHist
	Display/W=(PlotLimitLeft,PlotLimitTop+PlotSize + PlotInterval,PlotLimitLeft+PlotSize,PlotLimitTop+2*PlotSize + PlotInterval) AmplitudeHistogram as "Amplitude Histogram"
	DoWindow/C Window_AmpHist
	Label left "N", Label bottom "Amplitude (xx)"
	ModifyGraph rgb(AmplitudeHistogram)=(0,0,0)
	
	ControlBar 30
	CheckBox CheckBoxAmpHistGrid	pos={50,7},size={40,20},title="Grid", proc=ProcCheckBoxAmpHistGrid
	Button ButtonSaveFigAmpHist		pos={100,5},size={60,20},title="Save Fig.", proc=ProcSaveFig
	Button ButtonAmpHistClose		pos={160,5},size={40,20},title="Close", proc=ProcButtonAmpHistClose
	Button ButtonAmpHistCloseAll		pos={200,5},size={60,20},title="Close All", proc=ProcButtonFAvsTCloseAll
	
	// Plot cumulative amplitude
	DoWindow/K Window_CumAmp
	Display/W=(PlotLimitLeft + PlotSize + PlotInterval,PlotLimitTop + PlotSize + PlotInterval,PlotLimitLeft + 2*PlotSize + PlotInterval,PlotLimitTop+2*PlotSize + PlotInterval) WaveXCumAmp vs CumAmp as "Cumulative Amplitude Histogram"
	DoWindow/C Window_CumAmp
	Label left "Cumulative probability", Label bottom "Amplitude (--)"
	ModifyGraph/W=Window_CumAmp rgb(WaveXCumAmp)=(0,0,0)
	
	ControlBar 30
	CheckBox CheckBoxCumAmpGrid	pos={50,7},size={40,20},title="Grid", proc=ProcCheckBoxCumAmpGrid
	Button ButtonSaveCumAmpHist		pos={100,5},size={60,20},title="Save Fig.", proc=ProcSaveFig
	Button ButtonCumAmpClose		pos={160,5},size={40,20},title="Close", proc=ProcButtonCumAmpClose
	Button ButtonCumAmpCloseAll		pos={200,5},size={60,20},title="Close All", proc=ProcButtonFAvsTCloseAll

	// Plot ISI Histogram
	DoWindow/K Window_ISI_Histogram
	Display/W=(PlotLimitLeft + 2*PlotSize + 2*PlotInterval,PlotLimitTop,PlotLimitLeft+3*PlotSize + 2*PlotInterval,PlotLimitTop+PlotSize) ISI_Histogram as "ISI Histogram"
	DoWindow/C Window_ISI_Histogram
	ModifyGraph/W=Window_ISI_Histogram rgb(ISI_Histogram)=(0,0,0)
	
	ControlBar 30
	CheckBox CheckBoxISIHistoGrid	pos={50,7},size={40,20},title="Grid", proc=ProcCheckISIHistoGrid
	Button ButtonSaveISIHistoHist		pos={100,5},size={60,20},title="Save Fig.", proc=ProcSaveFig
	Button ButtonISIHistoClose		pos={160,5},size={40,20},title="Close", proc=ProcButtonISIHistoClose
	Button ButtonISIHistoCloseAll		pos={200,5},size={60,20},title="Close All", proc=ProcButtonFAvsTCloseAll
End

// ============ FUNCTIONS RELATED TO FAvsT(): buttons, checkboxes etc ============
FUNCTION ProcButtonFvsTClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_FreqvsTime
END

FUNCTION ProcButtonFAvsTCloseAll(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_FreqvsTime
	DoWindow/K Window_AmpvsTime
	DoWindow/K Window_AmpHist
	DoWindow/K Window_ISI_Histogram
	DoWindow/K Window_CumAmp
END

FUNCTION ProcCheckBoxFvsTGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_FreqvsTime grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_FreqvsTime grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_FreqvsTime grid(left)=0
		ModifyGraph/W=Window_FreqvsTime grid(bottom)=0
	endif
END

FUNCTION ProcButtonAvsTClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_AmpvsTime
END

FUNCTION ProcCheckBoxAvsTGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_AmpvsTime grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_AmpvsTime grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_AmpvsTime grid(left)=0
		ModifyGraph/W=Window_AmpvsTime grid(bottom)=0
	endif
END

FUNCTION ProcButtonAmpHistClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_AmpHist
END

FUNCTION ProcCheckBoxAmpHistGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_AmpHist grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_AmpHist grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_AmpHist grid(left)=0
		ModifyGraph/W=Window_AmpHist grid(bottom)=0
	endif
END

FUNCTION ProcButtonISIHistoClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_ISI_Histogram
END

FUNCTION ProcButtonCumAmpClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_CumAmp
END

FUNCTION ProcCheckISIHistoGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_ISI_Histogram grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_ISI_Histogram grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_ISI_Histogram grid(left)=0
		ModifyGraph/W=Window_ISI_Histogram grid(bottom)=0
	endif
END

FUNCTION ProcCheckBoxCumAmpGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_CumAmp grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_CumAmp grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_CumAmp grid(left)=0
		ModifyGraph/W=Window_CumAmp grid(bottom)=0
	endif
END
// ============ END OF FUNCTIONS RELATED TO FAvsT() ============

Function Horiz_Expand_KeepCursor() : GraphMarquee
		string format
		Nvar checkedCsr=checkedCsr,XA=XA,YA=YA
	GetMarquee/K left,bottom
	format = "flag: %g; left: %g; top: %g; right: %g; bottom: %g\r"
	if (checkedCsr==1)
		XA=hcsr(A)
		YA=vcsr(A)
		SetAxis/W=MainWindow bottom V_left,V_right
		cursor/F/A=1/W=MainWindow/A=1/H=1 A Smoothed_Trace XA,YA
	else
		SetAxis/W=MainWindow bottom V_left,V_right
	endif
End


//*****************************************************************************************************************************************************************
// New function for discrete correlation - with the right normalization : shuffling the ISIs
Function Correlogram(Wave_1_bis,Wave_1_tags,Wave_2_bis,Wave_2_tags, Length_Wave_1, Length_Wave_2)
	Wave Wave_1_bis, Wave_2_bis, Wave_1_tags,Wave_2_tags
	Variable Length_Wave_1, Length_Wave_2
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	// Length of recording. Should be identical
//	if (Length_Wave_1 != Length_Wave_2)
//		Abort "Length1 and length2 not identical !!"
//	endif
	// input waves will be modified (avoids overwritting the original)
	Duplicate/o Wave_1_bis, Wave_1; Duplicate/o Wave_2_bis, Wave_2
	// Variables
	Variable i,j , k, LEFT, RIGHT, jSTART, jMEM, VeryEnd, TimeWindow, Nlong, Bin, GraphWidth
	String CCGraph_Name
	Svar currentfilename
	Make/o/N=0 DT_Wave
	// Parameters set by the user in the main panel.
	Nvar CorrTimeWindow, CorrBin
	TimeWindow = CorrTimeWindow; Bin = CorrBin
	CCGraph_Name = "Correlogram, Bin = " + num2str(Bin) + " seconds"
	// Cleaning the input waves
	for (i=0; i < numpnts(Wave_1); i+=1)
	if ( numtype(Wave_1[i]) == 2)	// ... if Wave_1[i] is NaN
		Wave_1[i] = 0			// ... replaced by 0
	endif
	endfor
	for (i=0; i < numpnts(Wave_2); i+=1)
	if ( numtype(Wave_2[i]) == 2)   // ... if Wave_2[i] is NaN
		Wave_2[i] = 0			// ... replaced by 0
	endif
	endfor
	// Initialize variables
	i = 0 ; k = 0; jSTART = 0; jMEM = 0; VeryEnd = Length_Wave_1; Nlong=numpnts(Wave_2)
	// untagged occurrences will become 0
	Nvar CheckUseTagsCorr
	Nvar RefCh1BoxVar, RefCh2BoxVar
	Nvar TestCh1BoxVar, TestCh2BoxVar
	if (CheckUseTagsCorr)
		if (RefCh1BoxVar || TestCh1BoxVar)
			Wave_1 = Wave_1 * Wave_1_tags
		endif
		if (RefCh2BoxVar || TestCh2BoxVar)
			Wave_2 = Wave_2 * Wave_2_tags
		endif
	endif
	
	Variable/G NUMBER_WAVE_1	// number of valid events in Wave_1
	NUMBER_WAVE_1 = sum(Wave_1_tags,0,numpnts(Wave_1_tags)-1)
	FindLevel/P Wave_1_tags, 1	// Find first valid event.
	Variable FirstTaggedEventWave1; FirstTaggedEventWave1 = V_LevelX
	Duplicate/o Wave_1_tags, TEMPWAVE2
	TEMPWAVE2 = Wave_1_tags(numpnts(Wave_1_tags)-1-p)	// Flip the wave
	FindLevel/P TEMPWAVE2, 1	// Find first (last) valid event.
	Variable LastTaggedEventWave1; LastTaggedEventWave1 = numpnts(Wave_1_tags) - (V_LevelX + 1)
	
	// TEMPWAVE2 is re-used to determine the valid events in channel 2
	Duplicate/o Wave_2 TEMPWAVE2
	For (i = 0 ; TEMPWAVE2[i] < (Wave_1[FirstTaggedEventWave1] - (TimeWindow/2)) ; i += 1)
		TEMPWAVE2[i] = 0
	Endfor
	For (i = numpnts(TEMPWAVE2)-1 ; TEMPWAVE2[i] > (Wave_1[LastTaggedEventWave1] + (TimeWindow/2)) ; i -= 1)
		TEMPWAVE2[i] = 0
	Endfor
	TEMPWAVE2 = TEMPWAVE2 / TEMPWAVE2
	for (i=0; i < numpnts(TEMPWAVE2); i+=1)
		if ( numtype(TEMPWAVE2[i]) == 2)	// ... if NaN
			TEMPWAVE2[i] = 0			// ... replaced by 0
		endif
	endfor
	Variable/G NUMBER_WAVE_2	// number of valid events in Wave_2
	NUMBER_WAVE_2 = sum(TEMPWAVE2,0,numpnts(TEMPWAVE2)-1)
	Variable/G DURATION; DURATION = Wave_1[LastTaggedEventWave1] - Wave_1[FirstTaggedEventWave1]

	// Explanation :
	//                                                  |<------------- TimeWindow -------------->|
	//                                               LEFT               Wave_1[i]               RIGHT                                              
 	// Wave_1                *       *            [     *    *              *             *    *     ]             *                      *       Length_Wave_1 -> VeryEnd
	//
	// Wave_2          *          *               [      *         *           *        *            ]  *                    *            *       Length_Wave_2
	//                                                    jSTART
	//                                                    -> jMEM
	
	// The core code to calculate the CC
	For (i=0;i<numpnts(Wave_1);i+=1)
		if ( (Wave_1[i] != 0) && (Wave_1[i] != NaN) )	// Wave_1[i] has to be different from 0 or NaN
			// Reinitializing LEFT
			if ( (Wave_1[i] - (TimeWindow/2) ) > 0 )
				LEFT = Wave_1[i] - (TimeWindow/2)
				else
				LEFT = 0
			endif
			// Reinitializing RIGHT
			if ( (Wave_1[i] + (TimeWindow/2) ) < VeryEnd )
				RIGHT = LEFT + TimeWindow
				else
				RIGHT = VeryEnd
			endif
			// Reinitializing jSTART
			For ( jSTART = jMEM; Wave_2[jSTART] < LEFT && jSTART < numpnts(Wave_2); jSTART += 1)
				// as long as jSTART is strictly inferior to LEFT, keeps incrementing it.
				// ... a way to find the first event of Wave_2 in the [LEFT,RIGHT] window.
			Endfor
			if (Wave_2[jSTART] >= LEFT && Wave_2[jSTART] <= RIGHT)		// Make sure jSTART is in the TimeWindow.
				jMEM = jSTART		// For next loop (avoids starting back from 0)
				for( j = jSTART ; Wave_2[j] < RIGHT && j < numpnts(Wave_2); j += 1 )
					if (Wave_2[j] != 0)		// Occurrence has to be different from 0
						InsertPoints k, 1, DT_Wave
						DT_Wave[k] = Wave_2[j] - Wave_1[i]
						k+=1
					endif
				endfor
			endif
		endif
	Endfor
	
	Duplicate/o DT_Wave Corr
	Histogram/B={-(TimeWindow/2),Bin,(TimeWindow/Bin)} DT_Wave,Corr
	Duplicate/o Corr Corr_raw
	
	// The following line is for normalization using the expected number of spikes form poissonian firing.
	//Corr = (Corr - (NUMBER_WAVE_1*NUMBER_WAVE_2*Bin/DURATION)) / sqrt(NUMBER_WAVE_1*NUMBER_WAVE_2*Bin/DURATION)	// Normalization by offsetting the histogram

	// Now it's time to do the right normalization: shuffling the ISIs
	// 1. take the original wave containing the occurrences in channel 1: wave_1
	// and remove the zeros. Then calculate the ISIs
	For (i = numpnts(wave_1)-1; i > -1; i -= 1)
		if (wave_1[i] == 0)
			DeletePoints i, 1, wave_1
		endif
	Endfor
	Duplicate/o wave_1, TEMPWAVE1; Duplicate/o wave_1, TEMPWAVE2
	InsertPoints numpnts(TEMPWAVE1), 1, TEMPWAVE1
	InsertPoints 0, 1, TEMPWAVE2
	Duplicate/o TEMPWAVE1, INTERVALS1
	INTERVALS1 = TEMPWAVE1 - TEMPWAVE2
	DeletePoints 0, 1, INTERVALS1
	DeletePoints (numpnts(INTERVALS1)-1), 1, INTERVALS1
	
	// Same for channel 2
	For (i = numpnts(wave_2)-1; i > -1; i -= 1)
		if (wave_2[i] == 0)
			DeletePoints i, 1, wave_2
		endif
	Endfor
	Duplicate/o wave_2, TEMPWAVE1; Duplicate/o wave_2, TEMPWAVE2
	InsertPoints numpnts(TEMPWAVE1), 1, TEMPWAVE1
	InsertPoints 0, 1, TEMPWAVE2
	Duplicate/o TEMPWAVE1, INTERVALS2
	INTERVALS2 = TEMPWAVE1 - TEMPWAVE2
	DeletePoints 0, 1, INTERVALS2
	DeletePoints (numpnts(INTERVALS2)-1), 1, INTERVALS2

	Make/N=10/o horizon, SD_horizon
	Variable counter
	For (counter = 0; counter < 10; counter += 1)
		// Calculate the shuffled ISI waves
		Variable TheOneRandomlyChosen
		// For channel 1
		Duplicate/o INTERVALS1, TEMPWAVE
		Duplicate/o INTERVALS1 INTERVALS1_shuffled
		INTERVALS1_shuffled = 0
		For (i = 0; i < numpnts(INTERVALS1_shuffled); i += 1)
			TheOneRandomlyChosen = trunc(abs(enoise(numpnts(TEMPWAVE))))
			INTERVALS1_shuffled[i] = TEMPWAVE[TheOneRandomlyChosen]
			DeletePoints TheOneRandomlyChosen, 1, TEMPWAVE
		Endfor
		// For channel 2
		Duplicate/o INTERVALS2, TEMPWAVE
		Duplicate/o INTERVALS2 INTERVALS2_shuffled
		INTERVALS2_shuffled = 0
		For (i = 0; i < numpnts(INTERVALS2_shuffled); i += 1)
			TheOneRandomlyChosen = trunc(abs(enoise(numpnts(TEMPWAVE))))
			INTERVALS2_shuffled[i] = TEMPWAVE[TheOneRandomlyChosen]
			DeletePoints TheOneRandomlyChosen, 1, TEMPWAVE
		Endfor
	
		// Now we will reconstruct two lists of times of occurrence form the shuffled ISIs
		Duplicate/o wave_1, wave_1_shuffled
		wave_1_shuffled[0] = mean(INTERVALS1_shuffled, 0, numpnts(INTERVALS1_shuffled)-1)
		For (i = 1; i < numpnts(wave_1); i += 1)
			wave_1_shuffled[i] = wave_1_shuffled[i-1] + INTERVALS1_shuffled[i-1]
		Endfor

		// Now we will reconstruct two lists of times of occurrence form the shuffled ISIs
		Duplicate/o wave_2, wave_2_shuffled
		wave_2_shuffled[0] = mean(INTERVALS2_shuffled, 0, numpnts(INTERVALS2_shuffled)-1)
		For (i = 1; i < numpnts(wave_2); i += 1)
			wave_2_shuffled[i] = wave_2_shuffled[i-1] + INTERVALS2_shuffled[i-1]
		Endfor
			
		
		Make/o/N=0 DT_Wave
		// Initialize variables
		i = 0 ; k = 0; jSTART = 0; jMEM = 0; VeryEnd = Length_Wave_1; Nlong=numpnts(wave_2_shuffled)
		// Now calculate the CC
		// The core code to calculate the CC (again)
		For (i=0;i<numpnts(wave_1_shuffled);i+=1)
			if ( (wave_1_shuffled[i] != 0) && (wave_1_shuffled[i] != NaN) )	// Wave_1[i] has to be different from 0 or NaN
				// Reinitializing LEFT
				if ( (wave_1_shuffled[i] - (TimeWindow/2) ) > 0 )
					LEFT = wave_1_shuffled[i] - (TimeWindow/2)
					else
					LEFT = 0
				endif
				// Reinitializing RIGHT
				if ( (wave_1_shuffled[i] + (TimeWindow/2) ) < VeryEnd )
					RIGHT = LEFT + TimeWindow
					else
					RIGHT = VeryEnd
				endif
				// Reinitializing jSTART
				For ( jSTART = jMEM; wave_2_shuffled[jSTART] < LEFT && jSTART < numpnts(wave_2_shuffled); jSTART += 1)
					// as long as jSTART is strictly inferior to LEFT, keeps incrementing it.
					// ... a way to find the first event of Wave_2 in the [LEFT,RIGHT] window.
				Endfor
				if (wave_2_shuffled[jSTART] >= LEFT && wave_2_shuffled[jSTART] <= RIGHT)		// Make sure jSTART is in the TimeWindow.
					jMEM = jSTART		// For next loop (avoids starting back from 0)
					for( j = jSTART ; wave_2_shuffled[j] < RIGHT && j < numpnts(wave_2_shuffled); j += 1 )
						if (wave_2_shuffled[j] != 0)		// Occurrence has to be different from 0
							InsertPoints k, 1, DT_Wave
							DT_Wave[k] = wave_2_shuffled[j] - wave_1_shuffled[i]
							k+=1
						endif
					endfor
				endif
			endif
		Endfor
		Duplicate/o DT_Wave Corr_shuffled
		Histogram/B={-(TimeWindow/2),Bin,(TimeWindow/Bin)} DT_Wave,Corr_shuffled
		Wavestats/Q Corr_shuffled
		horizon[counter] = V_avg
		SD_horizon[counter] = V_sdev
	Endfor

	// Display
	DoWindow/K CrossCorrelogramGraph
	
	if (CCTypeNorm == 1)
		display/W=(220,0,650,300) Corr as CCGraph_Name
	endif
	if (CCTypeShuff == 1)
		display/W=(220,0,650,300) Corr_shuffled as CCGraph_Name
	endif
	if (CCTypeRaw == 1)
		display/W=(220,0,650,300) Corr_raw as CCGraph_Name
	endif
	
	DoWindow/C CrossCorrelogramGraph
	Controlbar/W=CrossCorrelogramGraph 70
	Valdisplay DispSegLength 		pos={10,10}, size={170,20}, title="Segment length (s) = "
	Execute "Valdisplay DispSegLength value=" + num2str(DURATION)
	Valdisplay DispBinSize 			pos={190,10}, size={100,20}, title="Bin (s) = "
	Execute "Valdisplay DispBinSize value=" + num2str(Bin)
	Valdisplay NEventsCh1 			pos={10,27}, size={170,20}, title="Number of events ch.1 = "
	Execute "Valdisplay NEventsCh1 value=" + num2str(NUMBER_WAVE_1)
	Valdisplay NEventsCh2 			pos={10,44}, size={170,20}, title="Number of events ch.2 = "
	Execute "Valdisplay NEventsCh2 value=" + num2str(NUMBER_WAVE_2)
	ValDisplay FreqEventsCh1 		pos={190,27}, size={120,20}, title="<-> Freq (Hz) = "
	Execute "Valdisplay FreqEventsCh1 value=" + num2str(NUMBER_WAVE_1/DURATION)
	ValDisplay FreqEventsCh2 		pos={190,44}, size={120,20}, title="<-> Freq (Hz) = "
	Execute "Valdisplay FreqEventsCh2 value=" + num2str(NUMBER_WAVE_2/DURATION)
	
	CheckBox CheckBoxCorr	pos={435,12},size={40,20},	title="Grid", 	proc=ProcCorrGrid
	
	Button CloseCorrGraph pos={480,9},size={40,20},title="close", proc=procButtonCloseCorrGraph
	
	ModifyGraph standoff=0,font="Arial"; SetAxis bottom -TimeWindow/2,TimeWindow/2; ModifyGraph rgb=(0,0,0); ModifyGraph mode=0
	
	// THE FOLLOWING NORMALIZATION WAS VALID WHEN ASSUMING POISSONIAN FIRING
	// NOTE : confidence interval corresponds to a z score of 2 (i.e. percentile 97.7%, i.e. 2 times SD)
	//Make/o/N=2 ConfidenceInterval_top_Y, ConfidenceInterval_top_X, ConfidenceInterval_bottom_Y, ConfidenceInterval_bottom_X
	//ConfidenceInterval_top_Y = 2
	//ConfidenceInterval_top_X[0] = -TimeWindow/2; ConfidenceInterval_top_X[1] = TimeWindow/2
	//ConfidenceInterval_bottom_Y = -2
	//ConfidenceInterval_bottom_X[0] = -TimeWindow/2; ConfidenceInterval_bottom_X[1] = TimeWindow/2
	//AppendToGraph ConfidenceInterval_top_Y vs ConfidenceInterval_top_X
	//AppendToGraph ConfidenceInterval_bottom_Y vs ConfidenceInterval_bottom_X
	//ModifyGraph lstyle(ConfidenceInterval_top_Y)=2
	//ModifyGraph lstyle(ConfidenceInterval_bottom_Y)=2

	// This one is for the shuffling method
	if (CCTypeNorm == 1)
		Corr = Corr - mean(horizon, 0, numpnts(horizon)-1)
		Corr = Corr / mean(SD_horizon, 0, numpnts(SD_horizon)-1)
		Make/o/N=2 ConfidenceInterval_top_Y, ConfidenceInterval_top_X, ConfidenceInterval_bottom_Y, ConfidenceInterval_bottom_X
		ConfidenceInterval_top_Y = 2
		ConfidenceInterval_top_X[0] = -TimeWindow/2; ConfidenceInterval_top_X[1] = TimeWindow/2
		ConfidenceInterval_bottom_Y = -2
		ConfidenceInterval_bottom_X[0] = -TimeWindow/2; ConfidenceInterval_bottom_X[1] = TimeWindow/2
		AppendToGraph ConfidenceInterval_top_Y vs ConfidenceInterval_top_X
		AppendToGraph ConfidenceInterval_bottom_Y vs ConfidenceInterval_bottom_X
		ModifyGraph lstyle(ConfidenceInterval_top_Y)=2
		ModifyGraph lstyle(ConfidenceInterval_bottom_Y)=2
	endif
	// Axis
	Wavestats/Q Corr; SetAxis left V_min*1.2,V_max*1.2; ModifyGraph tick=2; ModifyGraph mirror=1
	if (CCTypeNorm == 1)
		Label left "Z-score"
	else
		Label left "Count"
	endif
	Label bottom "Lag"
End

FUNCTION procButtonCloseCorrGraph(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K CrossCorrelogramGraph
END

Function ProcCorrGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=CrossCorrelogramGraph grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=CrossCorrelogramGraph grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=CrossCorrelogramGraph grid(left)=0
		ModifyGraph/W=CrossCorrelogramGraph grid(bottom)=0
	endif
End

// The function that calculates the synchronization width and strength (width and area of the central peak above z score = 2)
Function Synch_width_and_strength() : GraphMarquee	
	String Format
// First step: save the limits of the graph marquees
Variable Vleft, Vright, Vbottom, Vtop
GetMarquee/K bottom
Format = "flag: %g; left: %g; top: %g; right: %g; bottom: %g\r"
Vleft = V_left; Vright = V_right
GetMarquee/K left
Format = "flag: %g; left: %g; top: %g; right: %g; bottom: %g\r"
Vbottom = V_bottom; Vtop = V_top
Wave Corr, ConfidenceInterval_top_Y
// Crossing in terms of X
FindLevels/Q/R=(Vleft, Vright) Corr ConfidenceInterval_top_Y[0]
Duplicate/o W_FindLevels TEMPWAVE1
// Crossing in terms of points
FindLevels/P/Q/R=(Vleft, Vright) Corr ConfidenceInterval_top_Y[0]
Duplicate/o W_FindLevels TEMPWAVE2
// Calculate synchronization width (peak width at z score = 2)
Variable/G SynchWidth = TEMPWAVE1[numpnts(TEMPWAVE1)-1] - TEMPWAVE1[0]
DrawText 0,-0.09,"Synchronization width = " + num2str(SynchWidth) + " s"
// Calculate synchronization strength (area of peak)
Variable TEMPVAR, i, j
Nvar CorrBin
Make/o/N=0 COUNTER_WAVE
String TEMPSTRING1, TEMPSTRING2, MASKERNAME1, MASKERNAME2
// A big mess is following... there is the calculation of the synchronization strength
// but also some code to color the area of the peak above z score = 2
// --> totally useless... this was just a coding challenge... sorry
For (i = 0; i < numpnts(TEMPWAVE2); i += 2)
	Make/o/N=0 FILL_X; Make/o/N=0 FILL_Y
	InsertPoints 0, 1, FILL_X; InsertPoints 0, 1, FILL_Y
	FILL_X[0] = TEMPWAVE1[i]; FILL_Y[0] = ConfidenceInterval_top_Y[0]
	InsertPoints (i/2), 1, COUNTER_WAVE
	Variable k; k = 1
	For (j = floor(TEMPWAVE2[i])+1; j < floor(TEMPWAVE2[i+1])+1; j += 1)
		COUNTER_WAVE[i/2] = COUNTER_WAVE[i/2] + (Corr[j] - ConfidenceInterval_top_Y[0])*CorrBin
		InsertPoints k, 1, FILL_X; InsertPoints k, 1, FILL_Y
		FILL_X[k] = leftX(Corr) + j*deltaX(Corr); FILL_Y[k] = Corr[j]
		k += 1
	Endfor
	InsertPoints k, 1, FILL_X; InsertPoints k, 1, FILL_Y
	FILL_X[k] = TEMPWAVE1[i+1]; FILL_Y[k] = ConfidenceInterval_top_Y[0]
	InsertPoints k+1, 1, FILL_X; InsertPoints k+1, 1, FILL_Y
	FILL_X[k+1] = FILL_X[0]; FILL_Y[k+1] = ConfidenceInterval_top_Y[0]
	TEMPSTRING1 = "Fill" + num2str(i/2) + "_X"
	TEMPSTRING2 = "Fill" + num2str(i/2) + "_Y"
	Duplicate/o FILL_X $TEMPSTRING1; Duplicate/o FILL_Y $TEMPSTRING2
	Appendtograph $TEMPSTRING2 vs $TEMPSTRING1
	ModifyGraph mode($TEMPSTRING2)=7,hbFill($TEMPSTRING2)=4
	Make/o/N=4 MASKER1, MASKER2
	MASKER1[0] = FILL_X[0]; MASKER1[1] = FILL_X[0]
	MASKER1[2] = FILL_X[k]; MASKER1[3] = FILL_X[k]
	MASKER2[0] = 0; MASKER2[1] = ConfidenceInterval_top_Y[0]
	MASKER2[2] = ConfidenceInterval_top_Y[0]; MASKER2[3] = 0
	MASKERNAME1 = "Masker" + num2str(i/2) + "_X"
	MASKERNAME2 = "Masker" + num2str(i/2) + "_Y"
	Duplicate/o MASKER1 $MASKERNAME1
	Duplicate/o MASKER2 $MASKERNAME2
	Appendtograph $MASKERNAME2 vs $MASKERNAME1
	ModifyGraph mode($MASKERNAME2)=7,hbFill($MASKERNAME2)=2
	ModifyGraph rgb($MASKERNAME2)=(65535,65535,65535)
Endfor
KillWaves FILL_X, FILL_Y
removefromgraph Corr
appendtograph Corr; ModifyGraph rgb(Corr)=(0,0,0)
Variable/G SynchStrength = sum(COUNTER_WAVE, 0, numpnts(COUNTER_WAVE)-1)
DrawText 0,-0.02,"Synchronization strength = " + num2str(SynchStrength) + " s"
End

FUNCTION MannWhitney(wave1, wave2)
	Wave wave1, wave2
	Duplicate/o wave1, grouping1; grouping1 = 0
	Duplicate/o wave2, grouping2; grouping2 = 1
	Concatenate/o/NP {wave1, wave2}, combination
	Concatenate/o/NP {grouping1, grouping2}, grouping
	Sort combination combination, grouping
	Duplicate/o grouping grouping_wave2; Wave grouping_wave2
	Duplicate/o grouping_wave2 grouping_wave1
	grouping_wave1 = 1 - grouping_wave2
	Duplicate/o grouping_wave1 dummy; dummy = p+1
	grouping_wave1 = grouping_wave1 * dummy
	grouping_wave2 = grouping_wave2 * dummy
	Variable Wx // somme des rangs de grouping_wave1
	Variable Wy // somme des rangs de grouping_wave2
	Variable Null_Wx, Null_Wy
	Wx = sum(grouping_wave1, 0, numpnts(grouping_wave1))
	Wy = sum(grouping_wave2, 0, numpnts(grouping_wave2))
	//Killwaves grouping1, grouping2, combination, grouping, grouping_wave1, grouping_wave2, dummy
	Null_Wx = 0.5*numpnts(wave1)*(numpnts(wave1)+numpnts(wave2)+1)
	Null_Wy = 0.5*numpnts(wave2)*(numpnts(wave1)+numpnts(wave2)+1)
	Variable/G ecart_reduit_x
	ecart_reduit_x = abs( (Wx - Null_Wx) / sqrt( numpnts(wave1) * numpnts(wave2) * (numpnts(wave1)+numpnts(wave2)+1)/12 ) )
	print "Ecart reduit x = " + num2str(abs(ecart_reduit_x))
	Variable/G ecart_reduit_y
	ecart_reduit_y = abs( (Wy - Null_Wy) / sqrt( numpnts(wave1)*numpnts(wave2)*(numpnts(wave1)+numpnts(wave2)+1)/12 ) )
	//print "Ecart reduit y = " + num2str(abs(ecart_reduit_y))
	
END

FUNCTION ProcCB_Ticks_CH1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckTicksCh1
	CheckTicksCh1 = Checked 
END

FUNCTION ProcCB_Ticks_CH2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckTicksCh2
	CheckTicksCh2 = Checked
END

FUNCTION ProcCBRastersUseTags(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckRastersUseTags
	CheckRastersUseTags = Checked
END

FUNCTION ProcCB_TTL_CH1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckTTLch1, CheckTTLch2
	CheckTTLch1 = checked
	CheckTTLch2 = 1 - CheckTTLch1
	Execute "CheckBox CB_TTL_CH1 value=CheckTTLch1"
	Execute "CheckBox CB_TTL_CH2 value=CheckTTLch2"
END

FUNCTION ProcCB_TTL_CH2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckTTLch1, CheckTTLch2
	CheckTTLch2 = checked
	CheckTTLch1 = 1 - CheckTTLch2
	Execute "CheckBox CB_TTL_CH1 value=CheckTTLch1"
	Execute "CheckBox CB_TTL_CH2 value=CheckTTLch2"
END

// TOGGLE CHECKBOX FOR LIGHT PULSES SYMBOLS ABOVE
FUNCTION ToggleTicksAbove (ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar TickAbove, TickBelow, TickAcross
	if (TickAbove == 0)
		TickAbove = 1 - TickAbove
		TickBelow = 0; TickAcross = 0
		Execute "CheckBox CheckTicksBelow value=TickBelow"
		Execute "CheckBox CheckTicksAcross value=TickAcross"
	endif
	Execute "CheckBox CheckTicksAbove value=TickAbove"
END

// TOGGLE CHECKBOX FOR LIGHT PULSES SYMBOLS BELOW
FUNCTION ToggleTicksBelow (ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar TickAbove, TickBelow, TickAcross
	if (TickBelow == 0)
		TickBelow = 1 - TickBelow
		TickAbove = 0; TickAcross = 0
		Execute "CheckBox CheckTicksAbove value=TickAbove"
		Execute "CheckBox CheckTicksAcross value=TickAcross"
	endif
	Execute "CheckBox CheckTicksBelow value=TickBelow"
END

// TOGGLE CHECKBOX FOR LIGHT PULSES ACROSS
FUNCTION ToggleTicksAcross (ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar TickAbove, TickBelow, TickAcross
	if (TickAcross == 0)
		TickAcross = 1 - TickAcross
		TickAbove = 0; TickBelow = 0
		Execute "CheckBox CheckTicksAbove value=TickAbove"
		Execute "CheckBox CheckTicksBelow value=TickBelow"
	endif
	Execute "CheckBox CheckTicksAcross value=TickAcross"
END

FUNCTION ProcCBNormPSTH(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckNormPSTH
	CheckNormPSTH = checked
END

FUNCTION ProcCBPSTHUseTags(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckPSTHUseTags
	CheckPSTHUseTags = checked
END

FUNCTION ProcCBShowPulseNumber(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckShowPulseNumber
	CheckShowPulseNumber = checked
END

FUNCTION ToggleUseTagRange(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckRangeTTLs
	CheckRangeTTLs = checked
	if (CheckRangeTTLs)
		CheckBox CBNormPSTH		disable=0
		SetVariable SetPSTHbin		disable=0
		Button ButtonMakePSTH		disable=0
		CheckBox CBPSTHUseTags	disable=0
		SetVariable SetTagStartTTLs	disable=0
		SetVariable SetTagEndTTLs	disable=0
		SetVariable SetBeforeTTLs	disable=0
		SetVariable SetAfterTTLs		disable=0
		CheckBox CBShowPulseNumber disable=0
	else		
		CheckBox CBNormPSTH		disable=2
		SetVariable SetPSTHbin		disable=2
		Button ButtonMakePSTH		disable=2
		CheckBox CBPSTHUseTags	disable=2
		SetVariable SetTagStartTTLs	disable=2
		SetVariable SetTagEndTTLs	disable=2
		SetVariable SetBeforeTTLs	disable=2
		SetVariable SetAfterTTLs		disable=2
		CheckBox CBShowPulseNumber disable=2
	endif
	
END

FUNCTION ToggleCBFillTicks(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar FillTicks_WindowTTLs
	FillTicks_WindowTTLs = checked
	CheckBox CheckFillTicks value = FillTicks_WindowTTLs
END

// FUNCTION CALLED BY PANEL BUTTONS
FUNCTION ProcButtonCloseGraphsPanel (ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K GraphControlWindow
	DoWindow/K WindowTTLs
END

FUNCTION ProcButtonPlotTicks (ctrlName) : ButtonControl
	String ctrlName

	// Should check is both channels exist
	
	Svar Filename1, Filename2
	Nvar TicksNumberRows, CheckTicksCh1, CheckTicksCh2
	Nvar C1E, C2E, Ch1Detected, Ch2Detected
	Nvar SI1, SI2
	Nvar RowsSpacing
	Nvar CheckRastersUseTags
	
	Wave Occurrence1, Occurrence2
	if (CheckRastersUseTags)
		if (Ch1Detected)
			Wave Tags1
		endif
		if (Ch2Detected)
			Wave Tags2
		endif
	endif
	
	Wave/T Params
	Params[0][5] = num2str(CheckTicksCh1)
	Params[1][5] = num2str(CheckTicksCh2)
	Params[2][5] = num2str(TicksNumberRows)
	Params[3][5] = num2str(RowsSpacing)
	Params[18][5] = num2str(CheckRastersUseTags)
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	
	DoWindow/K WindowRasters
	KillDataFolder/Z root:spacan:WindowRasters
	NewDataFolder/o root:spacan:WindowRasters
	NewDataFolder/o root:spacan:WindowRasters:ScaleBars
	
	Variable i, j
	
	// Routine checks
	if (C1E == 0 && C2E == 0)
		Abort "No channel loaded!"
	endif
	
	if (C1E == 1 && CheckTicksCh1 == 1 && Ch1Detected == 0)
		Abort "To display events in ch1 as ticks, perform a detection first!"
	endif
	
	if (C2E == 1 && CheckTicksCh2 == 1 && Ch2Detected == 0)
		Abort "To display events in ch2 as ticks, perform a detection first!"
	endif	
	
	// Get the window limits from the active subwindow
	Variable ChNumber
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)	// If MainWindow#Channel1 is on top
		ChNumber = 1
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)	// If MainWindow#Channel2 is on top
		ChNumber = 2
	endif
	
	Variable V_min, V_max
	GetAxis/Q/W=MainWindow#$"Channel" + num2str(ChNumber) bottom
	
	Variable V_start = V_min
	Variable V_end = V_max
	Variable Xrange = V_end - V_start
//	Variable LimitUp, LimitDown
	Variable LineXbottom, LineXtop, LineYbottom, LineYtop
	
	if (C1E)
		Make/N=(TicksNumberRows)/o W_YRangeCh1
		For (i = 1; i < (TicksNumberRows + 1); i += 1)
			Wavestats/Q/R=(V_start + (i-1)*Xrange/TicksNumberRows,V_start+i*Xrange/TicksNumberRows) $Filename1
			W_YRangeCh1[i-1] = V_max - V_min
		Endfor
		WaveStats/Q W_YRangeCh1
		Variable MaxYRangeCh1 = V_max
		KillWaves W_YRangeCh1
	endif
	
	if (C2E)
		Make/N=(TicksNumberRows)/o W_YRangeCh2
		For (i = 1; i < (TicksNumberRows + 1); i += 1)
			Wavestats/Q/R=(V_start + (i-1)*Xrange/TicksNumberRows,V_start+i*Xrange/TicksNumberRows) $Filename2
			W_YRangeCh2[i-1] = V_max - V_min
		Endfor
		WaveStats/Q W_YRangeCh2
		Variable MaxYRangeCh2 = V_max
		KillWaves W_YRangeCh2	
	endif
	
	Display/W=(210,50,700,500) as "Rasters"
	DoWindow/C WindowRasters

	String Ch1LeftAxisName, Ch2LeftAxisName
	if (CheckTicksCh1 == 1 || CheckTicksCh2 == 1)
		Make/o/N=1 RasterDummy_Y
		Make/o/N=1 RasterDummy_X
		Variable EventNum_Start, EventNum_End
	endif
	
	For (i = 1; i < (TicksNumberRows + 1); i += 1)
	
		if (C1E)	// Channel 1 exists

			if (CheckTicksCh1 == 0)	// Display channel 1 as trace
				// Channel 1
				// Get segment from original trace
				Duplicate/o/R=(V_start + (i-1)*Xrange/TicksNumberRows,V_start+i*Xrange/TicksNumberRows) $Filename1 TEMPWAVE_A
				SetScale/P x 0,SI1*1e-6,"s", TEMPWAVE_A
				WaveStats/Q TEMPWAVE_A
				//TEMPWAVE_A = TEMPWAVE_A - V_max - (2*(i-1) + i*RowsSpacing/100)*MaxYRangeCh1
				TEMPWAVE_A = TEMPWAVE_A - V_max - 2*(i-1)*MaxYRangeCh1 - (i-1)*(RowsSpacing/100)*MaxYRangeCh1
				Duplicate/o TEMPWAVE_A $"root:spacan:WindowRasters:Ch1Segment" + num2str(i)
				AppendToGraph/W=WindowRasters $"root:spacan:WindowRasters:Ch1Segment" + num2str(i)
				ModifyGraph noLabel(Left)=2,axThick(Left)=0,standoff(Left)=0
				ModifyGraph/W=WindowRasters rgb($"Ch1Segment" + num2str(i))=(0,0,65535)
			else		// Display channel 1 as ticks
				AppendToGraph/W=WindowRasters RasterDummy_Y vs RasterDummy_X
				ModifyGraph noLabel(Left)=2,axThick(Left)=0,standoff(Left)=0
				ModifyGraph mode(RasterDummy_Y)=2,lsize(RasterDummy_Y)=0,rgb(RasterDummy_Y)=(65535,65535,65535)
				// Events as ticks
				// Get number of events falling into range
				if (i == 1)
					if (V_start < Occurrence1[0])
						EventNum_Start = 0
					endif
				else
					FindLevel/Q/P Occurrence1, (V_start + (i-1)*Xrange/TicksNumberRows)
					EventNum_Start = floor(V_LevelX) + 1
				endif
				if ( (V_start+i*Xrange/TicksNumberRows) > Occurrence1[numpnts(Occurrence1)-1])
					EventNum_End = numpnts(Occurrence1)-1
				else
					FindLevel/Q/P Occurrence1, (V_start+i*Xrange/TicksNumberRows)
					EventNum_End = floor(V_LevelX)
				endif
				For (j = EventNum_Start; j < EventNum_End + 1; j += 1)
					if (CheckRastersUseTags == 0)
						SetDrawLayer/W=WindowRasters UserBack
						SetDrawEnv/W=WindowRasters linefgc=(0,0,65535), linethick=2, xcoord=bottom, ycoord=Left
						LineXbottom = Occurrence1[j] - (V_start + (i-1)*Xrange/TicksNumberRows)
						LineYbottom = - 2*(i-1)*MaxYRangeCh1 - MaxYRangeCh1 - (i-1)*(RowsSpacing/100)*MaxYRangeCh1
						LineXtop = Occurrence1[j] - (V_start + (i-1)*Xrange/TicksNumberRows)
						LineYtop = - 2*(i-1)*MaxYRangeCh1 - (i-1)*(RowsSpacing/100)*MaxYRangeCh1
						DrawLine/W=WindowRasters LineXbottom, LineYbottom, LineXtop, LineYtop
					else
						if (Tags1[j] == 1)
							SetDrawLayer/W=WindowRasters UserBack
							SetDrawEnv/W=WindowRasters linefgc=(0,0,65535), linethick=2, xcoord=bottom, ycoord=Left
							LineXbottom = Occurrence1[j] - (V_start + (i-1)*Xrange/TicksNumberRows)
							LineYbottom = - 2*(i-1)*MaxYRangeCh1 - MaxYRangeCh1 - (i-1)*(RowsSpacing/100)*MaxYRangeCh1
							LineXtop = Occurrence1[j] - (V_start + (i-1)*Xrange/TicksNumberRows)
							LineYtop = - 2*(i-1)*MaxYRangeCh1 - (i-1)*(RowsSpacing/100)*MaxYRangeCh1
							DrawLine/W=WindowRasters LineXbottom, LineYbottom, LineXtop, LineYtop						
						endif
					endif
				endfor
			endif
			SetAxis/W=WindowRasters bottom 0, Xrange/TicksNumberRows
			SetAxis/W=WindowRasters Left -(MaxYRangeCh1*TicksNumberRows*2 + (RowsSpacing/100)*MaxYRangeCh1*(TicksNumberRows-1)), 0
		endif
		
		if (C2E)		// Channel 2 exists
			
			if (CheckTicksCh2 == 0)	// Display channel 2 as trace
				// Channel 2
				Duplicate/o/R=(V_start + (i-1)*Xrange/TicksNumberRows,V_start + i*Xrange/TicksNumberRows) $Filename2 TEMPWAVE_B
				SetScale/P x 0,SI2*1e-6,"s", TEMPWAVE_B
				WaveStats/Q TEMPWAVE_B
				//TEMPWAVE_B = TEMPWAVE_B - V_max - (2*(i-1) + 1 + i*RowsSpacing/100)*MaxYRangeCh2
				TEMPWAVE_B = TEMPWAVE_B - V_max - (2*(i-1) + 1)*MaxYRangeCh2 - (i-1)*(RowsSpacing/100)*MaxYRangeCh2
				Duplicate/o TEMPWAVE_B, $"root:spacan:WindowRasters:Ch2Segment" + num2str(i)
				AppendToGraph/W=WindowRasters/R $"root:spacan:WindowRasters:Ch2Segment" + num2str(i)
				ModifyGraph noLabel(Right)=2,axThick(Right)=0,standoff(Right)=0
				ModifyGraph/W=WindowRasters rgb($"Ch2Segment" + num2str(i))=(65535,0,0)
			else		// Display channel 2 as ticks
				AppendToGraph/R/W=WindowRasters RasterDummy_Y vs RasterDummy_X
				ModifyGraph noLabel(Right)=2,axThick(Right)=0,standoff(Right)=0
				ModifyGraph mode(RasterDummy_Y)=2,lsize(RasterDummy_Y)=0,rgb(RasterDummy_Y)=(65535,65535,65535)
				// Events as ticks
				// Get number of events falling into range				
				if (i == 1)
					if (V_start < Occurrence2[0])
						EventNum_Start = 0
					endif
				else
					FindLevel/Q/P Occurrence2, (V_start + (i-1)*Xrange/TicksNumberRows)
					EventNum_Start = floor(V_LevelX) + 1
				endif
				if ( (V_start+i*Xrange/TicksNumberRows) > Occurrence2[numpnts(Occurrence2)-1])
					EventNum_End = numpnts(Occurrence2)-1
				else
					FindLevel/Q/P Occurrence2, (V_start+i*Xrange/TicksNumberRows)
					EventNum_End = floor(V_LevelX)
				endif
				For (j = EventNum_Start; j < EventNum_End + 1; j += 1)
					if (CheckRastersUseTags == 0)
						SetDrawLayer/W=WindowRasters UserBack
						SetDrawEnv/W=WindowRasters linefgc=(65535,0,0), linethick=2, xcoord=bottom, ycoord=Right
						LineXbottom = Occurrence2[j] - (V_start + (i-1)*Xrange/TicksNumberRows)
						LineYbottom = - (2*(i-1) + 1)*MaxYRangeCh2 - MaxYRangeCh2 - (i-1)*(RowsSpacing/100)*MaxYRangeCh2
						LineXtop = Occurrence2[j] - (V_start + (i-1)*Xrange/TicksNumberRows)
						LineYtop = - (2*(i-1) + 1)*MaxYRangeCh2 - (i-1)*(RowsSpacing/100)*MaxYRangeCh2
						DrawLine/W=WindowRasters LineXbottom, LineYbottom, LineXtop, LineYtop
					else
						if (Tags2[j] == 1)
							SetDrawLayer/W=WindowRasters UserBack
							SetDrawEnv/W=WindowRasters linefgc=(65535,0,0), linethick=2, xcoord=bottom, ycoord=Right
							LineXbottom = Occurrence2[j] - (V_start + (i-1)*Xrange/TicksNumberRows)
							LineYbottom = - (2*(i-1) + 1)*MaxYRangeCh2 - MaxYRangeCh2 - (i-1)*(RowsSpacing/100)*MaxYRangeCh2
							LineXtop = Occurrence2[j] - (V_start + (i-1)*Xrange/TicksNumberRows)
							LineYtop = - (2*(i-1) + 1)*MaxYRangeCh2 - (i-1)*(RowsSpacing/100)*MaxYRangeCh2
							DrawLine/W=WindowRasters LineXbottom, LineYbottom, LineXtop, LineYtop										
						endif
					endif
				endfor		
			endif
			SetAxis/W=WindowRasters bottom 0, Xrange/TicksNumberRows
			SetAxis/W=WindowRasters Right -(MaxYRangeCh2*TicksNumberRows*2 + (RowsSpacing/100)*MaxYRangeCh2*(TicksNumberRows-1)), 0
		endif
		
	Endfor
	
	if (CheckTicksCh1 == 1 && CheckTicksCh2 == 1)
		SetAxis Bottom 0,Xrange/TicksNumberRows
	endif
	
	KillWaves TEMPWAVE_A, TEMPWAVE_B
	
	ModifyGraph/W=WindowRasters noLabel(bottom)=2,axThick(bottom)=0,standoff(bottom)=0
	ModifyGraph/W=WindowRasters margin=20
	
	ControlBar/W=WindowRasters 120
	ModifyGraph/W=WindowRasters cbRGB=(60000,60000,60000)
	Button	ButtonCloseWindowRasters	pos={605,5},size={40,20},title="Close",proc=ProcButtonCloseRasters, labelback=(60000,60000,60000)
	Button 	ButtonSaveRasters			pos={540,5},size={60,20},title="Save Fig.",proc=ProcSaveFig, labelback=(60000,60000,60000)
	Button	BGotoProcButtonPlotTicks		pos={540,30},size={105,20},title="Show function",proc=ProcGotoProcButtonPlotTicks, labelback=(60000,60000,60000)

	GroupBox GBScalesRasters			pos={5,2},size={500,110},title="Scale bars",fsize=12,fstyle=1
	
	SetDataFolder root:spacan:WindowRasters:ScaleBars

	Variable/G ScaleNum_WindowRasters = NaN
	Variable/G YScaleBar_WindowRasters = NaN
	Variable/G XScaleBar_WindowRasters = NaN
	TitleBox ScaleBarTitleRasters1			frame=0, fsize=11,	fstyle=1,pos={175,22},size={100,20},title="Use graph marquee to draw scale bars",frame=0,labelBack=(60000,60000,60000)
	TitleBox ScaleBarTitleRasters2			frame=0, fsize=11,	fstyle=1,pos={175,38},size={100,20},title="(click and drag diagonally, and select DrawScale)",frame=0,labelBack=(60000,60000,60000)
	
	SetVariable SetScaleNum_WindowRasters	pos={175,57},size={85,20},labelback=(60000,60000,60000),title="Bar #:",value=ScaleNum_WindowRasters,proc=ProcSetScaleNum,limits={1,inf,1}
	SetVariable SetYScaleB_WindowRasters 	pos={265,57},size={110,20},labelback=(60000,60000,60000),title="Y:",value=YScaleBar_WindowRasters,proc=ProcSetScaleB
	SetVariable SetXScaleB_WindowRasters 	pos={380,57},size={110,20},labelback=(60000,60000,60000),title="X:",value=XScaleBar_WindowRasters,proc=ProcSetScaleB
	Button ButtonDoneScaleBars			pos={175,80},size={140,20},labelback=(60000,60000,60000),title="Done editing scale bars",proc=ProcDoneScaleBars
	Button ButtonEraseScaleBars			pos={320,80},size={110,20},labelback=(60000,60000,60000),title="Delete scale bars",proc=ProcEraseScaleBars
	
	// Here must use AxisList to get list of axis
	String AxisListOutput = AxisList("WindowRasters")
	Make/N=0/o/T VertAxisList		// Will contain all axis starting with left
	Make/N=0/o/T HorizAxisList		// Will contain all axis starting with right
	Variable VertCounter = 0
	Variable HorizCounter = 0
	Variable CharCounter = 0
	String TempString = ""
	For (i = 0; i < strlen(AxisListOutput)+1; i += 1)
		if (stringmatch(AxisListOutput[i],";") == 0)		// If character is not ";"
			TempString[CharCounter] = AxisListOutput[i]
			CharCounter += 1
		else
			if (stringmatch(TempString[0],"l") == 1 || stringmatch(TempString[0],"r") == 1)
				InsertPoints VertCounter, 1, VertAxisList
				VertAxisList[VertCounter] = TempString
			endif
			if (stringmatch(TempString[0],"b"))
				InsertPoints HorizCounter, 1, HorizAxisList
				HorizAxisList[HorizCounter] = TempString
			endif			
			TempString = ""
		endif
	Endfor
	// The lists of vertical and horizontal axis are in VertAxisList and HorizAxisList
	
	Make/N=(numpnts(VertAxisList))/o VertAxisSelWave = p
	Make/N=(numpnts(HorizAxisList))/o HorizAxisSelWave = p
	
	Make/N=(numpnts(VertAxisList))/o/T LBVertAxis_tw
	LBVertAxis_tw[0] = "Ch2"
	LBVertAxis_tw[1] = "Ch1"
	
	TitleBox TBVertAxisRasters			pos={15,22},size={60,40},title="Vert. axis",frame=0,labelBack=(60000,60000,60000)
	ListBox LBVertAxis_WindowRasters labelBack=(60000,60000,60000), pos={15,44},size={70,40},frame=0,mode=1,editStyle=0,listWave=LBVertAxis_tw,selWave=VertAxisSelWave
	TitleBox TBHorizAxisRasters			pos={90,22},size={60,40},title="Horiz. axis",frame=0,labelBack=(60000,60000,60000)
	ListBox LBHorizAxis_WindowRasters labelBack=(60000,60000,60000), pos={90,44},size={70,40},frame=0,mode=1,editStyle=0,listWave=HorizAxisList,selWave=HorizAxisSelWave
	
	SetDataFolder root:spacan
	
END

Function ProcGotoProcButtonPlotTicks(ctrlName) : ButtonControl
	String ctrlName
	DisplayProcedure "ProcButtonPlotTicks"
End

Function ProcButtonCloseRasters(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K WindowRasters
End

Function ProcSaveFig(ctrlName) : ButtonControl
	String ctrlName
	SavePICT/EF=1/E=-3
End

FUNCTION ProcButtonMakePSTH (ctrlName) : ButtonControl
	String ctrlName
	
	Nvar C1E, C2E
	If (C1E == 0 && C2E == 0)
		Abort "No channel loaded!"
	endif
	If (C1E == 1 && C2E == 0)
		Abort "Channel 2 not loaded!"
	endif
	If (C1E == 0 && C2E == 1)
		Abort "Channel 1 not loaded!"
	endif	
	
	Nvar Ch1Detected, Ch2Detected
	if (Ch1Detected == 0 && Ch2Detected == 0)
		Abort "No events detected in either channel!"
	endif
	if (Ch1Detected == 0 && Ch2Detected == 1)
		Abort "Detect events in channel 1 first!"
	endif
	if (Ch1Detected == 1 && Ch2Detected == 0)
		Abort "Detect events in channel 2 first!"
	endif	
	
	Nvar CheckTTLch1, CheckTTLch2	// Tells you which ch has TTLs
	Variable ChNumber
	if (CheckTTLch1 == 1)
		ChNumber = 1
	endif
	if (CheckTTLch2 == 1)
		ChNumber = 2
	endif

	Nvar CheckTTLch1, CheckTTLch2
	Nvar CheckRangeTTLs
	Nvar CheckPSTHUseTags
	Nvar RangeStartTTLs, RangeEndTTLs
	Nvar BeforeTTLs, AfterTTLs
	Nvar BinSizePSTH
	Nvar CheckNormPSTH
	Nvar TickAbove, TickBelow, TickAcross
	Nvar FillTicks_WindowTTLs
	
	Wave/T Params
	Params[4][5] = num2str(CheckTTLch1)
	Params[5][5] = num2str(CheckTTLch2)
	Params[6][5] = num2str(TickAbove)
	Params[7][5] = num2str(TickBelow)
	Params[8][5] = num2str(TickAcross)
	Params[4][5] = num2str(CheckTTLch1)
	Params[5][5] = num2str(CheckTTLch2)
	Params[9][5] = num2str(FillTicks_WindowTTLs)
	Params[10][5] = num2str(CheckRangeTTLs)
	Params[11][5] = num2str(CheckPSTHUseTags)
	Params[12][5] = num2str(RangeStartTTLs)
	Params[13][5] = num2str(RangeEndTTLs)
	Params[14][5] = num2str(BeforeTTLs)
	Params[15][5] = num2str(AfterTTLs)
	Params[16][5] = num2str(BinSizePSTH)
	Params[17][5] = num2str(CheckNormPSTH)
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	
	MakePSTH(ChNumber)
END

FUNCTION MakePSTH (ChNumber)
	Variable ChNumber		// The ch where TTLs are
	
	KillDataFolder/Z root:spacan:WindowTTLs

	NewDataFolder/o root:spacan:WindowTTLs
	// Data folder name has to be the same as window name

	// Waves
	Wave OccurrenceA = $"Occurrence" + num2str(ChNumber)
	Wave OccurrenceB = $"Occurrence" + num2str(3 - ChNumber)
	//Wave TagsA = $"Tags" + num2str(ChNumber)
	Wave TagsA = $"Tags" + num2str(ChNumber)
	Wave TagsB = $"Tags" + num2str(3 - ChNumber)
	// Global variables
	Nvar SIb = $"SI" + num2str(3 - ChNumber)
	Nvar TickAbove, TickBelow, TickAcross
	Nvar FillTicks_WindowTTLs
	Nvar CheckRangeTTLs
	Nvar RangeStartTTLs
	Nvar RangeEndTTLs
	Nvar BeforeTTLs
	Nvar AfterTTLs
	Nvar Length
	Nvar BinSizePSTH
	Nvar CheckNormPSTH
	Nvar CheckPSTHUseTags
	// Global strings
	Svar FilenameA = $"Filename" + num2str(ChNumber)
	Svar FilenameB = $"Filename" + num2str(3 - ChNumber)

	Variable i
	Variable upperlimit, lowerlimit
		
	String Filename = "root:spacan:" + FilenameB
		
	GetPulses(ChNumber,0,Length)
	DoUpdate
	Wave PulseWidth, PulseBegin, PulseEnd
		
	// Warning here: pulses must have the same length !
	Variable PulseWidthVar = PulseWidth[RangeStartTTLs]

	// A set of variables for the for loops
	Variable CountStart, CountEnd
	Variable BinStart, BinEnd
	Variable SweepCounter, BinCounter
	
	// How many bins in the time window?
	Variable NumberOfBins = floor(BeforeTTLs/BinSizePSTH) + floor((PulseWidthVar + AfterTTLs)/BinSizePSTH)
	
	// Make array that will accept individual PSTHs (one for each "sweep" = pulse)
	Make/o/N=((RangeEndTTLs - RangeStartTTLs + 1), NumberOfBins) BinCountArray
	
	// Start the loop
	
	For (SweepCounter = RangeStartTTLs; SweepCounter < (RangeEndTTLs + 1); SweepCounter += 1)	
	
		For (BinCounter = 1; BinCounter < NumberOfBins+1; BinCounter += 1)
		
			if (CheckPSTHUseTags == 1)	// Use tags for both channels
			
				if (TagsA[SweepCounter] == 1)
			
					BinStart = PulseBegin[SweepCounter] - floor(BeforeTTLs/BinSizePSTH)*BinSizePSTH + (BinCounter-1)*BinSizePSTH
					BinEnd = BinStart + BinSizePSTH
					FindLevel/Q OccurrenceB BinStart
					Countstart = floor(V_LevelX)+1
					FindLevel/Q OccurrenceB BinEnd
					CountEnd = floor(V_LevelX)
					if (CountEnd >= Countstart)
						WaveStats/Q/R=[CountStart,CountEnd] TagsB
//						BinCountArray[SweepCounter-RangeStartTTLs][BinCounter-1] = sum(TagsB,CountStart,CountEnd)/BinSizePSTH
						BinCountArray[SweepCounter-RangeStartTTLs][BinCounter-1] = V_Sum/BinSizePSTH
					else
						BinCountArray[SweepCounter-RangeStartTTLs][BinCounter-1] = 0
					endif
	
				else
				
					BinCountArray[SweepCounter-RangeStartTTLs][BinCounter-1] = NaN
					
				endif
				
			else			// don't use tags for either channel 
			
				BinStart = PulseBegin[SweepCounter] - floor(BeforeTTLs/BinSizePSTH)*BinSizePSTH + (BinCounter-1)*BinSizePSTH
				BinEnd = BinStart + BinSizePSTH
				FindLevel/Q OccurrenceB BinStart
				Countstart = floor(V_LevelX)+1
				FindLevel/Q OccurrenceB BinEnd
				CountEnd = floor(V_LevelX)
				if (CountEnd >= Countstart)
					BinCountArray[SweepCounter-RangeStartTTLs][BinCounter-1] = (CountEnd - CountStart + 1)/BinSizePSTH
				else
					BinCountArray[SweepCounter-RangeStartTTLs][BinCounter-1] = 0
				endif
			
			endif	
				
		Endfor

		// Here do the normalization
		if (CheckNormPSTH)
			Duplicate/o/R=[SweepCounter - RangeStartTTLs][0,NumberOfBins] BinCountArray TEMPWAVE
			MatrixTranspose TEMPWAVE
			Redimension/N=(-1,0) TEMPWAVE
			WaveStats/Q/R=[0,floor(BeforeTTLs/BinSizePSTH)] TEMPWAVE
			TEMPWAVE = TEMPWAVE / V_avg
			For (BinCounter = 1; BinCounter < NumberOfBins+1; BinCounter += 1)
				BinCountArray[SweepCounter-RangeStartTTLs][BinCounter-1] = TEMPWAVE[BinCounter-1]
			endfor
		endif
	Endfor
	// Now we have the list of PSTHs for individual sweeps
	// Let's calculate the average PSTH with SD
	Make/o/N=(NumberOfBins) PSTH_avg
	Make/o/N=(NumberOfBins) PSTH_sd
	For (i = 0; i < NumberOfBins; i += 1)
		Make/o/N=(RangeEndTTLs - RangeStartTTLs + 1) TMP_PointAverage
		TMP_PointAverage = BinCountArray[p][i]
		Wavestats/Q TMP_PointAverage
		PSTH_avg[i] = V_avg
		PSTH_sd[i] = V_sdev
	Endfor
	KillWaves TMP_PointAverage

	SetScale/P x (-floor(BeforeTTLs/BinSizePSTH)*BinSizePSTH + BinSizePSTH/2), BinSizePSTH,"s", PSTH_avg

	DoWindow/K WindowTTLs
	Display/W=(50,50,600,360) as "Average PSTH"
	DoWindow/C WindowTTLs

	Appendtograph/W=WindowTTLs PSTH_avg
	ModifyGraph/W=WindowTTLs rgb(PSTH_avg)=(65535,0,0)
	ModifyGraph/W=WindowTTLs standoff=0
			
	// Put SD Box
	// The SD is represented as a polygon
	Duplicate/o PSTH_sd, PSTH_sd_top; PSTH_sd_top = PSTH_avg + PSTH_sd
	reverse PSTH_sd_top
	Duplicate/o PSTH_sd_top, PSTH_sd_top_x
	PSTH_sd_top_x = p*BinSizePSTH - floor(BeforeTTLs/BinSizePSTH)*BinSizePSTH + BinSizePSTH/2
	reverse PSTH_sd_top_x
	Duplicate/o PSTH_sd, PSTH_sd_bottom; PSTH_sd_bottom = PSTH_avg - PSTH_sd
	Duplicate/o PSTH_sd_bottom, PSTH_sd_bottom_x
	PSTH_sd_bottom_x = p*BinSizePSTH - floor(BeforeTTLs/BinSizePSTH)*BinSizePSTH + BinSizePSTH/2
	concatenate/NP/o {PSTH_sd_bottom, PSTH_sd_top}, PSTH_sd_box
	concatenate/NP/o {PSTH_sd_bottom_x, PSTH_sd_top_x}, PSTH_sd_box_x
	
	SetDrawLayer UserBack
	SetDrawEnv/W=WindowTTLs xcoord=bottom, ycoord=left, fillpat=3, fillfgc=(65280,48896,48896), linefgc=(65535,65535,65535)
	DrawPoly PSTH_sd_box_x[0], PSTH_sd_box[0], 1, 1, PSTH_sd_box_x, PSTH_sd_box
	
	Wavestats/Q PSTH_sd_box
	SetAxis/W=WindowTTLs left 0,V_max*1.1

	// Display pulse
	GetAxis/Q/W=WindowTTLs left
	if (TickAbove == 1)
		upperlimit = V_max; lowerlimit = V_max - (V_max - V_min)/15
	endif
	if (TickBelow == 1)
		upperlimit = V_min + (V_max - V_min)/15; lowerlimit = V_min
	endif
	if (TickAcross == 1)
		upperlimit = V_max; lowerlimit = V_min
	endif
			
	SetDrawEnv/W=WindowTTLs fillfgc=(0,0,65535), linefgc=(0,0,65535), xcoord=bottom, ycoord=left
	if (FillTicks_WindowTTLs)
		SetDrawEnv/W=WindowTTLs fillpat=1
	else
		SetDrawEnv/W=WindowTTLs fillpat=0
	endif
	DrawRect/W=WindowTTLs 0, upperlimit, PulseWidthVar, lowerlimit
		
	// ControlBar
	ControlBar/W=WindowTTLs 70
	ModifyGraph/W=WindowTTLs cbRGB=(60000,60000,60000)
	SetDataFolder root:spacan:WindowTTLs
	Variable/G ScaleNum_WindowTTLs = NaN
	Variable/G YScaleBar_WindowTTLs = NaN
	Variable/G XScaleBar_WindowTTLs = NaN
	SetVariable SetScaleNum_WindowTTLs	pos={10,10},size={120,20},labelback=(60000,60000,60000),title="Scale bar #:",value=ScaleNum_WindowTTLs,proc=ProcSetScaleNum,limits={1,inf,1}
	SetVariable SetYScaleB_WindowTTLs 	pos={140,10},size={90,20},labelback=(60000,60000,60000),title="Y =",value=YScaleBar_WindowTTLs,proc=ProcSetYScaleB
	SetVariable SetXScaleB_WindowTTLs 	pos={240,10},size={90,20},labelback=(60000,60000,60000),title="X =",value=XScaleBar_WindowTTLs,proc=ProcSetXScaleB
	Button ButtonDoneScaleBars			pos={10,35},size={140,20},labelback=(60000,60000,60000),title="Done editing scale bars",proc=ProcDoneScaleBars
	Button ButtonEraseScaleBars			pos={160,35},size={110,20},labelback=(60000,60000,60000),title="Delete scale bars",proc=ProcEraseScaleBars
	Button ButtonSaveWindowTTLs			pos={400,9},size={60,20},labelback=(60000,60000,60000),title="Save fig.",proc=ProcSaveWindowTTLs
	Button ButtonCloseWindowTTLs		pos={470,9},size={40,20},labelback=(60000,60000,60000),title="Close",proc=ProcCloseWindowTTLs
	CheckBox CBGridWindowTTLs			pos={350,10},size={40,20},labelback=(60000,60000,60000),title="Grid",proc=ProcGridWindowTTLs
	
	SetDataFolder root:spacan
END

FUNCTION ProcButtonPlotLightPulses (ctrlName) : ButtonControl
	String ctrlName
	
	Nvar C1E, C2E
	If (C1E == 0 && C2E == 0)
		Abort "No channel loaded!"
	endif
	If (C1E == 1 && C2E == 0)
		Abort "Channel 2 not loaded!"
	endif
	If (C1E == 0 && C2E == 1)
		Abort "Channel 1 not loaded!"
	endif	
	
	Nvar CheckTTLch1, CheckTTLch2
	Variable ChNumber
	if (CheckTTLch1 == 1)
		ChNumber = 1
	endif
	if (CheckTTLch2 == 1)
		ChNumber = 2
	endif
	
	Nvar Ch1Detected, Ch2Detected
	
	If (CheckTTLch1 == 1 && Ch1Detected == 0)
		Abort "Detect pulses in channel 1 first!"
	endif
	If (CheckTTLch2 == 1 && Ch2Detected == 0)
		Abort "Detect pulses in channel 2 first!"
	endif	
	
	Nvar TickAbove, TickBelow, TickAcross
	Nvar CheckTTLch1, CheckTTLch2
	Nvar FillTicks_WindowTTLs
	Nvar CheckRangeTTLs
	Nvar CheckPSTHUseTags
	Nvar RangeStartTTLs, RangeEndTTLs
	Nvar BeforeTTLs, AfterTTLs
	Nvar BinSizePSTH
	Nvar CheckNormPSTH
	Nvar CheckShowPulseNumber
	
	Wave/T Params
	Params[4][5] = num2str(CheckTTLch1)
	Params[5][5] = num2str(CheckTTLch2)
	Params[6][5] = num2str(TickAbove)
	Params[7][5] = num2str(TickBelow)
	Params[8][5] = num2str(TickAcross)
	Params[4][5] = num2str(CheckTTLch1)
	Params[5][5] = num2str(CheckTTLch2)
	Params[9][5] = num2str(FillTicks_WindowTTLs)
	Params[10][5] = num2str(CheckRangeTTLs)
	Params[11][5] = num2str(CheckPSTHUseTags)
	Params[12][5] = num2str(RangeStartTTLs)
	Params[13][5] = num2str(RangeEndTTLs)
	Params[14][5] = num2str(BeforeTTLs)
	Params[15][5] = num2str(AfterTTLs)
	Params[16][5] = num2str(BinSizePSTH)
	Params[17][5] = num2str(CheckNormPSTH)
	Params[19][5] = num2str(CheckShowPulseNumber)
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	
	PlotLightPulses(ChNumber)
END

FUNCTION GetPulses(ChNumber,Vmin,Vmax)
	Variable ChNumber
	Variable Vmin
	Variable Vmax
	
	// Svar
	Svar Filename = $"Filename" + num2str(ChNumber)
	// Waves
	Wave RealAmp = $"RealAmp" + num2str(ChNumber)
	Wave Occurrence = $"Occurrence" + num2str(ChNumber)
	Wave TTLWave = $Filename

	// Global variables
	Nvar SIa = $"SI" + num2str(ChNumber)

	Variable i

	Variable AvAmp
	Wavestats/Q/R=(Vmin,Vmax) RealAmp; AvAmp = V_avg

	// Look for crosses with increasing value
	FindLevels/Q/P/R=(Vmin,Vmax) TTLWave, (AvAmp/2)
	Wave W_FindLevels
	Variable PulseBegCounter = 0
	Variable PulseEndCounter = 0
	Make/O/N=0 PulseBegin
	Make/O/N=0 PulseEnd
	For (i = 0; i < numpnts(W_FindLevels); i += 1)
		if (TTLWave[floor(W_FindLevels[i])] < TTLWave[floor(W_FindLevels[i])+1])
			InsertPoints PulseBegCounter, 1, PulseBegin
			PulseBegin[i] = floor(W_FindLevels[i])*SIa*1e-6
			PulseBegCounter += 1
		endif
		if (TTLWave[floor(W_FindLevels[i])] > TTLWave[floor(W_FindLevels[i])+1])
			InsertPoints PulseEndCounter, 1, PulseEnd
			PulseEnd[i] = floor(W_FindLevels[i])*SIa*1e-6
			PulseEndCounter += 1
		endif		
	Endfor
	
//	Duplicate/o W_FindLevels PulseBegin
//	PulseBegin = floor(W_FindLevels[p])*SIa*1e-6
	
	// Look for crosses with decreading value
//	FindLevels/Q/P/EDGE=2/R=(Vmin,Vmax) TTLWave, (AvAmp/2)
//	Wave W_FindLevels
//	Duplicate/o W_FindLevels PulseEnd
//	PulseEnd = (floor(W_FindLevels[p]))*SIa*1e-6

	Duplicate/o PulseBegin PulseWidth
	PulseWidth = PulseEnd - PulseBegin
END

// FUNCTION THAT PLOTS THE TRACE WITH BLUE BOXES SYMBOLIZING THE TTL PULSES
FUNCTION PlotLightPulses(ChNumber)
	Variable ChNumber	// The channel were the TTL are
	
	KillDataFolder/Z root:spacan:WindowTTLs
	
	GetAxis/Q/W=MainWindow#$"Channel" + num2str(3 - ChNumber) bottom
	GetPulses(ChNumber,V_min,V_max)
	NewDataFolder/o root:spacan:WindowTTLs
	// Data folder name has to be the same as window name

	// Waves
	Wave PulseWidth, PulseBegin, PulseEnd
	Wave Occurrence = $"Occurrence" + num2str(ChNumber)
	Wave Tags = $"Tags" + num2str(ChNumber)
	// Global variables
	Nvar SIb = $"SI" + num2str(3 - ChNumber)
	Nvar TickAbove, TickBelow, TickAcross
	Nvar FillTicks_WindowTTLs
	Nvar CheckRangeTTLs
	Nvar RangeStartTTLs
	Nvar RangeEndTTLs
	Nvar BeforeTTLs
	Nvar AfterTTLs
	Nvar Length
	Nvar CheckPSTHUseTags
	Nvar CheckShowPulseNumber
	// Global strings
	Svar FilenameA = $"Filename" + num2str(ChNumber)
	Svar FilenameB = $"Filename" + num2str(3 - ChNumber)

	Variable i
	Variable upperlimit, lowerlimit

	if (CheckRangeTTLs == 0)

		// Display the trace in a new window and remove the axis
		GetAxis/Q/W=MainWindow#$"Channel" + num2str(3 - ChNumber) bottom
		Duplicate/R=(V_min,V_max)/o $FilenameB RecordedTrace
		SetScale/P x V_min, SIb*1e-6, "s", RecordedTrace
		DoWindow/K WindowTTLs
		Display/W=(50,50,600,360) RecordedTrace as "TTL as ticks"
		DoWindow/C WindowTTLs
		ModifyGraph/W=WindowTTLs rgb(RecordedTrace)=(0,0,0)
		ModifyGraph/W=WindowTTLs margin=20
		ModifyGraph/W=WindowTTLs noLabel=2,axThick=0,standoff=0
		// Set limits of the blue boxes according to the checkbox (TickAbove, TickBelow or TickAcross)
		GetAxis/W=MainWindow#$"Channel" + num2str(3 - ChNumber) $"Left" + num2str(3 - ChNumber)
		SetAxis/W=WindowTTLs left V_min,V_max
		if (TickAbove == 1)
			upperlimit = V_max; lowerlimit = V_max - (V_max - V_min)/15
		endif
		if (TickBelow == 1)
			upperlimit = V_min + (V_max - V_min)/15; lowerlimit = V_min
		endif
		if (TickAcross == 1)
			upperlimit = V_max; lowerlimit = V_min
		endif
		// Draw the blue boxes
		SetDrawLayer/W=WindowTTLs UserBack
		For (i = 0; i < numpnts(Pulsewidth); i += 1)
			SetDrawEnv/W=WindowTTLs fillfgc=(0,0,65535), linefgc=(0,0,65535), xcoord=bottom, ycoord=left
			if (FillTicks_WindowTTLs)
				SetDrawEnv/W=WindowTTLs fillpat=1
			else
				SetDrawEnv/W=WindowTTLs fillpat=0
			endif
			DrawRect/W=WindowTTLs PulseBegin[i], upperlimit, PulseEnd[i], lowerlimit
		Endfor
	
		GetAxis/Q/W=MainWindow#$"Channel" + num2str(3 - ChNumber) bottom
		SetAxis bottom V_min,V_max

	endif
	
	if (CheckRangeTTLs == 1)
		
		DoWindow/K WindowTTLs
		Display/W=(220,38,640,510)
		DoWindow/C WindowTTLs
		
		String Filename = "root:spacan:" + FilenameB
		
		GetPulses(ChNumber,0,Length)
		
	//	SetDataFolder root:spacan:WindowTTLs
		
		Variable RowCounter = 0
		
		SetDrawLayer/W=WindowTTLs UserBack
		For (i = RangeStartTTLs; i < (RangeEndTTLs + 1); i += 1)
			// For every line display segment of trace
			// But only if tag = 1 in the case that CheckPSTHUseTags = 1
			String LeftAxisName = "Left" + num2str(i)
			if (CheckPSTHUseTags == 0)
				Duplicate/o/R=(PulseBegin[i] - BeforeTTLs, PulseEnd[i] + AfterTTLs) $Filename $"root:spacan:WindowTTLs:RecSegment" + num2str(i)
				SetScale/P x -BeforeTTLs, SIb*1e-6,"s" $"root:spacan:WindowTTLs:RecSegment" + num2str(i)
				AppendToGraph/W=WindowTTLs/L=$LeftAxisName $"root:spacan:WindowTTLs:RecSegment" + num2str(i)
				UpperLimit = 1 - (i - RangeStartTTLs)/(RangeEndTTLs - RangeStartTTLs + 1)
				if (i == RangeEndTTLs)
					LowerLimit = 0
				endif
				if (i < RangeEndTTLs)
					LowerLimit = 1 - (i - RangeStartTTLs)/(RangeEndTTLs - RangeStartTTLs + 1) - 1/(RangeEndTTLs - RangeStartTTLs + 1)
				endif
				ModifyGraph/W=WindowTTLs axisEnab($LeftAxisName)={LowerLimit,UpperLimit}
				ModifyGraph/W=WindowTTLs rgb($"RecSegment" + num2str(i))=(0,0,0)
				ModifyGraph/W=WindowTTLs margin=20
				ModifyGraph/W=WindowTTLs noLabel=2,axThick=0,standoff=0
				
				// For every line display pulse
				DoUpdate
				GetAxis/Q/W=WindowTTLs $LeftAxisName
				if (TickAbove == 1)
					upperlimit = V_max; lowerlimit = V_max - (V_max - V_min)/15
				endif
				if (TickBelow == 1)
					upperlimit = V_min + (V_max - V_min)/15; lowerlimit = V_min
				endif
				if (TickAcross == 1)
					upperlimit = V_max; lowerlimit = V_min
				endif
				
				SetDrawEnv/W=WindowTTLs fillfgc=(0,0,65535), linefgc=(0,0,65535), xcoord=bottom, ycoord=$LeftAxisName
				if (FillTicks_WindowTTLs)
					SetDrawEnv/W=WindowTTLs fillpat=1
				else
					SetDrawEnv/W=WindowTTLs fillpat=0
				endif
				DrawRect/W=WindowTTLs 0, upperlimit, PulseWidth[i], lowerlimit
				if (CheckShowPulseNumber)	// add pulse number
					SetDrawLayer UserBack
					SetDrawEnv/W=WindowTTLs xcoord=bottom, ycoord=$LeftAxisName, fstyle= 1, fsize=12, textrgb=(0,0,65535)
					DrawText PulseWidth[i] + 0.05*(BeforeTTLs + PulseWidth[i] + AfterTTLs),lowerlimit,num2str(i)
				endif
			else
				if (Tags[i] == 1)
					Duplicate/o/R=(PulseBegin[i] - BeforeTTLs, PulseEnd[i] + AfterTTLs) $Filename $"root:spacan:WindowTTLs:RecSegment" + num2str(i)
					SetScale/P x -BeforeTTLs, SIb*1e-6,"s" $"root:spacan:WindowTTLs:RecSegment" + num2str(i)
					AppendToGraph/W=WindowTTLs/L=$LeftAxisName $"root:spacan:WindowTTLs:RecSegment" + num2str(i)
					UpperLimit = 1 - RowCounter/(RangeEndTTLs - RangeStartTTLs + 1)
					//if (i == RangeEndTTLs)
					//	LowerLimit = 0
					//endif
				//	if (i < RangeEndTTLs)
				
					if (RowCounter == RangeEndTTLs)
						LowerLimit = 0
					endif
					if (RowCounter < RangeEndTTLs)
						LowerLimit = UpperLimit - 1/(RangeEndTTLs - RangeStartTTLs + 1)
					endif
					print "LowerLimit:" + num2str(LowerLimit)
					print "UpperLimit:" + num2str(UpperLimit)
					ModifyGraph/W=WindowTTLs axisEnab($LeftAxisName)={LowerLimit,UpperLimit}
					ModifyGraph/W=WindowTTLs rgb($"RecSegment" + num2str(i))=(0,0,0)
					ModifyGraph/W=WindowTTLs margin=20
					ModifyGraph/W=WindowTTLs noLabel=2,axThick=0,standoff=0
					
					// For every line display pulse
					DoUpdate
					GetAxis/Q/W=WindowTTLs $LeftAxisName
					if (TickAbove == 1)
						upperlimit = V_max; lowerlimit = V_max - (V_max - V_min)/15
					endif
					if (TickBelow == 1)
						upperlimit = V_min + (V_max - V_min)/15; lowerlimit = V_min
					endif
					if (TickAcross == 1)
						upperlimit = V_max; lowerlimit = V_min
					endif
					
					SetDrawEnv/W=WindowTTLs fillfgc=(0,0,65535), linefgc=(0,0,65535), xcoord=bottom, ycoord=$LeftAxisName
					if (FillTicks_WindowTTLs)
						SetDrawEnv/W=WindowTTLs fillpat=1
					else
						SetDrawEnv/W=WindowTTLs fillpat=0
					endif
					DrawRect/W=WindowTTLs 0, upperlimit, PulseWidth[i], lowerlimit
					if (CheckShowPulseNumber)	// add pulse number
						SetDrawLayer UserBack
						SetDrawEnv/W=WindowTTLs xcoord=bottom, ycoord=$LeftAxisName, fstyle= 1, fsize=12, textrgb=(0,0,65535)
						DrawText PulseWidth[i] + 0.05*(BeforeTTLs + PulseWidth[i] + AfterTTLs),lowerlimit,num2str(i)
					endif
					RowCounter += 1
				endif
			endif
		Endfor
		
	//	SetDataFolder root:spacan
	endif

	ControlBar/W=WindowTTLs 70
	ModifyGraph/W=WindowTTLs cbRGB=(60000,60000,60000)
	SetDataFolder root:spacan:WindowTTLs
	Variable/G ScaleNum_WindowTTLs = NaN
	Variable/G YScaleBar_WindowTTLs = NaN
	Variable/G XScaleBar_WindowTTLs = NaN
	SetVariable SetScaleNum_WindowTTLs	pos={10,10},size={120,20},labelback=(60000,60000,60000),title="Scale bar #:",value=ScaleNum_WindowTTLs,proc=ProcSetScaleNum,limits={1,inf,1}
	SetVariable SetYScaleB_WindowTTLs 	pos={140,10},size={90,20},labelback=(60000,60000,60000),title="Y =",value=YScaleBar_WindowTTLs,proc=ProcSetYScaleB
	SetVariable SetXScaleB_WindowTTLs 	pos={240,10},size={90,20},labelback=(60000,60000,60000),title="X =",value=XScaleBar_WindowTTLs,proc=ProcSetXScaleB
	Button ButtonDoneScaleBars			pos={10,35},size={140,20},labelback=(60000,60000,60000),title="Done editing scale bars",proc=ProcDoneScaleBars
	Button ButtonEraseScaleBars			pos={160,35},size={110,20},labelback=(60000,60000,60000),title="Delete scale bars",proc=ProcEraseScaleBars
	Button ButtonSaveWindowTTLs			pos={400,9},size={60,20},labelback=(60000,60000,60000),title="Save fig.",proc=ProcSaveWindowTTLs
	Button ButtonCloseWindowTTLs		pos={470,9},size={40,20},labelback=(60000,60000,60000),title="Close",proc=ProcCloseWindowTTLs
	CheckBox CBGridWindowTTLs			pos={350,10},size={40,20},labelback=(60000,60000,60000),title="Grid",proc=ProcGridWindowTTLs
	
	SetDataFolder root:spacan
END

FUNCTION ProcGridWindowTTLs(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=WindowTTLs grid=1,tick=3,axThick=1
		ModifyGraph axRGB=(65535,65535,65535)
	else
		ModifyGraph/W=WindowTTLs grid=0,axThick=0
	endif
END

Function ProcSaveWindowTTLs(ctrlName) : ButtonControl
	String ctrlName
	SavePICT/EF=1/E=-3
end

Function ProcCloseWindowTTLs(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K WindowTTLs
	KillDataFolder root:spacan:WindowTTLs
end

// TODO
// round scale bar values
// explanation pics in panel
// eventually help buttons in panel to explain step by step
// See if scale managing functions can work on superimpose events
// Put Graph Control Panel functions in Plots tab in MainWindow.

Function ProcDoneScaleBars(ctrlName) : ButtonControl
	String ctrlName
	
	// Get name of the window
	String WindowName
	getwindow kwTopWin activeSW
	WindowName = S_Value
	
	Execute "SetDataFolder root:spacan:" + WindowName + ":ScaleBars"
	
	// Get the list of waves
	Variable Counter = 0
	GetWindow $WindowName, WaveList
	Wave/T W_WaveList
	Variable i
	String TEMPSTR
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,8], "YScaleBar" ) == 0)
			ModifyGraph/W=$WindowName rgb($TEMPSTR)=(0,0,0)
			ModifyGraph/W=$WindowName lsize($TEMPSTR)=1
		endif		
	endfor
	
	KillWaves W_WaveList
	
	SetDataFolder root:spacan
End

Function ProcEraseScaleBars(ctrlName) : ButtonControl
	String ctrlName

	// Get name of the window
	String WindowName
	getwindow kwTopWin activeSW
	WindowName = S_Value
	
	// Erase legends of scale bars
	SetDrawLayer/K UserFront
	
	Execute "SetDataFolder root:spacan:" + WindowName + ":ScaleBars"
	
	// Get the list of waves
	Variable Counter = 0
	GetWindow $WindowName, WaveList
	Wave/T W_WaveList
	Variable i
	String TEMPSTR
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,8], "YScaleBar" ) == 0)
			print TEMPSTR
			RemovefromGraph/W=$WindowName $TEMPSTR
			KillWaves $TEMPSTR
		endif		
	endfor
	
	Nvar ScaleNum = $"ScaleNum_" + WindowName
	Nvar YScaleBar = $"YScaleBar_" + WindowName
	Nvar XScaleBar = $"XScaleBar_" + WindowName
	ScaleNum = NaN; YScaleBar = NaN; XScaleBar = NaN
	
	SetDataFolder root:spacan
end

// THIS FUNCTION IS PART OF THE NEW SCALE BAR MANAGEMENT SYSTEM (version >= 1.04)
Function ProcSetScaleNum(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	// Get name of the window
	String WindowName
	getwindow kwTopWin activeSW
	WindowName = S_Value
	
	// Get the list of waves
	Variable Counter = 0
	GetWindow $WindowName, WaveList
	Wave/T W_WaveList
	Variable i
	String TEMPSTR
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,8], "YScaleBar" ) == 0)	// If name of wave begins with "YScaleBar"
			counter = str2num(TEMPSTR[9])
		endif		
	endfor
	// Counter gives the number of scale bars on the graph
	
	Execute "SetDataFolder root:spacan:" + WindowName + ":ScaleBars"
	
	// ScaleNum identifies the topmost scale bar
	
	Nvar ScaleNum = $"ScaleNum_" + WindowName
	
	if (ScaleNum > counter)
		ScaleNum = counter
	endif

	// Get the size of the scale bar identified by ScaleNum

	Nvar YScaleBarSize = $"YScaleBar_" + WindowName
	Nvar XScaleBarSize = $"XScaleBar_" + WindowName

	// need to get the whole name of the scale bar waves
	// Need to look into W_wavelist
	String YScaleBarWaveName
	String XScaleBarWaveName
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,9], "YScaleBar" + num2str(ScaleNum)) == 0)
			YScaleBarWaveName = TEMPSTR
		endif
		if (cmpstr(TEMPSTR[0,9], "XScaleBar" + num2str(ScaleNum)) == 0)
			XScaleBarWaveName = TEMPSTR
		endif		
	endfor

	Wave YScaleBarWave = $YScaleBarWaveName
	YScaleBarSize = YScaleBarWave[0] - YScaleBarWave[1]
	
	Wave XScaleBarWave = $XScaleBarWaveName
	XScaleBarSize = XScaleBarWave[2] - XScaleBarWave[1]

	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,8], "YScaleBar" ) == 0)
			if (str2num(TEMPSTR[9]) == ScaleNum)
				ModifyGraph/W=$WindowName rgb($TEMPSTR)=(65535,0,0)
				ModifyGraph/W=$WindowName lsize($TEMPSTR)=2
			else
				ModifyGraph/W=$WindowName rgb($TEMPSTR)=(0,0,0)
				ModifyGraph/W=$WindowName lsize($TEMPSTR)=1
			endif
		endif		
	endfor

	KillWaves W_WaveList

	SetDataFolder root:spacan

End

Function ProcSetScaleB(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
		
	// Get name of the window
	String WindowName
	getwindow kwTopWin activeSW
	WindowName = S_Value
	
	Execute "SetDataFolder root:spacan:" + WindowName + ":ScaleBars"
	
	// Get the name of the topmost scale bar (identified by ScaleNum)
	Nvar ScaleNum = $"ScaleNum_" + WindowName
	GetWindow $WindowName, WaveList
	Wave/T W_WaveList
	Variable i, j, k, m
	String TEMPSTR
	String YScaleBarWaveName
	String XScaleBarWaveName
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,9], "YScaleBar" + num2str(ScaleNum)) == 0)
			YScaleBarWaveName = TEMPSTR
		endif
		if (cmpstr(TEMPSTR[0,9], "XScaleBar" + num2str(ScaleNum)) == 0)
			XScaleBarWaveName = TEMPSTR
		endif		
	endfor
	
	Nvar XScaleBarSize = $"XScaleBar_" + WindowName
	Nvar YScaleBarSize = $"YScaleBar_" + WindowName
	
	Wave XScaleBarWave = $XScaleBarWaveName
	Wave YScaleBarWave = $YScaleBarWaveName
	XScaleBarWave[2] = XScaleBarWave[1] + XScaleBarSize
	YScaleBarWave[0] = YScaleBarWave[1] + YScaleBarSize
	
	// Now erase all text and redraw (only way to do it)
	String xAxis, yAxis
	SetDrawLayer/K UserFront
	GetWindow $WindowName, WaveList
	Wave/T W_WaveList
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,8], "YScaleBar" ) == 0)

			// Get axis names for Y wave
			k = 11
			yAxis = ""
			Do
				yAxis += TEMPSTR[k]
				k += 1
			While (cmpstr(TEMPSTR[k], "_") == -1)

			String Dummy = TEMPSTR[9]
			Duplicate/o $"YScaleBar" + Dummy + "_" + yAxis + "_" + WindowName YDummy

			// Find X wave with same ScaleNum and get axis
			string TEMPSTR_X
			for (j=0; j < numpnts(W_WaveList)/3; j+=1)
				TEMPSTR_X = W_WaveList[j][0]
				if (cmpstr(TEMPSTR_X[0,9], "XScaleBar" + TEMPSTR[9]) == 0)
					m = 11
					xAxis = ""
					Do
						xAxis += TEMPSTR_X[m]
						m += 1
					While (cmpstr(TEMPSTR_X[m], "_") == -1)
					Dummy = TEMPSTR[9]
					Duplicate/o $"XScaleBar" + Dummy + "_" + xAxis + "_" + WindowName XDummy
				endif
			endfor

			//  Redraw text on scale bar

			Variable xSize = XDummy[2] - XDummy[1]
			Variable ySize = YDummy[0] - YDummy[1]
			
			Variable left = XDummy[1]
			Variable bottom = YDummy[1]
			
			String xLabel = "s"
			String yLabel = ""
			
			String Label
			
			// Draw x-Axis Text
			label = num2str(xSize)
			if (strlen(xLabel) > 0)
				label += " "
				label += xLabel
			endif
			SetDrawEnv xcoord = $xAxis, ycoord = $yAxis, textxjust = 1, textyjust = 1
			DrawText left + xSize/2, bottom - 0.1*ySize, label

			// Draw y-Axis Text
			label = num2str (ySize)
			if (strlen (yLabel) > 0)
				label += " "
				label += yLabel
			endif
			SetDrawEnv xcoord = $xAxis, ycoord = $yAxis, textxjust = 1, textyjust = 1, textrot = 90
			DrawText left - 0.1*xSize, bottom + ySize/2, label

		endif
		
	endfor	
	
	SetDataFolder root:spacan
End

// THIS FUNCTION IS PART OF THE NEW SCALE BAR MANAGEMENT SYSTEM (version >= 1.04)
Function DrawScale() : GraphMarquee

	// Scale bar wave names use the following structure:
	// XScaleBar#_XAxisName_WindowName
	// YScaleBar#_YAxisName_WindowName
	// where # is the number of the scale bar

	// Get name of the window
	String WindowName
	getwindow kwTopWin activeSW
	WindowName = S_Value
	
	// Get the list of waves
	Variable Counter = 0
	GetWindow $WindowName, WaveList
	Wave/T W_WaveList
	Variable i
	String TEMPSTR
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,8], "YScaleBar" ) == 0)	// If wave name begins with "YScaleBar"
			Counter += 1
		endif		
	endfor
	// If counter stays at 0, then no previous scalebar on the graph
	// Otherwise, counter gives the number of the last scale bar
	
	// Update the global variable associated to the scale number
	Execute "SetDataFolder root:spacan:" + WindowName + ":ScaleBars"
	Nvar ScaleNum = $"ScaleNum_" + WindowName
	ScaleNum = Counter+1
	
	// Get the names of the selected axis (in List Box)
	Wave/T VertAxisList
	Wave/T HorizAxisList
	
	String LBHorizName = "LBHorizAxis_" + WindowName
	ControlInfo/W=$WindowName $LBHorizName
	String xAxis = HorizAxisList[V_value]
	
	String LBVertName = "LBVertAxis_" + WindowName
	ControlInfo/W=$WindowName $LBVertName
	String yAxis = VertAxisList[V_value]
	
	// Get the limits of the graph marquee
	String format
	GetMarquee/K $yAxis,$xAxis
	format = "flag: %g; left: %g; top: %g; right: %g; bottom: %g\r"
	
	Variable left = V_left
	Variable bottom = V_bottom
	Variable xSize = V_right - V_left
	Variable ySize = V_top - V_bottom
	
	// Prepare waves for new scale bar
	Make/N=3/o $"YScaleBar" + num2str(counter+1) + "_" + yAxis + "_" + WindowName
	Make/N=3/o $"XScaleBar" + num2str(counter+1) + "_" + xAxis + "_" + WindowName
	Wave YScaleBar = $"YScaleBar" + num2str(counter+1) + "_" + yAxis + "_" + WindowName
	Wave XScaleBar = $"XScaleBar" + num2str(counter+1) + "_" + xAxis + "_" + WindowName
	
	YScaleBar[0] = bottom + ySize; YScaleBar[1] = bottom; YScaleBar[2] = bottom
	XScaleBar[0] = left; XScaleBar[1] = left; XScaleBar[2] = left + xSize
	
	if (stringmatch(yAxis,"right") == 1)
		Appendtograph/W=$WindowName/R YScaleBar vs XScaleBar
	endif
	if (stringmatch(yAxis,"left") == 1)
		Appendtograph/W=$WindowName YScaleBar vs XScaleBar
	endif
	if (stringmatch(yAxis,"right") == 0 && stringmatch(yAxis,"left") == 0)
		Appendtograph/W=$WindowName/L=$yAxis YScaleBar vs XScaleBar
	endif	
	String YScaleBarName = "YScaleBar" + num2str(counter+1) + "_" + yAxis + "_" + WindowName
	ModifyGraph/W=$WindowName rgb($YScaleBarName)=(0,0,0)
	
	// Update the global variables associated with the topmost (latest) scale bar
	Nvar YScaleBarSize = $"YScaleBar_" + WindowName
	YScaleBarSize = ySize
	Nvar XScaleBarSize = $"XScaleBar_" + WindowName
	XScaleBarSize = xSize

	String xLabel = "s"
	//  Change this
	String yLabel = ""

	SetDrawLayer UserFront

	String label

	// Draw x-Axis Text
	label = num2str(xSize)
	if (strlen(xLabel) > 0)
		label += " "
		label += xLabel
	endif
	SetDrawEnv xcoord = $xAxis, ycoord = $yAxis, textxjust = 1, textyjust = 1
	DrawText left + xSize/2, bottom - 0.1*ySize, label

	// Draw y-Axis Text
	label = num2str (ySize)
	if (strlen (yLabel) > 0)
		label += " "
		label += yLabel
	endif
	SetDrawEnv xcoord = $xAxis, ycoord = $yAxis, textxjust = 1, textyjust = 1, textrot = 90
	DrawText left - 0.1*xSize, bottom + ySize/2, label
	
	SetDataFolder root:spacan

	ResumeUpdate
End
